package org.openxma.dsl.generator.impl;

import at.spardat.xma.guidesign.XMAComponent;
import com.google.inject.Singleton;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.ConditionsBlock;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.dom.model.Dependant;
import org.openxma.dsl.generator.impl.BackingBean;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.JsfData;
import org.openxma.dsl.generator.impl.JsfEvents;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.XmadslPage;

@Singleton
@SuppressWarnings("all")
public class BackingBeanImpl implements BackingBean {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private JsfEvents _jsfEvents;
  
  @Inject
  private JsfData _jsfData;
  
  public CharSequence toComponentBackingBean(final String namespace, final Component it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Backing Bean for Component ");
    XMAComponent _xmaComponent = it.getXmaComponent();
    String _namClass = _xmaComponent.getNamClass();
    _builder.append(_namClass, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("@");
    String _addImport = this._names.addImport(it, "org.springframework.stereotype.Component");
    _builder.append(_addImport, "");
    _builder.append("(\"");
    XMAComponent _xmaComponent_1 = it.getXmaComponent();
    String _namClass_1 = _xmaComponent_1.getNamClass();
    String _firstLower = StringExtensions.toFirstLower(_namClass_1);
    _builder.append(_firstLower, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("@");
    String _addImport_1 = this._names.addImport(it, "org.springframework.context.annotation.Scope");
    _builder.append(_addImport_1, "");
    _builder.append("(\"session\")");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    XMAComponent _xmaComponent_2 = it.getXmaComponent();
    String _namClass_2 = _xmaComponent_2.getNamClass();
    _builder.append(_namClass_2, "");
    _builder.append(" extends ");
    XMAComponent _xmaComponent_3 = it.getXmaComponent();
    String _namClass_3 = _xmaComponent_3.getNamClass();
    _builder.append(_namClass_3, "");
    _builder.append("Gen {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toBackingBean(final String namespace, final XmadslPage it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Backing Bean for Page ");
    String _name = it.getName();
    _builder.append(_name, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("@");
    String _addImport = this._names.addImport(it, "org.springframework.stereotype.Component");
    _builder.append(_addImport, "");
    _builder.append("(\"");
    String _name_1 = it.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("@");
    String _addImport_1 = this._names.addImport(it, "org.springframework.context.annotation.Scope");
    _builder.append(_addImport_1, "");
    _builder.append("(\"access\")");
    _builder.newLineIfNotEmpty();
    _builder.append("@");
    String _addImport_2 = this._names.addImport(it, "org.apache.myfaces.orchestra.viewController.annotations.ViewController");
    _builder.append(_addImport_2, "");
    _builder.append("(viewIds={\"");
    String _faceletPageFileName = this._files.faceletPageFileName(it, false);
    _builder.append(_faceletPageFileName, "");
    _builder.append("\"})");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "");
    _builder.append(" extends ");
    String _name_3 = it.getName();
    _builder.append(_name_3, "");
    _builder.append("Gen {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toAbstractComponentBackingBean(final String namespace, final Component it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Abstract Backing Bean Base for Component ");
    XMAComponent _xmaComponent = it.getXmaComponent();
    String _namClass = _xmaComponent.getNamClass();
    _builder.append(_namClass, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("public abstract class ");
    XMAComponent _xmaComponent_1 = it.getXmaComponent();
    String _namClass_1 = _xmaComponent_1.getNamClass();
    _builder.append(_namClass_1, "");
    _builder.append("Gen extends ");
    String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.jsf.beans.ComponentBackingBean");
    _builder.append(_addImport, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    EList<Dependant> _dependencies = it.getDependencies();
    CharSequence _injectedDependencies = this._jsfData.toInjectedDependencies(_dependencies);
    _builder.append(_injectedDependencies, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    EList<ObjectProperty> _componentProperties = it.getComponentProperties();
    CharSequence _objectPropertyVars = this._jsfData.toObjectPropertyVars(_componentProperties);
    _builder.append(_objectPropertyVars, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    EList<DataObjectVariable> _dataobjects = it.getDataobjects();
    CharSequence _dataObjectVars = this._jsfData.toDataObjectVars(_dataobjects);
    _builder.append(_dataObjectVars, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    EventMappingList _events = it.getEvents();
    CharSequence _lifecycleMethods = _events==null?(CharSequence)null:this._jsfEvents.toLifecycleMethods(_events);
    _builder.append(_lifecycleMethods, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// commands");
    _builder.newLine();
    _builder.append("\t");
    EList<Command> _commands = it.getCommands();
    CharSequence _abstractCommands = this.toAbstractCommands(_commands);
    _builder.append(_abstractCommands, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    {
      EList<ObjectProperty> _componentProperties_1 = it.getComponentProperties();
      for(final ObjectProperty compProperty : _componentProperties_1) {
        _builder.append("    ");
        CharSequence _getterAndSetter = this._jsfData.toGetterAndSetter(compProperty);
        _builder.append(_getterAndSetter, "    ");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    {
      EList<DataObjectVariable> _dataobjects_1 = it.getDataobjects();
      for(final DataObjectVariable dataObject : _dataobjects_1) {
        _builder.append("\t");
        CharSequence _getterAndSetter_1 = this._jsfData.toGetterAndSetter(dataObject);
        _builder.append(_getterAndSetter_1, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toAbstractBackingBean(final String namespace, final XmadslPage it, final Component comp) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Abstract Backing Bean Base for Page ");
    String _name = it.getName();
    _builder.append(_name, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("public abstract class ");
    String _name_1 = it.getName();
    _builder.append(_name_1, "");
    _builder.append("Gen extends ");
    String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.jsf.beans.PageBackingBean");
    _builder.append(_addImport, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      if (false) {
        _builder.append("public static String VIEW_ID = \"");
        String _replaceAll = namespace.replaceAll("[.]", "/");
        _builder.append(_replaceAll, "	");
        _builder.append("/");
        String _name_2 = it.getName();
        String _firstLower = StringExtensions.toFirstLower(_name_2);
        _builder.append(_firstLower, "	");
        _builder.append("\";");
      }
    }
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@");
    String _addImport_1 = this._names.addImport(it, "org.springframework.beans.factory.annotation.Autowired");
    _builder.append(_addImport_1, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("protected ");
    XMAComponent _xmaComponent = comp.getXmaComponent();
    String _namClass = _xmaComponent.getNamClass();
    _builder.append(_namClass, "	");
    _builder.append(" component;");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public ");
    XMAComponent _xmaComponent_1 = comp.getXmaComponent();
    String _namClass_1 = _xmaComponent_1.getNamClass();
    _builder.append(_namClass_1, "	");
    _builder.append(" getTypedComponent() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return component;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    EList<Dependant> _dependencies = it.getDependencies();
    CharSequence _injectedDependencies = this._jsfData.toInjectedDependencies(_dependencies);
    _builder.append(_injectedDependencies, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    EList<ObjectProperty> _pageProperties = it.getPageProperties();
    CharSequence _objectPropertyVars = this._jsfData.toObjectPropertyVars(_pageProperties);
    _builder.append(_objectPropertyVars, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// data objects");
    _builder.newLine();
    _builder.append("    ");
    EList<DataObjectVariable> _dataobjects = it.getDataobjects();
    CharSequence _dataObjectVars = this._jsfData.toDataObjectVars(_dataobjects);
    _builder.append(_dataObjectVars, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// jsf data models");
    _builder.newLine();
    _builder.append("    ");
    Content _content = it.getContent();
    CharSequence _jsfDataModel = _content==null?(CharSequence)null:this._jsfData.toJsfDataModel(_content);
    _builder.append(_jsfDataModel, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// lifecycle events");
    _builder.newLine();
    _builder.append("    ");
    EventMappingList _events = it.getEvents();
    CharSequence _lifecycleMethods = _events==null?(CharSequence)null:this._jsfEvents.toLifecycleMethods(_events);
    _builder.append(_lifecycleMethods, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// commands");
    _builder.newLine();
    _builder.append("\t");
    EList<Command> _commands = it.getCommands();
    CharSequence _abstractCommands = this.toAbstractCommands(_commands);
    _builder.append(_abstractCommands, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// events mappings");
    _builder.newLine();
    _builder.append("    ");
    EventMappingList _events_1 = it.getEvents();
    CharSequence _eventMappings = _events_1==null?(CharSequence)null:this._jsfEvents.toEventMappings(_events_1);
    _builder.append(_eventMappings, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// conditions");
    _builder.newLine();
    _builder.append("    ");
    ConditionsBlock _conditionsBlock = it.getConditionsBlock();
    CharSequence _conditions = _conditionsBlock==null?(CharSequence)null:this.toConditions(_conditionsBlock);
    _builder.append(_conditions, "    ");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      EList<ObjectProperty> _pageProperties_1 = it.getPageProperties();
      for(final ObjectProperty pageProperty : _pageProperties_1) {
        _builder.append("    ");
        CharSequence _getterAndSetter = this._jsfData.toGetterAndSetter(pageProperty);
        _builder.append(_getterAndSetter, "    ");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    {
      EList<DataObjectVariable> _dataobjects_1 = it.getDataobjects();
      for(final DataObjectVariable dataObject : _dataobjects_1) {
        _builder.append("\t");
        CharSequence _getterAndSetter_1 = this._jsfData.toGetterAndSetter(dataObject);
        _builder.append(_getterAndSetter_1, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toConditions(final ConditionsBlock conditionsBlock) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<StatusFlag> _statusFlags = conditionsBlock.getStatusFlags();
      for(final StatusFlag statusFlag : _statusFlags) {
        _builder.append("private boolean ");
        String _name = statusFlag.getName();
        _builder.append(_name, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("public boolean is");
        String _name_1 = statusFlag.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper, "");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("return ");
        String _name_2 = statusFlag.getName();
        _builder.append(_name_2, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}   ");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence toAbstractCommands(final EList<Command> commands) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final Command command : commands) {
        {
          boolean _isClientOnly = command.isClientOnly();
          boolean _not = (!_isClientOnly);
          if (_not) {
            _builder.append("public abstract void ");
            String _name = command.getName();
            _builder.append(_name, "");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
}
