package org.openxma.dsl.generator.impl

import com.google.inject.Singleton
import javax.inject.Inject
import org.eclipse.emf.common.util.EList
import org.openxma.dsl.core.model.ConditionsBlock
import org.openxma.dsl.pom.model.Command
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.XmadslPage

import static extension org.openxma.dsl.generator.lib.ImportAdapter.*


@Singleton
class BackingBeanImpl implements BackingBean {
	@Inject
	extension Names
	@Inject
	extension Files
	@Inject
	extension JsfEvents
	@Inject
	extension JsfData
	
	override CharSequence toComponentBackingBean(String namespace, Component it) { '''
package namespace.backingbean;

addImportsHere

/** 
 * Backing Bean for Component xmaComponent.namClass 
 */
@addImport("org.springframework.stereotype.Component")("xmaComponent.namClass.toFirstLower")
@addImport("org.springframework.context.annotation.Scope")("session")
public class xmaComponent.namClass extends xmaComponent.namClassGen {
}
'''
	}
	
	override CharSequence toBackingBean(String namespace, XmadslPage it) { '''
package namespace.backingbean;

addImportsHere

/** 
 * Backing Bean for Page name 
 */
@addImport("org.springframework.stereotype.Component")("name.toFirstLower")
@addImport("org.springframework.context.annotation.Scope")("access")
@addImport("org.apache.myfaces.orchestra.viewController.annotations.ViewController")(viewIds={"faceletPageFileName(false)"})
public class name extends nameGen {
}
'''
	}
	

	override CharSequence toAbstractComponentBackingBean(String namespace, Component it) {'''
package namespace.backingbean;
addImportsHere
/** 
 * Abstract Backing Bean Base for Component xmaComponent.namClass 
 */
public abstract class xmaComponent.namClassGen extends addImport("org.openxma.dsl.platform.jsf.beans.ComponentBackingBean") {
	dependencies.toInjectedDependencies
	componentProperties.toObjectPropertyVars
	
	dataobjects.toDataObjectVars
	events?.toLifecycleMethods
	
    // commands
	commands.toAbstractCommands
	
	
    FOR compProperty : componentProperties
    	compProperty.toGetterAndSetter
	ENDFOR

	FOR dataObject : dataobjects
		dataObject.toGetterAndSetter
	ENDFOR
}
	''' 
	}
	override CharSequence toAbstractBackingBean(String namespace, XmadslPage it, Component comp) {'''
package namespace.backingbean;
addImportsHere
/** 
 * Abstract Backing Bean Base for Page name 
 */
public abstract class nameGen extends addImport("org.openxma.dsl.platform.jsf.beans.PageBackingBean") {
	IF falsepublic static String VIEW_ID = "namespace.replaceAll("[.]", "/")/name.toFirstLower()";ENDIF
	
	@addImport("org.springframework.beans.factory.annotation.Autowired")
	protected comp.xmaComponent.namClass component;
	public comp.xmaComponent.namClass getTypedComponent() {
		return component;
	}
	
	dependencies.toInjectedDependencies
	pageProperties.toObjectPropertyVars
	
    // data objects
    dataobjects.toDataObjectVars
    
    // jsf data models
    content?.toJsfDataModel
    
    // lifecycle events
    events?.toLifecycleMethods
    
    // commands
	commands.toAbstractCommands
    
    // events mappings
    events?.toEventMappings
    /*content.createChildEventMappings(events)*/
    
    // conditions
    conditionsBlock?.toConditions

    FOR pageProperty : pageProperties
    	pageProperty.toGetterAndSetter
	ENDFOR

	FOR dataObject : dataobjects
		dataObject.toGetterAndSetter
	ENDFOR

}
'''		
	}
	
	
	
	override CharSequence toConditions(ConditionsBlock conditionsBlock) {'''
    	FOR statusFlag : conditionsBlock.statusFlags
    		private boolean statusFlag.name;
    		public boolean isstatusFlag.name.toFirstUpper() {
    			return statusFlag.name;
    		}   
    	ENDFOR
'''
	}
	
	
	override CharSequence toAbstractCommands(EList<Command> commands) { '''
	    FOR command : commands
			IF !command.clientOnly
    public abstract void command.name();
    		ENDIF
    	ENDFOR
	'''
	}
	
	
	
 
}