package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.openxma.dsl.core.model.EnumLiteral;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.SimpleTypeExtension;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;

@SuppressWarnings("all")
public class DataTypeGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<EnumType> _filter = Iterables.<EnumType>filter(_elements, EnumType.class);
    final Procedure1<EnumType> _function = new Procedure1<EnumType>() {
        public void apply(final EnumType it) {
          String _qualifiedClassName = DataTypeGenerator.this.getQualifiedClassName(it);
          String _javaFilePath = DataTypeGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _enumClass = DataTypeGenerator.this.toEnumClass(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _enumClass);
          String _qualifiedClassName_1 = DataTypeGenerator.this.getQualifiedClassName(it);
          String _filePath = DataTypeGenerator.this._files.toFilePath(_qualifiedClassName_1, "properties");
          CharSequence _resourceBundle = DataTypeGenerator.this.toResourceBundle(it);
          fsa.generateFile(_filePath, GeneratorConstants.OUTLET_GENERATED_RESOURCES, _resourceBundle);
        }
      };
    IterableExtensions.<EnumType>forEach(_filter, _function);
    EList<ModelElement> _elements_1 = model.getElements();
    Iterable<SimpleType> _filter_1 = Iterables.<SimpleType>filter(_elements_1, SimpleType.class);
    final Function1<SimpleType,Boolean> _function_1 = new Function1<SimpleType,Boolean>() {
        public Boolean apply(final SimpleType it) {
          boolean _isCustom = it.isCustom();
          return Boolean.valueOf(_isCustom);
        }
      };
    Iterable<SimpleType> _filter_2 = IterableExtensions.<SimpleType>filter(_filter_1, _function_1);
    final Procedure1<SimpleType> _function_2 = new Procedure1<SimpleType>() {
        public void apply(final SimpleType it) {
          String _qualifiedClassName = DataTypeGenerator.this.getQualifiedClassName(it);
          String _javaFilePath = DataTypeGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _dataTypeClass = DataTypeGenerator.this.toDataTypeClass(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _dataTypeClass);
        }
      };
    IterableExtensions.<SimpleType>forEach(_filter_2, _function_2);
  }
  
  public CharSequence toEnumClass(final EnumType it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import java.util.Locale;");
    _builder.newLine();
    _builder.append("import java.util.Map;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("import org.openxma.dsl.platform.enums.I18nEnum;");
    _builder.newLine();
    _builder.append("import org.openxma.dsl.platform.enums.I18nEnums;");
    _builder.newLine();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public enum ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" implements I18nEnum {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    {
      EList<EnumLiteral> _literals = it.getLiterals();
      boolean _hasElements = false;
      for(final EnumLiteral literal : _literals) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",\n", "    ");
        }
        String _name_1 = literal.getName();
        _builder.append(_name_1, "    ");
        _builder.append("(\"");
        String _key = this.getKey(literal);
        _builder.append(_key, "    ");
        _builder.append("\")");
      }
    }
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public static final Map<String, ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "");
    _builder.append("> keyMap = I18nEnums.createKeyMap(");
    String _name_3 = it.getName();
    _builder.append(_name_3, "");
    _builder.append(".class);");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("private String key;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("private ");
    String _name_4 = it.getName();
    _builder.append(_name_4, "    ");
    _builder.append("(String key) {");
    _builder.newLineIfNotEmpty();
    _builder.append("        ");
    _builder.append("this.key = key;");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public String getKey() {");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("return this.key;");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public String getLongValue(Locale locale) {");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("return I18nEnums.longValueFromProvider(this, locale);");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public String getShortValue(Locale locale) {");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("return I18nEnums.shortValueFromProvider(this, locale);");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public static ");
    String _name_5 = it.getName();
    _builder.append(_name_5, "    ");
    _builder.append(" valueOfKey(String key) {");
    _builder.newLineIfNotEmpty();
    _builder.append("        ");
    _builder.append("return I18nEnums.valueOfKey(keyMap, key);");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toResourceBundle(final EnumType it) {
    StringBuilder _xblockexpression = null;
    {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("COLS  = COD_KEY,SHORT_VALUE,LONG_VALUE,VALID_FROM,VALID_TO");
      _builder.newLine();
      StringBuilder _stringBuilder = new StringBuilder(_builder);
      StringBuilder content = _stringBuilder;
      int index1 = 1;
      EList<EnumLiteral> _literals = it.getLiterals();
      for (final EnumLiteral literal : _literals) {
        {
          StringConcatenation _builder_1 = new StringConcatenation();
          int _multiply = (index1 * 10);
          _builder_1.append(_multiply, "");
          _builder_1.append("    = \"");
          String _key = this.getKey(literal);
          _builder_1.append(_key, "");
          _builder_1.append("\", \"");
          String _shortValue = this.getShortValue(literal);
          _builder_1.append(_shortValue, "");
          _builder_1.append("\", \"");
          String _longValue = this.getLongValue(literal);
          _builder_1.append(_longValue, "");
          _builder_1.append("\", D19700101, D99991231");
          _builder_1.newLineIfNotEmpty();
          content.append(_builder_1);
          int _plus = (index1 + 1);
          index1 = _plus;
        }
      }
      _xblockexpression = (content);
    }
    return _xblockexpression;
  }
  
  public CharSequence toDataTypeClass(final SimpleType it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Represents a custom DataType for <code>");
    String _name = it.getName();
    _builder.append(_name, " ");
    _builder.append("</code>.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("public class ");
    String _name_1 = it.getName();
    _builder.append(_name_1, "");
    _builder.append(" extends ");
    String _javaType = SimpleTypeExtension.getJavaType(it);
    _builder.append(_javaType, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("/**");
    _builder.newLine();
    _builder.append("     ");
    _builder.append("* Put the control and responsibility on the developer to determine the version of the");
    _builder.newLine();
    _builder.append("     ");
    _builder.append("* serialization class with which a serialized object is compatible.");
    _builder.newLine();
    _builder.append("     ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("private static final long serialVersionUID = 100L;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("/**");
    _builder.newLine();
    _builder.append("     ");
    _builder.append("* default ctor");
    _builder.newLine();
    _builder.append("     ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "    ");
    _builder.append("(");
    String _javaType_1 = SimpleTypeExtension.getJavaType(it);
    String _nativeJavaTypeForValueObject = SimpleTypeExtension.getNativeJavaTypeForValueObject(_javaType_1);
    _builder.append(_nativeJavaTypeForValueObject, "    ");
    _builder.append(" ");
    String _javaType_2 = SimpleTypeExtension.getJavaType(it);
    String _generateVariableNameForValueObject = SimpleTypeExtension.generateVariableNameForValueObject(_javaType_2);
    _builder.append(_generateVariableNameForValueObject, "    ");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("    \t");
    _builder.append("super(");
    String _javaType_3 = SimpleTypeExtension.getJavaType(it);
    String _generateVariableNameForValueObject_1 = SimpleTypeExtension.generateVariableNameForValueObject(_javaType_3);
    _builder.append(_generateVariableNameForValueObject_1, "    	");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("    ");
    _builder.append("/**");
    _builder.newLine();
    _builder.append("     ");
    _builder.append("* Create a new instance of ");
    String _name_3 = it.getName();
    _builder.append(_name_3, "     ");
    _builder.append(" or return null depending on input.");
    _builder.newLineIfNotEmpty();
    _builder.append("     ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public static ");
    String _name_4 = it.getName();
    _builder.append(_name_4, "    ");
    _builder.append(" newOrNull(");
    String _javaType_4 = SimpleTypeExtension.getJavaType(it);
    String _nativeJavaTypeForValueObject_1 = SimpleTypeExtension.getNativeJavaTypeForValueObject(_javaType_4);
    _builder.append(_nativeJavaTypeForValueObject_1, "    ");
    _builder.append(" ");
    String _javaType_5 = SimpleTypeExtension.getJavaType(it);
    String _generateVariableNameForValueObject_2 = SimpleTypeExtension.generateVariableNameForValueObject(_javaType_5);
    _builder.append(_generateVariableNameForValueObject_2, "    ");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("        ");
    _builder.append("if (");
    String _javaType_6 = SimpleTypeExtension.getJavaType(it);
    String _generateVariableNameForValueObject_3 = SimpleTypeExtension.generateVariableNameForValueObject(_javaType_6);
    _builder.append(_generateVariableNameForValueObject_3, "        ");
    _builder.append(" != null) {");
    _builder.newLineIfNotEmpty();
    _builder.append("            ");
    _builder.append("return new ");
    String _name_5 = it.getName();
    _builder.append(_name_5, "            ");
    _builder.append("(");
    String _javaType_7 = SimpleTypeExtension.getJavaType(it);
    String _generateVariableNameForValueObject_4 = SimpleTypeExtension.generateVariableNameForValueObject(_javaType_7);
    _builder.append(_generateVariableNameForValueObject_4, "            ");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("        ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    {
      String _javaType_8 = SimpleTypeExtension.getJavaType(it);
      String _nativeJavaTypeForValueObject_2 = SimpleTypeExtension.getNativeJavaTypeForValueObject(_javaType_8);
      boolean _contains = "Boolean,Float,Double,Integer,Long".contains(_nativeJavaTypeForValueObject_2);
      if (_contains) {
        _builder.append("/**");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("* Create a new instance of ");
        String _name_6 = it.getName();
        _builder.append(_name_6, " ");
        _builder.append(" or return null depending on input.");
        _builder.newLineIfNotEmpty();
        _builder.append(" ");
        _builder.append("*/");
        _builder.newLine();
        _builder.append("public static ");
        String _name_7 = it.getName();
        _builder.append(_name_7, "");
        _builder.append(" newOrNull(String value) {");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("if (value != null && value.length() > 0) {");
        _builder.newLine();
        _builder.append("        ");
        _builder.append("return new ");
        String _name_8 = it.getName();
        _builder.append(_name_8, "        ");
        _builder.append("(");
        String _javaType_9 = SimpleTypeExtension.getJavaType(it);
        String _nativeJavaTypeForValueObject_3 = SimpleTypeExtension.getNativeJavaTypeForValueObject(_javaType_9);
        _builder.append(_nativeJavaTypeForValueObject_3, "        ");
        _builder.append(".valueOf(value));");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("}");
        _builder.newLine();
        _builder.append("    ");
        _builder.append("return null;");
        _builder.newLine();
        _builder.append("}");
        _builder.newLine();
        _builder.newLine();
      } else {
        String _javaType_10 = SimpleTypeExtension.getJavaType(it);
        String _nativeJavaTypeForValueObject_4 = SimpleTypeExtension.getNativeJavaTypeForValueObject(_javaType_10);
        boolean _equals = "java.math.BigDecimal".equals(_nativeJavaTypeForValueObject_4);
        if (_equals) {
          _builder.append("/**");
          _builder.newLine();
          _builder.append(" ");
          _builder.append("* Create a new instance of ");
          String _name_9 = it.getName();
          _builder.append(_name_9, " ");
          _builder.append(" or return null depending on input.");
          _builder.newLineIfNotEmpty();
          _builder.append(" ");
          _builder.append("*/");
          _builder.newLine();
          _builder.append("public static ");
          String _name_10 = it.getName();
          _builder.append(_name_10, "");
          _builder.append(" newOrNull(String value) {");
          _builder.newLineIfNotEmpty();
          _builder.append("    ");
          _builder.append("if (value != null && value.length() > 0) {");
          _builder.newLine();
          _builder.append("        ");
          _builder.append("return new ");
          String _name_11 = it.getName();
          _builder.append(_name_11, "        ");
          _builder.append("(new ");
          String _javaType_11 = SimpleTypeExtension.getJavaType(it);
          String _nativeJavaTypeForValueObject_5 = SimpleTypeExtension.getNativeJavaTypeForValueObject(_javaType_11);
          _builder.append(_nativeJavaTypeForValueObject_5, "        ");
          _builder.append("(value));");
          _builder.newLineIfNotEmpty();
          _builder.append("    ");
          _builder.append("}");
          _builder.newLine();
          _builder.append("    ");
          _builder.append("return null;");
          _builder.newLine();
          _builder.append("}");
          _builder.newLine();
          _builder.newLine();
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private String getQualifiedClassName(final SimpleType it) {
    String _packageName = this._names.getPackageName(it);
    String _plus = (_packageName + ".");
    String _name = it.getName();
    String _plus_1 = (_plus + _name);
    return _plus_1;
  }
  
  private String getKey(final EnumLiteral it) {
    String _xifexpression = null;
    String _key = it.getKey();
    boolean _equals = Objects.equal(_key, null);
    if (_equals) {
      String _name = it.getName();
      _xifexpression = _name;
    } else {
      String _key_1 = it.getKey();
      _xifexpression = _key_1;
    }
    return _xifexpression;
  }
  
  private String getShortValue(final EnumLiteral it) {
    String _xifexpression = null;
    String _shortValue = it.getShortValue();
    boolean _equals = Objects.equal(_shortValue, null);
    if (_equals) {
      String _name = it.getName();
      _xifexpression = _name;
    } else {
      String _shortValue_1 = it.getShortValue();
      _xifexpression = _shortValue_1;
    }
    return _xifexpression;
  }
  
  private String getLongValue(final EnumLiteral it) {
    String _xifexpression = null;
    String _longValue = it.getLongValue();
    boolean _equals = Objects.equal(_longValue, null);
    if (_equals) {
      String _name = it.getName();
      _xifexpression = _name;
    } else {
      String _longValue_1 = it.getLongValue();
      _xifexpression = _longValue_1;
    }
    return _xifexpression;
  }
}
