package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.generator.IGenerator
import org.openxma.dsl.core.model.EnumLiteral
import org.openxma.dsl.core.model.EnumType
import org.openxma.dsl.core.model.Model
import org.openxma.dsl.core.model.SimpleType

import static org.openxma.dsl.generator.GeneratorConstants.*

import static extension org.openxma.dsl.generator.helper.SimpleTypeExtension.*

class DataTypeGenerator implements IGenerator {
	@Inject
    extension Names
	@Inject
    extension Files
	@Inject
    extension Features

	override doGenerate(Resource resource, IFileSystemAccess fsa) {
		val model = resource.getEObject("/") as Model
		model.elements.filter(typeof(EnumType)).forEach(
			[
				fsa.generateFile(qualifiedClassName.toJavaFilePath, OUTLET_GENERATED_JAVA, toEnumClass)
				fsa.generateFile(qualifiedClassName.toFilePath("properties"), OUTLET_GENERATED_RESOURCES, toResourceBundle)
			]
		)
		model.elements.filter(typeof(SimpleType)).filter([custom]).forEach(
			[
				fsa.generateFile(qualifiedClassName.toJavaFilePath, OUTLET_GENERATED_JAVA, toDataTypeClass)
			]
		)
	}


	def CharSequence toEnumClass(EnumType it) {
		'''
			package packageName;
			
			import java.util.Locale;
			import java.util.Map;
			
			import org.openxma.dsl.platform.enums.I18nEnum;
			import org.openxma.dsl.platform.enums.I18nEnums;
			
			documentation
			public enum name implements I18nEnum {
			    FOR literal : literals SEPARATOR ",\n"literal.name("getKey(literal)")ENDFOR;
			
			public static final Map<String, name> keyMap = I18nEnums.createKeyMap(name.class);
			
			    private String key;
			
			    private name(String key) {
			        this.key = key;
			    }
			
			    public String getKey() {
			        return this.key;
			    }
			
			    public String getLongValue(Locale locale) {
			        return I18nEnums.longValueFromProvider(this, locale);
			    }
			
			    public String getShortValue(Locale locale) {
			        return I18nEnums.shortValueFromProvider(this, locale);
			    }
			
			    public static name valueOfKey(String key) {
			        return I18nEnums.valueOfKey(keyMap, key);
			    }
			}
		'''
	}

	def CharSequence toResourceBundle(EnumType it) {
		var StringBuilder content = new StringBuilder('''
			COLS  = COD_KEY,SHORT_VALUE,LONG_VALUE,VALID_FROM,VALID_TO
		''')
		var index1 = 1
		for (literal : literals) {
			content.append('''
				index1*10    = "getKey(literal)", "getShortValue(literal)", "getLongValue(literal)", D19700101, D99991231
			''')
			index1 = index1 + 1
		}
		content
	}
	
	def CharSequence toDataTypeClass(SimpleType it) {
		'''
		package packageName;

		/**
		 * Represents a custom DataType for <code>name</code>.
		 */
		public class name extends javaType {
		
		    /**
		     * Put the control and responsibility on the developer to determine the version of the
		     * serialization class with which a serialized object is compatible.
		     */
		    private static final long serialVersionUID = 100L;
		
		    /**
		     * default ctor
		     */
		    public name(getNativeJavaTypeForValueObject(javaType) generateVariableNameForValueObject(javaType)) {
		    	super(generateVariableNameForValueObject(javaType));
		    }
		
		    /**
		     * Create a new instance of name or return null depending on input.
		     */
		    public static name newOrNull(getNativeJavaTypeForValueObject(javaType) generateVariableNameForValueObject(javaType)) {
		        if (generateVariableNameForValueObject(javaType) != null) {
		            return new name(generateVariableNameForValueObject(javaType));
		        }
		        return null;
		    }
		
		IF  "Boolean,Float,Double,Integer,Long".contains(getNativeJavaTypeForValueObject(javaType))
		    /**
		     * Create a new instance of name or return null depending on input.
		     */
		    public static name newOrNull(String value) {
		        if (value != null && value.length() > 0) {
		            return new name(getNativeJavaTypeForValueObject(javaType).valueOf(value));
		        }
		        return null;
		    }
		    
		ELSEIF "java.math.BigDecimal".equals(getNativeJavaTypeForValueObject(getJavaType))
		    /**
		     * Create a new instance of name or return null depending on input.
		     */
		    public static name newOrNull(String value) {
		        if (value != null && value.length() > 0) {
		            return new name(new getNativeJavaTypeForValueObject(getJavaType)(value));
		        }
		        return null;
		    }
		    
		ENDIF
		}
		'''
	}

	def private getQualifiedClassName(SimpleType it) {
		packageName + "." + name
	}
	
	def private getKey(EnumLiteral it) {
		if (key==null) name else key
	}

	def private getShortValue(EnumLiteral it) {
		if (shortValue==null) name else shortValue
	}
	
	def private String getLongValue(EnumLiteral it) {
		if (longValue==null) name else longValue
	}
    
}
