package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import java.util.List;
import java.util.Set;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.common.Strings;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.FeatureReference;
import org.openxma.dsl.dom.model.Mapper;
import org.openxma.dsl.dom.model.PropertyMapping;
import org.openxma.dsl.generator.FeatureExtensions;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.DataViewExtension;
import org.openxma.dsl.generator.helper.EntityExtension;
import org.openxma.dsl.generator.helper.MapperExtension;
import org.openxma.dsl.generator.helper.NameExtension;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;
import org.openxma.dsl.generator.lib.Index;

@SuppressWarnings("all")
public class DataViewGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  @Inject
  private FeatureExtensions _featureExtensions;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    Boolean _isNameSpaceGenerated = DataViewExtension.isNameSpaceGenerated(model);
    boolean _not = (!(_isNameSpaceGenerated).booleanValue());
    if (_not) {
      DataViewExtension.addGeneratedNameSpaceName(model);
      final Set<DataView> dataViews = DataViewExtension.getNameSpaceDataViewsFromModel(model);
      boolean _isEmpty = dataViews.isEmpty();
      boolean _not_1 = (!_isEmpty);
      if (_not_1) {
        String _qualifiedGenAssemblerClassName = this.getQualifiedGenAssemblerClassName(model);
        String _javaFilePath = this._files.toJavaFilePath(_qualifiedGenAssemblerClassName);
        CharSequence _genAssemblerClass = this.toGenAssemblerClass(model, dataViews);
        fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genAssemblerClass);
        String _qualifiedAssemblerClassName = this.getQualifiedAssemblerClassName(model);
        String _javaFilePath_1 = this._files.toJavaFilePath(_qualifiedAssemblerClassName);
        CharSequence _assemblerClass = this.toAssemblerClass(model, dataViews);
        fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _assemblerClass);
      }
    }
    EList<ModelElement> _elements = model.getElements();
    Iterable<DataView> _filter = Iterables.<DataView>filter(_elements, DataView.class);
    final Procedure1<DataView> _function = new Procedure1<DataView>() {
        public void apply(final DataView it) {
          String _qualifiedGenClassName = DataViewGenerator.this._names.getQualifiedGenClassName(it);
          String _javaFilePath = DataViewGenerator.this._files.toJavaFilePath(_qualifiedGenClassName);
          CharSequence _genClass = DataViewGenerator.this.toGenClass(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _qualifiedClassName = DataViewGenerator.this._names.getQualifiedClassName(it);
          String _javaFilePath_1 = DataViewGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _class = DataViewGenerator.this.toClass(it);
          fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _class);
        }
      };
    IterableExtensions.<DataView>forEach(_filter, _function);
  }
  
  public CharSequence toGenAssemblerClass(final Model it, final Iterable<DataView> dataViews) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.openxma.dsl.platform.service.MapperContextHolder;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public class ");
    String _genAssemblerClassName = this.getGenAssemblerClassName(it);
    _builder.append(_genAssemblerClassName, "");
    _builder.append(" implements ");
    String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.assembler.DtoAssemblerFactory");
    _builder.append(_addImport, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("private ");
    String _addImport_1 = this._names.addImport(it, "java.util.List");
    _builder.append(_addImport_1, "	");
    _builder.append("<");
    String _addImport_2 = this._names.addImport(it, "org.openxma.dsl.platform.assembler.DtoAssembler");
    _builder.append(_addImport_2, "	");
    _builder.append("<?>> dtoAssemblers;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@");
    String _addImport_3 = this._names.addImport(it, "org.springframework.beans.factory.annotation.Autowired");
    _builder.append(_addImport_3, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private ");
    String _addImport_4 = this._names.addImport(it, "org.openxma.dsl.platform.assembler.DtoAssemblerRegistry");
    _builder.append(_addImport_4, "	");
    _builder.append(" dtoAssemblerRegistry;");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@");
    String _addImport_5 = this._names.addImport(it, "org.springframework.beans.factory.annotation.Autowired");
    _builder.append(_addImport_5, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private ");
    String _addImport_6 = this._names.addImport(it, "org.openxma.dsl.platform.converter.Converter");
    _builder.append(_addImport_6, "	");
    _builder.append(" converter;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public ");
    String _genAssemblerClassName_1 = this.getGenAssemblerClassName(it);
    _builder.append(_genAssemblerClassName_1, "	");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("dtoAssemblers = new ");
    String _addImport_7 = this._names.addImport(it, "java.util.ArrayList");
    _builder.append(_addImport_7, "		");
    _builder.append("<DtoAssembler<?>>(");
    int _size = IterableExtensions.size(dataViews);
    _builder.append(_size, "		");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("initDtoAssemblers();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public List<DtoAssembler<?>> getDtoAssemblers() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return dtoAssemblers;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected DtoAssemblerRegistry getDtoAssemblerRegistry() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return dtoAssemblerRegistry;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected Converter getConverter() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return converter;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("private void initDtoAssemblers() {");
    _builder.newLine();
    {
      for(final DataView dataView : dataViews) {
        _builder.append("\t\t");
        _builder.append("dtoAssemblers.add(createDtoAssembler");
        String _name_1 = dataView.getName();
        _builder.append(_name_1, "		");
        _builder.append("());");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.newLine();
    {
      for(final DataView dataView_1 : dataViews) {
        _builder.append("\t");
        _builder.append("private DtoAssembler<");
        String _qualifiedName = this._names.getQualifiedName(dataView_1);
        _builder.append(_qualifiedName, "	");
        _builder.append("> createDtoAssembler");
        String _name_2 = dataView_1.getName();
        _builder.append(_name_2, "	");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("return new DtoAssembler<");
        String _name_3 = dataView_1.getName();
        _builder.append(_name_3, "		");
        _builder.append(">() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t\t");
        CharSequence _anonymousDtoAssemblerClass = this.toAnonymousDtoAssemblerClass(dataView_1);
        _builder.append(_anonymousDtoAssemblerClass, "			");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("};");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.newLine();
        _builder.append("\t");
        CharSequence _mapping = this.toMapping(dataView_1);
        _builder.append(_mapping, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toAnonymousDtoAssemblerClass(final DataView it) {
    CharSequence _xblockexpression = null;
    {
      EList<FeatureReference> _featureReferences = it.getFeatureReferences();
      final Function1<FeatureReference,Entity> _function = new Function1<FeatureReference,Entity>() {
          public Entity apply(final FeatureReference e) {
            Entity _source = e.getSource();
            return _source;
          }
        };
      List<Entity> _map = ListExtensions.<FeatureReference, Entity>map(_featureReferences, _function);
      final Iterable<Entity> entities = IterableExtensions.<Entity>toSet(_map);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("public ");
      String _className = this._names.getClassName(it);
      _builder.append(_className, "");
      _builder.append(" mapFromEntity(Object sourceEntity) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      String _addImport = this._names.addImport(it, "org.springframework.util.Assert");
      _builder.append(_addImport, "	");
      _builder.append(".notNull(sourceEntity, \"parameter \'sourceEntity\' must not be null\");");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      Index _index = new Index();
      final Index iterator = _index;
      _builder.newLineIfNotEmpty();
      {
        for(final Entity entity : entities) {
          {
            int _counter0 = iterator.getCounter0();
            boolean _greaterThan = (_counter0 > 0);
            if (_greaterThan) {
              _builder.append("\t");
              _builder.append("} else if (sourceEntity instanceof ");
              String _qualifiedName = this._names.getQualifiedName(entity);
              _builder.append(_qualifiedName, "	");
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("\t");
              _builder.append("if (sourceEntity instanceof ");
              String _qualifiedName_1 = this._names.getQualifiedName(entity);
              _builder.append(_qualifiedName_1, "	");
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          String _qualifiedName_2 = this._names.getQualifiedName(entity);
          _builder.append(_qualifiedName_2, "		");
          _builder.append(" ");
          String _name = entity.getName();
          String _variableName = this._names.toVariableName(_name);
          _builder.append(_variableName, "		");
          _builder.append(" = (");
          String _name_1 = entity.getName();
          _builder.append(_name_1, "		");
          _builder.append(") sourceEntity;");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          String _className_1 = this._names.getClassName(it);
          _builder.append(_className_1, "		");
          _builder.append(" result = mapFrom");
          String _name_2 = entity.getName();
          _builder.append(_name_2, "		");
          _builder.append("To");
          String _className_2 = this._names.getClassName(it);
          _builder.append(_className_2, "		");
          _builder.append("(");
          String _name_3 = entity.getName();
          String _variableName_1 = this._names.toVariableName(_name_3);
          _builder.append(_variableName_1, "		");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          {
            EList<Attribute> _allAttributes = entity.getAllAttributes();
            final Function1<Attribute,Boolean> _function_1 = new Function1<Attribute,Boolean>() {
                public Boolean apply(final Attribute reference) {
                  EList<FeatureReference> _featureReferences = it.getFeatureReferences();
                  final Function1<FeatureReference,Boolean> _function = new Function1<FeatureReference,Boolean>() {
                      public Boolean apply(final FeatureReference e) {
                        Object _property = EObjectPropertiesAdapter.getProperty(e, "sourceReference");
                        boolean _equals = Objects.equal(_property, reference);
                        return Boolean.valueOf(_equals);
                      }
                    };
                  boolean _exists = IterableExtensions.<FeatureReference>exists(_featureReferences, _function);
                  return Boolean.valueOf(_exists);
                }
              };
            Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_allAttributes, _function_1);
            for(final Attribute requiredReference : _filter) {
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("if (null != ");
              String _variableName_2 = this._names.toVariableName(entity);
              _builder.append(_variableName_2, "		");
              _builder.append(".");
              String _accessorMethodName = this._features.getAccessorMethodName(requiredReference);
              _builder.append(_accessorMethodName, "		");
              _builder.append("()) {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("result.");
              EList<FeatureReference> _featureReferences_1 = it.getFeatureReferences();
              final Function1<FeatureReference,Boolean> _function_2 = new Function1<FeatureReference,Boolean>() {
                  public Boolean apply(final FeatureReference e) {
                    Object _property = EObjectPropertiesAdapter.getProperty(e, "sourceReference");
                    boolean _equals = Objects.equal(_property, requiredReference);
                    return Boolean.valueOf(_equals);
                  }
                };
              FeatureReference _findFirst = IterableExtensions.<FeatureReference>findFirst(_featureReferences_1, _function_2);
              Attribute _target = _findFirst.getTarget();
              String _mutatorMethodName = this._features.getMutatorMethodName(_target);
              _builder.append(_mutatorMethodName, "			");
              _builder.append("(");
              String _variableName_3 = this._names.toVariableName(entity);
              _builder.append(_variableName_3, "			");
              _builder.append(".");
              String _accessorMethodName_1 = this._features.getAccessorMethodName(requiredReference);
              _builder.append(_accessorMethodName_1, "			");
              _builder.append("().");
              Type _dataType = requiredReference.getDataType();
              Attribute _identifier = ((Entity) _dataType).getIdentifier();
              String _accessorMethodName_2 = this._features.getAccessorMethodName(_identifier);
              _builder.append(_accessorMethodName_2, "			");
              _builder.append("());");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return result;");
          _builder.newLine();
          _builder.append("\t");
          iterator.increment();
          _builder.newLineIfNotEmpty();
        }
      }
      {
        boolean _and = false;
        DataView _superType = it.getSuperType();
        boolean _notEquals = (!Objects.equal(_superType, null));
        if (!_notEquals) {
          _and = false;
        } else {
          DataView _superType_1 = it.getSuperType();
          EList<FeatureReference> _featureReferences_2 = _superType_1.getFeatureReferences();
          boolean _isEmpty = _featureReferences_2.isEmpty();
          boolean _not = (!_isEmpty);
          _and = (_notEquals && _not);
        }
        if (_and) {
          _builder.append("\t");
          _builder.append("} else {");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t\t");
          String _className_3 = this._names.getClassName(it);
          _builder.append(_className_3, "			");
          _builder.append(" result = new ");
          String _className_4 = this._names.getClassName(it);
          _builder.append(_className_4, "			");
          _builder.append("();");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t\t");
          _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_2 = it.getSuperType();
          String _qualifiedName_3 = this._names.getQualifiedName(_superType_2);
          _builder.append(_qualifiedName_3, "			");
          _builder.append(".class)");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t\t\t");
          _builder.append(".mapTo(dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_3 = it.getSuperType();
          String _name_4 = _superType_3.getName();
          _builder.append(_name_4, "				");
          _builder.append(".class)");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t\t\t\t\t");
          _builder.append(".mapFromEntity(sourceEntity), result);");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t\t");
          _builder.append("return result;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        } else {
          {
            int _counter0_1 = iterator.getCounter0();
            boolean _greaterThan_1 = (_counter0_1 > 0);
            if (_greaterThan_1) {
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            } else {
              _builder.append("\t");
              _builder.newLine();
            }
          }
          _builder.append("\t");
          _builder.append("throw new IllegalArgumentException(\"mapping from object \'\"+sourceEntity+\"\' is not supported\");");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.newLine();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _builder.append("public ");
      String _className_5 = this._names.getClassName(it);
      _builder.append(_className_5, "");
      _builder.append(" mapFromEntity(Object sourceEntity, ");
      String _className_6 = this._names.getClassName(it);
      _builder.append(_className_6, "");
      _builder.append(" targetDto) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      String _addImport_1 = this._names.addImport(it, "org.springframework.util.Assert");
      _builder.append(_addImport_1, "	");
      _builder.append(".notNull(sourceEntity, \"parameter \'sourceEntity\' must not be null\");");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      String _addImport_2 = this._names.addImport(it, "org.springframework.util.Assert");
      _builder.append(_addImport_2, "	");
      _builder.append(".notNull(targetDto, \"parameter \'targetDto\' must not be null\");");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      Index _index_1 = new Index();
      final Index index = _index_1;
      {
        for(final Entity entity_1 : entities) {
          _builder.newLineIfNotEmpty();
          {
            int _counter0_2 = index.getCounter0();
            boolean _greaterThan_2 = (_counter0_2 > 0);
            if (_greaterThan_2) {
              _builder.append("\t");
              _builder.append("} else if (sourceEntity instanceof ");
              String _qualifiedName_4 = this._names.getQualifiedName(entity_1);
              _builder.append(_qualifiedName_4, "	");
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("\t");
              _builder.append("if (sourceEntity instanceof ");
              String _qualifiedName_5 = this._names.getQualifiedName(entity_1);
              _builder.append(_qualifiedName_5, "	");
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          String _qualifiedName_6 = this._names.getQualifiedName(entity_1);
          _builder.append(_qualifiedName_6, "		");
          _builder.append(" ");
          String _variableName_4 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_4, "		");
          _builder.append(" = (");
          String _name_5 = entity_1.getName();
          _builder.append(_name_5, "		");
          _builder.append(") sourceEntity;");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("mapFrom");
          String _name_6 = entity_1.getName();
          _builder.append(_name_6, "		");
          _builder.append("To");
          String _className_7 = this._names.getClassName(it);
          _builder.append(_className_7, "		");
          _builder.append("(");
          String _variableName_5 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_5, "		");
          _builder.append(", targetDto);");
          _builder.newLineIfNotEmpty();
          {
            EList<Attribute> _allAttributes_1 = entity_1.getAllAttributes();
            final Function1<Attribute,Boolean> _function_3 = new Function1<Attribute,Boolean>() {
                public Boolean apply(final Attribute reference) {
                  EList<FeatureReference> _featureReferences = it.getFeatureReferences();
                  final Function1<FeatureReference,Boolean> _function = new Function1<FeatureReference,Boolean>() {
                      public Boolean apply(final FeatureReference e) {
                        Object _property = EObjectPropertiesAdapter.getProperty(e, "sourceReference");
                        boolean _equals = Objects.equal(_property, reference);
                        return Boolean.valueOf(_equals);
                      }
                    };
                  boolean _exists = IterableExtensions.<FeatureReference>exists(_featureReferences, _function);
                  return Boolean.valueOf(_exists);
                }
              };
            Iterable<Attribute> _filter_1 = IterableExtensions.<Attribute>filter(_allAttributes_1, _function_3);
            for(final Attribute requiredReference_1 : _filter_1) {
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("if (null != ");
              String _variableName_6 = this._names.toVariableName(entity_1);
              _builder.append(_variableName_6, "		");
              _builder.append(".");
              String _accessorMethodName_3 = this._features.getAccessorMethodName(requiredReference_1);
              _builder.append(_accessorMethodName_3, "		");
              _builder.append("()) {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("targetDto.");
              EList<FeatureReference> _featureReferences_3 = it.getFeatureReferences();
              final Function1<FeatureReference,Boolean> _function_4 = new Function1<FeatureReference,Boolean>() {
                  public Boolean apply(final FeatureReference e) {
                    Object _property = EObjectPropertiesAdapter.getProperty(e, "sourceReference");
                    boolean _equals = Objects.equal(_property, requiredReference_1);
                    return Boolean.valueOf(_equals);
                  }
                };
              FeatureReference _findFirst_1 = IterableExtensions.<FeatureReference>findFirst(_featureReferences_3, _function_4);
              Attribute _target_1 = _findFirst_1.getTarget();
              String _mutatorMethodName_1 = this._features.getMutatorMethodName(_target_1);
              _builder.append(_mutatorMethodName_1, "			");
              _builder.append("(");
              String _variableName_7 = this._names.toVariableName(entity_1);
              _builder.append(_variableName_7, "			");
              _builder.append(".");
              String _accessorMethodName_4 = this._features.getAccessorMethodName(requiredReference_1);
              _builder.append(_accessorMethodName_4, "			");
              _builder.append("().");
              Type _dataType_1 = requiredReference_1.getDataType();
              Attribute _identifier_1 = ((Entity) _dataType_1).getIdentifier();
              String _accessorMethodName_5 = this._features.getAccessorMethodName(_identifier_1);
              _builder.append(_accessorMethodName_5, "			");
              _builder.append("());");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return targetDto;");
          _builder.newLine();
          _builder.append("\t");
          index.increment();
          _builder.newLineIfNotEmpty();
        }
      }
      {
        boolean _and_1 = false;
        DataView _superType_4 = it.getSuperType();
        boolean _notEquals_1 = (!Objects.equal(_superType_4, null));
        if (!_notEquals_1) {
          _and_1 = false;
        } else {
          DataView _superType_5 = it.getSuperType();
          EList<FeatureReference> _featureReferences_4 = _superType_5.getFeatureReferences();
          boolean _isEmpty_1 = _featureReferences_4.isEmpty();
          boolean _not_1 = (!_isEmpty_1);
          _and_1 = (_notEquals_1 && _not_1);
        }
        if (_and_1) {
          _builder.append("\t");
          _builder.append("} else {");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t\t");
          _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_6 = it.getSuperType();
          String _qualifiedName_7 = this._names.getQualifiedName(_superType_6);
          _builder.append(_qualifiedName_7, "			");
          _builder.append(".class)");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t\t\t");
          _builder.append(".mapTo(dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_7 = it.getSuperType();
          String _name_7 = _superType_7.getName();
          _builder.append(_name_7, "				");
          _builder.append(".class)");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t\t\t\t\t");
          _builder.append(".mapFromEntity(sourceEntity), targetDto);");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t\t");
          _builder.append("return targetDto;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        } else {
          {
            int _counter0_3 = index.getCounter0();
            boolean _greaterThan_3 = (_counter0_3 > 0);
            if (_greaterThan_3) {
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            } else {
              _builder.append("\t");
              _builder.newLine();
            }
          }
          _builder.append("\t");
          _builder.append("throw new IllegalArgumentException(\"mapping from object \'\"+sourceEntity+\"\' is not supported\");");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.newLine();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _builder.append("public <T> T mapTo(");
      String _className_8 = this._names.getClassName(it);
      _builder.append(_className_8, "");
      _builder.append(" sourceDto, Class<T> target) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("if (");
      String _className_9 = this._names.getClassName(it);
      _builder.append(_className_9, "	");
      _builder.append(".class.equals(target)) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      String _className_10 = this._names.getClassName(it);
      _builder.append(_className_10, "		");
      _builder.append(" result = new ");
      String _className_11 = this._names.getClassName(it);
      _builder.append(_className_11, "		");
      _builder.append("();");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("mapFrom");
      String _className_12 = this._names.getClassName(it);
      _builder.append(_className_12, "		");
      _builder.append("To");
      String _className_13 = this._names.getClassName(it);
      _builder.append(_className_13, "		");
      _builder.append("(sourceDto, result);");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("return target.cast(result);");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t");
      final Set<Mapper> mappers = MapperExtension.getMappersByDataView(it);
      _builder.newLineIfNotEmpty();
      {
        for(final Mapper mapper : mappers) {
          {
            Boolean _dataViewIsMapperSource = MapperExtension.dataViewIsMapperSource(it, mapper);
            if ((_dataViewIsMapperSource).booleanValue()) {
              _builder.append("\t");
              _builder.append("else if (");
              String _oppositeElementFullyQualifiedName = MapperExtension.getOppositeElementFullyQualifiedName(it, mapper);
              String _addImport_3 = this._names.addImport(it, _oppositeElementFullyQualifiedName);
              _builder.append(_addImport_3, "	");
              _builder.append(".class.equals(target) && MapperContextHolder.getContext().containsKey(\"mapper:");
              String _name_8 = mapper.getName();
              _builder.append(_name_8, "	");
              _builder.append("\")) {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("return target.cast(mapper");
              String _name_9 = mapper.getName();
              _builder.append(_name_9, "		");
              _builder.append("MapsFrom");
              String _className_14 = this._names.getClassName(it);
              _builder.append(_className_14, "		");
              _builder.append("To");
              String _oppositeElementName = MapperExtension.getOppositeElementName(it, mapper);
              _builder.append(_oppositeElementName, "		");
              _builder.append("(sourceDto));");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            } else {
              _builder.append("\t");
              _builder.append("//dataView: \'");
              String _name_10 = it.getName();
              _builder.append(_name_10, "	");
              _builder.append("\' is not a mapping source in mapper: \'");
              String _name_11 = mapper.getName();
              _builder.append(_name_11, "	");
              _builder.append("\'");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.newLine();
        }
      }
      {
        for(final Entity entity_2 : entities) {
          _builder.append("\t");
          _builder.append("else if (");
          String _name_12 = entity_2.getName();
          _builder.append(_name_12, "	");
          _builder.append(".class.equals(target)) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return target.cast(mapFrom");
          String _className_15 = this._names.getClassName(it);
          _builder.append(_className_15, "		");
          _builder.append("To");
          String _name_13 = entity_2.getName();
          _builder.append(_name_13, "		");
          _builder.append("(sourceDto));");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      {
        boolean _and_2 = false;
        DataView _superType_8 = it.getSuperType();
        boolean _notEquals_2 = (!Objects.equal(_superType_8, null));
        if (!_notEquals_2) {
          _and_2 = false;
        } else {
          DataView _superType_9 = it.getSuperType();
          EList<FeatureReference> _featureReferences_5 = _superType_9.getFeatureReferences();
          boolean _isEmpty_2 = _featureReferences_5.isEmpty();
          boolean _not_2 = (!_isEmpty_2);
          _and_2 = (_notEquals_2 && _not_2);
        }
        if (_and_2) {
          _builder.append("\t");
          _builder.append("else {");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_10 = it.getSuperType();
          String _name_14 = _superType_10.getName();
          _builder.append(_name_14, "		");
          _builder.append(".class).mapTo(sourceDto, target);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        } else {
          _builder.append("\t");
          _builder.append("else {");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("throw new IllegalArgumentException(\"mapping to class \'\"+target+\"\' is not supported\");");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _builder.append("public void mapTo(");
      String _className_16 = this._names.getClassName(it);
      _builder.append(_className_16, "");
      _builder.append(" sourceDto, Object targetObject) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("if (targetObject instanceof ");
      String _className_17 = this._names.getClassName(it);
      _builder.append(_className_17, "	");
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("mapFrom");
      String _className_18 = this._names.getClassName(it);
      _builder.append(_className_18, "		");
      _builder.append("To");
      String _className_19 = this._names.getClassName(it);
      _builder.append(_className_19, "		");
      _builder.append("(sourceDto, (");
      String _className_20 = this._names.getClassName(it);
      _builder.append(_className_20, "		");
      _builder.append(") targetObject);");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      {
        for(final Mapper mapper_1 : mappers) {
          {
            Boolean _dataViewIsMapperSource_1 = MapperExtension.dataViewIsMapperSource(it, mapper_1);
            if ((_dataViewIsMapperSource_1).booleanValue()) {
              _builder.append("\t");
              _builder.append("else if (targetObject instanceof ");
              String _oppositeElementFullyQualifiedName_1 = MapperExtension.getOppositeElementFullyQualifiedName(it, mapper_1);
              String _addImport_4 = this._names.addImport(it, _oppositeElementFullyQualifiedName_1);
              _builder.append(_addImport_4, "	");
              _builder.append(" && MapperContextHolder.getContext().containsKey(\"mapper:");
              String _name_15 = mapper_1.getName();
              _builder.append(_name_15, "	");
              _builder.append("\")) {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("mapper");
              String _name_16 = mapper_1.getName();
              _builder.append(_name_16, "		");
              _builder.append("MapsFrom");
              String _className_21 = this._names.getClassName(it);
              _builder.append(_className_21, "		");
              _builder.append("To");
              String _oppositeElementName_1 = MapperExtension.getOppositeElementName(it, mapper_1);
              _builder.append(_oppositeElementName_1, "		");
              _builder.append("(sourceDto, (");
              String _oppositeElementName_2 = MapperExtension.getOppositeElementName(it, mapper_1);
              _builder.append(_oppositeElementName_2, "		");
              _builder.append(")targetObject);");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            } else {
              _builder.append("\t");
              _builder.append("//dataView: \'");
              String _name_17 = it.getName();
              _builder.append(_name_17, "	");
              _builder.append("\' is not a mapping source in mapper: \'");
              String _name_18 = mapper_1.getName();
              _builder.append(_name_18, "	");
              _builder.append("\'");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.newLine();
        }
      }
      {
        for(final Entity entity_3 : entities) {
          _builder.append("\t");
          _builder.append("else if (targetObject instanceof ");
          String _name_19 = entity_3.getName();
          _builder.append(_name_19, "	");
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("mapFrom");
          String _className_22 = this._names.getClassName(it);
          _builder.append(_className_22, "		");
          _builder.append("To");
          String _name_20 = entity_3.getName();
          _builder.append(_name_20, "		");
          _builder.append("(sourceDto, (");
          String _name_21 = entity_3.getName();
          _builder.append(_name_21, "		");
          _builder.append(")targetObject);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}\t\t\t   ");
          _builder.newLine();
        }
      }
      {
        boolean _and_3 = false;
        DataView _superType_11 = it.getSuperType();
        boolean _notEquals_3 = (!Objects.equal(_superType_11, null));
        if (!_notEquals_3) {
          _and_3 = false;
        } else {
          DataView _superType_12 = it.getSuperType();
          EList<FeatureReference> _featureReferences_6 = _superType_12.getFeatureReferences();
          boolean _isEmpty_3 = _featureReferences_6.isEmpty();
          boolean _not_3 = (!_isEmpty_3);
          _and_3 = (_notEquals_3 && _not_3);
        }
        if (_and_3) {
          _builder.append("\t");
          _builder.append("else {");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_13 = it.getSuperType();
          String _name_22 = _superType_13.getName();
          _builder.append(_name_22, "		");
          _builder.append(".class).mapTo(sourceDto, targetObject);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        } else {
          _builder.append("\t");
          _builder.append("else if (targetObject != null) {");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("throw new IllegalArgumentException(\"mapping to object \'\"+targetObject+\"\' is not supported\");");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence toMapping(final DataView it) {
    CharSequence _xblockexpression = null;
    {
      EList<FeatureReference> _featureReferences = it.getFeatureReferences();
      final Function1<FeatureReference,Entity> _function = new Function1<FeatureReference,Entity>() {
          public Entity apply(final FeatureReference e) {
            Entity _source = e.getSource();
            return _source;
          }
        };
      List<Entity> _map = ListExtensions.<FeatureReference, Entity>map(_featureReferences, _function);
      final Iterable<Entity> entities = IterableExtensions.<Entity>toSet(_map);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("protected void mapFrom");
      String _className = this._names.getClassName(it);
      _builder.append(_className, "");
      _builder.append("To");
      String _className_1 = this._names.getClassName(it);
      _builder.append(_className_1, "");
      _builder.append("(");
      String _className_2 = this._names.getClassName(it);
      _builder.append(_className_2, "");
      _builder.append(" sourceDto, ");
      String _className_3 = this._names.getClassName(it);
      _builder.append(_className_3, "");
      _builder.append(" ");
      String _className_4 = this._names.getClassName(it);
      String _firstLower = StringExtensions.toFirstLower(_className_4);
      _builder.append(_firstLower, "");
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      {
        DataView _superType = it.getSuperType();
        boolean _notEquals = (!Objects.equal(_superType, null));
        if (_notEquals) {
          _builder.append("\t");
          _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
          DataView _superType_1 = it.getSuperType();
          String _name = _superType_1.getName();
          _builder.append(_name, "	");
          _builder.append(".class).mapTo(sourceDto, ");
          String _className_5 = this._names.getClassName(it);
          String _firstLower_1 = StringExtensions.toFirstLower(_className_5);
          _builder.append(_firstLower_1, "	");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        EList<FeatureReference> _featureReferences_1 = it.getFeatureReferences();
        final Function1<FeatureReference,Boolean> _function_1 = new Function1<FeatureReference,Boolean>() {
            public Boolean apply(final FeatureReference e) {
              boolean _and = false;
              boolean _and_1 = false;
              Attribute _target = e.getTarget();
              boolean _notEquals = (!Objects.equal(_target, null));
              if (!_notEquals) {
                _and_1 = false;
              } else {
                Attribute _attribute = e.getAttribute();
                boolean _notEquals_1 = (!Objects.equal(_attribute, null));
                _and_1 = (_notEquals && _notEquals_1);
              }
              if (!_and_1) {
                _and = false;
              } else {
                Attribute _attribute_1 = e.getAttribute();
                boolean _isReadOnly = _attribute_1.isReadOnly();
                boolean _not = (!_isReadOnly);
                _and = (_and_1 && _not);
              }
              return Boolean.valueOf(_and);
            }
          };
        Iterable<FeatureReference> _filter = IterableExtensions.<FeatureReference>filter(_featureReferences_1, _function_1);
        for(final FeatureReference featureReference : _filter) {
          _builder.append("\t");
          String _className_6 = this._names.getClassName(it);
          String _firstLower_2 = StringExtensions.toFirstLower(_className_6);
          _builder.append(_firstLower_2, "	");
          _builder.append(".");
          Attribute _target = featureReference.getTarget();
          String _mutatorMethodName = this._features.getMutatorMethodName(_target);
          _builder.append(_mutatorMethodName, "	");
          _builder.append("(sourceDto.");
          Attribute _target_1 = featureReference.getTarget();
          String _accessorMethodName = this._features.getAccessorMethodName(_target_1);
          _builder.append(_accessorMethodName, "	");
          _builder.append("());");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("}");
      _builder.newLine();
      {
        for(final Entity entity : entities) {
          _builder.newLine();
          _builder.append("protected ");
          String _name_1 = entity.getName();
          _builder.append(_name_1, "");
          _builder.append(" mapFrom");
          String _className_7 = this._names.getClassName(it);
          _builder.append(_className_7, "");
          _builder.append("To");
          String _name_2 = entity.getName();
          _builder.append(_name_2, "");
          _builder.append("(");
          String _className_8 = this._names.getClassName(it);
          _builder.append(_className_8, "");
          _builder.append(" sourceDto) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("return mapFrom");
          String _className_9 = this._names.getClassName(it);
          _builder.append(_className_9, "	");
          _builder.append("To");
          String _name_3 = entity.getName();
          _builder.append(_name_3, "	");
          _builder.append("(sourceDto, MapperContextHolder.createEntity(");
          String _name_4 = entity.getName();
          _builder.append(_name_4, "	");
          _builder.append(".class));");
          _builder.newLineIfNotEmpty();
          _builder.append("}");
          _builder.newLine();
          _builder.newLine();
          _builder.append("protected ");
          String _name_5 = entity.getName();
          _builder.append(_name_5, "");
          _builder.append(" mapFrom");
          String _className_10 = this._names.getClassName(it);
          _builder.append(_className_10, "");
          _builder.append("To");
          String _name_6 = entity.getName();
          _builder.append(_name_6, "");
          _builder.append("(");
          String _className_11 = this._names.getClassName(it);
          _builder.append(_className_11, "");
          _builder.append(" sourceDto, ");
          String _name_7 = entity.getName();
          _builder.append(_name_7, "");
          _builder.append(" ");
          String _variableName = this._names.toVariableName(entity);
          _builder.append(_variableName, "");
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("if (MapperContextHolder.getContext().containsKey(sourceDto)) {");
          _builder.newLine();
          _builder.append("\t\t");
          _builder.append("return (");
          String _name_8 = entity.getName();
          _builder.append(_name_8, "		");
          _builder.append(") MapperContextHolder.getContext().get(sourceDto);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          {
            boolean _and = false;
            DataView _superType_2 = it.getSuperType();
            boolean _notEquals_1 = (!Objects.equal(_superType_2, null));
            if (!_notEquals_1) {
              _and = false;
            } else {
              DataView _superType_3 = it.getSuperType();
              EList<FeatureReference> _featureReferences_2 = _superType_3.getFeatureReferences();
              final Function1<FeatureReference,Entity> _function_2 = new Function1<FeatureReference,Entity>() {
                  public Entity apply(final FeatureReference e) {
                    Entity _source = e.getSource();
                    return _source;
                  }
                };
              List<Entity> _map_1 = ListExtensions.<FeatureReference, Entity>map(_featureReferences_2, _function_2);
              boolean _contains = _map_1.contains(entity);
              _and = (_notEquals_1 && _contains);
            }
            if (_and) {
              _builder.append("\t");
              _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
              DataView _superType_4 = it.getSuperType();
              String _name_9 = _superType_4.getName();
              _builder.append(_name_9, "	");
              _builder.append(".class).mapTo(sourceDto, ");
              String _variableName_1 = this._names.toVariableName(entity);
              _builder.append(_variableName_1, "	");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("\t");
              _builder.append("MapperContextHolder.getContext().put(sourceDto, ");
              String _variableName_2 = this._names.toVariableName(entity);
              _builder.append(_variableName_2, "	");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
            }
          }
          {
            EList<FeatureReference> _featureReferences_3 = it.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function_3 = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  boolean _and_2 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity);
                  if (!_equals) {
                    _and_2 = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _notEquals = (!Objects.equal(_attribute, null));
                    _and_2 = (_equals && _notEquals);
                  }
                  if (!_and_2) {
                    _and_1 = false;
                  } else {
                    Attribute _attribute_1 = e.getAttribute();
                    boolean _isReference = _attribute_1==null?false:_attribute_1.isReference();
                    boolean _not = (!_isReference);
                    _and_1 = (_and_2 && _not);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    Attribute _attribute_2 = e.getAttribute();
                    boolean _isReadOnly = _attribute_2==null?false:_attribute_2.isReadOnly();
                    boolean _not_1 = (!_isReadOnly);
                    _and = (_and_1 && _not_1);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<FeatureReference> _filter_1 = IterableExtensions.<FeatureReference>filter(_featureReferences_3, _function_3);
            for(final FeatureReference featureReference_1 : _filter_1) {
              {
                Attribute _attribute = featureReference_1.getAttribute();
                boolean _isIdentifier = _attribute.isIdentifier();
                if (_isIdentifier) {
                  _builder.append("\t");
                  _builder.append("if (sourceDto.");
                  Attribute _target_2 = featureReference_1.getTarget();
                  String _accessorMethodName_1 = this._features.getAccessorMethodName(_target_2);
                  _builder.append(_accessorMethodName_1, "	");
                  _builder.append("() != null");
                  {
                    Object _property = EObjectPropertiesAdapter.getProperty(featureReference_1, "sourceReference");
                    boolean _notEquals_2 = (!Objects.equal(_property, null));
                    if (_notEquals_2) {
                      _builder.append(" ");
                      EObjectPropertiesAdapter.setProperty(it, "generateIsContextMethod", "true");
                      _builder.append(" && MapperContextHolder.isContext(\"");
                      Object _property_1 = EObjectPropertiesAdapter.getProperty(featureReference_1, "sourceReference");
                      String _name_10 = ((Attribute) _property_1).getName();
                      _builder.append(_name_10, "	");
                      _builder.append("\")");
                    }
                  }
                  _builder.append(") {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  String _variableName_3 = this._names.toVariableName(entity);
                  _builder.append(_variableName_3, "		");
                  _builder.append(".");
                  Attribute _identifier = entity.getIdentifier();
                  String _mutatorMethodName_1 = this._features.getMutatorMethodName(_identifier);
                  _builder.append(_mutatorMethodName_1, "		");
                  _builder.append("(sourceDto.");
                  Attribute _target_3 = featureReference_1.getTarget();
                  String _accessorMethodName_2 = this._features.getAccessorMethodName(_target_3);
                  _builder.append(_accessorMethodName_2, "		");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                } else {
                  _builder.append("\t");
                  String _variableName_4 = this._names.toVariableName(entity);
                  _builder.append(_variableName_4, "	");
                  _builder.append(".");
                  Attribute _attribute_1 = featureReference_1.getAttribute();
                  String _mutatorMethodName_2 = this._features.getMutatorMethodName(_attribute_1);
                  _builder.append(_mutatorMethodName_2, "	");
                  _builder.append("(sourceDto.");
                  Attribute _target_4 = featureReference_1.getTarget();
                  String _accessorMethodName_3 = this._features.getAccessorMethodName(_target_4);
                  _builder.append(_accessorMethodName_3, "	");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                }
              }
            }
          }
          _builder.append("\t");
          Index _index = new Index();
          final Index iterator = _index;
          _builder.newLineIfNotEmpty();
          {
            EList<FeatureReference> _featureReferences_4 = it.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function_4 = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity);
                  if (!_equals) {
                    _and_1 = false;
                  } else {
                    DataView _view = e.getView();
                    boolean _notEquals = (!Objects.equal(_view, null));
                    _and_1 = (_equals && _notEquals);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _isReference = _attribute==null?false:_attribute.isReference();
                    _and = (_and_1 && _isReference);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<FeatureReference> _filter_2 = IterableExtensions.<FeatureReference>filter(_featureReferences_4, _function_4);
            for(final FeatureReference includedFeatureReference : _filter_2) {
              {
                boolean _firstIteration = iterator.firstIteration();
                if (_firstIteration) {
                  _builder.append("\t");
                  EObjectPropertiesAdapter.setProperty(it, "generateIsContextMethod", "true");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("if (!MapperContextHolder.isContext(\"save");
                  String _name_11 = entity.getName();
                  _builder.append(_name_11, "	");
                  _builder.append("\") && !MapperContextHolder.isContext(\"update");
                  String _name_12 = entity.getName();
                  _builder.append(_name_12, "	");
                  _builder.append("\")) {");
                  _builder.newLineIfNotEmpty();
                }
              }
              {
                Attribute _attribute_2 = includedFeatureReference.getAttribute();
                boolean _isMany = _attribute_2.isMany();
                boolean _not = (!_isMany);
                if (_not) {
                  _builder.append("\t");
                  _builder.append("if (null != sourceDto.");
                  Attribute _target_5 = includedFeatureReference.getTarget();
                  String _accessorMethodName_4 = this._features.getAccessorMethodName(_target_5);
                  _builder.append(_accessorMethodName_4, "	");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  Attribute _attribute_3 = includedFeatureReference.getAttribute();
                  Type _dataType = _attribute_3.getDataType();
                  String _name_13 = _dataType.getName();
                  _builder.append(_name_13, "		");
                  _builder.append(" ");
                  Attribute _attribute_4 = includedFeatureReference.getAttribute();
                  String _name_14 = _attribute_4.getName();
                  _builder.append(_name_14, "		");
                  _builder.append(" = (");
                  Attribute _attribute_5 = includedFeatureReference.getAttribute();
                  Type _dataType_1 = _attribute_5.getDataType();
                  String _name_15 = _dataType_1.getName();
                  _builder.append(_name_15, "		");
                  _builder.append(") dtoAssemblerRegistry.getDtoAssembler(");
                  DataView _view = includedFeatureReference.getView();
                  String _name_16 = _view.getName();
                  _builder.append(_name_16, "		");
                  _builder.append(".class).mapTo(sourceDto.");
                  Attribute _target_6 = includedFeatureReference.getTarget();
                  String _accessorMethodName_5 = this._features.getAccessorMethodName(_target_6);
                  _builder.append(_accessorMethodName_5, "		");
                  _builder.append("(), ");
                  Attribute _attribute_6 = includedFeatureReference.getAttribute();
                  Type _dataType_2 = _attribute_6.getDataType();
                  String _name_17 = _dataType_2.getName();
                  _builder.append(_name_17, "		");
                  _builder.append(".class);");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  String _variableName_5 = this._names.toVariableName(entity);
                  _builder.append(_variableName_5, "		");
                  _builder.append(".");
                  Attribute _attribute_7 = includedFeatureReference.getAttribute();
                  String _mutatorMethodName_3 = this._features.getMutatorMethodName(_attribute_7);
                  _builder.append(_mutatorMethodName_3, "		");
                  _builder.append("(");
                  Attribute _attribute_8 = includedFeatureReference.getAttribute();
                  String _name_18 = _attribute_8.getName();
                  _builder.append(_name_18, "		");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                } else {
                  _builder.append("\t");
                  _builder.append("if (null != sourceDto.");
                  Attribute _target_7 = includedFeatureReference.getTarget();
                  String _accessorMethodName_6 = this._features.getAccessorMethodName(_target_7);
                  _builder.append(_accessorMethodName_6, "	");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("for (");
                  DataView _view_1 = includedFeatureReference.getView();
                  String _fullClassNameForDatamapping = NameExtension.getFullClassNameForDatamapping(_view_1);
                  String _addImport = this._names.addImport(it, _fullClassNameForDatamapping);
                  _builder.append(_addImport, "		");
                  _builder.append(" ");
                  DataView _view_2 = includedFeatureReference.getView();
                  String _name_19 = _view_2.getName();
                  String _firstLower_3 = StringExtensions.toFirstLower(_name_19);
                  _builder.append(_firstLower_3, "		");
                  _builder.append(" : sourceDto.");
                  Attribute _target_8 = includedFeatureReference.getTarget();
                  String _accessorMethodName_7 = this._features.getAccessorMethodName(_target_8);
                  _builder.append(_accessorMethodName_7, "		");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  Attribute _attribute_9 = includedFeatureReference.getAttribute();
                  Type _dataType_3 = _attribute_9.getDataType();
                  String _name_20 = _dataType_3.getName();
                  _builder.append(_name_20, "			");
                  _builder.append(" ");
                  Attribute _attribute_10 = includedFeatureReference.getAttribute();
                  String _name_21 = _attribute_10.getName();
                  _builder.append(_name_21, "			");
                  _builder.append(" = (");
                  Attribute _attribute_11 = includedFeatureReference.getAttribute();
                  Type _dataType_4 = _attribute_11.getDataType();
                  String _name_22 = _dataType_4.getName();
                  _builder.append(_name_22, "			");
                  _builder.append(") dtoAssemblerRegistry.getDtoAssembler(");
                  DataView _view_3 = includedFeatureReference.getView();
                  String _name_23 = _view_3.getName();
                  _builder.append(_name_23, "			");
                  _builder.append(".class).mapTo(");
                  DataView _view_4 = includedFeatureReference.getView();
                  String _name_24 = _view_4.getName();
                  String _firstLower_4 = StringExtensions.toFirstLower(_name_24);
                  _builder.append(_firstLower_4, "			");
                  _builder.append(", ");
                  Attribute _attribute_12 = includedFeatureReference.getAttribute();
                  Type _dataType_5 = _attribute_12.getDataType();
                  String _name_25 = _dataType_5.getName();
                  _builder.append(_name_25, "			");
                  _builder.append(".class);");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  String _variableName_6 = this._names.toVariableName(entity);
                  _builder.append(_variableName_6, "			");
                  _builder.append(".");
                  Attribute _attribute_13 = includedFeatureReference.getAttribute();
                  String _addMethodName = this._features.getAddMethodName(_attribute_13);
                  _builder.append(_addMethodName, "			");
                  _builder.append("(");
                  Attribute _attribute_14 = includedFeatureReference.getAttribute();
                  String _name_26 = _attribute_14.getName();
                  _builder.append(_name_26, "			");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                }
              }
              _builder.append("\t");
              iterator.increment();
              _builder.newLineIfNotEmpty();
            }
          }
          {
            int _counter0 = iterator.getCounter0();
            boolean _greaterThan = (_counter0 > 0);
            if (_greaterThan) {
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            }
          }
          _builder.append("\t");
          _builder.append("return ");
          String _variableName_7 = this._names.toVariableName(entity);
          _builder.append(_variableName_7, "	");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.append("}");
          _builder.newLine();
        }
      }
      {
        for(final Entity entity_1 : entities) {
          _builder.newLine();
          _builder.append("protected ");
          String _className_12 = this._names.getClassName(it);
          _builder.append(_className_12, "");
          _builder.append(" mapFrom");
          String _name_27 = entity_1.getName();
          _builder.append(_name_27, "");
          _builder.append("To");
          String _className_13 = this._names.getClassName(it);
          _builder.append(_className_13, "");
          _builder.append("(");
          String _name_28 = entity_1.getName();
          _builder.append(_name_28, "");
          _builder.append(" ");
          String _variableName_8 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_8, "");
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("if (MapperContextHolder.getContext().containsKey(");
          String _variableName_9 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_9, "	");
          _builder.append(")) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t");
          _builder.append("return (");
          String _className_14 = this._names.getClassName(it);
          _builder.append(_className_14, "		");
          _builder.append(") MapperContextHolder.getContext().get(");
          String _variableName_10 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_10, "		");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          _builder.append("\t");
          String _className_15 = this._names.getClassName(it);
          _builder.append(_className_15, "	");
          _builder.append(" ");
          String _className_16 = this._names.getClassName(it);
          String _firstLower_5 = StringExtensions.toFirstLower(_className_16);
          _builder.append(_firstLower_5, "	");
          _builder.append(" = new ");
          String _className_17 = this._names.getClassName(it);
          _builder.append(_className_17, "	");
          _builder.append("();");
          _builder.newLineIfNotEmpty();
          {
            boolean _and_1 = false;
            DataView _superType_5 = it.getSuperType();
            boolean _notEquals_3 = (!Objects.equal(_superType_5, null));
            if (!_notEquals_3) {
              _and_1 = false;
            } else {
              DataView _superType_6 = it.getSuperType();
              EList<FeatureReference> _featureReferences_5 = _superType_6.getFeatureReferences();
              final Function1<FeatureReference,Entity> _function_5 = new Function1<FeatureReference,Entity>() {
                  public Entity apply(final FeatureReference e) {
                    Entity _source = e.getSource();
                    return _source;
                  }
                };
              List<Entity> _map_2 = ListExtensions.<FeatureReference, Entity>map(_featureReferences_5, _function_5);
              boolean _contains_1 = _map_2.contains(entity_1);
              _and_1 = (_notEquals_3 && _contains_1);
            }
            if (_and_1) {
              _builder.append("\t");
              _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
              DataView _superType_7 = it.getSuperType();
              String _name_29 = _superType_7.getName();
              _builder.append(_name_29, "	");
              _builder.append(".class)");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append(".mapTo(dtoAssemblerRegistry.getDtoAssembler(");
              DataView _superType_8 = it.getSuperType();
              String _name_30 = _superType_8.getName();
              _builder.append(_name_30, "		");
              _builder.append(".class)");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t\t\t");
              _builder.append(".mapFromEntity(");
              String _variableName_11 = this._names.toVariableName(entity_1);
              _builder.append(_variableName_11, "				");
              _builder.append("), ");
              String _className_18 = this._names.getClassName(it);
              String _firstLower_6 = StringExtensions.toFirstLower(_className_18);
              _builder.append(_firstLower_6, "				");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("MapperContextHolder.getContext().put(");
          String _variableName_12 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_12, "	");
          _builder.append(", ");
          String _className_19 = this._names.getClassName(it);
          String _firstLower_7 = StringExtensions.toFirstLower(_className_19);
          _builder.append(_firstLower_7, "	");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          {
            EList<FeatureReference> _featureReferences_6 = it.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function_6 = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  boolean _and_2 = false;
                  boolean _and_3 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity_1);
                  if (!_equals) {
                    _and_3 = false;
                  } else {
                    Object _property = EObjectPropertiesAdapter.getProperty(e, "sourceReference");
                    boolean _equals_1 = Objects.equal(_property, null);
                    _and_3 = (_equals && _equals_1);
                  }
                  if (!_and_3) {
                    _and_2 = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _notEquals = (!Objects.equal(_attribute, null));
                    _and_2 = (_and_3 && _notEquals);
                  }
                  if (!_and_2) {
                    _and_1 = false;
                  } else {
                    Attribute _attribute_1 = e.getAttribute();
                    boolean _isReference = _attribute_1.isReference();
                    boolean _not = (!_isReference);
                    _and_1 = (_and_2 && _not);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    Attribute _attribute_2 = e.getAttribute();
                    boolean _isReadOnly = _attribute_2.isReadOnly();
                    boolean _not_1 = (!_isReadOnly);
                    _and = (_and_1 && _not_1);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<FeatureReference> _filter_3 = IterableExtensions.<FeatureReference>filter(_featureReferences_6, _function_6);
            for(final FeatureReference featureReference_2 : _filter_3) {
              _builder.append("\t");
              String _className_20 = this._names.getClassName(it);
              String _firstLower_8 = StringExtensions.toFirstLower(_className_20);
              _builder.append(_firstLower_8, "	");
              _builder.append(".");
              Attribute _target_9 = featureReference_2.getTarget();
              String _mutatorMethodName_4 = this._features.getMutatorMethodName(_target_9);
              _builder.append(_mutatorMethodName_4, "	");
              _builder.append("(");
              String _variableName_13 = this._names.toVariableName(entity_1);
              _builder.append(_variableName_13, "	");
              _builder.append(".");
              Attribute _attribute_15 = featureReference_2.getAttribute();
              String _accessorMethodName_8 = this._features.getAccessorMethodName(_attribute_15);
              _builder.append(_accessorMethodName_8, "	");
              _builder.append("());");
              _builder.newLineIfNotEmpty();
            }
          }
          {
            EList<FeatureReference> _featureReferences_7 = it.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function_7 = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity_1);
                  if (!_equals) {
                    _and_1 = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _isReference = _attribute==null?false:_attribute.isReference();
                    _and_1 = (_equals && _isReference);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    DataView _view = e.getView();
                    boolean _notEquals = (!Objects.equal(_view, null));
                    _and = (_and_1 && _notEquals);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<FeatureReference> _filter_4 = IterableExtensions.<FeatureReference>filter(_featureReferences_7, _function_7);
            for(final FeatureReference includedFeatureReference_1 : _filter_4) {
              {
                Attribute _attribute_16 = includedFeatureReference_1.getAttribute();
                boolean _isMany_1 = _attribute_16.isMany();
                boolean _not_1 = (!_isMany_1);
                if (_not_1) {
                  _builder.append("\t");
                  _builder.append("if (null != ");
                  String _variableName_14 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_14, "	");
                  _builder.append(".");
                  Attribute _attribute_17 = includedFeatureReference_1.getAttribute();
                  String _accessorMethodName_9 = this._features.getAccessorMethodName(_attribute_17);
                  _builder.append(_accessorMethodName_9, "	");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  DataView _view_5 = includedFeatureReference_1.getView();
                  String _fullyQualifiedName = NameExtension.getFullyQualifiedName(_view_5, Boolean.valueOf(true));
                  String _addImport_1 = this._names.addImport(it, _fullyQualifiedName);
                  _builder.append(_addImport_1, "		");
                  _builder.append(" ");
                  Attribute _attribute_18 = includedFeatureReference_1.getAttribute();
                  String _name_31 = _attribute_18.getName();
                  _builder.append(_name_31, "		");
                  _builder.append(" = (");
                  DataView _view_6 = includedFeatureReference_1.getView();
                  String _name_32 = _view_6.getName();
                  _builder.append(_name_32, "		");
                  _builder.append(") MapperContextHolder.getContext().get(");
                  String _variableName_15 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_15, "		");
                  _builder.append(".");
                  Attribute _attribute_19 = includedFeatureReference_1.getAttribute();
                  String _accessorMethodName_10 = this._features.getAccessorMethodName(_attribute_19);
                  _builder.append(_accessorMethodName_10, "		");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("if (null == ");
                  Attribute _attribute_20 = includedFeatureReference_1.getAttribute();
                  String _name_33 = _attribute_20.getName();
                  _builder.append(_name_33, "		");
                  _builder.append(") {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  Attribute _attribute_21 = includedFeatureReference_1.getAttribute();
                  String _name_34 = _attribute_21.getName();
                  _builder.append(_name_34, "			");
                  _builder.append(" = dtoAssemblerRegistry.getDtoAssembler(");
                  DataView _view_7 = includedFeatureReference_1.getView();
                  String _name_35 = _view_7.getName();
                  _builder.append(_name_35, "			");
                  _builder.append(".class).mapFromEntity(");
                  String _variableName_16 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_16, "			");
                  _builder.append(".");
                  Attribute _attribute_22 = includedFeatureReference_1.getAttribute();
                  String _accessorMethodName_11 = this._features.getAccessorMethodName(_attribute_22);
                  _builder.append(_accessorMethodName_11, "			");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("\t");
                  String _className_21 = this._names.getClassName(it);
                  String _firstLower_9 = StringExtensions.toFirstLower(_className_21);
                  _builder.append(_firstLower_9, "		");
                  _builder.append(".");
                  Attribute _target_10 = includedFeatureReference_1.getTarget();
                  String _mutatorMethodName_5 = this._features.getMutatorMethodName(_target_10);
                  _builder.append(_mutatorMethodName_5, "		");
                  _builder.append("(");
                  Attribute _attribute_23 = includedFeatureReference_1.getAttribute();
                  String _name_36 = _attribute_23.getName();
                  _builder.append(_name_36, "		");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                } else {
                  _builder.append("\t");
                  _builder.append("if (");
                  String _variableName_17 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_17, "	");
                  _builder.append(".");
                  Attribute _attribute_24 = includedFeatureReference_1.getAttribute();
                  String _hasMethodName = this._features.getHasMethodName(_attribute_24);
                  _builder.append(_hasMethodName, "	");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("for (");
                  Attribute _attribute_25 = includedFeatureReference_1.getAttribute();
                  Type _dataType_6 = _attribute_25.getDataType();
                  String _name_37 = _dataType_6.getName();
                  _builder.append(_name_37, "		");
                  _builder.append(" next : ");
                  String _variableName_18 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_18, "		");
                  _builder.append(".");
                  Attribute _attribute_26 = includedFeatureReference_1.getAttribute();
                  String _accessorMethodName_12 = this._features.getAccessorMethodName(_attribute_26);
                  _builder.append(_accessorMethodName_12, "		");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  DataView _view_8 = includedFeatureReference_1.getView();
                  String _name_38 = _view_8.getName();
                  _builder.append(_name_38, "			");
                  _builder.append(" next");
                  DataView _view_9 = includedFeatureReference_1.getView();
                  String _name_39 = _view_9.getName();
                  _builder.append(_name_39, "			");
                  _builder.append(" = (");
                  DataView _view_10 = includedFeatureReference_1.getView();
                  String _name_40 = _view_10.getName();
                  _builder.append(_name_40, "			");
                  _builder.append(") MapperContextHolder.getContext().get(next);");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  _builder.append("if (null == next");
                  DataView _view_11 = includedFeatureReference_1.getView();
                  String _name_41 = _view_11.getName();
                  _builder.append(_name_41, "			");
                  _builder.append(") {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t\t");
                  _builder.append("next");
                  DataView _view_12 = includedFeatureReference_1.getView();
                  String _name_42 = _view_12.getName();
                  _builder.append(_name_42, "				");
                  _builder.append(" = dtoAssemblerRegistry.getDtoAssembler(");
                  DataView _view_13 = includedFeatureReference_1.getView();
                  String _name_43 = _view_13.getName();
                  _builder.append(_name_43, "				");
                  _builder.append(".class).mapFromEntity(next);");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  String _className_22 = this._names.getClassName(it);
                  String _firstLower_10 = StringExtensions.toFirstLower(_className_22);
                  _builder.append(_firstLower_10, "			");
                  _builder.append(".");
                  Attribute _target_11 = includedFeatureReference_1.getTarget();
                  String _addMethodName_1 = this._features.getAddMethodName(_target_11);
                  _builder.append(_addMethodName_1, "			");
                  _builder.append("(next");
                  DataView _view_14 = includedFeatureReference_1.getView();
                  String _name_44 = _view_14.getName();
                  _builder.append(_name_44, "			");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                }
              }
            }
          }
          _builder.append("\t");
          _builder.append("return ");
          String _className_23 = this._names.getClassName(it);
          String _firstLower_11 = StringExtensions.toFirstLower(_className_23);
          _builder.append(_firstLower_11, "	");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.append("}");
          _builder.newLine();
          _builder.newLine();
          _builder.append("protected ");
          String _className_24 = this._names.getClassName(it);
          _builder.append(_className_24, "");
          _builder.append(" mapFrom");
          String _name_45 = entity_1.getName();
          _builder.append(_name_45, "");
          _builder.append("To");
          String _className_25 = this._names.getClassName(it);
          _builder.append(_className_25, "");
          _builder.append("(");
          String _name_46 = entity_1.getName();
          _builder.append(_name_46, "");
          _builder.append(" ");
          String _variableName_19 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_19, "");
          _builder.append(", ");
          String _className_26 = this._names.getClassName(it);
          _builder.append(_className_26, "");
          _builder.append(" ");
          String _className_27 = this._names.getClassName(it);
          String _firstLower_12 = StringExtensions.toFirstLower(_className_27);
          _builder.append(_firstLower_12, "");
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("if (MapperContextHolder.getContext().containsKey(");
          String _variableName_20 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_20, "	");
          _builder.append(")) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t");
          _builder.append("return (");
          String _className_28 = this._names.getClassName(it);
          _builder.append(_className_28, "		");
          _builder.append(") MapperContextHolder.getContext().get(");
          String _variableName_21 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_21, "		");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          {
            boolean _and_2 = false;
            DataView _superType_9 = it.getSuperType();
            boolean _notEquals_4 = (!Objects.equal(_superType_9, null));
            if (!_notEquals_4) {
              _and_2 = false;
            } else {
              DataView _superType_10 = it.getSuperType();
              EList<FeatureReference> _featureReferences_8 = _superType_10.getFeatureReferences();
              final Function1<FeatureReference,Entity> _function_8 = new Function1<FeatureReference,Entity>() {
                  public Entity apply(final FeatureReference e) {
                    Entity _source = e.getSource();
                    return _source;
                  }
                };
              List<Entity> _map_3 = ListExtensions.<FeatureReference, Entity>map(_featureReferences_8, _function_8);
              boolean _contains_2 = _map_3.contains(entity_1);
              _and_2 = (_notEquals_4 && _contains_2);
            }
            if (_and_2) {
              _builder.append("\t");
              _builder.append("dtoAssemblerRegistry.getDtoAssembler(");
              DataView _superType_11 = it.getSuperType();
              String _name_47 = _superType_11.getName();
              _builder.append(_name_47, "	");
              _builder.append(".class)");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append(".mapTo(dtoAssemblerRegistry.getDtoAssembler(");
              DataView _superType_12 = it.getSuperType();
              String _name_48 = _superType_12.getName();
              _builder.append(_name_48, "		");
              _builder.append(".class)");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t\t\t");
              _builder.append(".mapFromEntity(");
              String _variableName_22 = this._names.toVariableName(entity_1);
              _builder.append(_variableName_22, "				");
              _builder.append("), ");
              String _className_29 = this._names.getClassName(it);
              String _firstLower_13 = StringExtensions.toFirstLower(_className_29);
              _builder.append(_firstLower_13, "				");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("MapperContextHolder.getContext().put(");
          String _variableName_23 = this._names.toVariableName(entity_1);
          _builder.append(_variableName_23, "	");
          _builder.append(", ");
          String _className_30 = this._names.getClassName(it);
          String _firstLower_14 = StringExtensions.toFirstLower(_className_30);
          _builder.append(_firstLower_14, "	");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          {
            EList<FeatureReference> _featureReferences_9 = it.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function_9 = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  boolean _and_2 = false;
                  boolean _and_3 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity_1);
                  if (!_equals) {
                    _and_3 = false;
                  } else {
                    Object _property = EObjectPropertiesAdapter.getProperty(e, "sourceReference");
                    boolean _equals_1 = Objects.equal(_property, null);
                    _and_3 = (_equals && _equals_1);
                  }
                  if (!_and_3) {
                    _and_2 = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _notEquals = (!Objects.equal(_attribute, null));
                    _and_2 = (_and_3 && _notEquals);
                  }
                  if (!_and_2) {
                    _and_1 = false;
                  } else {
                    Attribute _attribute_1 = e.getAttribute();
                    boolean _isReference = _attribute_1.isReference();
                    boolean _not = (!_isReference);
                    _and_1 = (_and_2 && _not);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    Attribute _attribute_2 = e.getAttribute();
                    boolean _isReadOnly = _attribute_2.isReadOnly();
                    boolean _not_1 = (!_isReadOnly);
                    _and = (_and_1 && _not_1);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<FeatureReference> _filter_5 = IterableExtensions.<FeatureReference>filter(_featureReferences_9, _function_9);
            for(final FeatureReference featureReference_3 : _filter_5) {
              _builder.append("\t");
              String _className_31 = this._names.getClassName(it);
              String _firstLower_15 = StringExtensions.toFirstLower(_className_31);
              _builder.append(_firstLower_15, "	");
              _builder.append(".");
              Attribute _target_12 = featureReference_3.getTarget();
              String _mutatorMethodName_6 = this._features.getMutatorMethodName(_target_12);
              _builder.append(_mutatorMethodName_6, "	");
              _builder.append("(");
              String _variableName_24 = this._names.toVariableName(entity_1);
              _builder.append(_variableName_24, "	");
              _builder.append(".");
              Attribute _attribute_27 = featureReference_3.getAttribute();
              String _accessorMethodName_13 = this._features.getAccessorMethodName(_attribute_27);
              _builder.append(_accessorMethodName_13, "	");
              _builder.append("());");
              _builder.newLineIfNotEmpty();
            }
          }
          {
            EList<FeatureReference> _featureReferences_10 = it.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function_10 = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity_1);
                  if (!_equals) {
                    _and_1 = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _isReference = _attribute==null?false:_attribute.isReference();
                    _and_1 = (_equals && _isReference);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    DataView _view = e.getView();
                    boolean _notEquals = (!Objects.equal(_view, null));
                    _and = (_and_1 && _notEquals);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<FeatureReference> _filter_6 = IterableExtensions.<FeatureReference>filter(_featureReferences_10, _function_10);
            for(final FeatureReference includedFeatureReference_2 : _filter_6) {
              {
                Attribute _attribute_28 = includedFeatureReference_2.getAttribute();
                boolean _isMany_2 = _attribute_28.isMany();
                boolean _not_2 = (!_isMany_2);
                if (_not_2) {
                  _builder.append("\t");
                  _builder.append("if (null != ");
                  String _variableName_25 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_25, "	");
                  _builder.append(".");
                  Attribute _attribute_29 = includedFeatureReference_2.getAttribute();
                  String _accessorMethodName_14 = this._features.getAccessorMethodName(_attribute_29);
                  _builder.append(_accessorMethodName_14, "	");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  DataView _view_15 = includedFeatureReference_2.getView();
                  String _fullyQualifiedName_1 = NameExtension.getFullyQualifiedName(_view_15, Boolean.valueOf(true));
                  String _addImport_2 = this._names.addImport(it, _fullyQualifiedName_1);
                  _builder.append(_addImport_2, "		");
                  _builder.append(" ");
                  Attribute _attribute_30 = includedFeatureReference_2.getAttribute();
                  String _name_49 = _attribute_30.getName();
                  _builder.append(_name_49, "		");
                  _builder.append(" = (");
                  DataView _view_16 = includedFeatureReference_2.getView();
                  String _name_50 = _view_16.getName();
                  _builder.append(_name_50, "		");
                  _builder.append(") MapperContextHolder.getContext().get(");
                  String _variableName_26 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_26, "		");
                  _builder.append(".");
                  Attribute _attribute_31 = includedFeatureReference_2.getAttribute();
                  String _accessorMethodName_15 = this._features.getAccessorMethodName(_attribute_31);
                  _builder.append(_accessorMethodName_15, "		");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("if (null == ");
                  Attribute _attribute_32 = includedFeatureReference_2.getAttribute();
                  String _name_51 = _attribute_32.getName();
                  _builder.append(_name_51, "		");
                  _builder.append(") {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  Attribute _attribute_33 = includedFeatureReference_2.getAttribute();
                  String _name_52 = _attribute_33.getName();
                  _builder.append(_name_52, "			");
                  _builder.append(" = dtoAssemblerRegistry.getDtoAssembler(");
                  DataView _view_17 = includedFeatureReference_2.getView();
                  String _name_53 = _view_17.getName();
                  _builder.append(_name_53, "			");
                  _builder.append(".class).mapFromEntity(");
                  String _variableName_27 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_27, "			");
                  _builder.append(".");
                  Attribute _attribute_34 = includedFeatureReference_2.getAttribute();
                  String _accessorMethodName_16 = this._features.getAccessorMethodName(_attribute_34);
                  _builder.append(_accessorMethodName_16, "			");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("\t");
                  String _className_32 = this._names.getClassName(it);
                  String _firstLower_16 = StringExtensions.toFirstLower(_className_32);
                  _builder.append(_firstLower_16, "		");
                  _builder.append(".");
                  Attribute _target_13 = includedFeatureReference_2.getTarget();
                  String _mutatorMethodName_7 = this._features.getMutatorMethodName(_target_13);
                  _builder.append(_mutatorMethodName_7, "		");
                  _builder.append("(");
                  Attribute _attribute_35 = includedFeatureReference_2.getAttribute();
                  String _name_54 = _attribute_35.getName();
                  _builder.append(_name_54, "		");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                } else {
                  _builder.append("\t");
                  _builder.append("if (");
                  String _variableName_28 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_28, "	");
                  _builder.append(".");
                  Attribute _attribute_36 = includedFeatureReference_2.getAttribute();
                  String _hasMethodName_1 = this._features.getHasMethodName(_attribute_36);
                  _builder.append(_hasMethodName_1, "	");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("for (");
                  Attribute _attribute_37 = includedFeatureReference_2.getAttribute();
                  Type _dataType_7 = _attribute_37.getDataType();
                  String _name_55 = _dataType_7.getName();
                  _builder.append(_name_55, "		");
                  _builder.append(" next : ");
                  String _variableName_29 = this._names.toVariableName(entity_1);
                  _builder.append(_variableName_29, "		");
                  _builder.append(".");
                  Attribute _attribute_38 = includedFeatureReference_2.getAttribute();
                  String _accessorMethodName_17 = this._features.getAccessorMethodName(_attribute_38);
                  _builder.append(_accessorMethodName_17, "		");
                  _builder.append("()) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  DataView _view_18 = includedFeatureReference_2.getView();
                  String _name_56 = _view_18.getName();
                  _builder.append(_name_56, "			");
                  _builder.append(" next");
                  DataView _view_19 = includedFeatureReference_2.getView();
                  String _name_57 = _view_19.getName();
                  _builder.append(_name_57, "			");
                  _builder.append(" = (");
                  DataView _view_20 = includedFeatureReference_2.getView();
                  String _name_58 = _view_20.getName();
                  _builder.append(_name_58, "			");
                  _builder.append(") MapperContextHolder.getContext().get(next);");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  _builder.append("if (null == next");
                  DataView _view_21 = includedFeatureReference_2.getView();
                  String _name_59 = _view_21.getName();
                  _builder.append(_name_59, "			");
                  _builder.append(") {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t\t");
                  _builder.append("next");
                  DataView _view_22 = includedFeatureReference_2.getView();
                  String _name_60 = _view_22.getName();
                  _builder.append(_name_60, "				");
                  _builder.append(" = dtoAssemblerRegistry.getDtoAssembler(");
                  DataView _view_23 = includedFeatureReference_2.getView();
                  String _name_61 = _view_23.getName();
                  _builder.append(_name_61, "				");
                  _builder.append(".class).mapFromEntity(next);");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("\t\t");
                  String _className_33 = this._names.getClassName(it);
                  String _firstLower_17 = StringExtensions.toFirstLower(_className_33);
                  _builder.append(_firstLower_17, "			");
                  _builder.append(".");
                  Attribute _target_14 = includedFeatureReference_2.getTarget();
                  String _addMethodName_2 = this._features.getAddMethodName(_target_14);
                  _builder.append(_addMethodName_2, "			");
                  _builder.append("(next");
                  DataView _view_24 = includedFeatureReference_2.getView();
                  String _name_62 = _view_24.getName();
                  _builder.append(_name_62, "			");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                }
              }
            }
          }
          _builder.append("\t");
          _builder.append("return ");
          String _className_34 = this._names.getClassName(it);
          String _firstLower_18 = StringExtensions.toFirstLower(_className_34);
          _builder.append(_firstLower_18, "	");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.append("}");
          _builder.newLine();
        }
      }
      _builder.newLine();
      final Set<Mapper> mappers = MapperExtension.getMappersByDataView(it);
      _builder.newLineIfNotEmpty();
      {
        for(final Mapper mapper : mappers) {
          _builder.newLine();
          _builder.append("protected ");
          String _oppositeElementName = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName, "");
          _builder.append(" mapper");
          String _name_63 = mapper.getName();
          _builder.append(_name_63, "");
          _builder.append("MapsFrom");
          String _className_35 = this._names.getClassName(it);
          _builder.append(_className_35, "");
          _builder.append("To");
          String _oppositeElementName_1 = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName_1, "");
          _builder.append("(");
          String _className_36 = this._names.getClassName(it);
          _builder.append(_className_36, "");
          _builder.append(" sourceDto) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          String _oppositeElementName_2 = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName_2, "	");
          _builder.append(" target = MapperContextHolder.createComplexType(");
          String _oppositeElementName_3 = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName_3, "	");
          _builder.append(".class);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("mapper");
          String _name_64 = mapper.getName();
          _builder.append(_name_64, "	");
          _builder.append("MapsFrom");
          String _className_37 = this._names.getClassName(it);
          _builder.append(_className_37, "	");
          _builder.append("To");
          String _oppositeElementName_4 = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName_4, "	");
          _builder.append("(sourceDto, target);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("return target;");
          _builder.newLine();
          _builder.append("}");
          _builder.newLine();
          _builder.newLine();
          _builder.append("protected void mapper");
          String _name_65 = mapper.getName();
          _builder.append(_name_65, "");
          _builder.append("MapsFrom");
          String _className_38 = this._names.getClassName(it);
          _builder.append(_className_38, "");
          _builder.append("To");
          String _oppositeElementName_5 = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName_5, "");
          _builder.append("(");
          String _className_39 = this._names.getClassName(it);
          _builder.append(_className_39, "");
          _builder.append(" sourceDto, ");
          String _oppositeElementName_6 = MapperExtension.getOppositeElementName(it, mapper);
          _builder.append(_oppositeElementName_6, "");
          _builder.append(" target ) {");
          _builder.newLineIfNotEmpty();
          {
            EList<PropertyMapping> _propertyMappings = mapper.getPropertyMappings();
            boolean _isEmpty = _propertyMappings.isEmpty();
            if (_isEmpty) {
              {
                Boolean _dataViewIsMapperSource = MapperExtension.dataViewIsMapperSource(it, mapper);
                if ((_dataViewIsMapperSource).booleanValue()) {
                  {
                    Boolean _dataViewIsLeft = MapperExtension.dataViewIsLeft(it, mapper);
                    if ((_dataViewIsLeft).booleanValue()) {
                      {
                        ComplexType _left = mapper.getLeft();
                        EList<Attribute> _attributes = _left.getAttributes();
                        final Function1<Attribute,Boolean> _function_11 = new Function1<Attribute,Boolean>() {
                            public Boolean apply(final Attribute lA) {
                              boolean _and = false;
                              boolean _and_1 = false;
                              boolean _and_2 = false;
                              boolean _notEquals = (!Objects.equal(lA, null));
                              if (!_notEquals) {
                                _and_2 = false;
                              } else {
                                boolean _isReference = lA.isReference();
                                boolean _not = (!_isReference);
                                _and_2 = (_notEquals && _not);
                              }
                              if (!_and_2) {
                                _and_1 = false;
                              } else {
                                EObject _eContainer = lA.eContainer();
                                ComplexType _left = mapper.getLeft();
                                boolean _equals = Objects.equal(_eContainer, _left);
                                _and_1 = (_and_2 && _equals);
                              }
                              if (!_and_1) {
                                _and = false;
                              } else {
                                ComplexType _right = mapper.getRight();
                                EList<Attribute> _attributes = _right.getAttributes();
                                final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
                                    public Boolean apply(final Attribute rA) {
                                      boolean _and = false;
                                      boolean _and_1 = false;
                                      boolean _and_2 = false;
                                      boolean _notEquals = (!Objects.equal(rA, null));
                                      if (!_notEquals) {
                                        _and_2 = false;
                                      } else {
                                        boolean _isReference = rA.isReference();
                                        boolean _not = (!_isReference);
                                        _and_2 = (_notEquals && _not);
                                      }
                                      if (!_and_2) {
                                        _and_1 = false;
                                      } else {
                                        boolean _isReadOnly = rA.isReadOnly();
                                        boolean _not_1 = (!_isReadOnly);
                                        _and_1 = (_and_2 && _not_1);
                                      }
                                      if (!_and_1) {
                                        _and = false;
                                      } else {
                                        String _accessorMethodName = DataViewGenerator.this._features.getAccessorMethodName(rA);
                                        String _accessorMethodName_1 = DataViewGenerator.this._features.getAccessorMethodName(lA);
                                        boolean _equals = Objects.equal(_accessorMethodName, _accessorMethodName_1);
                                        _and = (_and_1 && _equals);
                                      }
                                      return Boolean.valueOf(_and);
                                    }
                                  };
                                boolean _exists = IterableExtensions.<Attribute>exists(_attributes, _function);
                                _and = (_and_1 && _exists);
                              }
                              return Boolean.valueOf(_and);
                            }
                          };
                        Iterable<Attribute> _filter_7 = IterableExtensions.<Attribute>filter(_attributes, _function_11);
                        for(final Attribute sA : _filter_7) {
                          ComplexType _right = mapper.getRight();
                          EList<Attribute> _attributes_1 = _right.getAttributes();
                          final Function1<Attribute,Boolean> _function_12 = new Function1<Attribute,Boolean>() {
                              public Boolean apply(final Attribute rA) {
                                String _accessorMethodName = DataViewGenerator.this._features.getAccessorMethodName(rA);
                                String _accessorMethodName_1 = DataViewGenerator.this._features.getAccessorMethodName(sA);
                                boolean _equals = _accessorMethodName.equals(_accessorMethodName_1);
                                return Boolean.valueOf(_equals);
                              }
                            };
                          final Attribute tA = IterableExtensions.<Attribute>findFirst(_attributes_1, _function_12);
                          _builder.newLineIfNotEmpty();
                          {
                            String _dataTypeName = sA.getDataTypeName();
                            String _dataTypeName_1 = tA.getDataTypeName();
                            boolean _equals = Objects.equal(_dataTypeName, _dataTypeName_1);
                            if (_equals) {
                              _builder.append("target.");
                              String _mutatorMethodName_8 = this._features.getMutatorMethodName(tA);
                              _builder.append(_mutatorMethodName_8, "");
                              _builder.append("(sourceDto.");
                              String _accessorMethodName_18 = this._features.getAccessorMethodName(sA);
                              _builder.append(_accessorMethodName_18, "");
                              _builder.append("());");
                              _builder.newLineIfNotEmpty();
                            } else {
                              {
                                Boolean _isBigDecimalSimpleType = EntityExtension.isBigDecimalSimpleType(tA);
                                if ((_isBigDecimalSimpleType).booleanValue()) {
                                  _builder.append("target.");
                                  String _mutatorMethodName_9 = this._features.getMutatorMethodName(tA);
                                  _builder.append(_mutatorMethodName_9, "");
                                  _builder.append("(converter.convert(");
                                  String _addImport_3 = this._names.addImport(it, "java.math.BigDecimal");
                                  _builder.append(_addImport_3, "");
                                  _builder.append(".class, sourceDto.");
                                  String _accessorMethodName_19 = this._features.getAccessorMethodName(sA);
                                  _builder.append(_accessorMethodName_19, "");
                                  _builder.append("()));");
                                  _builder.newLineIfNotEmpty();
                                } else {
                                  Boolean _isTimestampSimpleType = EntityExtension.isTimestampSimpleType(tA);
                                  if ((_isTimestampSimpleType).booleanValue()) {
                                    _builder.append("target.");
                                    String _mutatorMethodName_10 = this._features.getMutatorMethodName(tA);
                                    _builder.append(_mutatorMethodName_10, "");
                                    _builder.append("(converter.convert(");
                                    String _addImport_4 = this._names.addImport(it, "java.sql.Timestamp");
                                    _builder.append(_addImport_4, "");
                                    _builder.append(".class, sourceDto.");
                                    String _accessorMethodName_20 = this._features.getAccessorMethodName(sA);
                                    _builder.append(_accessorMethodName_20, "");
                                    _builder.append("()));");
                                    _builder.newLineIfNotEmpty();
                                  } else {
                                    Boolean _isDateSimpleType = EntityExtension.isDateSimpleType(tA);
                                    if ((_isDateSimpleType).booleanValue()) {
                                      _builder.append("target.");
                                      String _mutatorMethodName_11 = this._features.getMutatorMethodName(tA);
                                      _builder.append(_mutatorMethodName_11, "");
                                      _builder.append("(converter.convert(");
                                      String _addImport_5 = this._names.addImport(it, "java.util.Date");
                                      _builder.append(_addImport_5, "");
                                      _builder.append(".class, sourceDto.");
                                      String _accessorMethodName_21 = this._features.getAccessorMethodName(sA);
                                      _builder.append(_accessorMethodName_21, "");
                                      _builder.append("()));");
                                      _builder.newLineIfNotEmpty();
                                    } else {
                                      _builder.append("target.");
                                      String _mutatorMethodName_12 = this._features.getMutatorMethodName(tA);
                                      _builder.append(_mutatorMethodName_12, "");
                                      _builder.append("(converter.convert(");
                                      String _dataTypeName_2 = tA.getDataTypeName();
                                      _builder.append(_dataTypeName_2, "");
                                      _builder.append(".class, sourceDto.");
                                      String _accessorMethodName_22 = this._features.getAccessorMethodName(sA);
                                      _builder.append(_accessorMethodName_22, "");
                                      _builder.append("()));");
                                      _builder.newLineIfNotEmpty();
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                  {
                    Boolean _dataViewIsRight = MapperExtension.dataViewIsRight(it, mapper);
                    if ((_dataViewIsRight).booleanValue()) {
                      {
                        ComplexType _right_1 = mapper.getRight();
                        EList<Attribute> _attributes_2 = _right_1.getAttributes();
                        final Function1<Attribute,Boolean> _function_13 = new Function1<Attribute,Boolean>() {
                            public Boolean apply(final Attribute rA) {
                              boolean _and = false;
                              boolean _and_1 = false;
                              boolean _and_2 = false;
                              boolean _notEquals = (!Objects.equal(rA, null));
                              if (!_notEquals) {
                                _and_2 = false;
                              } else {
                                boolean _isReference = rA.isReference();
                                boolean _not = (!_isReference);
                                _and_2 = (_notEquals && _not);
                              }
                              if (!_and_2) {
                                _and_1 = false;
                              } else {
                                EObject _eContainer = rA.eContainer();
                                ComplexType _right = mapper.getRight();
                                boolean _equals = Objects.equal(_eContainer, _right);
                                _and_1 = (_and_2 && _equals);
                              }
                              if (!_and_1) {
                                _and = false;
                              } else {
                                ComplexType _left = mapper.getLeft();
                                EList<Attribute> _attributes = _left.getAttributes();
                                final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
                                    public Boolean apply(final Attribute lA) {
                                      boolean _and = false;
                                      boolean _and_1 = false;
                                      boolean _and_2 = false;
                                      boolean _notEquals = (!Objects.equal(lA, null));
                                      if (!_notEquals) {
                                        _and_2 = false;
                                      } else {
                                        boolean _isReference = lA.isReference();
                                        boolean _not = (!_isReference);
                                        _and_2 = (_notEquals && _not);
                                      }
                                      if (!_and_2) {
                                        _and_1 = false;
                                      } else {
                                        boolean _isReadOnly = lA.isReadOnly();
                                        boolean _not_1 = (!_isReadOnly);
                                        _and_1 = (_and_2 && _not_1);
                                      }
                                      if (!_and_1) {
                                        _and = false;
                                      } else {
                                        String _accessorMethodName = DataViewGenerator.this._features.getAccessorMethodName(lA);
                                        String _accessorMethodName_1 = DataViewGenerator.this._features.getAccessorMethodName(rA);
                                        boolean _equals = Objects.equal(_accessorMethodName, _accessorMethodName_1);
                                        _and = (_and_1 && _equals);
                                      }
                                      return Boolean.valueOf(_and);
                                    }
                                  };
                                boolean _exists = IterableExtensions.<Attribute>exists(_attributes, _function);
                                _and = (_and_1 && _exists);
                              }
                              return Boolean.valueOf(_and);
                            }
                          };
                        Iterable<Attribute> _filter_8 = IterableExtensions.<Attribute>filter(_attributes_2, _function_13);
                        for(final Attribute sA_1 : _filter_8) {
                          ComplexType _left_1 = mapper.getLeft();
                          EList<Attribute> _attributes_3 = _left_1.getAttributes();
                          final Function1<Attribute,Boolean> _function_14 = new Function1<Attribute,Boolean>() {
                              public Boolean apply(final Attribute lA) {
                                String _accessorMethodName = DataViewGenerator.this._features.getAccessorMethodName(lA);
                                String _accessorMethodName_1 = DataViewGenerator.this._features.getAccessorMethodName(sA_1);
                                boolean _equals = Objects.equal(_accessorMethodName, _accessorMethodName_1);
                                return Boolean.valueOf(_equals);
                              }
                            };
                          final Attribute tA_1 = IterableExtensions.<Attribute>findFirst(_attributes_3, _function_14);
                          _builder.newLineIfNotEmpty();
                          {
                            String _dataTypeName_3 = sA_1.getDataTypeName();
                            String _dataTypeName_4 = tA_1.getDataTypeName();
                            boolean _equals_1 = Objects.equal(_dataTypeName_3, _dataTypeName_4);
                            if (_equals_1) {
                              _builder.append("target.");
                              String _mutatorMethodName_13 = this._features.getMutatorMethodName(tA_1);
                              _builder.append(_mutatorMethodName_13, "");
                              _builder.append("(sourceDto.");
                              String _accessorMethodName_23 = this._features.getAccessorMethodName(sA_1);
                              _builder.append(_accessorMethodName_23, "");
                              _builder.append("());");
                              _builder.newLineIfNotEmpty();
                            } else {
                              {
                                Boolean _isBigDecimalSimpleType_1 = EntityExtension.isBigDecimalSimpleType(tA_1);
                                if ((_isBigDecimalSimpleType_1).booleanValue()) {
                                  _builder.append("target.");
                                  String _mutatorMethodName_14 = this._features.getMutatorMethodName(tA_1);
                                  _builder.append(_mutatorMethodName_14, "");
                                  _builder.append("(converter.convert(");
                                  String _addImport_6 = this._names.addImport(it, "java.math.BigDecimal");
                                  _builder.append(_addImport_6, "");
                                  _builder.append(".class, sourceDto.");
                                  String _accessorMethodName_24 = this._features.getAccessorMethodName(sA_1);
                                  _builder.append(_accessorMethodName_24, "");
                                  _builder.append("()));");
                                  _builder.newLineIfNotEmpty();
                                } else {
                                  Boolean _isTimestampSimpleType_1 = EntityExtension.isTimestampSimpleType(tA_1);
                                  if ((_isTimestampSimpleType_1).booleanValue()) {
                                    _builder.append("target.");
                                    String _mutatorMethodName_15 = this._features.getMutatorMethodName(tA_1);
                                    _builder.append(_mutatorMethodName_15, "");
                                    _builder.append("(converter.convert(");
                                    String _addImport_7 = this._names.addImport(it, "java.sql.Timestamp");
                                    _builder.append(_addImport_7, "");
                                    _builder.append(".class, sourceDto.");
                                    String _accessorMethodName_25 = this._features.getAccessorMethodName(sA_1);
                                    _builder.append(_accessorMethodName_25, "");
                                    _builder.append("()));");
                                    _builder.newLineIfNotEmpty();
                                  } else {
                                    Boolean _isDateSimpleType_1 = EntityExtension.isDateSimpleType(tA_1);
                                    if ((_isDateSimpleType_1).booleanValue()) {
                                      _builder.append("target.");
                                      String _mutatorMethodName_16 = this._features.getMutatorMethodName(tA_1);
                                      _builder.append(_mutatorMethodName_16, "");
                                      _builder.append("(converter.convert(");
                                      String _addImport_8 = this._names.addImport(it, "java.util.Date");
                                      _builder.append(_addImport_8, "");
                                      _builder.append(".class, sourceDto.");
                                      String _accessorMethodName_26 = this._features.getAccessorMethodName(sA_1);
                                      _builder.append(_accessorMethodName_26, "");
                                      _builder.append("()));");
                                      _builder.newLineIfNotEmpty();
                                    } else {
                                      _builder.append("target.");
                                      String _mutatorMethodName_17 = this._features.getMutatorMethodName(tA_1);
                                      _builder.append(_mutatorMethodName_17, "");
                                      _builder.append("(converter.convert(");
                                      String _dataTypeName_5 = tA_1.getDataTypeName();
                                      _builder.append(_dataTypeName_5, "");
                                      _builder.append(".class, sourceDto.");
                                      String _accessorMethodName_27 = this._features.getAccessorMethodName(sA_1);
                                      _builder.append(_accessorMethodName_27, "");
                                      _builder.append("()));");
                                      _builder.newLineIfNotEmpty();
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            } else {
              {
                EList<PropertyMapping> _propertyMappings_1 = mapper.getPropertyMappings();
                final Function1<PropertyMapping,Boolean> _function_15 = new Function1<PropertyMapping,Boolean>() {
                    public Boolean apply(final PropertyMapping e) {
                      Boolean _dataViewIsPropertyMappingSource = MapperExtension.dataViewIsPropertyMappingSource(it, mapper, e);
                      return _dataViewIsPropertyMappingSource;
                    }
                  };
                Iterable<PropertyMapping> _filter_9 = IterableExtensions.<PropertyMapping>filter(_propertyMappings_1, _function_15);
                for(final PropertyMapping propertyMapping : _filter_9) {
                  {
                    Boolean _dataViewIsLeft_1 = MapperExtension.dataViewIsLeft(it, mapper);
                    if ((_dataViewIsLeft_1).booleanValue()) {
                      {
                        Attribute _left_2 = propertyMapping.getLeft();
                        String _dataTypeName_6 = _left_2.getDataTypeName();
                        Attribute _right_2 = propertyMapping.getRight();
                        String _dataTypeName_7 = _right_2.getDataTypeName();
                        boolean _matches = _dataTypeName_6.matches(_dataTypeName_7);
                        if (_matches) {
                          _builder.append("\t");
                          _builder.append("target.");
                          Attribute _right_3 = propertyMapping.getRight();
                          Attribute _derivedAttributeFromMapperRight = MapperExtension.getDerivedAttributeFromMapperRight(_right_3, mapper);
                          String _mutatorMethodName_18 = this._features.getMutatorMethodName(_derivedAttributeFromMapperRight);
                          _builder.append(_mutatorMethodName_18, "	");
                          _builder.append("(sourceDto.");
                          Attribute _left_3 = propertyMapping.getLeft();
                          Attribute _derivedAttributeFromMapperLeft = MapperExtension.getDerivedAttributeFromMapperLeft(_left_3, mapper);
                          String _accessorMethodName_28 = this._features.getAccessorMethodName(_derivedAttributeFromMapperLeft);
                          _builder.append(_accessorMethodName_28, "	");
                          _builder.append("());");
                          _builder.newLineIfNotEmpty();
                        } else {
                          {
                            Attribute _right_4 = propertyMapping.getRight();
                            Attribute _derivedAttributeFromMapperRight_1 = MapperExtension.getDerivedAttributeFromMapperRight(_right_4, mapper);
                            Boolean _isBigDecimalSimpleType_2 = EntityExtension.isBigDecimalSimpleType(_derivedAttributeFromMapperRight_1);
                            if ((_isBigDecimalSimpleType_2).booleanValue()) {
                              _builder.append("\t");
                              _builder.append("target.");
                              Attribute _right_5 = propertyMapping.getRight();
                              Attribute _derivedAttributeFromMapperRight_2 = MapperExtension.getDerivedAttributeFromMapperRight(_right_5, mapper);
                              String _mutatorMethodName_19 = this._features.getMutatorMethodName(_derivedAttributeFromMapperRight_2);
                              _builder.append(_mutatorMethodName_19, "	");
                              _builder.append("(converter.convert(");
                              String _addImport_9 = this._names.addImport(it, "java.math.BigDecimal");
                              _builder.append(_addImport_9, "	");
                              _builder.append(".class, sourceDto.");
                              Attribute _left_4 = propertyMapping.getLeft();
                              Attribute _derivedAttributeFromMapperLeft_1 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_4, mapper);
                              String _accessorMethodName_29 = this._features.getAccessorMethodName(_derivedAttributeFromMapperLeft_1);
                              _builder.append(_accessorMethodName_29, "	");
                              _builder.append("()));");
                              _builder.newLineIfNotEmpty();
                            } else {
                              Attribute _right_6 = propertyMapping.getRight();
                              Attribute _derivedAttributeFromMapperRight_3 = MapperExtension.getDerivedAttributeFromMapperRight(_right_6, mapper);
                              Boolean _isTimestampSimpleType_2 = EntityExtension.isTimestampSimpleType(_derivedAttributeFromMapperRight_3);
                              if ((_isTimestampSimpleType_2).booleanValue()) {
                                _builder.append("\t");
                                _builder.append("target.");
                                Attribute _right_7 = propertyMapping.getRight();
                                Attribute _derivedAttributeFromMapperRight_4 = MapperExtension.getDerivedAttributeFromMapperRight(_right_7, mapper);
                                String _mutatorMethodName_20 = this._features.getMutatorMethodName(_derivedAttributeFromMapperRight_4);
                                _builder.append(_mutatorMethodName_20, "	");
                                _builder.append("(converter.convert(");
                                String _addImport_10 = this._names.addImport(it, "java.sql.Timestamp");
                                _builder.append(_addImport_10, "	");
                                _builder.append(".class, sourceDto.");
                                Attribute _left_5 = propertyMapping.getLeft();
                                Attribute _derivedAttributeFromMapperLeft_2 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_5, mapper);
                                String _accessorMethodName_30 = this._features.getAccessorMethodName(_derivedAttributeFromMapperLeft_2);
                                _builder.append(_accessorMethodName_30, "	");
                                _builder.append("()));");
                                _builder.newLineIfNotEmpty();
                              } else {
                                Attribute _right_8 = propertyMapping.getRight();
                                Attribute _derivedAttributeFromMapperRight_5 = MapperExtension.getDerivedAttributeFromMapperRight(_right_8, mapper);
                                Boolean _isDateSimpleType_2 = EntityExtension.isDateSimpleType(_derivedAttributeFromMapperRight_5);
                                if ((_isDateSimpleType_2).booleanValue()) {
                                  _builder.append("\t");
                                  _builder.append("target.");
                                  Attribute _right_9 = propertyMapping.getRight();
                                  Attribute _derivedAttributeFromMapperRight_6 = MapperExtension.getDerivedAttributeFromMapperRight(_right_9, mapper);
                                  String _mutatorMethodName_21 = this._features.getMutatorMethodName(_derivedAttributeFromMapperRight_6);
                                  _builder.append(_mutatorMethodName_21, "	");
                                  _builder.append("(converter.convert(");
                                  String _addImport_11 = this._names.addImport(it, "java.util.Date");
                                  _builder.append(_addImport_11, "	");
                                  _builder.append(".class, sourceDto.");
                                  Attribute _left_6 = propertyMapping.getLeft();
                                  Attribute _derivedAttributeFromMapperLeft_3 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_6, mapper);
                                  String _accessorMethodName_31 = this._features.getAccessorMethodName(_derivedAttributeFromMapperLeft_3);
                                  _builder.append(_accessorMethodName_31, "	");
                                  _builder.append("()));");
                                  _builder.newLineIfNotEmpty();
                                } else {
                                  _builder.append("\t");
                                  _builder.append("target.");
                                  Attribute _right_10 = propertyMapping.getRight();
                                  Attribute _derivedAttributeFromMapperRight_7 = MapperExtension.getDerivedAttributeFromMapperRight(_right_10, mapper);
                                  String _mutatorMethodName_22 = this._features.getMutatorMethodName(_derivedAttributeFromMapperRight_7);
                                  _builder.append(_mutatorMethodName_22, "	");
                                  _builder.append("(converter.convert(");
                                  Attribute _right_11 = propertyMapping.getRight();
                                  String _dataTypeName_8 = _right_11.getDataTypeName();
                                  _builder.append(_dataTypeName_8, "	");
                                  _builder.append(".class, sourceDto.");
                                  Attribute _left_7 = propertyMapping.getLeft();
                                  Attribute _derivedAttributeFromMapperLeft_4 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_7, mapper);
                                  String _accessorMethodName_32 = this._features.getAccessorMethodName(_derivedAttributeFromMapperLeft_4);
                                  _builder.append(_accessorMethodName_32, "	");
                                  _builder.append("()));");
                                  _builder.newLineIfNotEmpty();
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                  {
                    Boolean _dataViewIsRight_1 = MapperExtension.dataViewIsRight(it, mapper);
                    if ((_dataViewIsRight_1).booleanValue()) {
                      {
                        Attribute _right_12 = propertyMapping.getRight();
                        String _dataTypeName_9 = _right_12.getDataTypeName();
                        Attribute _left_8 = propertyMapping.getLeft();
                        String _dataTypeName_10 = _left_8.getDataTypeName();
                        boolean _matches_1 = _dataTypeName_9.matches(_dataTypeName_10);
                        if (_matches_1) {
                          _builder.append("\t");
                          _builder.append("target.");
                          Attribute _left_9 = propertyMapping.getLeft();
                          Attribute _derivedAttributeFromMapperLeft_5 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_9, mapper);
                          String _mutatorMethodName_23 = this._features.getMutatorMethodName(_derivedAttributeFromMapperLeft_5);
                          _builder.append(_mutatorMethodName_23, "	");
                          _builder.append("(sourceDto.");
                          Attribute _right_13 = propertyMapping.getRight();
                          Attribute _derivedAttributeFromMapperRight_8 = MapperExtension.getDerivedAttributeFromMapperRight(_right_13, mapper);
                          String _accessorMethodName_33 = this._features.getAccessorMethodName(_derivedAttributeFromMapperRight_8);
                          _builder.append(_accessorMethodName_33, "	");
                          _builder.append("());");
                          _builder.newLineIfNotEmpty();
                        } else {
                          {
                            Attribute _left_10 = propertyMapping.getLeft();
                            Attribute _derivedAttributeFromMapperLeft_6 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_10, mapper);
                            Boolean _isBigDecimalSimpleType_3 = EntityExtension.isBigDecimalSimpleType(_derivedAttributeFromMapperLeft_6);
                            if ((_isBigDecimalSimpleType_3).booleanValue()) {
                              _builder.append("\t");
                              _builder.append("target.");
                              Attribute _left_11 = propertyMapping.getLeft();
                              Attribute _derivedAttributeFromMapperLeft_7 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_11, mapper);
                              String _mutatorMethodName_24 = this._features.getMutatorMethodName(_derivedAttributeFromMapperLeft_7);
                              _builder.append(_mutatorMethodName_24, "	");
                              _builder.append("(converter.convert(");
                              String _addImport_12 = this._names.addImport(it, "java.math.BigDecimal");
                              _builder.append(_addImport_12, "	");
                              _builder.append(".class, sourceDto.");
                              Attribute _right_14 = propertyMapping.getRight();
                              Attribute _derivedAttributeFromMapperRight_9 = MapperExtension.getDerivedAttributeFromMapperRight(_right_14, mapper);
                              String _accessorMethodName_34 = this._features.getAccessorMethodName(_derivedAttributeFromMapperRight_9);
                              _builder.append(_accessorMethodName_34, "	");
                              _builder.append("()));");
                              _builder.newLineIfNotEmpty();
                            } else {
                              Attribute _left_12 = propertyMapping.getLeft();
                              Attribute _derivedAttributeFromMapperLeft_8 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_12, mapper);
                              Boolean _isTimestampSimpleType_3 = EntityExtension.isTimestampSimpleType(_derivedAttributeFromMapperLeft_8);
                              if ((_isTimestampSimpleType_3).booleanValue()) {
                                _builder.append("\t");
                                _builder.append("target.");
                                Attribute _left_13 = propertyMapping.getLeft();
                                Attribute _derivedAttributeFromMapperLeft_9 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_13, mapper);
                                String _mutatorMethodName_25 = this._features.getMutatorMethodName(_derivedAttributeFromMapperLeft_9);
                                _builder.append(_mutatorMethodName_25, "	");
                                _builder.append("(converter.convert(");
                                String _addImport_13 = this._names.addImport(it, "java.sql.Timestamp");
                                _builder.append(_addImport_13, "	");
                                _builder.append(".class, sourceDto.");
                                Attribute _right_15 = propertyMapping.getRight();
                                Attribute _derivedAttributeFromMapperRight_10 = MapperExtension.getDerivedAttributeFromMapperRight(_right_15, mapper);
                                String _accessorMethodName_35 = this._features.getAccessorMethodName(_derivedAttributeFromMapperRight_10);
                                _builder.append(_accessorMethodName_35, "	");
                                _builder.append("()));");
                                _builder.newLineIfNotEmpty();
                              } else {
                                Attribute _left_14 = propertyMapping.getLeft();
                                Attribute _derivedAttributeFromMapperLeft_10 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_14, mapper);
                                Boolean _isDateSimpleType_3 = EntityExtension.isDateSimpleType(_derivedAttributeFromMapperLeft_10);
                                if ((_isDateSimpleType_3).booleanValue()) {
                                  _builder.append("\t");
                                  _builder.append("target.");
                                  Attribute _left_15 = propertyMapping.getLeft();
                                  Attribute _derivedAttributeFromMapperLeft_11 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_15, mapper);
                                  String _mutatorMethodName_26 = this._features.getMutatorMethodName(_derivedAttributeFromMapperLeft_11);
                                  _builder.append(_mutatorMethodName_26, "	");
                                  _builder.append("(converter.convert(");
                                  String _addImport_14 = this._names.addImport(it, "java.util.Date");
                                  _builder.append(_addImport_14, "	");
                                  _builder.append(".class, sourceDto.");
                                  Attribute _right_16 = propertyMapping.getRight();
                                  Attribute _derivedAttributeFromMapperRight_11 = MapperExtension.getDerivedAttributeFromMapperRight(_right_16, mapper);
                                  String _accessorMethodName_36 = this._features.getAccessorMethodName(_derivedAttributeFromMapperRight_11);
                                  _builder.append(_accessorMethodName_36, "	");
                                  _builder.append("()));");
                                  _builder.newLineIfNotEmpty();
                                } else {
                                  _builder.append("\t");
                                  _builder.append("target.");
                                  Attribute _left_16 = propertyMapping.getLeft();
                                  Attribute _derivedAttributeFromMapperLeft_12 = MapperExtension.getDerivedAttributeFromMapperLeft(_left_16, mapper);
                                  String _mutatorMethodName_27 = this._features.getMutatorMethodName(_derivedAttributeFromMapperLeft_12);
                                  _builder.append(_mutatorMethodName_27, "	");
                                  _builder.append("(converter.convert(");
                                  Attribute _left_17 = propertyMapping.getLeft();
                                  String _dataTypeName_11 = _left_17.getDataTypeName();
                                  _builder.append(_dataTypeName_11, "	");
                                  _builder.append(".class, sourceDto.");
                                  Attribute _right_17 = propertyMapping.getRight();
                                  Attribute _derivedAttributeFromMapperRight_12 = MapperExtension.getDerivedAttributeFromMapperRight(_right_17, mapper);
                                  String _accessorMethodName_37 = this._features.getAccessorMethodName(_derivedAttributeFromMapperRight_12);
                                  _builder.append(_accessorMethodName_37, "	");
                                  _builder.append("()));");
                                  _builder.newLineIfNotEmpty();
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          _builder.append("}");
          _builder.newLine();
        }
      }
      _builder.newLine();
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence toAssemblerClass(final Model it, final Iterable<DataView> dataViews) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append("* implementation of {@link org.openxma.dsl.platform.assembler.DtoAssemblerFactory}<p>");
    _builder.newLine();
    _builder.append("* you can override dto mapping functionality here");
    _builder.newLine();
    _builder.append("*/");
    _builder.newLine();
    _builder.append("@");
    String _addImport = this._names.addImport(it, "org.springframework.stereotype.Component");
    _builder.append(_addImport, "");
    _builder.append("(\"");
    String _name_1 = it.getName();
    String _camelCasedModelNameSpaceName = NameExtension.getCamelCasedModelNameSpaceName(_name_1);
    _builder.append(_camelCasedModelNameSpaceName, "");
    _builder.append("Daf\")");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _assemblerClassName = this.getAssemblerClassName(it);
    _builder.append(_assemblerClassName, "");
    _builder.append(" extends ");
    String _genAssemblerClassName = this.getGenAssemblerClassName(it);
    _builder.append(_genAssemblerClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toClass(final DataView it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" extends ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _serialVersionUID = this._features.toSerialVersionUID(it);
    _builder.append(_serialVersionUID, "	");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final DataView it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import java.io.Serializable;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    CharSequence _typeExtension = this._featureExtensions.toTypeExtension(it);
    _builder.append(_typeExtension, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public abstract class ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    {
      DataView _superType = it.getSuperType();
      boolean _notEquals = (!Objects.equal(_superType, null));
      if (_notEquals) {
        _builder.append(" extends ");
        DataView _superType_1 = it.getSuperType();
        String _qualifiedName = this._names.getQualifiedName(_superType_1);
        _builder.append(_qualifiedName, "");
      }
    }
    _builder.append(" implements Serializable {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _serialVersionUID = this._features.toSerialVersionUID(it);
    _builder.append(_serialVersionUID, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _generateAttributes = this._features.generateAttributes(it);
    _builder.append(_generateAttributes, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _generateProperties = this._features.generateProperties(it);
    _builder.append(_generateProperties, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generateToString = this._features.generateToString(it);
    _builder.append(_generateToString, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private String getQualifiedGenAssemblerClassName(final Model it) {
    String _qualifiedAssemblerClassName = this.getQualifiedAssemblerClassName(it);
    String _plus = (_qualifiedAssemblerClassName + "Gen");
    return _plus;
  }
  
  private String getQualifiedAssemblerClassName(final Model it) {
    String _name = it.getName();
    String _plus = (_name + ".");
    String _assemblerClassName = this.getAssemblerClassName(it);
    String _plus_1 = (_plus + _assemblerClassName);
    return _plus_1;
  }
  
  private String getAssemblerClassName(final Model it) {
    String _name = it.getName();
    String _lastToken = Strings.lastToken(_name, ".");
    String _firstUpper = StringExtensions.toFirstUpper(_lastToken);
    String _plus = ("DtoAssemblerFactory" + _firstUpper);
    return _plus;
  }
  
  private String getGenAssemblerClassName(final Model it) {
    String _assemblerClassName = this.getAssemblerClassName(it);
    String _plus = (_assemblerClassName + "Gen");
    return _plus;
  }
}
