package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.generator.IGenerator
import org.openxma.dsl.core.model.Model
import org.openxma.dsl.dom.model.Attribute
import org.openxma.dsl.dom.model.DataView
import org.openxma.dsl.dom.model.Entity
import org.openxma.dsl.generator.lib.Index

import static org.openxma.dsl.generator.GeneratorConstants.*
import static org.openxma.dsl.generator.helper.EntityExtension.*

import static extension org.openxma.dsl.common.Strings.*
import static extension org.openxma.dsl.common.extensions.EObjectPropertiesAdapter.*
import static extension org.openxma.dsl.generator.helper.DataViewExtension.*
import static extension org.openxma.dsl.generator.helper.MapperExtension.*
import static extension org.openxma.dsl.generator.helper.NameExtension.*
import static extension org.openxma.dsl.generator.lib.ImportAdapter.*
import org.openxma.dsl.generator.FeatureExtensions

class DataViewGenerator implements IGenerator {
	@Inject
	extension Names
	@Inject
	extension Files
	@Inject
	extension Features
	@Inject
	extension FeatureExtensions
	
	override doGenerate(Resource resource, IFileSystemAccess fsa) {
		val model = resource.getEObject("/") as Model
		if (!model.isNameSpaceGenerated()) {
			model.addGeneratedNameSpaceName() 
			val dataViews = model.nameSpaceDataViewsFromModel
			if (!dataViews.empty) {
				fsa.generateFile(model.qualifiedGenAssemblerClassName.toJavaFilePath, OUTLET_GENERATED_JAVA, model.toGenAssemblerClass(dataViews))
				fsa.generateFile(model.qualifiedAssemblerClassName.toJavaFilePath, OUTLET_JAVA, model.toAssemblerClass(dataViews))
			}
		}
		model.elements.filter(typeof(DataView)).forEach(
			[
				fsa.generateFile(qualifiedGenClassName.toJavaFilePath, OUTLET_GENERATED_JAVA, toGenClass)
				fsa.generateFile(qualifiedClassName.toJavaFilePath, OUTLET_JAVA, toClass)
			]
		)
	}

	def CharSequence toGenAssemblerClass(Model it, Iterable<DataView> dataViews) {
		'''
		package name;

		import org.openxma.dsl.platform.service.MapperContextHolder;
		addImportsHere
		
		public class genAssemblerClassName implements addImport("org.openxma.dsl.platform.assembler.DtoAssemblerFactory") {

			private addImport("java.util.List")<addImport("org.openxma.dsl.platform.assembler.DtoAssembler")<?>> dtoAssemblers;
		
			@addImport("org.springframework.beans.factory.annotation.Autowired")
			private addImport("org.openxma.dsl.platform.assembler.DtoAssemblerRegistry") dtoAssemblerRegistry;
			
			@addImport("org.springframework.beans.factory.annotation.Autowired")
			private addImport("org.openxma.dsl.platform.converter.Converter") converter;
		
			public genAssemblerClassName() {
				dtoAssemblers = new addImport("java.util.ArrayList")<DtoAssembler<?>>(dataViews.size);
				initDtoAssemblers();
			}
		
		
			public List<DtoAssembler<?>> getDtoAssemblers() {
				return dtoAssemblers;
			}
		
			protected DtoAssemblerRegistry getDtoAssemblerRegistry() {
				return dtoAssemblerRegistry;
			}
			
			protected Converter getConverter() {
				return converter;
			}
		
			private void initDtoAssemblers() {
				FOR dataView : dataViews
				dtoAssemblers.add(createDtoAssemblerdataView.name());
				ENDFOR
			}
		
		
			FOR dataView : dataViews
			private DtoAssembler<dataView.qualifiedName> createDtoAssemblerdataView.name() {
				return new DtoAssembler<dataView.name>() {
					dataView.toAnonymousDtoAssemblerClass
				};
			}
			
			dataView.toMapping
			ENDFOR
		}
		'''
	}
	
	def CharSequence toAnonymousDtoAssemblerClass(DataView it) {
		val Iterable<Entity> entities = featureReferences.map(e|e.source).toSet
		'''
		public className mapFromEntity(Object sourceEntity) {
			addImport("org.springframework.util.Assert").notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
			val iterator = new Index
			FOR entity : entities
			IF iterator.counter0>0
			} else if (sourceEntity instanceof entity.qualifiedName) {
			ELSE
			if (sourceEntity instanceof entity.qualifiedName) {
			ENDIF
				entity.qualifiedName entity.name.toVariableName() = (entity.name) sourceEntity;
				getClassName() result = mapFromentity.nameTogetClassName()(entity.name.toVariableName());
				FOR requiredReference : entity.allAttributes.filter(reference|featureReferences.exists(e|e.getProperty("sourceReference")==reference))
				if (null != entity.toVariableName().requiredReference.getAccessorMethodName()()) {
					result.featureReferences.findFirst(e|e.getProperty("sourceReference")==requiredReference).target.getMutatorMethodName()(entity.toVariableName().requiredReference.getAccessorMethodName()().(requiredReference.dataType as Entity).identifier.getAccessorMethodName()());
				}
				ENDFOR
				return result;
			iterator.increment
			ENDFORIF superType!=null && !superType.featureReferences.isEmpty
			} else {
					className result = new className();
					dtoAssemblerRegistry.getDtoAssembler(superType.qualifiedName.class)
						.mapTo(dtoAssemblerRegistry.getDtoAssembler(superType.name.class)
								.mapFromEntity(sourceEntity), result);
					return result;
			}
			ELSE
			IF iterator.counter0>0
			}
			ELSE
			
			ENDIF
			throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
				
			ENDIF
		}
		
		public className mapFromEntity(Object sourceEntity, className targetDto) {
			addImport("org.springframework.util.Assert").notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
			addImport("org.springframework.util.Assert").notNull(targetDto, "parameter 'targetDto' must not be null");
			val index = new IndexFOR entity : entities
			IF index.counter0>0
			} else if (sourceEntity instanceof entity.qualifiedName) {
			ELSE
			if (sourceEntity instanceof entity.qualifiedName) {
			ENDIF
				entity.qualifiedName entity.toVariableName() = (entity.name) sourceEntity;
				mapFromentity.nameToclassName(entity.toVariableName(), targetDto);
				FOR requiredReference : entity.allAttributes.filter(reference|featureReferences.exists(e|e.getProperty("sourceReference")==reference))
				if (null != entity.toVariableName().requiredReference.getAccessorMethodName()()) {
					targetDto.featureReferences.findFirst(e|e.getProperty("sourceReference")==requiredReference).target.getMutatorMethodName()(entity.toVariableName().requiredReference.getAccessorMethodName()().(requiredReference.dataType as Entity).identifier.getAccessorMethodName()());
				}
				ENDFOR
				return targetDto;
			index.increment
			ENDFOR
			IF superType!=null && !superType.featureReferences.isEmpty 
			} else {
					dtoAssemblerRegistry.getDtoAssembler(superType.qualifiedName.class)
						.mapTo(dtoAssemblerRegistry.getDtoAssembler(superType.name.class)
								.mapFromEntity(sourceEntity), targetDto);
					return targetDto;
			}
			ELSE
			IF index.counter0>0
			}
			ELSE
			
			ENDIF
			throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
				
			ENDIF
		}
		
		public <T> T mapTo(className sourceDto, Class<T> target) {
			if (className.class.equals(target)) {
				className result = new className();
				mapFromclassNameToclassName(sourceDto, result);
				return target.cast(result);
			}
			val mappers = getMappersByDataView()
			FOR mapper: mappers
			IF dataViewIsMapperSource(mapper)
			else if (addImport(getOppositeElementFullyQualifiedName(mapper)).class.equals(target) && MapperContextHolder.getContext().containsKey("mapper:mapper.name")) {
				return target.cast(mappermapper.nameMapsFromclassNameTogetOppositeElementName(mapper)(sourceDto));
			}
			ELSE
			//dataView: 'name' is not a mapping source in mapper: 'mapper.name'
			ENDIF
			
			ENDFOR
			FOR entity : entities
			else if (entity.name.class.equals(target)) {
				return target.cast(mapFromclassNameToentity.name(sourceDto));
			}
			ENDFOR
			IF superType!=null && !superType.featureReferences.isEmpty
			else {
				return dtoAssemblerRegistry.getDtoAssembler(superType.name.class).mapTo(sourceDto, target);
			}
			ELSE
			else {
				throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
			}
			ENDIF
		}

		public void mapTo(className sourceDto, Object targetObject) {
			if (targetObject instanceof className) {
				mapFromclassNameToclassName(sourceDto, (className) targetObject);
			}
			FOR mapper : mappers
			IF dataViewIsMapperSource(mapper)
			else if (targetObject instanceof addImport(getOppositeElementFullyQualifiedName(mapper)) && MapperContextHolder.getContext().containsKey("mapper:mapper.name")) {
				mappermapper.nameMapsFromclassNameTogetOppositeElementName(mapper)(sourceDto, (getOppositeElementName(mapper))targetObject);
			}
			ELSE
			//dataView: 'name' is not a mapping source in mapper: 'mapper.name'
			ENDIF
			
			ENDFOR
			FOR entity : entities
			else if (targetObject instanceof entity.name) {
				mapFromclassNameToentity.name(sourceDto, (entity.name)targetObject);
			}			   
			ENDFOR
			IF superType!=null && !superType.featureReferences.isEmpty
			else {
				dtoAssemblerRegistry.getDtoAssembler(superType.name.class).mapTo(sourceDto, targetObject);
			}
			ELSE
			else if (targetObject != null) {
				throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
			}
			ENDIF
		}
		'''
	}
	
	def CharSequence toMapping(DataView it) {
		val Iterable<Entity> entities = featureReferences.map(e|e.source).toSet
		'''
		protected void mapFromclassNameToclassName(className sourceDto, className className.toFirstLower()) {
			IF superType!=null
			dtoAssemblerRegistry.getDtoAssembler(superType.name.class).mapTo(sourceDto, className.toFirstLower());
			ENDIF
			FOR featureReference : featureReferences.filter(e|e.target!=null && e.attribute!=null && !e.attribute.readOnly)
			className.toFirstLower().featureReference.target.getMutatorMethodName()(sourceDto.featureReference.target.getAccessorMethodName()());
			ENDFOR
		}
		FOR entity : entities
		
		protected entity.name mapFromclassNameToentity.name(className sourceDto) {
			return mapFromclassNameToentity.name(sourceDto, MapperContextHolder.createEntity(entity.name.class));
		}
		
		protected entity.name mapFromclassNameToentity.name(className sourceDto, entity.name entity.toVariableName()) {
			if (MapperContextHolder.getContext().containsKey(sourceDto)) {
				return (entity.name) MapperContextHolder.getContext().get(sourceDto);
			}
			IF superType!=null && superType.featureReferences.map(e|e.source).contains(entity)
			dtoAssemblerRegistry.getDtoAssembler(superType.name.class).mapTo(sourceDto, entity.toVariableName());
			ELSE
			MapperContextHolder.getContext().put(sourceDto, entity.toVariableName());
			ENDIF
			FOR featureReference : featureReferences.filter(e|e.source==entity && e.attribute!=null && !e.attribute?.reference && !e.attribute?.readOnly)
			IF featureReference.attribute.identifier
			if (sourceDto.featureReference.target.getAccessorMethodName()() != nullIF featureReference.getProperty("sourceReference")!=null setProperty("generateIsContextMethod","true") && MapperContextHolder.isContext("(featureReference.getProperty("sourceReference") as Attribute).name")ENDIF) {
				entity.toVariableName().entity.identifier.getMutatorMethodName()(sourceDto.featureReference.target.getAccessorMethodName()());
			}
			ELSE
			entity.toVariableName().featureReference.attribute.getMutatorMethodName()(sourceDto.featureReference.target.getAccessorMethodName()());
			ENDIF
			ENDFOR
			val iterator = new Index
			FOR includedFeatureReference : featureReferences.filter(e|e.source==entity && e.view!=null && e.attribute?.reference)
			IF iterator.firstIteration
			setProperty("generateIsContextMethod","true")
			if (!MapperContextHolder.isContext("saveentity.name") && !MapperContextHolder.isContext("updateentity.name")) {
			ENDIF
			IF !includedFeatureReference.attribute.many
				if (null != sourceDto.includedFeatureReference.target.getAccessorMethodName()()) {
					includedFeatureReference.attribute.dataType.name includedFeatureReference.attribute.name = (includedFeatureReference.attribute.dataType.name) dtoAssemblerRegistry.getDtoAssembler(includedFeatureReference.view.name.class).mapTo(sourceDto.includedFeatureReference.target.getAccessorMethodName()(), includedFeatureReference.attribute.dataType.name.class);
					entity.toVariableName().includedFeatureReference.attribute.getMutatorMethodName()(includedFeatureReference.attribute.name);
				}
			ELSE
				if (null != sourceDto.includedFeatureReference.target.getAccessorMethodName()()) {
					for (addImport(includedFeatureReference.view.getFullClassNameForDatamapping()) includedFeatureReference.view.name.toFirstLower() : sourceDto.includedFeatureReference.target.getAccessorMethodName()()) {
						includedFeatureReference.attribute.dataType.name includedFeatureReference.attribute.name = (includedFeatureReference.attribute.dataType.name) dtoAssemblerRegistry.getDtoAssembler(includedFeatureReference.view.name.class).mapTo(includedFeatureReference.view.name.toFirstLower(), includedFeatureReference.attribute.dataType.name.class);
						entity.toVariableName().includedFeatureReference.attribute.getAddMethodName()(includedFeatureReference.attribute.name);
					}
				}
			ENDIF
			iterator.increment
			ENDFOR
			IF iterator.counter0>0
			}
			ENDIF
			return entity.toVariableName();
		}
		ENDFOR
		FOR entity : entities
		
		protected className mapFromentity.nameToclassName(entity.name entity.toVariableName()) {
			if (MapperContextHolder.getContext().containsKey(entity.toVariableName())) {
				return (className) MapperContextHolder.getContext().get(entity.toVariableName());
			}
			className className.toFirstLower() = new className();
			IF superType!=null && superType.featureReferences.map(e|e.source).contains(entity)
			dtoAssemblerRegistry.getDtoAssembler(superType.name.class)
				.mapTo(dtoAssemblerRegistry.getDtoAssembler(superType.name.class)
						.mapFromEntity(entity.toVariableName()), className.toFirstLower());
			ENDIF
			MapperContextHolder.getContext().put(entity.toVariableName(), className.toFirstLower());
			FOR featureReference : featureReferences.filter(e|e.source==entity && e.getProperty("sourceReference")==null && e.attribute != null && !e.attribute.reference && !e.attribute.readOnly)
			className.toFirstLower().featureReference.target.getMutatorMethodName()(entity.toVariableName().featureReference.attribute.getAccessorMethodName()());
			ENDFOR
			FOR includedFeatureReference : featureReferences.filter(e|e.source==entity && e.attribute?.reference && e.view!=null)
			IF !includedFeatureReference.attribute.many
			if (null != entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()()) {
				addImport(getFullyQualifiedName(includedFeatureReference.view,true)) includedFeatureReference.attribute.name = (includedFeatureReference.view.name) MapperContextHolder.getContext().get(entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()());
				if (null == includedFeatureReference.attribute.name) {
					includedFeatureReference.attribute.name = dtoAssemblerRegistry.getDtoAssembler(includedFeatureReference.view.name.class).mapFromEntity(entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()());
				}
				className.toFirstLower().includedFeatureReference.target.getMutatorMethodName()(includedFeatureReference.attribute.name);
			}
			ELSE
			if (entity.toVariableName().includedFeatureReference.attribute.getHasMethodName()()) {
				for (includedFeatureReference.attribute.dataType.name next : entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()()) {
					includedFeatureReference.view.name nextincludedFeatureReference.view.name = (includedFeatureReference.view.name) MapperContextHolder.getContext().get(next);
					if (null == nextincludedFeatureReference.view.name) {
						nextincludedFeatureReference.view.name = dtoAssemblerRegistry.getDtoAssembler(includedFeatureReference.view.name.class).mapFromEntity(next);
					}
					className.toFirstLower().includedFeatureReference.target.getAddMethodName()(nextincludedFeatureReference.view.name);
				}
			}
			ENDIF
			ENDFOR
			return className.toFirstLower();
		}
		
		protected className mapFromentity.nameToclassName(entity.name entity.toVariableName(), className className.toFirstLower()) {
			if (MapperContextHolder.getContext().containsKey(entity.toVariableName())) {
				return (className) MapperContextHolder.getContext().get(entity.toVariableName());
			}
			IF superType!=null && superType.featureReferences.map(e|e.source).contains(entity)
			dtoAssemblerRegistry.getDtoAssembler(superType.name.class)
				.mapTo(dtoAssemblerRegistry.getDtoAssembler(superType.name.class)
						.mapFromEntity(entity.toVariableName()), className.toFirstLower());
			ENDIF
			MapperContextHolder.getContext().put(entity.toVariableName(), className.toFirstLower());
			FOR featureReference : featureReferences.filter(e|(e.source==entity && e.getProperty("sourceReference")==null) && e.attribute != null && !e.attribute.reference && !e.attribute.readOnly)	  
			className.toFirstLower().featureReference.target.getMutatorMethodName()(entity.toVariableName().featureReference.attribute.getAccessorMethodName()());
			ENDFOR
			FOR includedFeatureReference : featureReferences.filter(e|e.source==entity && e.attribute?.reference && e.view!=null)
			IF !includedFeatureReference.attribute.many
			if (null != entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()()) {
				addImport(getFullyQualifiedName(includedFeatureReference.view,true)) includedFeatureReference.attribute.name = (includedFeatureReference.view.name) MapperContextHolder.getContext().get(entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()());
				if (null == includedFeatureReference.attribute.name) {
					includedFeatureReference.attribute.name = dtoAssemblerRegistry.getDtoAssembler(includedFeatureReference.view.name.class).mapFromEntity(entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()());
				}
				className.toFirstLower().includedFeatureReference.target.getMutatorMethodName()(includedFeatureReference.attribute.name);
			}
			ELSE
			if (entity.toVariableName().includedFeatureReference.attribute.getHasMethodName()()) {
				for (includedFeatureReference.attribute.dataType.name next : entity.toVariableName().includedFeatureReference.attribute.getAccessorMethodName()()) {
					includedFeatureReference.view.name nextincludedFeatureReference.view.name = (includedFeatureReference.view.name) MapperContextHolder.getContext().get(next);
					if (null == nextincludedFeatureReference.view.name) {
						nextincludedFeatureReference.view.name = dtoAssemblerRegistry.getDtoAssembler(includedFeatureReference.view.name.class).mapFromEntity(next);
					}
					className.toFirstLower().includedFeatureReference.target.getAddMethodName()(nextincludedFeatureReference.view.name);
				}
			}
			ENDIF
			ENDFOR
			return className.toFirstLower();
		}
		ENDFOR
		
		val mappers = getMappersByDataView
		FOR mapper : mappers
		
		protected getOppositeElementName(mapper) mappermapper.nameMapsFromgetClassName()TogetOppositeElementName(mapper)(getClassName() sourceDto) {
			getOppositeElementName(mapper) target = MapperContextHolder.createComplexType(getOppositeElementName(mapper).class);
			mappermapper.nameMapsFromgetClassName()TogetOppositeElementName(mapper)(sourceDto, target);
			return target;
		}
		
		protected void mappermapper.nameMapsFromgetClassName()TogetOppositeElementName(mapper)(getClassName() sourceDto, getOppositeElementName(mapper) target ) {
			IF mapper.propertyMappings.isEmpty
				IF dataViewIsMapperSource(mapper)	   
					IF dataViewIsLeft(mapper)
			FOR sA : mapper.left.attributes.filter(lA|lA!=null && !lA.reference && lA.eContainer==mapper.left 
								&& mapper.right.attributes.exists(rA|rA!=null && !rA.reference && !rA.readOnly
								&& rA.getAccessorMethodName()==lA.getAccessorMethodName())) 
			val tA = mapper.right.attributes.findFirst(rA|rA.getAccessorMethodName().equals(sA.getAccessorMethodName()))
			IF sA.dataTypeName==tA.dataTypeName 
			target.tA.getMutatorMethodName()(sourceDto.sA.getAccessorMethodName()());
			ELSE
			IF isBigDecimalSimpleType(tA)
			target.tA.getMutatorMethodName()(converter.convert(addImport("java.math.BigDecimal").class, sourceDto.sA.getAccessorMethodName()()));
			ELSEIF isTimestampSimpleType(tA)
			target.tA.getMutatorMethodName()(converter.convert(addImport("java.sql.Timestamp").class, sourceDto.sA.getAccessorMethodName()()));
			ELSEIF isDateSimpleType(tA)
			target.tA.getMutatorMethodName()(converter.convert(addImport("java.util.Date").class, sourceDto.sA.getAccessorMethodName()()));
			ELSE	 
			target.tA.getMutatorMethodName()(converter.convert(tA.dataTypeName.class, sourceDto.sA.getAccessorMethodName()()));
			ENDIF
			ENDIF
			ENDFOR
			ENDIF
			IF dataViewIsRight(mapper)	
			FOR sA : mapper.right.attributes.filter(rA|rA!=null && !rA.reference && rA.eContainer==mapper.right 
								&& mapper.left.attributes.exists(lA|lA!=null && !lA.reference && !lA.readOnly
								&& lA.getAccessorMethodName()==rA.getAccessorMethodName())) 
			val tA = mapper.left.attributes.findFirst(lA|lA.getAccessorMethodName()==sA.getAccessorMethodName())
			IF sA.dataTypeName==tA.dataTypeName 
			target.tA.getMutatorMethodName()(sourceDto.sA.getAccessorMethodName()());
			ELSE
			IF isBigDecimalSimpleType(tA)
			target.tA.getMutatorMethodName()(converter.convert(addImport("java.math.BigDecimal").class, sourceDto.sA.getAccessorMethodName()()));
			ELSEIF isTimestampSimpleType(tA)
			target.tA.getMutatorMethodName()(converter.convert(addImport("java.sql.Timestamp").class, sourceDto.sA.getAccessorMethodName()()));
			ELSEIF isDateSimpleType(tA)
			target.tA.getMutatorMethodName()(converter.convert(addImport("java.util.Date").class, sourceDto.sA.getAccessorMethodName()()));
			ELSE	 
			target.tA.getMutatorMethodName()(converter.convert(tA.dataTypeName.class, sourceDto.sA.getAccessorMethodName()()));
			ENDIF
			ENDIF
			ENDFOR
					ENDIF		
				ENDIF
			ELSE		 
			FOR propertyMapping : mapper.propertyMappings.filter(e|dataViewIsPropertyMappingSource(mapper,e))			
			IF dataViewIsLeft(mapper)			   
			IF propertyMapping.left.dataTypeName.matches(propertyMapping.right.dataTypeName) 
			target.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getMutatorMethodName()(sourceDto.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getAccessorMethodName()());
			ELSE
			IF isBigDecimalSimpleType(getDerivedAttributeFromMapperRight(propertyMapping.right,mapper))
			target.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getMutatorMethodName()(converter.convert(addImport("java.math.BigDecimal").class, sourceDto.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getAccessorMethodName()()));
			ELSEIF isTimestampSimpleType(getDerivedAttributeFromMapperRight(propertyMapping.right,mapper))
			target.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getMutatorMethodName()(converter.convert(addImport("java.sql.Timestamp").class, sourceDto.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getAccessorMethodName()()));
			ELSEIF isDateSimpleType(getDerivedAttributeFromMapperRight(propertyMapping.right,mapper))
			target.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getMutatorMethodName()(converter.convert(addImport("java.util.Date").class, sourceDto.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getAccessorMethodName()()));
			ELSE	 
			target.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getMutatorMethodName()(converter.convert(propertyMapping.right.dataTypeName.class, sourceDto.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getAccessorMethodName()()));
			ENDIF
			ENDIF
			ENDIF
			IF dataViewIsRight(mapper)
			IF propertyMapping.right.dataTypeName.matches(propertyMapping.left.dataTypeName) 
			target.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getMutatorMethodName()(sourceDto.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getAccessorMethodName()());
			ELSE
			IF isBigDecimalSimpleType(getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper))
			target.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getMutatorMethodName()(converter.convert(addImport("java.math.BigDecimal").class, sourceDto.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getAccessorMethodName()()));
			ELSEIF isTimestampSimpleType(getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper))
			target.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getMutatorMethodName()(converter.convert(addImport("java.sql.Timestamp").class, sourceDto.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getAccessorMethodName()()));
			ELSEIF isDateSimpleType(getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper))
			target.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getMutatorMethodName()(converter.convert(addImport("java.util.Date").class, sourceDto.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getAccessorMethodName()()));
			ELSE	 
			target.getDerivedAttributeFromMapperLeft(propertyMapping.left,mapper).getMutatorMethodName()(converter.convert(propertyMapping.left.dataTypeName.class, sourceDto.getDerivedAttributeFromMapperRight(propertyMapping.right,mapper).getAccessorMethodName()()));
							ENDIF
						ENDIF
					ENDIF		
				ENDFOR			   
			ENDIF
		}
		ENDFOR
		
		
		'''
	}
	

	def CharSequence toAssemblerClass(Model it, Iterable<DataView> dataViews) {
		'''
		package name;

		addImportsHere
		
		/**
		* implementation of {@link org.openxma.dsl.platform.assembler.DtoAssemblerFactory}<p>
		* you can override dto mapping functionality here
		*/
		@addImport("org.springframework.stereotype.Component")("getCamelCasedModelNameSpaceName(name)Daf")
		public class getAssemblerClassName() extends getGenAssemblerClassName() {
		
		}
		'''
	}

	def CharSequence toClass(DataView it) {
		'''
		package classPackageName;

		documentation
		public class name extends genClassName {
			toSerialVersionUID
		
		}
		'''
	}

	def CharSequence toGenClass(DataView it) {
		'''
		package classPackageName;
		
		import java.io.Serializable;
		addImportsHere

		documentation
		toTypeExtension
		public abstract class getGenClassName()IF superType!=null extends superType.qualifiedNameENDIF implements Serializable {
			toSerialVersionUID
			generateAttributes
			generateProperties
			
			
			generateToString
		}
		'''
	}
	
	def private getQualifiedGenAssemblerClassName(Model it) {
		qualifiedAssemblerClassName + "Gen"
	}

	def private getQualifiedAssemblerClassName(Model it) {
		name + "." + assemblerClassName
	}

	def private getAssemblerClassName(Model it) {
		"DtoAssemblerFactory" + name.lastToken(".").toFirstUpper
	}

	def private getGenAssemblerClassName(Model it) {
		getAssemblerClassName + "Gen"
	}

}
