package org.openxma.dsl.generator.impl;

import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_GENERATED_JAVA;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_GENERATED_RESOURCES;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_GENERATED_TEST;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_GENERATED_WEBAPP;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_JAVA;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_RESOURCES;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_TEST;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_WEBAPP;

import java.util.Map;

import javax.inject.Inject;
import javax.inject.Provider;

import org.eclipse.emf.common.util.URI;
import org.eclipse.xtext.generator.AbstractFileSystemAccess;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.OutputConfiguration;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.IFileSystemAccessProvider;
import org.openxma.dsl.generator.WebAppGeneratorConfiguration;

import com.google.common.collect.Maps;

public class DefaultFileSystemAccessProvider implements IFileSystemAccessProvider {

    @Inject
    private Provider<DefaultFileSystemAccess> fileSystemAccess;

    public IFileSystemAccess get(GeneratorConfiguration generatorConfiguration) {
        AbstractFileSystemAccess javaIoFileSystemAccess = createFileSystemAccess(generatorConfiguration);
        Map<String, OutputConfiguration> outputConfigurations = Maps.newHashMap();
        OutputConfiguration javaOutputConfiguration = createOutputConfiguration(OUTLET_JAVA,
                generatorConfiguration.getSourceFolder());
        outputConfigurations.put(javaOutputConfiguration.getName(), javaOutputConfiguration);
        OutputConfiguration resourcesConfiguration = createOutputConfiguration(OUTLET_RESOURCES,
                generatorConfiguration.getResourceFolder());
        outputConfigurations.put(resourcesConfiguration.getName(), resourcesConfiguration);
        if (generatorConfiguration.getTestSourceFolder() != null) {
            OutputConfiguration testOutputConfiguration = createOutputConfiguration(OUTLET_TEST,
                    generatorConfiguration.getTestSourceFolder());
            outputConfigurations.put(testOutputConfiguration.getName(), testOutputConfiguration);
        }
        OutputConfiguration genJavaOutputConfiguration = createGenOutputConfiguration(OUTLET_GENERATED_JAVA,
                generatorConfiguration.getGenSourceFolder());
        outputConfigurations.put(genJavaOutputConfiguration.getName(), genJavaOutputConfiguration);
        OutputConfiguration genResourcesConfiguration = createGenOutputConfiguration(OUTLET_GENERATED_RESOURCES,
                generatorConfiguration.getGenResourceFolder());
        outputConfigurations.put(genResourcesConfiguration.getName(), genResourcesConfiguration);
        if (generatorConfiguration.getGenTestSourceFolder() != null) {
            OutputConfiguration genTestOutputConfiguration = createGenOutputConfiguration(OUTLET_GENERATED_TEST,
                    generatorConfiguration.getGenTestSourceFolder());
            outputConfigurations.put(genTestOutputConfiguration.getName(), genTestOutputConfiguration);
        }
        if (generatorConfiguration instanceof WebAppGeneratorConfiguration) {
            WebAppGeneratorConfiguration webAppGeneratorConfiguration = (WebAppGeneratorConfiguration) generatorConfiguration;
            setOutputConfiguration(outputConfigurations, webAppGeneratorConfiguration);
        }
        javaIoFileSystemAccess.setOutputConfigurations(outputConfigurations);
        return javaIoFileSystemAccess;
    }

    protected void setOutputConfiguration(Map<String, OutputConfiguration> outputConfigurations,
            WebAppGeneratorConfiguration webAppGeneratorConfiguration) {
        if (webAppGeneratorConfiguration.getWebAppFolder() != null) {
            OutputConfiguration webApplicationConfiguration = createOutputConfiguration(OUTLET_WEBAPP,
                    webAppGeneratorConfiguration.getWebAppFolder());
            outputConfigurations.put(webApplicationConfiguration.getName(), webApplicationConfiguration);
        }
        if (webAppGeneratorConfiguration.getGenWebAppFolder() != null) {
            OutputConfiguration genWebApplicationConfiguration = createGenOutputConfiguration(
                    OUTLET_GENERATED_WEBAPP, webAppGeneratorConfiguration.getGenWebAppFolder());
            outputConfigurations.put(genWebApplicationConfiguration.getName(), genWebApplicationConfiguration);
        }
    }

    protected AbstractFileSystemAccess createFileSystemAccess(GeneratorConfiguration generatorConfiguration) {
        return fileSystemAccess.get();
    }

    protected OutputConfiguration createOutputConfiguration(String name, URI uri) {
        OutputConfiguration outputConfiguration = new OutputConfiguration(name);
        outputConfiguration.setOutputDirectory(getDirectory(uri));
        outputConfiguration.setSetDerivedProperty(false);
        outputConfiguration.setOverrideExistingResources(false);
        return outputConfiguration;
    }

    protected OutputConfiguration createGenOutputConfiguration(String name, URI uri) {
        OutputConfiguration outputConfiguration = new OutputConfiguration(name);
        outputConfiguration.setOutputDirectory(getDirectory(uri));
        outputConfiguration.setSetDerivedProperty(false);
        outputConfiguration.setOverrideExistingResources(true);
        outputConfiguration.setCreateOutputDirectory(true);
        return outputConfiguration;
    }

    protected String getDirectory(URI uri) {
        return uri.toFileString();
    }

}
