package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import java.util.List;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeGroup;
import org.openxma.dsl.dom.model.AttributeProperty;
import org.openxma.dsl.dom.model.AttributeSortOrder;
import org.openxma.dsl.dom.model.AttributeTextProperty;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.generator.FeatureExtensions;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.EntityExtension;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;

@SuppressWarnings("all")
public class EntityGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  @Inject
  private FeatureExtensions _featureExtensions;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<Entity> _filter = Iterables.<Entity>filter(_elements, Entity.class);
    final Procedure1<Entity> _function = new Procedure1<Entity>() {
        public void apply(final Entity it) {
          String _qualifiedGenInterfaceName = EntityGenerator.this._names.getQualifiedGenInterfaceName(it);
          String _javaFilePath = EntityGenerator.this._files.toJavaFilePath(_qualifiedGenInterfaceName);
          CharSequence _genInterface = EntityGenerator.this.toGenInterface(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genInterface);
          String _qualifiedInterfaceName = EntityGenerator.this._names.getQualifiedInterfaceName(it);
          String _javaFilePath_1 = EntityGenerator.this._files.toJavaFilePath(_qualifiedInterfaceName);
          CharSequence _interface = EntityGenerator.this.toInterface(it);
          fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _interface);
          String _qualifiedGenClassName = EntityGenerator.this._names.getQualifiedGenClassName(it);
          String _javaFilePath_2 = EntityGenerator.this._files.toJavaFilePath(_qualifiedGenClassName);
          CharSequence _genClass = EntityGenerator.this.toGenClass(it);
          fsa.generateFile(_javaFilePath_2, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _qualifiedClassName = EntityGenerator.this._names.getQualifiedClassName(it);
          String _javaFilePath_3 = EntityGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _class = EntityGenerator.this.toClass(it);
          fsa.generateFile(_javaFilePath_3, GeneratorConstants.OUTLET_JAVA, _class);
          boolean _and = false;
          Attribute _identifier = it.getIdentifier();
          boolean _equals = Objects.equal(_identifier, null);
          if (!_equals) {
            _and = false;
          } else {
            AttributeGroup _key = it.getKey();
            boolean _notEquals = (!Objects.equal(_key, null));
            _and = (_equals && _notEquals);
          }
          if (_and) {
            AttributeGroup _key_1 = it.getKey();
            String _qualifiedKeyClassName = EntityGenerator.this._features.getQualifiedKeyClassName(_key_1);
            String _javaFilePath_4 = EntityGenerator.this._files.toJavaFilePath(_qualifiedKeyClassName);
            CharSequence _keyClass = EntityGenerator.this.toKeyClass(it);
            fsa.generateFile(_javaFilePath_4, GeneratorConstants.OUTLET_GENERATED_JAVA, _keyClass);
          }
          String _attributePropertiesFileName = EntityGenerator.this.attributePropertiesFileName(it);
          CharSequence _attributeProperties = EntityGenerator.this.toAttributeProperties(it);
          fsa.generateFile(_attributePropertiesFileName, GeneratorConstants.OUTLET_GENERATED_RESOURCES, _attributeProperties);
        }
      };
    IterableExtensions.<Entity>forEach(_filter, _function);
  }
  
  public CharSequence toKeyClass(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import java.io.Serializable;");
    _builder.newLine();
    _builder.append("import org.springframework.util.Assert;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Represents a composite identifier for entity <code>");
    String _name = it.getName();
    _builder.append(_name, " ");
    _builder.append("</code>.");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("public class ");
    AttributeGroup _key = it.getKey();
    String _keyClassName = this._features.getKeyClassName(_key);
    _builder.append(_keyClassName, "");
    _builder.append(" implements Serializable {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _serialVersionUID = this._features.toSerialVersionUID(it);
    _builder.append(_serialVersionUID, "	");
    _builder.newLineIfNotEmpty();
    {
      AttributeGroup _key_1 = it.getKey();
      EList<AttributeSortOrder> _attributes = _key_1.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes, _function);
      for(final Attribute attribute : _map) {
        _builder.append("\t");
        CharSequence _field = this._features.toField(attribute);
        _builder.append(_field, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      AttributeGroup _key_2 = it.getKey();
      EList<AttributeSortOrder> _attributes_1 = _key_2.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_1 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_1 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_1, _function_1);
      for(final Attribute attribute_1 : _map_1) {
        _builder.append("\t");
        _builder.newLine();
        _builder.append("\t");
        CharSequence _getterMethod = this._features.toGetterMethod(attribute_1);
        _builder.append(_getterMethod, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        CharSequence _setterMethod = this._features.toSetterMethod(attribute_1);
        _builder.append(_setterMethod, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public ");
    AttributeGroup _key_3 = it.getKey();
    String _keyClassName_1 = this._features.getKeyClassName(_key_3);
    _builder.append(_keyClassName_1, "	");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public ");
    AttributeGroup _key_4 = it.getKey();
    String _keyClassName_2 = this._features.getKeyClassName(_key_4);
    _builder.append(_keyClassName_2, "	");
    _builder.append("(");
    {
      AttributeGroup _key_5 = it.getKey();
      EList<AttributeSortOrder> _attributes_2 = _key_5.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_2 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_2 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_2, _function_2);
      boolean _hasElements = false;
      for(final Attribute attribute_2 : _map_2) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "	");
        }
        CharSequence _methodParameter = this._features.methodParameter(attribute_2);
        _builder.append(_methodParameter, "	");
      }
    }
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    {
      AttributeGroup _key_6 = it.getKey();
      EList<AttributeSortOrder> _attributes_3 = _key_6.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_3 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_3 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_3, _function_3);
      for(final Attribute attribute_3 : _map_3) {
        _builder.append("\t\t");
        _builder.append("Assert.notNull(");
        String _name_1 = attribute_3.getName();
        _builder.append(_name_1, "		");
        _builder.append(", \"parameter \'");
        String _name_2 = attribute_3.getName();
        _builder.append(_name_2, "		");
        _builder.append("\' must not be null\");");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      AttributeGroup _key_7 = it.getKey();
      EList<AttributeSortOrder> _attributes_4 = _key_7.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_4 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_4 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_4, _function_4);
      for(final Attribute attribute_4 : _map_4) {
        _builder.append("\t\t");
        _builder.append("this.");
        String _name_3 = attribute_4.getName();
        _builder.append(_name_3, "		");
        _builder.append("=");
        String _name_4 = attribute_4.getName();
        _builder.append(_name_4, "		");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public int hashCode() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("final int prime = 31;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("int result = 1;");
    _builder.newLine();
    {
      AttributeGroup _key_8 = it.getKey();
      EList<AttributeSortOrder> _attributes_5 = _key_8.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_5 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_5 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_5, _function_5);
      for(final Attribute attribute_5 : _map_5) {
        _builder.append("\t\t");
        _builder.append("result = prime * result + ((");
        String _name_5 = attribute_5.getName();
        _builder.append(_name_5, "		");
        _builder.append(" == null) ? 0 : ");
        String _name_6 = attribute_5.getName();
        _builder.append(_name_6, "		");
        _builder.append(".hashCode());");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t\t");
    _builder.append("return result;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public boolean equals(Object obj) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (this == obj) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return true;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (obj == null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return false;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (getClass() != obj.getClass()) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return false;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    AttributeGroup _key_9 = it.getKey();
    String _keyClassName_3 = this._features.getKeyClassName(_key_9);
    _builder.append(_keyClassName_3, "		");
    _builder.append(" other = (");
    AttributeGroup _key_10 = it.getKey();
    String _keyClassName_4 = this._features.getKeyClassName(_key_10);
    _builder.append(_keyClassName_4, "		");
    _builder.append(") obj;");
    _builder.newLineIfNotEmpty();
    {
      AttributeGroup _key_11 = it.getKey();
      EList<AttributeSortOrder> _attributes_6 = _key_11.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_6 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_6 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_6, _function_6);
      for(final Attribute attribute_6 : _map_6) {
        _builder.append("\t\t");
        _builder.append("if (");
        String _name_7 = attribute_6.getName();
        _builder.append(_name_7, "		");
        _builder.append(" == null) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("if (other.");
        String _name_8 = attribute_6.getName();
        _builder.append(_name_8, "			");
        _builder.append(" != null) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t\t");
        _builder.append("return false;");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("} else if (!");
        String _name_9 = attribute_6.getName();
        _builder.append(_name_9, "		");
        _builder.append(".equals(other.");
        String _name_10 = attribute_6.getName();
        _builder.append(_name_10, "		");
        _builder.append(")) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("return false;");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("return true;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public String toString() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return new StringBuilder(\"");
    String _name_11 = it.getName();
    _builder.append(_name_11, "		");
    _builder.append(" [\")");
    _builder.newLineIfNotEmpty();
    {
      AttributeGroup _key_12 = it.getKey();
      EList<AttributeSortOrder> _attributes_7 = _key_12.getAttributes();
      final Function1<AttributeSortOrder,Attribute> _function_7 = new Function1<AttributeSortOrder,Attribute>() {
          public Attribute apply(final AttributeSortOrder e) {
            Attribute _attribute = e.getAttribute();
            return _attribute;
          }
        };
      List<Attribute> _map_7 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_7, _function_7);
      boolean _hasElements_1 = false;
      for(final Attribute attribute_7 : _map_7) {
        if (!_hasElements_1) {
          _hasElements_1 = true;
        } else {
          _builder.appendImmediate(".append(\",\")", "			");
        }
        _builder.append("\t\t\t");
        _builder.append(".append(\"");
        String _name_12 = attribute_7.getName();
        _builder.append(_name_12, "			");
        _builder.append("=\").append(");
        String _name_13 = attribute_7.getName();
        _builder.append(_name_13, "			");
        _builder.append(")");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t\t\t");
    _builder.append(".append(\"]\")");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append(".toString();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toAttributeProperties(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Attribute> _allAttributes = it.getAllAttributes();
      for(final Attribute attribute : _allAttributes) {
        {
          EList<AttributeProperty> _attributProperties = attribute.getAttributProperties();
          Iterable<AttributeTextProperty> _filter = Iterables.<AttributeTextProperty>filter(_attributProperties, AttributeTextProperty.class);
          final Function1<AttributeTextProperty,Boolean> _function = new Function1<AttributeTextProperty,Boolean>() {
              public Boolean apply(final AttributeTextProperty it) {
                String _labelText = it.getLabelText();
                boolean _notEquals = (!Objects.equal(_labelText, null));
                return Boolean.valueOf(_notEquals);
              }
            };
          Iterable<AttributeTextProperty> _filter_1 = IterableExtensions.<AttributeTextProperty>filter(_filter, _function);
          for(final AttributeTextProperty atp : _filter_1) {
            String _name = attribute.getName();
            _builder.append(_name, "");
            _builder.append(".title = ");
            String _unicode = EntityExtension.toUnicode(atp);
            _builder.append(_unicode, "");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          EList<AttributeProperty> _attributProperties_1 = attribute.getAttributProperties();
          Iterable<AttributeTextProperty> _filter_2 = Iterables.<AttributeTextProperty>filter(_attributProperties_1, AttributeTextProperty.class);
          final Function1<AttributeTextProperty,Boolean> _function_1 = new Function1<AttributeTextProperty,Boolean>() {
              public Boolean apply(final AttributeTextProperty it) {
                String _tooltipText = it.getTooltipText();
                boolean _notEquals = (!Objects.equal(_tooltipText, null));
                return Boolean.valueOf(_notEquals);
              }
            };
          Iterable<AttributeTextProperty> _filter_3 = IterableExtensions.<AttributeTextProperty>filter(_filter_2, _function_1);
          for(final AttributeTextProperty atp_1 : _filter_3) {
            _builder.append("\t");
            String _name_1 = attribute.getName();
            _builder.append(_name_1, "	");
            _builder.append(".description = ");
            String _tooltipText = atp_1.getTooltipText();
            _builder.append(_tooltipText, "	");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("\t");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toClass(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.springframework.context.annotation.Scope;");
    _builder.newLine();
    _builder.append("import org.springframework.stereotype.Component;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("@Component(\"");
    String _name = it.getName();
    String _firstLower = StringExtensions.toFirstLower(_name);
    _builder.append(_firstLower, "");
    _builder.append("Prototype\")");
    _builder.newLineIfNotEmpty();
    _builder.append("@Scope(\"prototype\")");
    _builder.newLine();
    _builder.append("public class ");
    String _className = this._names.getClassName(it);
    _builder.append(_className, "");
    _builder.append(" extends ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isDerived = it.isDerived();
            return Boolean.valueOf(_isDerived);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      for(final Attribute derivedAttribute : _filter) {
        {
          boolean _isReadOnly = derivedAttribute.isReadOnly();
          boolean _not = (!_isReadOnly);
          if (_not) {
            _builder.append("\t");
            _builder.append("/**");
            _builder.newLine();
            _builder.append("\t");
            _builder.append(" ");
            _builder.append("* @param ");
            String _name_1 = derivedAttribute.getName();
            _builder.append(_name_1, "	 ");
            _builder.append(" the ");
            String _name_2 = derivedAttribute.getName();
            _builder.append(_name_2, "	 ");
            _builder.append(" to set");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append(" ");
            _builder.append("*/");
            _builder.newLine();
            _builder.append("\t");
            _builder.append(" ");
            _builder.append("public void ");
            String _mutatorMethodName = this._features.getMutatorMethodName(derivedAttribute);
            _builder.append(_mutatorMethodName, "	 ");
            _builder.append("(");
            Type _dataType = derivedAttribute.getDataType();
            String _qualifiedName = this._names.getQualifiedName(_dataType);
            _builder.append(_qualifiedName, "	 ");
            _builder.append(" ");
            String _name_3 = derivedAttribute.getName();
            _builder.append(_name_3, "	 ");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append(" ");
            _builder.append("}");
            _builder.newLine();
          }
        }
        _builder.append("\t");
        _builder.append("/**");
        _builder.newLine();
        _builder.append("\t");
        _builder.append(" ");
        _builder.append("* @return the ");
        String _name_4 = derivedAttribute.getName();
        _builder.append(_name_4, "	 ");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append(" ");
        _builder.append("*/");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("public ");
        Type _dataType_1 = derivedAttribute.getDataType();
        String _qualifiedName_1 = this._names.getQualifiedName(_dataType_1);
        _builder.append(_qualifiedName_1, "	");
        _builder.append(" ");
        String _accessorMethodName = this._features.getAccessorMethodName(derivedAttribute);
        _builder.append(_accessorMethodName, "	");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("return null;");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toInterface(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _interfaceName = this._names.getInterfaceName(it);
    _builder.append(_interfaceName, "");
    _builder.append(" extends ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenInterface(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    CharSequence _xifexpression = null;
    Entity _superType = it.getSuperType();
    boolean _notEquals = (!Objects.equal(_superType, null));
    if (_notEquals) {
      StringConcatenation _builder_1 = new StringConcatenation();
      _builder_1.append(" ");
      _builder_1.append("extends ");
      Entity _superType_1 = it.getSuperType();
      String _qualifiedName = this._names.getQualifiedName(_superType_1);
      _builder_1.append(_qualifiedName, " ");
      _xifexpression = _builder_1;
    }
    _builder.append(_xifexpression, "");
    _builder.append("  {");
    _builder.newLineIfNotEmpty();
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isReference = it.isReference();
            boolean _not = (!_isReference);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      for(final Attribute attribute : _filter) {
        _builder.newLine();
        _builder.append("\t");
        CharSequence _accessorMethod = this.accessorMethod(attribute);
        _builder.append(_accessorMethod, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        CharSequence _mutatorMethod = this.mutatorMethod(attribute);
        _builder.append(_mutatorMethod, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Attribute> _attributes_1 = it.getAttributes();
      final Function1<Attribute,Boolean> _function_1 = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isReference = it.isReference();
            return Boolean.valueOf(_isReference);
          }
        };
      Iterable<Attribute> _filter_1 = IterableExtensions.<Attribute>filter(_attributes_1, _function_1);
      for(final Attribute attribute_1 : _filter_1) {
        _builder.newLine();
        _builder.append("\t");
        CharSequence _accessorMethod_1 = this.accessorMethod(attribute_1);
        _builder.append(_accessorMethod_1, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        CharSequence _mutatorMethod_1 = this.mutatorMethod(attribute_1);
        _builder.append(_mutatorMethod_1, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      boolean _and = false;
      boolean _and_1 = false;
      boolean _and_2 = false;
      AttributeGroup _key = it.getKey();
      boolean _notEquals_1 = (!Objects.equal(_key, null));
      if (!_notEquals_1) {
        _and_2 = false;
      } else {
        boolean _or = false;
        Entity _superType_2 = it.getSuperType();
        boolean _equals = Objects.equal(_superType_2, null);
        if (_equals) {
          _or = true;
        } else {
          Entity _superType_3 = it.getSuperType();
          AttributeGroup _key_1 = _superType_3.getKey();
          boolean _equals_1 = Objects.equal(_key_1, null);
          _or = (_equals || _equals_1);
        }
        _and_2 = (_notEquals_1 && _or);
      }
      if (!_and_2) {
        _and_1 = false;
      } else {
        Attribute _identifier = it.getIdentifier();
        boolean _equals_2 = Objects.equal(_identifier, null);
        _and_1 = (_and_2 && _equals_2);
      }
      if (!_and_1) {
        _and = false;
      } else {
        boolean _or_1 = false;
        Entity _superType_4 = it.getSuperType();
        boolean _equals_3 = Objects.equal(_superType_4, null);
        if (_equals_3) {
          _or_1 = true;
        } else {
          Entity _superType_5 = it.getSuperType();
          Attribute _identifier_1 = _superType_5.getIdentifier();
          boolean _equals_4 = Objects.equal(_identifier_1, null);
          _or_1 = (_equals_3 || _equals_4);
        }
        _and = (_and_1 && _or_1);
      }
      if (_and) {
        _builder.append("\t");
        AttributeGroup _key_2 = it.getKey();
        String _qualifiedKeyClassName = this._features.getQualifiedKeyClassName(_key_2);
        String _addImport = this._names.addImport(it, _qualifiedKeyClassName);
        _builder.append(_addImport, "	");
        _builder.append(" get");
        AttributeGroup _key_3 = it.getKey();
        String _name = _key_3.getName();
        _builder.append(_name, "	");
        _builder.append("();");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    CharSequence _typeExtension = this._featureExtensions.toTypeExtension(it);
    _builder.append(_typeExtension, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public abstract class ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    CharSequence _xifexpression = null;
    Entity _superType = it.getSuperType();
    boolean _notEquals = (!Objects.equal(_superType, null));
    if (_notEquals) {
      StringConcatenation _builder_1 = new StringConcatenation();
      _builder_1.append(" ");
      _builder_1.append("extends ");
      Entity _superType_1 = it.getSuperType();
      String _qualifiedClassName = this._names.getQualifiedClassName(_superType_1);
      String _addImport = this._names.addImport(it, _qualifiedClassName);
      _builder_1.append(_addImport, " ");
      _xifexpression = _builder_1;
    }
    _builder.append(_xifexpression, "");
    _builder.append(" implements ");
    String _qualifiedName = this._names.getQualifiedName(it);
    _builder.append(_qualifiedName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _generateAttributes = this._features.generateAttributes(it);
    _builder.append(_generateAttributes, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _generateProperties = this._features.generateProperties(it);
    _builder.append(_generateProperties, "	");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      boolean _and = false;
      boolean _and_1 = false;
      boolean _and_2 = false;
      AttributeGroup _key = it.getKey();
      boolean _notEquals_1 = (!Objects.equal(_key, null));
      if (!_notEquals_1) {
        _and_2 = false;
      } else {
        boolean _or = false;
        Entity _superType_2 = it.getSuperType();
        boolean _equals = Objects.equal(_superType_2, null);
        if (_equals) {
          _or = true;
        } else {
          Entity _superType_3 = it.getSuperType();
          AttributeGroup _key_1 = _superType_3.getKey();
          boolean _equals_1 = Objects.equal(_key_1, null);
          _or = (_equals || _equals_1);
        }
        _and_2 = (_notEquals_1 && _or);
      }
      if (!_and_2) {
        _and_1 = false;
      } else {
        Attribute _identifier = it.getIdentifier();
        boolean _equals_2 = Objects.equal(_identifier, null);
        _and_1 = (_and_2 && _equals_2);
      }
      if (!_and_1) {
        _and = false;
      } else {
        boolean _or_1 = false;
        Entity _superType_4 = it.getSuperType();
        boolean _equals_3 = Objects.equal(_superType_4, null);
        if (_equals_3) {
          _or_1 = true;
        } else {
          Entity _superType_5 = it.getSuperType();
          Attribute _identifier_1 = _superType_5.getIdentifier();
          boolean _equals_4 = Objects.equal(_identifier_1, null);
          _or_1 = (_equals_3 || _equals_4);
        }
        _and = (_and_1 && _or_1);
      }
      if (_and) {
        _builder.append("\t");
        _builder.append("public ");
        AttributeGroup _key_2 = it.getKey();
        String _keyClassName = this._features.getKeyClassName(_key_2);
        _builder.append(_keyClassName, "	");
        _builder.append(" get");
        AttributeGroup _key_3 = it.getKey();
        String _name = _key_3.getName();
        _builder.append(_name, "	");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("return new ");
        AttributeGroup _key_4 = it.getKey();
        String _keyClassName_1 = this._features.getKeyClassName(_key_4);
        _builder.append(_keyClassName_1, "		");
        _builder.append("(");
        {
          AttributeGroup _key_5 = it.getKey();
          EList<AttributeSortOrder> _attributes = _key_5.getAttributes();
          final Function1<AttributeSortOrder,Attribute> _function = new Function1<AttributeSortOrder,Attribute>() {
              public Attribute apply(final AttributeSortOrder e) {
                Attribute _attribute = e.getAttribute();
                return _attribute;
              }
            };
          List<Attribute> _map = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes, _function);
          boolean _hasElements = false;
          for(final Attribute attribute : _map) {
            if (!_hasElements) {
              _hasElements = true;
            } else {
              _builder.appendImmediate(",", "		");
            }
            String _name_1 = attribute.getName();
            _builder.append(_name_1, "		");
          }
        }
        _builder.append(");");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.newLine();
      }
    }
    {
      boolean _and_3 = false;
      boolean _hasIdentifierOrKey = this.hasIdentifierOrKey(it);
      if (!_hasIdentifierOrKey) {
        _and_3 = false;
      } else {
        boolean _or_2 = false;
        Entity _superType_6 = it.getSuperType();
        boolean _equals_5 = Objects.equal(_superType_6, null);
        if (_equals_5) {
          _or_2 = true;
        } else {
          Entity _superType_7 = it.getSuperType();
          boolean _hasIdentifierOrKey_1 = this.hasIdentifierOrKey(_superType_7);
          boolean _not = (!_hasIdentifierOrKey_1);
          _or_2 = (_equals_5 || _not);
        }
        _and_3 = (_hasIdentifierOrKey && _or_2);
      }
      if (_and_3) {
        _builder.append("\t");
        CharSequence _generateEqualsHashCode = this.generateEqualsHashCode(it);
        _builder.append(_generateEqualsHashCode, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    CharSequence _generateToString = this._features.generateToString(it);
    _builder.append(_generateToString, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence generateEqualsHashCode(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("public int hashCode() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("final int prime = 31;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("int result = 1;");
    _builder.newLine();
    {
      Attribute _identifier = it.getIdentifier();
      boolean _notEquals = (!Objects.equal(_identifier, null));
      if (_notEquals) {
        _builder.append("\t");
        _builder.append("result = prime * result + ((");
        Attribute _identifier_1 = it.getIdentifier();
        String _accessorMethodName = this._features.getAccessorMethodName(_identifier_1);
        _builder.append(_accessorMethodName, "	");
        _builder.append("() == null) ? 0 : ");
        Attribute _identifier_2 = it.getIdentifier();
        String _accessorMethodName_1 = this._features.getAccessorMethodName(_identifier_2);
        _builder.append(_accessorMethodName_1, "	");
        _builder.append("().hashCode());");
        _builder.newLineIfNotEmpty();
      } else {
        AttributeGroup _key = it.getKey();
        boolean _notEquals_1 = (!Objects.equal(_key, null));
        if (_notEquals_1) {
          _builder.append("\t");
          _builder.append("result = prime * result + ((get");
          AttributeGroup _key_1 = it.getKey();
          String _name = _key_1.getName();
          _builder.append(_name, "	");
          _builder.append("() == null) ? 0 : get");
          AttributeGroup _key_2 = it.getKey();
          String _name_1 = _key_2.getName();
          _builder.append(_name_1, "	");
          _builder.append("().hashCode());");
          _builder.newLineIfNotEmpty();
        }
      }
    }
    _builder.append("\t");
    _builder.append("return result;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("public boolean equals(Object obj) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (this == obj) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return true;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (obj == null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return false;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (!(obj instanceof ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "	");
    _builder.append(")) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return false;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    String _name_3 = it.getName();
    _builder.append(_name_3, "	");
    _builder.append(" other = (");
    String _name_4 = it.getName();
    _builder.append(_name_4, "	");
    _builder.append(") obj;");
    _builder.newLineIfNotEmpty();
    {
      Attribute _identifier_3 = it.getIdentifier();
      boolean _notEquals_2 = (!Objects.equal(_identifier_3, null));
      if (_notEquals_2) {
        _builder.append("\t");
        _builder.append("if (");
        Attribute _identifier_4 = it.getIdentifier();
        String _accessorMethodName_2 = this._features.getAccessorMethodName(_identifier_4);
        _builder.append(_accessorMethodName_2, "	");
        _builder.append("() == null) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("if (other.");
        Attribute _identifier_5 = it.getIdentifier();
        String _accessorMethodName_3 = this._features.getAccessorMethodName(_identifier_5);
        _builder.append(_accessorMethodName_3, "		");
        _builder.append("() != null) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t\t");
        _builder.append("return false;");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("} else if (!");
        Attribute _identifier_6 = it.getIdentifier();
        String _accessorMethodName_4 = this._features.getAccessorMethodName(_identifier_6);
        _builder.append(_accessorMethodName_4, "	");
        _builder.append("().equals(other.");
        Attribute _identifier_7 = it.getIdentifier();
        String _accessorMethodName_5 = this._features.getAccessorMethodName(_identifier_7);
        _builder.append(_accessorMethodName_5, "	");
        _builder.append("())) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("return false;");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      } else {
        AttributeGroup _key_3 = it.getKey();
        boolean _notEquals_3 = (!Objects.equal(_key_3, null));
        if (_notEquals_3) {
          _builder.append("\t");
          _builder.append("if (get");
          AttributeGroup _key_4 = it.getKey();
          String _name_5 = _key_4.getName();
          _builder.append(_name_5, "	");
          _builder.append("() == null) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("if (((");
          String _className = this._names.getClassName(it);
          _builder.append(_className, "		");
          _builder.append(")other).get");
          AttributeGroup _key_5 = it.getKey();
          String _name_6 = _key_5.getName();
          _builder.append(_name_6, "		");
          _builder.append("() != null) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t\t");
          _builder.append("return false;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("} else if (!get");
          AttributeGroup _key_6 = it.getKey();
          String _name_7 = _key_6.getName();
          _builder.append(_name_7, "	");
          _builder.append("().equals(((");
          String _className_1 = this._names.getClassName(it);
          _builder.append(_className_1, "	");
          _builder.append(")other).get");
          AttributeGroup _key_7 = it.getKey();
          String _name_8 = _key_7.getName();
          _builder.append(_name_8, "	");
          _builder.append("())) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return false;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
    }
    _builder.append("\t");
    _builder.append("return true;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence accessorMethod(final Attribute it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _documentationIndented = this._features.getDocumentationIndented(it);
    _builder.append(_documentationIndented, "");
    {
      boolean _isMany = it.isMany();
      if (_isMany) {
        _builder.newLineIfNotEmpty();
        String _addImport = this._names.addImport(it, "java.util.Set");
        _builder.append(_addImport, "");
        _builder.append("<");
        DataTypeAndTypeParameter _type = it.getType();
        Type _dataType = _type.getDataType();
        String _qualifiedName = this._names.getQualifiedName(_dataType);
        _builder.append(_qualifiedName, "");
        _builder.append("> ");
        String _accessorMethodName = this._features.getAccessorMethodName(it);
        _builder.append(_accessorMethodName, "");
        _builder.append("();");
      } else {
        _builder.newLineIfNotEmpty();
        DataTypeAndTypeParameter _type_1 = it.getType();
        Type _dataType_1 = _type_1.getDataType();
        String _qualifiedName_1 = this._names.getQualifiedName(_dataType_1);
        _builder.append(_qualifiedName_1, "");
        _builder.append(" ");
        String _accessorMethodName_1 = this._features.getAccessorMethodName(it);
        _builder.append(_accessorMethodName_1, "");
        _builder.append("();");
      }
    }
    return _builder;
  }
  
  private CharSequence mutatorMethod(final Attribute it) {
    CharSequence _xifexpression = null;
    boolean _isMany = it.isMany();
    if (_isMany) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("boolean ");
      String _hasMethodName = this._features.getHasMethodName(it);
      _builder.append(_hasMethodName, "");
      _builder.append("();");
      _builder.newLineIfNotEmpty();
      _builder.append("void ");
      String _addMethodName = this._features.getAddMethodName(it);
      _builder.append(_addMethodName, "");
      _builder.append("(");
      DataTypeAndTypeParameter _type = it.getType();
      Type _dataType = _type.getDataType();
      String _qualifiedName = this._names.getQualifiedName(_dataType);
      _builder.append(_qualifiedName, "");
      _builder.append(" ");
      String _name = it.getName();
      _builder.append(_name, "");
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      _builder.append("void ");
      String _removeMethodName = this._features.getRemoveMethodName(it);
      _builder.append(_removeMethodName, "");
      _builder.append("(");
      DataTypeAndTypeParameter _type_1 = it.getType();
      Type _dataType_1 = _type_1.getDataType();
      String _qualifiedName_1 = this._names.getQualifiedName(_dataType_1);
      _builder.append(_qualifiedName_1, "");
      _builder.append(" ");
      String _name_1 = it.getName();
      _builder.append(_name_1, "");
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      _builder.append("void ");
      String _removeAllMethodName = this._features.getRemoveAllMethodName(it);
      _builder.append(_removeAllMethodName, "");
      _builder.append("();");
      _builder.newLineIfNotEmpty();
      _xifexpression = _builder;
    } else {
      CharSequence _xifexpression_1 = null;
      boolean _isReadOnly = it.isReadOnly();
      boolean _not = (!_isReadOnly);
      if (_not) {
        StringConcatenation _builder_1 = new StringConcatenation();
        _builder_1.append("void ");
        String _mutatorMethodName = this._features.getMutatorMethodName(it);
        _builder_1.append(_mutatorMethodName, "");
        _builder_1.append("(");
        DataTypeAndTypeParameter _type_2 = it.getType();
        Type _dataType_2 = _type_2.getDataType();
        String _qualifiedName_2 = this._names.getQualifiedName(_dataType_2);
        _builder_1.append(_qualifiedName_2, "");
        _builder_1.append(" ");
        String _name_2 = it.getName();
        String _firstLower = StringExtensions.toFirstLower(_name_2);
        _builder_1.append(_firstLower, "");
        _builder_1.append(");");
        _xifexpression_1 = _builder_1;
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  private boolean hasIdentifierOrKey(final Entity it) {
    boolean _or = false;
    Attribute _identifier = it.getIdentifier();
    boolean _notEquals = (!Objects.equal(null, _identifier));
    if (_notEquals) {
      _or = true;
    } else {
      AttributeGroup _key = it.getKey();
      boolean _notEquals_1 = (!Objects.equal(null, _key));
      _or = (_notEquals || _notEquals_1);
    }
    return _or;
  }
  
  private String attributePropertiesFileName(final Entity it) {
    String _packageName = this._names.getPackageName(it);
    String _plus = (_packageName + ".");
    String _interfaceName = this._names.getInterfaceName(it);
    String _plus_1 = (_plus + _interfaceName);
    String _filePath = this._files.toFilePath(_plus_1, "properties");
    return _filePath;
  }
}
