package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.inject.Singleton;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.BoolLiteral;
import org.openxma.dsl.core.model.CondORExpr;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.EqualityOp;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.core.model.IntegerParameterValue;
import org.openxma.dsl.core.model.ParameterDefinition;
import org.openxma.dsl.core.model.ParameterValue;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.dom.model.ReadOnlyFlag;
import org.openxma.dsl.generator.impl.Facelet;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.JsfData;
import org.openxma.dsl.generator.impl.JsfNames;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.CommandImplementation;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.ComposedElement;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.EditableFlag;
import org.openxma.dsl.pom.model.EnabledFlag;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.FieldFeature;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.FieldReference;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.IComposite;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.InvokeStatement;
import org.openxma.dsl.pom.model.Invokeable;
import org.openxma.dsl.pom.model.Label;
import org.openxma.dsl.pom.model.LabelText;
import org.openxma.dsl.pom.model.MultiselectionFlag;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.PredefinedCommand;
import org.openxma.dsl.pom.model.PropertyJoin;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.Seperator;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.Statement;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;

@Singleton
@SuppressWarnings("all")
public class FaceletImpl implements Facelet {
  @Inject
  private JsfData data;
  
  @Inject
  private Files files;
  
  @Inject
  private JsfNames jsfNames;
  
  protected CharSequence _createContents(final XmadslPage page) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    _builder.newLine();
    _builder.append("<!DOCTYPE html");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
    _builder.newLine();
    _builder.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:f=\"http://java.sun.com/jsf/core\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:h=\"http://java.sun.com/jsf/html\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:ui=\"http://java.sun.com/jsf/facelets\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:p=\"http://primefaces.org/ui\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:i18n=\"http://fwv.spardat.at/i18n\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:fn=\"http://java.sun.com/jsp/jstl/functions\">");
    _builder.newLine();
    _builder.append("<head>");
    _builder.newLine();
    _builder.append("</head>");
    _builder.newLine();
    _builder.append("<body>");
    _builder.newLine();
    _builder.append("<ui:composition template=\"${template}\">");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("<ui:param name=\"title\" value=\"");
    LabelText _labelText = page.getLabelText();
    String _label = _labelText==null?(String)null:_labelText.getLabel();
    _builder.append(_label, "		");
    _builder.append("\" />");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("<ui:param name=\"dialogId\" value=\"");
    String _name = page.getName();
    String _firstLower = StringExtensions.toFirstLower(_name);
    _builder.append(_firstLower, "		");
    _builder.append("\" />");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("<ui:param name=\"dialogWidgetVar\" value=\"");
    CharSequence _dialogWidgetVar = this.jsfNames.getDialogWidgetVar(page);
    _builder.append(_dialogWidgetVar, "		");
    _builder.append("\" />");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("<ui:define name=\"dialogs\">");
    _builder.newLine();
    {
      Iterable<XmadslPage> _invokablePages = this.getInvokablePages(page);
      for(final XmadslPage invokeablePage : _invokablePages) {
        _builder.append("\t\t\t");
        _builder.append("<ui:remove>TODO: Copy the section \'dialogs\' to custom facelet. (otherwise not recognized)</ui:remove>");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("<ui:include src=\"/");
        String _faceletPageFileName = this.files.faceletPageFileName(invokeablePage, false);
        _builder.append(_faceletPageFileName, "			");
        _builder.append("\" />");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t\t");
    _builder.append("</ui:define>");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("<ui:define name=\"genscripts\">");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("<script type=\"text/javascript\">");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("if (!App.");
    Component _component = this.jsfNames.component(page);
    CharSequence _jSClass = this.jsfNames.getJSClass(_component);
    _builder.append(_jSClass, "				");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t\t");
    _builder.append("App.");
    Component _component_1 = this.jsfNames.component(page);
    CharSequence _jSClass_1 = this.jsfNames.getJSClass(_component_1);
    _builder.append(_jSClass_1, "					");
    _builder.append(" = Xma.Component.extend( {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t\t\t");
    Component _component_2 = this.jsfNames.component(page);
    EList<Command> _commands = _component_2.getCommands();
    CharSequence _uiCommandPrototypes = _commands==null?(CharSequence)null:this.toUiCommandPrototypes(_commands);
    _builder.append(_uiCommandPrototypes, "						");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t\t\t");
    _builder.append("getMainPage : function() { return App.");
    Component _component_3 = this.jsfNames.component(page);
    XmadslPage _mainPage = this.jsfNames.mainPage(_component_3);
    CharSequence _widgetVar = this.jsfNames.getWidgetVar(_mainPage);
    _builder.append(_widgetVar, "						");
    _builder.append("; }");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t\t");
    _builder.append("});");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("App.");
    Component _component_4 = this.jsfNames.component(page);
    CharSequence _widgetVar_1 = this.jsfNames.getWidgetVar(_component_4);
    _builder.append(_widgetVar_1, "					");
    _builder.append(" = new App.");
    Component _component_5 = this.jsfNames.component(page);
    CharSequence _jSClass_2 = this.jsfNames.getJSClass(_component_5);
    _builder.append(_jSClass_2, "					");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("App.");
    Component _component_6 = this.jsfNames.component(page);
    CharSequence _jSClass_3 = this.jsfNames.getJSClass(_component_6);
    _builder.append(_jSClass_3, "				");
    _builder.append(".");
    CharSequence _jSClass_4 = this.jsfNames.getJSClass(page);
    _builder.append(_jSClass_4, "				");
    _builder.append(" = Xma.Page.extend({");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t\t");
    EList<Command> _commands_1 = page.getCommands();
    CharSequence _uiCommandPrototypes_1 = _commands_1==null?(CharSequence)null:this.toUiCommandPrototypes(_commands_1);
    _builder.append(_uiCommandPrototypes_1, "					");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t\t");
    _builder.append("getDialogWidget : \tfunction() { return ");
    CharSequence _dialogWidgetVar_1 = this.jsfNames.getDialogWidgetVar(page);
    _builder.append(_dialogWidgetVar_1, "					");
    _builder.append(";}");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.append("});");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("App.");
    CharSequence _widgetVar_2 = this.jsfNames.getWidgetVar(page);
    _builder.append(_widgetVar_2, "				");
    _builder.append(" = new App.");
    Component _component_7 = this.jsfNames.component(page);
    CharSequence _jSClass_5 = this.jsfNames.getJSClass(_component_7);
    _builder.append(_jSClass_5, "				");
    _builder.append(".");
    CharSequence _jSClass_6 = this.jsfNames.getJSClass(page);
    _builder.append(_jSClass_6, "				");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("</script>");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("</ui:define>");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("<ui:define name=\"center\">");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("<h:form id=\"");
    String _name_1 = page.getName();
    _builder.append(_name_1, "			");
    _builder.append("Form\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    Content _content = page.getContent();
    CharSequence _createContents = _content==null?(CharSequence)null:this.createContents(_content);
    _builder.append(_createContents, "				");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    EList<Command> _commands_2 = page.getCommands();
    CharSequence _remoteCommands = _commands_2==null?(CharSequence)null:this.toRemoteCommands(_commands_2);
    _builder.append(_remoteCommands, "				");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    EList<ObjectProperty> _pageProperties = page.getPageProperties();
    CharSequence _remoteProperties = _pageProperties==null?(CharSequence)null:this.toRemoteProperties(_pageProperties);
    _builder.append(_remoteProperties, "				");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.append("<ui:insert name=\"additionalWidgets\" />");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("</h:form>");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("</ui:define>");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("<ui:define name=\"dialogs\">");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("</ui:define>");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("</ui:composition>");
    _builder.newLine();
    _builder.append("</body>");
    _builder.newLine();
    _builder.append("</html>");
    _builder.newLine();
    return _builder;
  }
  
  public Iterable<XmadslPage> getInvokablePages(final XmadslPage page) {
    Iterable<XmadslPage> _xblockexpression = null;
    {
      EList<Command> _commands = page.getCommands();
      final Function1<Command,CommandImplementation> _function = new Function1<Command,CommandImplementation>() {
          public CommandImplementation apply(final Command it) {
            CommandImplementation _implementation = it.getImplementation();
            return _implementation;
          }
        };
      List<CommandImplementation> _map = ListExtensions.<Command, CommandImplementation>map(_commands, _function);
      final Iterable<CommandImplementation> implementations = IterableExtensions.<CommandImplementation>filterNull(_map);
      final Function1<CommandImplementation,Iterable<InvokeStatement>> _function_1 = new Function1<CommandImplementation,Iterable<InvokeStatement>>() {
          public Iterable<InvokeStatement> apply(final CommandImplementation it) {
            EList<Statement> _statements = it.getStatements();
            Iterable<InvokeStatement> _filter = Iterables.<InvokeStatement>filter(_statements, InvokeStatement.class);
            return _filter;
          }
        };
      Iterable<Iterable<InvokeStatement>> _map_1 = IterableExtensions.<CommandImplementation, Iterable<InvokeStatement>>map(implementations, _function_1);
      final Iterable<InvokeStatement> statements = Iterables.<InvokeStatement>concat(_map_1);
      final Function1<InvokeStatement,Invokeable> _function_2 = new Function1<InvokeStatement,Invokeable>() {
          public Invokeable apply(final InvokeStatement it) {
            Invokeable _invokeAble = it.getInvokeAble();
            return _invokeAble;
          }
        };
      final Iterable<Invokeable> invokeables = IterableExtensions.<InvokeStatement, Invokeable>map(statements, _function_2);
      Iterable<XmadslPage> _filter = Iterables.<XmadslPage>filter(invokeables, XmadslPage.class);
      _xblockexpression = (_filter);
    }
    return _xblockexpression;
  }
  
  public CharSequence toRemoteProperties(final EList<ObjectProperty> properties) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final ObjectProperty property : properties) {
        _builder.append("<p:inputText value=\"#{");
        XmadslPage _parentPage = this.jsfNames.getParentPage(property);
        CharSequence _bean = this.jsfNames.getBean(_parentPage);
        _builder.append(_bean, "");
        _builder.append(".");
        String _name = property.getName();
        _builder.append(_name, "");
        _builder.append("}\" style=\"display:none;\" widgetVar=\"");
        CharSequence _widgetVar = this.jsfNames.getWidgetVar(property);
        _builder.append(_widgetVar, "");
        _builder.append("\" />");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toRemoteCommands(final EList<Command> commands) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final Command command : commands) {
        {
          boolean _isClientOnly = command.isClientOnly();
          boolean _not = (!_isClientOnly);
          if (_not) {
            _builder.append("<p:remoteCommand action=\"#{");
            XmadslPage _parentPage = this.jsfNames.getParentPage(command);
            CharSequence _bean = this.jsfNames.getBean(_parentPage);
            _builder.append(_bean, "");
            _builder.append(".");
            String _name = command.getName();
            _builder.append(_name, "");
            _builder.append("}\" update=\"@form\" name=\"");
            CharSequence _widgetVar = this.jsfNames.getWidgetVar(command);
            _builder.append(_widgetVar, "");
            _builder.append("\" />");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final Table table) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:dataTable id=\"");
    String _name = table.getName();
    _builder.append(_name, "");
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    {
      boolean _or = false;
      MultiselectionFlag _multiselectionFlag = table.getMultiselectionFlag();
      BoolLiteral _multiSelection = _multiselectionFlag==null?(BoolLiteral)null:_multiselectionFlag.getMultiSelection();
      if ((_multiSelection instanceof TrueLiteral)) {
        _or = true;
      } else {
        BoolLiteral _checkSelection = table.getCheckSelection();
        _or = ((_multiSelection instanceof TrueLiteral) || (_checkSelection instanceof TrueLiteral));
      }
      if (_or) {
        _builder.append("selectionMode=\"multiple\" selection=\"#{");
        CharSequence _modelProperty = this.getModelProperty(table);
        _builder.append(_modelProperty, "");
        _builder.append(".selectedItems}\"");
        _builder.newLineIfNotEmpty();
      } else {
        _builder.append("selectionMode=\"single\" selection=\"#{");
        CharSequence _modelProperty_1 = this.getModelProperty(table);
        _builder.append(_modelProperty_1, "");
        _builder.append(".selectedItem}\"");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append(" \t");
    _builder.append("widgetVar=\"");
    CharSequence _widgetVar = this.jsfNames.getWidgetVar(table);
    _builder.append(_widgetVar, " 	");
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    _builder.append("value=\"#{");
    CharSequence _modelProperty_2 = this.getModelProperty(table);
    _builder.append(_modelProperty_2, "");
    _builder.append("}\" var=\"item\"");
    _builder.newLineIfNotEmpty();
    _builder.append("rowKey=\"#{item.oid}\"");
    _builder.newLine();
    {
      BoolLiteral _pageable = table.getPageable();
      if ((_pageable instanceof TrueLiteral)) {
        _builder.append("\t\t\t    ");
        _builder.append("paginator=\"true\" rows=\"10\" paginatorPosition=\"top\" paginatorTemplate=\"{CurrentPageReport}  {FirstPageLink} {PreviousPageLink} {PageLinks} {NextPageLink} {LastPageLink}\"");
        _builder.newLine();
      }
    }
    _builder.append("\t\t\t    ");
    _builder.append("resizableColumns=\"true\" >");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    CharSequence _createEventMappingsForControl = this.createEventMappingsForControl(table);
    _builder.append(_createEventMappingsForControl, "			    ");
    _builder.newLineIfNotEmpty();
    {
      EList<TableColumn> _columns = table.getColumns();
      for(final TableColumn column : _columns) {
        CharSequence _createContents = this.createContents(column);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("</p:dataTable>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:tree id=\"");
    String _name = tree.getName();
    _builder.append(_name, "");
    _builder.append("\" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" \t");
    _builder.append("selectionMode=\"single\" widgetVar=\"");
    String _name_1 = tree.getName();
    _builder.append(_name_1, " 	");
    _builder.append("WidgetVar\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    _builder.append("value=\"#{");
    CharSequence _rootNodeProperty = this.getRootNodeProperty(tree);
    _builder.append(_rootNodeProperty, "			    ");
    _builder.append("}\" var=\"node\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    _builder.append("selection=\"#{");
    CharSequence _selectedNodeProperty = this.getSelectedNodeProperty(tree);
    _builder.append(_selectedNodeProperty, "			    ");
    _builder.append("}\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    CharSequence _createEventMappingsForControl = this.createEventMappingsForControl(tree);
    _builder.append(_createEventMappingsForControl, "			    ");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    _builder.append("<p:treeNode>");
    _builder.newLine();
    _builder.append("\t\t\t    \t");
    _builder.append("<h:outputText value=\"#{node}\" />");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("</p:treeNode>");
    _builder.newLine();
    _builder.append("</p:tree>");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getModelProperty(final Table table) {
    StringConcatenation _builder = new StringConcatenation();
    XmadslPage _parentPage = this.jsfNames.getParentPage(table);
    CharSequence _bean = this.jsfNames.getBean(_parentPage);
    _builder.append(_bean, "");
    _builder.append(".");
    CharSequence _modelPropertyName = this.data.getModelPropertyName(table);
    _builder.append(_modelPropertyName, "");
    return _builder;
  }
  
  public CharSequence getRootNodeProperty(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    XmadslPage _parentPage = this.jsfNames.getParentPage(tree);
    CharSequence _bean = this.jsfNames.getBean(_parentPage);
    _builder.append(_bean, "");
    _builder.append(".");
    CharSequence _rootNodePropertyName = this.data.getRootNodePropertyName(tree);
    _builder.append(_rootNodePropertyName, "");
    return _builder;
  }
  
  public CharSequence getSelectedNodeProperty(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    XmadslPage _parentPage = this.jsfNames.getParentPage(tree);
    CharSequence _bean = this.jsfNames.getBean(_parentPage);
    _builder.append(_bean, "");
    _builder.append(".");
    CharSequence _selectedNodePropertyName = this.data.getSelectedNodePropertyName(tree);
    _builder.append(_selectedNodePropertyName, "");
    return _builder;
  }
  
  public CharSequence getSelectItemsProperty(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    XmadslPage _parentPage = this.jsfNames.getParentPage(field);
    CharSequence _bean = this.jsfNames.getBean(_parentPage);
    _builder.append(_bean, "");
    _builder.append(".");
    CharSequence _selectItemsPropertyName = this.data.getSelectItemsPropertyName(field);
    _builder.append(_selectItemsPropertyName, "");
    return _builder;
  }
  
  protected CharSequence _createEventMapping(final IGuiElementWithEvent control, final GuiElementEventMapping mapping) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<!-- error not control event mapping for control type ");
    Class<? extends IGuiElementWithEvent> _class = control.getClass();
    String _name = _class.getName();
    _builder.append(_name, "");
    _builder.append(" -->");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createEventMapping(final Table table, final GuiElementEventMapping mapping) {
    CharSequence _switchResult = null;
    EventType _event = mapping.getEvent();
    final EventType _switchValue = _event;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(_switchValue,EventType.ON_SELECTION)) {
        _matched=true;
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("<p:ajax event=\"rowSelect\" listener=\"#{");
        XmadslPage _parentPage = this.jsfNames.getParentPage(table);
        CharSequence _bean = this.jsfNames.getBean(_parentPage);
        _builder.append(_bean, "");
        _builder.append(".");
        String _name = table.getName();
        _builder.append(_name, "");
        _builder.append("_onSelection}\" update=\"@form\" />");
        _switchResult = _builder;
      }
    }
    if (!_matched) {
      StringConcatenation _builder_1 = new StringConcatenation();
      _switchResult = _builder_1;
    }
    return _switchResult;
  }
  
  public CharSequence createEventMappingsForControl(final IGuiElementWithEvent control) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Iterable<GuiElementEventMapping> _eventMappingsForControl = this.getEventMappingsForControl(control);
      for(final GuiElementEventMapping mapping : _eventMappingsForControl) {
        CharSequence _createEventMapping = this.createEventMapping(control, mapping);
        _builder.append(_createEventMapping, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public Iterable<GuiElementEventMapping> getEventMappingsForControl(final IGuiElementWithEvent control) {
    XmadslPage _parentPage = this.jsfNames.getParentPage(control);
    EventMappingList _events = _parentPage.getEvents();
    EList<EventMapping> _mapping = _events.getMapping();
    Iterable<GuiElementEventMapping> _filter = Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
    final Function1<GuiElementEventMapping,Boolean> _function = new Function1<GuiElementEventMapping,Boolean>() {
        public Boolean apply(final GuiElementEventMapping m) {
          IGuiElementWithEvent _control = m.getControl();
          boolean _equals = Objects.equal(_control, control);
          return Boolean.valueOf(_equals);
        }
      };
    Iterable<GuiElementEventMapping> _filter_1 = IterableExtensions.<GuiElementEventMapping>filter(_filter, _function);
    return _filter_1;
  }
  
  protected CharSequence _createContents(final TableCombo table) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(" ");
    _builder.append("<p:outputLabel>not supported: TableCombo</p:outputLabel>");
    return _builder;
  }
  
  public ParameterValue getParameter(final EList<ParameterValue> list, final String name) {
    final Function1<ParameterValue,Boolean> _function = new Function1<ParameterValue,Boolean>() {
        public Boolean apply(final ParameterValue p) {
          ParameterDefinition _referencedParameter = p.getReferencedParameter();
          String _name = _referencedParameter.getName();
          boolean _equals = Objects.equal(_name, name);
          return Boolean.valueOf(_equals);
        }
      };
    ParameterValue _findFirst = IterableExtensions.<ParameterValue>findFirst(list, _function);
    return _findFirst;
  }
  
  public int getParameterValue(final EList<ParameterValue> list, final String name) {
    ParameterValue _parameter = this.getParameter(list, name);
    int _intValue = ((IntegerParameterValue) _parameter).getIntValue();
    return _intValue;
  }
  
  public CharSequence createConverter(final ValidatorReference validatorRef) {
    CharSequence _xifexpression = null;
    boolean _equals = Objects.equal(validatorRef, null);
    if (_equals) {
      StringConcatenation _builder = new StringConcatenation();
      _xifexpression = _builder;
    } else {
      CharSequence _xifexpression_1 = null;
      boolean _isDateValidator = validatorRef.isDateValidator();
      if (_isDateValidator) {
        StringConcatenation _builder_1 = new StringConcatenation();
        _builder_1.append("<f:convertDateTime pattern=\"dd.MM.yyyy\" />");
        _xifexpression_1 = _builder_1;
      } else {
        CharSequence _xifexpression_2 = null;
        boolean _isNumberValidator = validatorRef.isNumberValidator();
        if (_isNumberValidator) {
          StringConcatenation _builder_2 = new StringConcatenation();
          _builder_2.append("<f:convertNumber minFractionDigits=\"");
          _builder_2.append(2, "");
          _builder_2.append("\" maxFractionDigits=\"");
          _builder_2.append(2, "");
          _builder_2.append("\" ");
          _builder_2.newLineIfNotEmpty();
          _builder_2.append("\t\t\t\t\t\t");
          _builder_2.append("maxIntegerDigits=\"");
          _builder_2.append(10, "						");
          _builder_2.append("\" />");
          _builder_2.newLineIfNotEmpty();
          _builder_2.append("\t\t\t\t\t\t");
          _builder_2.newLine();
          _builder_2.append("\t\t\t\t\t\t");
          _builder_2.newLine();
          _xifexpression_2 = _builder_2;
        } else {
          StringConcatenation _builder_3 = new StringConcatenation();
          _xifexpression_2 = _builder_3;
        }
        _xifexpression_1 = _xifexpression_2;
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  protected CharSequence _createContents(final TableColumn column) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:column headerText=\"");
    String _labelText = column.getLabelText();
    CharSequence _encodedHtml = this.toEncodedHtml(_labelText);
    _builder.append(_encodedHtml, "");
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _and = false;
      boolean _equals = "styling".equals("styling");
      if (!_equals) {
        _and = false;
      } else {
        Attribute _attribute = column.getAttribute();
        boolean _notEquals = (!Objects.equal(_attribute, null));
        _and = (_equals && _notEquals);
      }
      if (_and) {
        _builder.append("styleClass=\"");
        Attribute _attribute_1 = column.getAttribute();
        String _name = _attribute_1.getName();
        _builder.append(_name, "	");
        _builder.append("\"");
      }
    }
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t    ");
    {
      boolean _and_1 = false;
      FieldFeature _feature = column.getFeature();
      boolean _notEquals_1 = (!Objects.equal(_feature, null));
      if (!_notEquals_1) {
        _and_1 = false;
      } else {
        FieldFeature _feature_1 = column.getFeature();
        PresentableFeature _attributeHolder = _feature_1.getAttributeHolder();
        boolean _notEquals_2 = (!Objects.equal(_attributeHolder, null));
        _and_1 = (_notEquals_1 && _notEquals_2);
      }
      if (_and_1) {
        _builder.append("sortBy=\"#{");
        CharSequence _columnValueExpression = this.columnValueExpression(column);
        _builder.append(_columnValueExpression, "				    ");
        _builder.append("}\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t\t    ");
        _builder.append("<h:outputText value=\"#{");
        CharSequence _columnValueExpression_1 = this.columnValueExpression(column);
        _builder.append(_columnValueExpression_1, "				    ");
        _builder.append("}\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t\t    ");
        _builder.append("\t");
        FieldFeature _feature_2 = column.getFeature();
        PresentableFeature _attributeHolder_1 = _feature_2.getAttributeHolder();
        Attribute _attribute_2 = _attributeHolder_1.getAttribute();
        DataTypeAndTypeParameter _type = _attribute_2.getType();
        ValidatorReference _validatorReferenceDefinition = _type.getValidatorReferenceDefinition();
        CharSequence _createConverter = this.createConverter(_validatorReferenceDefinition);
        _builder.append(_createConverter, "				    	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t\t    ");
        _builder.append("</h:outputText>");
        _builder.newLine();
      } else {
        _builder.append("\t\t\t\t    ");
        _builder.append(">");
        _builder.newLine();
        _builder.append("\t\t\t\t    ");
        _builder.append("<h:outputText value=\"hardcodedtext\" />");
        _builder.newLine();
      }
    }
    _builder.append("</p:column>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final Text text) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:inputText id=\"");
    String _name = text.getName();
    _builder.append(_name, "");
    _builder.append("\" />");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final TabFolder tabFolder) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:tabView id=\"");
    String _name = tabFolder.getName();
    CharSequence _asIdentifier = this.jsfNames.asIdentifier(_name);
    _builder.append(_asIdentifier, "");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    {
      EList<EObject> _eContents = tabFolder.eContents();
      for(final EObject item : _eContents) {
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("</p:tabView>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final TabPage tabPage) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:tab id=\"");
    String _name = tabPage.getName();
    _builder.append(_name, "");
    _builder.append("\" title=\"");
    LabelText _labelText = tabPage.getLabelText();
    String _string = _labelText.getString();
    _builder.append(_string, "");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    {
      EList<EObject> _eContents = tabPage.eContents();
      for(final EObject item : _eContents) {
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("</p:tab>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final Button button) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:commandButton id=\"");
    String _name = button.getName();
    _builder.append(_name, "");
    _builder.append("\" value=\"");
    LabelText _labelText = button.getLabelText();
    String _label = _labelText.getLabel();
    _builder.append(_label, "");
    _builder.append("\" onclick=\"");
    CharSequence _expandCommandMappings = this.expandCommandMappings(button);
    _builder.append(_expandCommandMappings, "");
    _builder.append("\" type=\"button\" />");
    return _builder;
  }
  
  protected CharSequence _createContents(final Seperator seperator) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:separator />");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final FieldReference field) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<h:field attribute=\"");
    Attribute _attribute = field.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final Combo combo) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:selectOneMenu id=\"");
    String _name = combo.getName();
    _builder.append(_name, "");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final Label label) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<h:outputLabel value=\"");
    LabelText _labelText = label.getLabelText();
    String _label = _labelText.getLabel();
    CharSequence _encodedHtml = this.toEncodedHtml(_label);
    _builder.append(_encodedHtml, "");
    _builder.append("\" ");
    {
      String _name = label.getName();
      boolean _notEquals = (!Objects.equal(_name, null));
      if (_notEquals) {
        _builder.append("id=\"");
        String _name_1 = label.getName();
        _builder.append(_name_1, "");
        _builder.append("\"");
      }
    }
    _builder.append(" />");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final LabelText label) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _and = false;
      EObject _eContainer = label.eContainer();
      if (!(_eContainer instanceof TabPage)) {
        _and = false;
      } else {
        EObject _eContainer_1 = label.eContainer();
        LabelText _labelText = ((TabPage) _eContainer_1).getLabelText();
        boolean _equals = Objects.equal(_labelText, label);
        _and = ((_eContainer instanceof TabPage) && _equals);
      }
      if (_and) {
      } else {
        _builder.append("<h:outputLabel value=\"");
        String _label = label.getLabel();
        CharSequence _encodedHtml = this.toEncodedHtml(_label);
        _builder.append(_encodedHtml, "");
        _builder.append("\" />");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence debugFieldPart(final FieldPartSpecification part) {
    FieldPart _part = part.getPart();
    String _literal = _part.getLiteral();
    String _plus = ("lit" + _literal);
    String _plus_1 = (_plus + "pname");
    FieldPart _part_1 = part.getPart();
    String _name = _part_1.name();
    String _plus_2 = (_plus_1 + _name);
    return _plus_2;
  }
  
  protected CharSequence _createContents(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _or = false;
      boolean _or_1 = false;
      EList<FieldPartSpecification> _parts = field.getParts();
      boolean _equals = Objects.equal(_parts, null);
      if (_equals) {
        _or_1 = true;
      } else {
        EList<FieldPartSpecification> _parts_1 = field.getParts();
        int _size = _parts_1.size();
        boolean _equals_1 = (_size == 0);
        _or_1 = (_equals || _equals_1);
      }
      if (_or_1) {
        _or = true;
      } else {
        EList<FieldPartSpecification> _parts_2 = field.getParts();
        final Function1<FieldPartSpecification,Boolean> _function = new Function1<FieldPartSpecification,Boolean>() {
            public Boolean apply(final FieldPartSpecification fps) {
              FieldPart _part = fps.getPart();
              boolean _equals = Objects.equal(_part, FieldPart.LABEL);
              return Boolean.valueOf(_equals);
            }
          };
        Iterable<FieldPartSpecification> _filter = IterableExtensions.<FieldPartSpecification>filter(_parts_2, _function);
        int _size_1 = IterableExtensions.size(_filter);
        boolean _greaterThan = (_size_1 > 0);
        _or = (_or_1 || _greaterThan);
      }
      if (_or) {
        _builder.append("<p:column>");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("<p:outputLabel value=\"");
        String _labelText = field.getLabelText();
        CharSequence _encodedHtml = this.toEncodedHtml(_labelText);
        _builder.append(_encodedHtml, "	");
        _builder.append(": \" for=\"");
        Attribute _attribute = field.getAttribute();
        String _name = _attribute.getName();
        _builder.append(_name, "	");
        _builder.append("1\" />");
        _builder.newLineIfNotEmpty();
        _builder.append("</p:column>");
        _builder.newLine();
      }
    }
    {
      boolean _or_2 = false;
      boolean _or_3 = false;
      EList<FieldPartSpecification> _parts_3 = field.getParts();
      boolean _equals_2 = Objects.equal(_parts_3, null);
      if (_equals_2) {
        _or_3 = true;
      } else {
        EList<FieldPartSpecification> _parts_4 = field.getParts();
        int _size_2 = _parts_4.size();
        boolean _equals_3 = (_size_2 == 0);
        _or_3 = (_equals_2 || _equals_3);
      }
      if (_or_3) {
        _or_2 = true;
      } else {
        EList<FieldPartSpecification> _parts_5 = field.getParts();
        final Function1<FieldPartSpecification,Boolean> _function_1 = new Function1<FieldPartSpecification,Boolean>() {
            public Boolean apply(final FieldPartSpecification fps) {
              FieldPart _part = fps.getPart();
              boolean _equals = Objects.equal(_part, FieldPart.CONTROL);
              return Boolean.valueOf(_equals);
            }
          };
        Iterable<FieldPartSpecification> _filter_1 = IterableExtensions.<FieldPartSpecification>filter(_parts_5, _function_1);
        int _size_3 = IterableExtensions.size(_filter_1);
        boolean _greaterThan_1 = (_size_3 > 0);
        _or_2 = (_or_3 || _greaterThan_1);
      }
      if (_or_2) {
        _builder.append("<p:column>");
        _builder.newLine();
        {
          Attribute _attribute_1 = field.getAttribute();
          DataTypeAndTypeParameter _type = _attribute_1.getType();
          Type _dataType = _type.getDataType();
          if ((_dataType instanceof EnumType)) {
            _builder.append("    ");
            _builder.append("<p:selectOneMenu id=\"");
            Attribute _attribute_2 = field.getAttribute();
            String _name_1 = _attribute_2.getName();
            _builder.append(_name_1, "    ");
            _builder.append("1\" value=\"#{");
            CharSequence _fieldValueExpression = this.fieldValueExpression(field);
            _builder.append(_fieldValueExpression, "    ");
            _builder.append("}\" ");
            CharSequence _disabledFlag = this.disabledFlag(field);
            _builder.append(_disabledFlag, "    ");
            _builder.append(">");
            _builder.newLineIfNotEmpty();
            _builder.append("    ");
            _builder.append("\t");
            _builder.append("<f:selectItem itemLabel=\"Select value\" itemValue=\"\" />");
            _builder.newLine();
            _builder.append("    ");
            _builder.append("    ");
            _builder.append("<f:selectItems value=\"#{");
            CharSequence _selectItemsProperty = this.getSelectItemsProperty(field);
            _builder.append(_selectItemsProperty, "        ");
            _builder.append("}\" var=\"item\" itemLabel=\"#{item.name}\" itemValue=\"#{item}\" />");
            _builder.newLineIfNotEmpty();
            _builder.append("    ");
            _builder.append("</p:selectOneMenu>");
            _builder.newLine();
          } else {
            Attribute _attribute_3 = field.getAttribute();
            DataTypeAndTypeParameter _type_1 = _attribute_3.getType();
            ValidatorReference _validatorReferenceDefinition = _type_1.getValidatorReferenceDefinition();
            boolean _isBooleanValidator = _validatorReferenceDefinition.isBooleanValidator();
            if (_isBooleanValidator) {
              _builder.append("    ");
              _builder.append("<p:selectBooleanCheckbox id=\"");
              Attribute _attribute_4 = field.getAttribute();
              String _name_2 = _attribute_4.getName();
              _builder.append(_name_2, "    ");
              _builder.append("1\" value=\"#{");
              CharSequence _fieldValueExpression_1 = this.fieldValueExpression(field);
              _builder.append(_fieldValueExpression_1, "    ");
              _builder.append("}\" ");
              CharSequence _disabledFlag_1 = this.disabledFlag(field);
              _builder.append(_disabledFlag_1, "    ");
              _builder.append(" />");
              _builder.newLineIfNotEmpty();
            } else {
              Attribute _attribute_5 = field.getAttribute();
              DataTypeAndTypeParameter _type_2 = _attribute_5.getType();
              ValidatorReference _validatorReferenceDefinition_1 = _type_2.getValidatorReferenceDefinition();
              boolean _isDateValidator = _validatorReferenceDefinition_1.isDateValidator();
              if (_isDateValidator) {
                _builder.append("    ");
                _builder.append("<p:calendar id=\"");
                Attribute _attribute_6 = field.getAttribute();
                String _name_3 = _attribute_6.getName();
                _builder.append(_name_3, "    ");
                _builder.append("1\" value=\"#{");
                CharSequence _fieldValueExpression_2 = this.fieldValueExpression(field);
                _builder.append(_fieldValueExpression_2, "    ");
                _builder.append("}\" ");
                CharSequence _disabledFlag_2 = this.disabledFlag(field);
                _builder.append(_disabledFlag_2, "    ");
                _builder.append(" />");
                _builder.newLineIfNotEmpty();
              } else {
                _builder.append("    ");
                _builder.append("<p:inputText id=\"");
                Attribute _attribute_7 = field.getAttribute();
                String _name_4 = _attribute_7.getName();
                _builder.append(_name_4, "    ");
                _builder.append("1\" value=\"#{");
                CharSequence _fieldValueExpression_3 = this.fieldValueExpression(field);
                _builder.append(_fieldValueExpression_3, "    ");
                _builder.append("}\" ");
                CharSequence _disabledFlag_3 = this.disabledFlag(field);
                _builder.append(_disabledFlag_3, "    ");
                _builder.append(">");
                ValidatorReference _format = field.getFormat();
                CharSequence _createConverter = this.createConverter(_format);
                _builder.append(_createConverter, "    ");
                _builder.newLineIfNotEmpty();
                _builder.append("    ");
                FieldFeature _feature = field.getFeature();
                PresentableFeature _attributeHolder = _feature.getAttributeHolder();
                Attribute _attribute_8 = _attributeHolder.getAttribute();
                DataTypeAndTypeParameter _type_3 = _attribute_8.getType();
                ValidatorReference _validatorReferenceDefinition_2 = _type_3.getValidatorReferenceDefinition();
                CharSequence _createConverter_1 = this.createConverter(_validatorReferenceDefinition_2);
                _builder.append(_createConverter_1, "    ");
                _builder.append(" ");
                _builder.newLineIfNotEmpty();
                _builder.append("    ");
                _builder.append("</p:inputText>");
                _builder.newLine();
              }
            }
          }
        }
        _builder.append("</p:column>");
        _builder.newLine();
      }
    }
    _builder.append("<p:column>");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("<p:message for=\"");
    Attribute _attribute_9 = field.getAttribute();
    String _name_5 = _attribute_9.getName();
    _builder.append(_name_5, "	");
    _builder.append("1\" display=\"icon\" />");
    _builder.newLineIfNotEmpty();
    _builder.append("</p:column>\t");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence disabledFlag(final CustomizeableField field) {
    CharSequence _xblockexpression = null;
    {
      EList<FieldFlag> _fieldFlags = field.getFieldFlags();
      final Function1<FieldFlag,Boolean> _function = new Function1<FieldFlag,Boolean>() {
          public Boolean apply(final FieldFlag f) {
            return Boolean.valueOf((f instanceof EnabledFlag));
          }
        };
      final FieldFlag flag = IterableExtensions.<FieldFlag>findFirst(_fieldFlags, _function);
      CharSequence _xifexpression = null;
      boolean _and = false;
      boolean _notEquals = (!Objects.equal(flag, null));
      if (!_notEquals) {
        _and = false;
      } else {
        boolean _not = (!(flag instanceof TrueLiteral));
        _and = (_notEquals && _not);
      }
      if (_and) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("disabled=\"!");
        EqualityExpr _expression = flag.getExpression();
        CharSequence _expressionLanguage = this.getExpressionLanguage(_expression, "false");
        _builder.append(_expressionLanguage, "");
        _builder.append("\"");
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public CharSequence readonlyFlag(final CustomizeableField field) {
    CharSequence _xblockexpression = null;
    {
      EList<FieldFlag> _fieldFlags = field.getFieldFlags();
      final Function1<FieldFlag,Boolean> _function = new Function1<FieldFlag,Boolean>() {
          public Boolean apply(final FieldFlag f) {
            return Boolean.valueOf((f instanceof ReadOnlyFlag));
          }
        };
      final FieldFlag flag = IterableExtensions.<FieldFlag>findFirst(_fieldFlags, _function);
      CharSequence _xifexpression = null;
      boolean _and = false;
      boolean _notEquals = (!Objects.equal(flag, null));
      if (!_notEquals) {
        _and = false;
      } else {
        boolean _not = (!(flag instanceof FalseLiteral));
        _and = (_notEquals && _not);
      }
      if (_and) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("readonly=\"");
        EqualityExpr _expression = flag.getExpression();
        CharSequence _expressionLanguage = this.getExpressionLanguage(_expression, "false");
        _builder.append(_expressionLanguage, "");
        _builder.append("\"");
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _getExpressionLanguage(final EqualityExpr it, final CharSequence defaultVal) {
    return defaultVal;
  }
  
  protected CharSequence _getExpressionLanguage(final FalseLiteral it, final CharSequence defaultVal) {
    return "false";
  }
  
  protected CharSequence _getExpressionLanguage(final TrueLiteral it, final CharSequence defaultVal) {
    return "true";
  }
  
  /**
   * «field.fieldFlags.createDisabledFlag(field.pageBean)»
   */
  public CharSequence createDisabledFlag(final EList<FieldFlag> fieldFlags, final CharSequence pageBean) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Iterable<EditableFlag> _filter = Iterables.<EditableFlag>filter(fieldFlags, EditableFlag.class);
      for(final EditableFlag flag : _filter) {
        _builder.append("disabled=\"#{not ");
        _builder.append(pageBean, "");
        _builder.append(".");
        EqualityExpr _expression = flag.getExpression();
        CharSequence _expressionLanguage = this.toExpressionLanguage(_expression);
        _builder.append(_expressionLanguage, "");
        _builder.append("}\" ");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toExpressionLanguage(final EqualityExpr expr) {
    StringConcatenation _builder = new StringConcatenation();
    CondORExpr _left = expr.getLeft();
    _builder.append(_left, "");
    _builder.newLineIfNotEmpty();
    EqualityOp _op = expr.getOp();
    _builder.append(_op, "");
    _builder.newLineIfNotEmpty();
    CondORExpr _right = expr.getRight();
    _builder.append(_right, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final Content content) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = content.eContents();
      for(final EObject item : _eContents) {
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final SetOfGuiElements s) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<ComposedElement> _elements = s.getElements();
      for(final ComposedElement item : _elements) {
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final ReferencedXMAComposite composite) {
    CharSequence _createChildContents = this.createChildContents(composite);
    return _createChildContents;
  }
  
  protected CharSequence _createContents(final XmadslComposite composite) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _and = false;
      boolean _isHorizontalPartitioning = composite.isHorizontalPartitioning();
      boolean _not = (!_isHorizontalPartitioning);
      if (!_not) {
        _and = false;
      } else {
        boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
        boolean _not_1 = (!_isVerticalPartitioning);
        _and = (_not && _not_1);
      }
      if (_and) {
        {
          CharSequence _compositeLabel = this.getCompositeLabel(composite);
          boolean _notEquals = (!Objects.equal(_compositeLabel, null));
          if (_notEquals) {
            _builder.append("<p:panel id=\"pg");
            String _name = composite.getName();
            CharSequence _asIdentifier = this.jsfNames.asIdentifier(_name);
            _builder.append(_asIdentifier, "");
            _builder.append("\">");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("<f:facet name=\"header\">");
            CharSequence _compositeLabel_1 = this.getCompositeLabel(composite);
            _builder.append(_compositeLabel_1, "	");
            _builder.append("</f:facet>");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            CharSequence _createChildContents = this.createChildContents(composite);
            _builder.append(_createChildContents, "	");
            _builder.newLineIfNotEmpty();
            _builder.append("</p:panel>");
            _builder.newLine();
          } else {
            _builder.append("<h:panelGroup id=\"pg");
            String _name_1 = composite.getName();
            CharSequence _asIdentifier_1 = this.jsfNames.asIdentifier(_name_1);
            _builder.append(_asIdentifier_1, "");
            _builder.append("\">");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            CharSequence _createChildContents_1 = this.createChildContents(composite);
            _builder.append(_createChildContents_1, "	");
            _builder.newLineIfNotEmpty();
            _builder.append("</h:panelGroup>\t\t\t");
            _builder.newLine();
          }
        }
      } else {
        boolean _isHorizontalPartitioning_1 = composite.isHorizontalPartitioning();
        if (_isHorizontalPartitioning_1) {
          _builder.append("<p:panelGrid columns=\"");
          Integer _columns = this.getColumns(composite);
          _builder.append(_columns, "");
          _builder.append("\" id=\"pg");
          String _name_2 = composite.getName();
          CharSequence _asIdentifier_2 = this.jsfNames.asIdentifier(_name_2);
          _builder.append(_asIdentifier_2, "");
          _builder.append("\">");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          {
            CharSequence _compositeLabel_2 = this.getCompositeLabel(composite);
            boolean _notEquals_1 = (!Objects.equal(_compositeLabel_2, null));
            if (_notEquals_1) {
              _builder.append("<f:facet name=\"header\">");
              CharSequence _compositeLabel_3 = this.getCompositeLabel(composite);
              _builder.append(_compositeLabel_3, "	");
              _builder.append("</f:facet>");
            }
          }
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          CharSequence _createChildContents_2 = this.createChildContents(composite);
          _builder.append(_createChildContents_2, "	");
          _builder.append(" ");
          _builder.newLineIfNotEmpty();
          _builder.append("</p:panelGrid>");
          _builder.newLine();
        } else {
          _builder.append("<p:panelGrid id=\"pg");
          String _name_3 = composite.getName();
          CharSequence _asIdentifier_3 = this.jsfNames.asIdentifier(_name_3);
          _builder.append(_asIdentifier_3, "");
          _builder.append("\">");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          {
            CharSequence _compositeLabel_4 = this.getCompositeLabel(composite);
            boolean _notEquals_2 = (!Objects.equal(_compositeLabel_4, null));
            if (_notEquals_2) {
              _builder.append("<f:facet name=\"header\"><p:row><p:column colspan=\"");
              Integer _columns_1 = this.getColumns(composite);
              _builder.append(_columns_1, "	");
              _builder.append("\">");
              CharSequence _compositeLabel_5 = this.getCompositeLabel(composite);
              _builder.append(_compositeLabel_5, "	");
              _builder.append("</p:column></p:row></f:facet>");
            }
          }
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          CharSequence _createChildContents_3 = this.createChildContents(composite);
          _builder.append(_createChildContents_3, "	");
          _builder.append(" ");
          _builder.newLineIfNotEmpty();
          _builder.append("</p:panelGrid>\t\t\t");
          _builder.newLine();
        }
      }
    }
    return _builder;
  }
  
  /**
   * This methods returns the number of columns a PanelGrid should have.
   * 
   * In case of vertical panel, PanelGrid should have 3 columns if there is a customizeable field and 1 if there is no customizeable field
   * In case of horizontal panel, PanelGrid should have 3 columns for each customizeable field plus one column of every other field
   */
  public Integer getColumns(final XmadslComposite composite) {
    int columns = 1;
    Integer numberOfCustomizableFields = this.numberOfCustomizableFields(composite);
    boolean _isHorizontalPartitioning = composite.isHorizontalPartitioning();
    if (_isHorizontalPartitioning) {
      boolean _and = false;
      Content _content = composite.getContent();
      boolean _notEquals = (!Objects.equal(_content, null));
      if (!_notEquals) {
        _and = false;
      } else {
        Content _content_1 = composite.getContent();
        EList<EObject> _eContents = _content_1.eContents();
        boolean _notEquals_1 = (!Objects.equal(_eContents, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        Content _content_2 = composite.getContent();
        EList<EObject> _eContents_1 = _content_2.eContents();
        final int totalComponents = _eContents_1.size();
        int _multiply = ((numberOfCustomizableFields).intValue() * 2);
        int _plus = (totalComponents + _multiply);
        columns = _plus;
      } else {
        columns = 0;
      }
    } else {
      boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
      if (_isVerticalPartitioning) {
        boolean _greaterThan = ((numberOfCustomizableFields).intValue() > 0);
        if (_greaterThan) {
          columns = 3;
        }
      }
    }
    return Integer.valueOf(columns);
  }
  
  /**
   * This method returns the number of customizeable fields in the composite.
   */
  public Integer numberOfCustomizableFields(final XmadslComposite composite) {
    int _xifexpression = (int) 0;
    boolean _and = false;
    boolean _and_1 = false;
    boolean _notEquals = (!Objects.equal(composite, null));
    if (!_notEquals) {
      _and_1 = false;
    } else {
      Content _content = composite.getContent();
      boolean _notEquals_1 = (!Objects.equal(_content, null));
      _and_1 = (_notEquals && _notEquals_1);
    }
    if (!_and_1) {
      _and = false;
    } else {
      Content _content_1 = composite.getContent();
      EList<EObject> _eContents = _content_1.eContents();
      boolean _notEquals_2 = (!Objects.equal(_eContents, null));
      _and = (_and_1 && _notEquals_2);
    }
    if (_and) {
      Content _content_2 = composite.getContent();
      EList<EObject> _eContents_1 = _content_2.eContents();
      final Function1<EObject,Boolean> _function = new Function1<EObject,Boolean>() {
          public Boolean apply(final EObject item) {
            boolean _xifexpression = false;
            EList<EObject> _eContents = item.eContents();
            boolean _notEquals = (!Objects.equal(_eContents, null));
            if (_notEquals) {
              EList<EObject> _eContents_1 = item.eContents();
              Iterable<CustomizeableField> _filter = Iterables.<CustomizeableField>filter(_eContents_1, CustomizeableField.class);
              int _size = IterableExtensions.size(_filter);
              boolean _greaterThan = (_size > 0);
              _xifexpression = _greaterThan;
            } else {
              _xifexpression = false;
            }
            return Boolean.valueOf(_xifexpression);
          }
        };
      Iterable<EObject> _filter = IterableExtensions.<EObject>filter(_eContents_1, _function);
      int _size = IterableExtensions.size(_filter);
      _xifexpression = _size;
    } else {
      _xifexpression = 0;
    }
    return Integer.valueOf(_xifexpression);
  }
  
  protected CharSequence _createContents(final ComposeData data) {
    return null;
  }
  
  protected CharSequence _createContents(final EObject eObject) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<!-- !!! No implementation for type \'");
    EClass _eClass = eObject.eClass();
    String _name = _eClass.getName();
    _builder.append(_name, "");
    _builder.append("\' !!! -->");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence beforeChildContents(final boolean vertical, final int tabulators, final CharSequence label) {
    return null;
  }
  
  public CharSequence afterChildContents(final boolean vertical, final int tabulators) {
    return null;
  }
  
  public CharSequence beforeChildItem(final boolean vertical, final boolean isCustomizeableField, final int numberOfColumns) {
    CharSequence _xifexpression = null;
    if (vertical) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:row>");
      _builder.newLine();
      {
        boolean _not = (!isCustomizeableField);
        if (_not) {
          _builder.append("<p:column colspan=\"");
          _builder.append(numberOfColumns, "");
          _builder.append("\">");
          _builder.newLineIfNotEmpty();
        }
      }
      _xifexpression = _builder;
    } else {
      StringConcatenation _builder_1 = new StringConcatenation();
      _xifexpression = _builder_1;
    }
    return _xifexpression;
  }
  
  public CharSequence afterChildItem(final boolean vertical, final boolean isCustomizeableField, final int numberOfColumns) {
    CharSequence _xifexpression = null;
    if (vertical) {
      StringConcatenation _builder = new StringConcatenation();
      {
        boolean _not = (!isCustomizeableField);
        if (_not) {
          _builder.append("</p:column>");
          _builder.newLine();
        }
      }
      _builder.append("</p:row>");
      _builder.newLine();
      _builder.newLine();
      _xifexpression = _builder;
    } else {
      StringConcatenation _builder_1 = new StringConcatenation();
      _xifexpression = _builder_1;
    }
    return _xifexpression;
  }
  
  protected CharSequence _getCompositeLabel(final ReferencedXMAComposite composite) {
    return null;
  }
  
  protected CharSequence _getCompositeLabel(final XmadslComposite composite) {
    LabelText _labelText = composite.getLabelText();
    String _label = _labelText==null?(String)null:_labelText.getLabel();
    return _label;
  }
  
  protected CharSequence _createChildContents(final ReferencedXMAComposite composite) {
    CharSequence _xblockexpression = null;
    {
      Content _content = composite.getContent();
      boolean _equals = Objects.equal(_content, null);
      if (_equals) {
        StringConcatenation _builder = new StringConcatenation();
        return _builder;
      }
      int tabulators = 1;
      boolean _and = false;
      Content _content_1 = composite.getContent();
      ComposeData _composeLayout = _content_1.getComposeLayout();
      boolean _notEquals = (!Objects.equal(_composeLayout, null));
      if (!_notEquals) {
        _and = false;
      } else {
        Content _content_2 = composite.getContent();
        ComposeData _composeLayout_1 = _content_2.getComposeLayout();
        EList<TabulatorPosition> _tabulators = _composeLayout_1.getTabulators();
        boolean _notEquals_1 = (!Objects.equal(_tabulators, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        Content _content_3 = composite.getContent();
        ComposeData _composeLayout_2 = _content_3.getComposeLayout();
        EList<TabulatorPosition> _tabulators_1 = _composeLayout_2.getTabulators();
        int _size = _tabulators_1.size();
        tabulators = _size;
      }
      StringConcatenation _builder_1 = new StringConcatenation();
      boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
      CharSequence _beforeChildContents = this.beforeChildContents(_isVerticalPartitioning, tabulators, null);
      _builder_1.append(_beforeChildContents, "");
      _builder_1.newLineIfNotEmpty();
      {
        Content _content_4 = composite.getContent();
        EList<EObject> _eContents = _content_4.eContents();
        for(final EObject item : _eContents) {
          boolean _isVerticalPartitioning_1 = composite.isVerticalPartitioning();
          CharSequence _beforeChildItem = this.beforeChildItem(_isVerticalPartitioning_1, false, tabulators);
          _builder_1.append(_beforeChildItem, "");
          _builder_1.newLineIfNotEmpty();
          CharSequence _createContents = this.createContents(item);
          _builder_1.append(_createContents, "");
          _builder_1.newLineIfNotEmpty();
          boolean _isVerticalPartitioning_2 = composite.isVerticalPartitioning();
          CharSequence _afterChildItem = this.afterChildItem(_isVerticalPartitioning_2, false, tabulators);
          _builder_1.append(_afterChildItem, "");
          _builder_1.newLineIfNotEmpty();
        }
      }
      boolean _isVerticalPartitioning_3 = composite.isVerticalPartitioning();
      CharSequence _afterChildContents = this.afterChildContents(_isVerticalPartitioning_3, tabulators);
      _builder_1.append(_afterChildContents, "");
      _builder_1.newLineIfNotEmpty();
      _xblockexpression = (_builder_1);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createChildContents(final XmadslComposite composite) {
    CharSequence _xblockexpression = null;
    {
      Content _content = composite.getContent();
      boolean _equals = Objects.equal(_content, null);
      if (_equals) {
        StringConcatenation _builder = new StringConcatenation();
        return _builder;
      }
      final Integer numberOfColumns = this.getColumns(composite);
      int tabulators = 1;
      boolean _and = false;
      Content _content_1 = composite.getContent();
      ComposeData _composeLayout = _content_1.getComposeLayout();
      boolean _notEquals = (!Objects.equal(_composeLayout, null));
      if (!_notEquals) {
        _and = false;
      } else {
        Content _content_2 = composite.getContent();
        ComposeData _composeLayout_1 = _content_2.getComposeLayout();
        EList<TabulatorPosition> _tabulators = _composeLayout_1.getTabulators();
        boolean _notEquals_1 = (!Objects.equal(_tabulators, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        Content _content_3 = composite.getContent();
        ComposeData _composeLayout_2 = _content_3.getComposeLayout();
        EList<TabulatorPosition> _tabulators_1 = _composeLayout_2.getTabulators();
        int _size = _tabulators_1.size();
        tabulators = _size;
      }
      StringConcatenation _builder_1 = new StringConcatenation();
      boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
      CharSequence _compositeLabel = this.getCompositeLabel(composite);
      CharSequence _beforeChildContents = this.beforeChildContents(_isVerticalPartitioning, tabulators, _compositeLabel);
      _builder_1.append(_beforeChildContents, "");
      _builder_1.newLineIfNotEmpty();
      {
        Content _content_4 = composite.getContent();
        EList<EObject> _eContents = _content_4.eContents();
        for(final EObject item : _eContents) {
          final boolean isCustomizeableField = this.isCustomizeableField(item);
          _builder_1.newLineIfNotEmpty();
          boolean _isVerticalPartitioning_1 = composite.isVerticalPartitioning();
          CharSequence _beforeChildItem = this.beforeChildItem(_isVerticalPartitioning_1, isCustomizeableField, (numberOfColumns).intValue());
          _builder_1.append(_beforeChildItem, "");
          _builder_1.newLineIfNotEmpty();
          CharSequence _createContents = this.createContents(item);
          _builder_1.append(_createContents, "");
          _builder_1.newLineIfNotEmpty();
          boolean _isVerticalPartitioning_2 = composite.isVerticalPartitioning();
          CharSequence _afterChildItem = this.afterChildItem(_isVerticalPartitioning_2, isCustomizeableField, (numberOfColumns).intValue());
          _builder_1.append(_afterChildItem, "");
          _builder_1.newLineIfNotEmpty();
        }
      }
      boolean _isVerticalPartitioning_3 = composite.isVerticalPartitioning();
      CharSequence _afterChildContents = this.afterChildContents(_isVerticalPartitioning_3, tabulators);
      _builder_1.append(_afterChildContents, "");
      _builder_1.newLineIfNotEmpty();
      _xblockexpression = (_builder_1);
    }
    return _xblockexpression;
  }
  
  /**
   * Checks whether the item is a customizeable field
   */
  public boolean isCustomizeableField(final EObject item) {
    EList<EObject> _eContents = item.eContents();
    Iterable<CustomizeableField> _filter = Iterables.<CustomizeableField>filter(_eContents, CustomizeableField.class);
    int _size = IterableExtensions.size(_filter);
    boolean _greaterThan = (_size > 0);
    return _greaterThan;
  }
  
  public boolean hasEventMappings(final IGuiElementWithEvent control, final boolean client) {
    boolean _xblockexpression = false;
    {
      XmadslPage _parentPage = this.jsfNames.getParentPage(control);
      EventMappingList _events = _parentPage.getEvents();
      EList<EventMapping> _mapping = _events.getMapping();
      Iterable<GuiElementEventMapping> _filter = Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
      for (final GuiElementEventMapping mapping : _filter) {
        EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
        for (final EventFunction function : _eventFunctions) {
          IGuiElementWithEvent _control = mapping.getControl();
          boolean _equals = Objects.equal(control, _control);
          if (_equals) {
            boolean _and = false;
            Command _command = function.getCommand();
            boolean _notEquals = (!Objects.equal(_command, null));
            if (!_notEquals) {
              _and = false;
            } else {
              Command _command_1 = function.getCommand();
              boolean _isClientOnly = _command_1.isClientOnly();
              boolean _equals_1 = (_isClientOnly == client);
              _and = (_notEquals && _equals_1);
            }
            if (_and) {
              return true;
            }
            boolean _and_1 = false;
            boolean _and_2 = false;
            PredefinedCommand _predefinedCommand = function.getPredefinedCommand();
            boolean _notEquals_1 = (!Objects.equal(_predefinedCommand, null));
            if (!_notEquals_1) {
              _and_2 = false;
            } else {
              Command _command_2 = function.getCommand();
              boolean _equals_2 = Objects.equal(_command_2, null);
              _and_2 = (_notEquals_1 && _equals_2);
            }
            if (!_and_2) {
              _and_1 = false;
            } else {
              _and_1 = (_and_2 && client);
            }
            if (_and_1) {
              return true;
            }
          }
        }
      }
      _xblockexpression = (false);
    }
    return _xblockexpression;
  }
  
  public CharSequence expandCommandMappings(final IGuiElementWithEvent control) {
    CharSequence _xblockexpression = null;
    {
      ArrayList<String> _arrayList = new ArrayList<String>();
      final ArrayList<String> calls = _arrayList;
      XmadslPage _parentPage = this.jsfNames.getParentPage(control);
      EventMappingList _events = _parentPage.getEvents();
      EList<EventMapping> _mapping = _events.getMapping();
      Iterable<GuiElementEventMapping> _filter = Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
      for (final GuiElementEventMapping mapping : _filter) {
        EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
        for (final EventFunction function : _eventFunctions) {
          IGuiElementWithEvent _control = mapping.getControl();
          boolean _equals = Objects.equal(control, _control);
          if (_equals) {
            Command _command = function.getCommand();
            boolean _notEquals = (!Objects.equal(_command, null));
            if (_notEquals) {
              StringConcatenation _builder = new StringConcatenation();
              _builder.append("App.");
              Command _command_1 = function.getCommand();
              EObject _eContainer = _command_1.eContainer();
              CharSequence _widgetVar = this.jsfNames.getWidgetVar(_eContainer);
              _builder.append(_widgetVar, "");
              _builder.append(".");
              Command _command_2 = function.getCommand();
              String _name = _command_2.getName();
              _builder.append(_name, "");
              _builder.append("()");
              calls.add(_builder.toString());
            } else {
              PredefinedCommand _predefinedCommand = function.getPredefinedCommand();
              boolean _notEquals_1 = (!Objects.equal(_predefinedCommand, null));
              if (_notEquals_1) {
                StringConcatenation _builder_1 = new StringConcatenation();
                _builder_1.append("App.");
                XmadslPage _parentPage_1 = this.jsfNames.getParentPage(control);
                CharSequence _widgetVar_1 = this.jsfNames.getWidgetVar(_parentPage_1);
                _builder_1.append(_widgetVar_1, "");
                _builder_1.append(".");
                PredefinedCommand _predefinedCommand_1 = function.getPredefinedCommand();
                String _literal = _predefinedCommand_1.getLiteral();
                _builder_1.append(_literal, "");
                _builder_1.append("()");
                calls.add(_builder_1.toString());
              }
            }
          }
        }
      }
      StringConcatenation _builder_2 = new StringConcatenation();
      String _join = IterableExtensions.join(calls, ";");
      _builder_2.append(_join, "");
      _builder_2.append(";return false");
      _xblockexpression = (_builder_2);
    }
    return _xblockexpression;
  }
  
  public CharSequence columnValueExpression(final TableColumn column) {
    CharSequence _xifexpression = null;
    FieldFeature _feature = column.getFeature();
    boolean _notEquals = (!Objects.equal(_feature, null));
    if (_notEquals) {
      CharSequence _xifexpression_1 = null;
      boolean _and = false;
      FieldFeature _feature_1 = column.getFeature();
      PresentableFeature _feature_2 = _feature_1.getFeature();
      boolean _notEquals_1 = (!Objects.equal(_feature_2, null));
      if (!_notEquals_1) {
        _and = false;
      } else {
        FieldFeature _feature_3 = column.getFeature();
        PresentableFeature _feature_4 = _feature_3.getFeature();
        String _name = _feature_4.getName();
        boolean _notEquals_2 = (!Objects.equal(_name, null));
        _and = (_notEquals_1 && _notEquals_2);
      }
      if (_and) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("item.");
        FieldFeature _feature_5 = column.getFeature();
        PresentableFeature _feature_6 = _feature_5.getFeature();
        String _name_1 = _feature_6.getName();
        _builder.append(_name_1, "");
        _xifexpression_1 = _builder;
      } else {
        CharSequence _xifexpression_2 = null;
        boolean _and_1 = false;
        boolean _and_2 = false;
        FieldFeature _feature_7 = column.getFeature();
        PresentableFeature _feature_8 = _feature_7.getFeature();
        boolean _notEquals_3 = (!Objects.equal(_feature_8, null));
        if (!_notEquals_3) {
          _and_2 = false;
        } else {
          FieldFeature _feature_9 = column.getFeature();
          PresentableFeature _feature_10 = _feature_9.getFeature();
          Attribute _attribute = _feature_10.getAttribute();
          boolean _notEquals_4 = (!Objects.equal(_attribute, null));
          _and_2 = (_notEquals_3 && _notEquals_4);
        }
        if (!_and_2) {
          _and_1 = false;
        } else {
          FieldFeature _feature_11 = column.getFeature();
          PresentableFeature _feature_12 = _feature_11.getFeature();
          Attribute _attribute_1 = _feature_12.getAttribute();
          String _name_2 = _attribute_1.getName();
          boolean _notEquals_5 = (!Objects.equal(_name_2, null));
          _and_1 = (_and_2 && _notEquals_5);
        }
        if (_and_1) {
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("item.");
          FieldFeature _feature_13 = column.getFeature();
          PresentableFeature _feature_14 = _feature_13.getFeature();
          Attribute _attribute_2 = _feature_14.getAttribute();
          String _name_3 = _attribute_2.getName();
          _builder_1.append(_name_3, "");
          _xifexpression_2 = _builder_1;
        }
        _xifexpression_1 = _xifexpression_2;
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  public CharSequence fieldValueExpression(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _and = false;
      FieldFeature _feature = field.getFeature();
      boolean _notEquals = (!Objects.equal(_feature, null));
      if (!_notEquals) {
        _and = false;
      } else {
        FieldFeature _feature_1 = field.getFeature();
        PresentableFeature _attributeHolder = _feature_1.getAttributeHolder();
        boolean _notEquals_1 = (!Objects.equal(_attributeHolder, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        DataObjectVariable _object = field.getObject();
        EObject _eContainer = _object.eContainer();
        CharSequence _referencedBean = this.jsfNames.referencedBean(_eContainer);
        _builder.append(_referencedBean, "");
        _builder.append(".");
        DataObjectVariable _object_1 = field.getObject();
        String _name = _object_1.getName();
        String _faceletName = this.jsfNames.getFaceletName(_name);
        _builder.append(_faceletName, "");
        _builder.append(".");
        FieldFeature _feature_2 = field.getFeature();
        PresentableFeature _attributeHolder_1 = _feature_2.getAttributeHolder();
        String _name_1 = _attributeHolder_1.getName();
        String _faceletName_1 = this.jsfNames.getFaceletName(_name_1);
        _builder.append(_faceletName_1, "");
      }
    }
    return _builder;
  }
  
  public CharSequence toUiCommandPrototypes(final EList<Command> commands) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final Command command : commands) {
        String _name = command.getName();
        _builder.append(_name, "");
        _builder.append(" : function() {");
        _builder.newLineIfNotEmpty();
        {
          boolean _isClientOnly = command.isClientOnly();
          if (_isClientOnly) {
            {
              boolean _and = false;
              CommandImplementation _implementation = command.getImplementation();
              boolean _notEquals = (!Objects.equal(_implementation, null));
              if (!_notEquals) {
                _and = false;
              } else {
                CommandImplementation _implementation_1 = command.getImplementation();
                EList<Statement> _statements = _implementation_1.getStatements();
                boolean _isEmpty = _statements==null?false:_statements.isEmpty();
                boolean _not = (!_isEmpty);
                _and = (_notEquals && _not);
              }
              if (_and) {
                _builder.append("\t\t\t");
                CommandImplementation _implementation_2 = command.getImplementation();
                EList<Statement> _statements_1 = _implementation_2.getStatements();
                final Procedure1<Statement> _function = new Procedure1<Statement>() {
                    public void apply(final Statement statement) {
                      FaceletImpl.this.createContents(statement);
                    }
                  };
                IterableExtensions.<Statement>forEach(_statements_1, _function);
                _builder.newLineIfNotEmpty();
              } else {
                _builder.append("\t\t\t");
                _builder.append("alert(\'uicommand \"");
                String _name_1 = command.getName();
                _builder.append(_name_1, "			");
                _builder.append("\" is not implemented. Please define it in custom .xhtml page \');");
                _builder.newLineIfNotEmpty();
              }
            }
          } else {
            CharSequence _widgetVar = this.jsfNames.getWidgetVar(command);
            _builder.append(_widgetVar, "");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("},");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final Statement stmt) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Other statement:");
    _builder.append(stmt, "");
    return _builder;
  }
  
  protected CharSequence _createContents(final InvokeStatement stmt) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// InvokeStatement");
    _builder.newLine();
    {
      EList<PropertyJoin> _propertyJoin = stmt.getPropertyJoin();
      for(final PropertyJoin propertyJoin : _propertyJoin) {
        _builder.append("\t\t");
        _builder.append("//TODO: ");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("App.");
    Invokeable _invokeAble = stmt.getInvokeAble();
    CharSequence _widgetVar = this.jsfNames.getWidgetVar(_invokeAble);
    _builder.append(_widgetVar, "		");
    _builder.append(".invoke();");
    return _builder;
  }
  
  /**
   * Remove method after introducing i18n internationlization (texts are outsourced to language files!)
   */
  public CharSequence toEncodedHtml(final String seq) {
    return seq.replaceAll("&", "&amp;");
  }
  
  public CharSequence createContents(final EObject composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _createContents((ReferencedXMAComposite)composite);
    } else if (composite instanceof TabFolder) {
      return _createContents((TabFolder)composite);
    } else if (composite instanceof TabPage) {
      return _createContents((TabPage)composite);
    } else if (composite instanceof XmadslComposite) {
      return _createContents((XmadslComposite)composite);
    } else if (composite instanceof Button) {
      return _createContents((Button)composite);
    } else if (composite instanceof Combo) {
      return _createContents((Combo)composite);
    } else if (composite instanceof CustomizeableField) {
      return _createContents((CustomizeableField)composite);
    } else if (composite instanceof Label) {
      return _createContents((Label)composite);
    } else if (composite instanceof Seperator) {
      return _createContents((Seperator)composite);
    } else if (composite instanceof Table) {
      return _createContents((Table)composite);
    } else if (composite instanceof TableCombo) {
      return _createContents((TableCombo)composite);
    } else if (composite instanceof Text) {
      return _createContents((Text)composite);
    } else if (composite instanceof Tree) {
      return _createContents((Tree)composite);
    } else if (composite instanceof XmadslPage) {
      return _createContents((XmadslPage)composite);
    } else if (composite instanceof FieldReference) {
      return _createContents((FieldReference)composite);
    } else if (composite instanceof InvokeStatement) {
      return _createContents((InvokeStatement)composite);
    } else if (composite instanceof ComposeData) {
      return _createContents((ComposeData)composite);
    } else if (composite instanceof Content) {
      return _createContents((Content)composite);
    } else if (composite instanceof LabelText) {
      return _createContents((LabelText)composite);
    } else if (composite instanceof SetOfGuiElements) {
      return _createContents((SetOfGuiElements)composite);
    } else if (composite instanceof Statement) {
      return _createContents((Statement)composite);
    } else if (composite instanceof TableColumn) {
      return _createContents((TableColumn)composite);
    } else if (composite != null) {
      return _createContents(composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
  
  public CharSequence createEventMapping(final IGuiElementWithEvent table, final GuiElementEventMapping mapping) {
    if (table instanceof Table) {
      return _createEventMapping((Table)table, mapping);
    } else if (table != null) {
      return _createEventMapping(table, mapping);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(table, mapping).toString());
    }
  }
  
  public CharSequence getExpressionLanguage(final EObject it, final CharSequence defaultVal) {
    if (it instanceof FalseLiteral) {
      return _getExpressionLanguage((FalseLiteral)it, defaultVal);
    } else if (it instanceof TrueLiteral) {
      return _getExpressionLanguage((TrueLiteral)it, defaultVal);
    } else if (it instanceof EqualityExpr) {
      return _getExpressionLanguage((EqualityExpr)it, defaultVal);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it, defaultVal).toString());
    }
  }
  
  public CharSequence getCompositeLabel(final IComposite composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _getCompositeLabel((ReferencedXMAComposite)composite);
    } else if (composite instanceof XmadslComposite) {
      return _getCompositeLabel((XmadslComposite)composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
  
  public CharSequence createChildContents(final IComposite composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _createChildContents((ReferencedXMAComposite)composite);
    } else if (composite instanceof XmadslComposite) {
      return _createChildContents((XmadslComposite)composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
}
