package org.openxma.dsl.generator.impl

import com.google.inject.Singleton
import javax.inject.Inject
import org.eclipse.emf.common.util.EList
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.core.model.EnumType
import org.openxma.dsl.core.model.EqualityExpr
import org.openxma.dsl.core.model.IntegerParameterValue
import org.openxma.dsl.core.model.ParameterValue
import org.openxma.dsl.core.model.TrueLiteral
import org.openxma.dsl.core.model.ValidatorReference
import org.openxma.dsl.pom.model.Button
import org.openxma.dsl.pom.model.Combo
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.ComposeData
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.EditableFlag
import org.openxma.dsl.pom.model.EventType
import org.openxma.dsl.pom.model.FieldFlag
import org.openxma.dsl.pom.model.FieldReference
import org.openxma.dsl.pom.model.GuiElementEventMapping
import org.openxma.dsl.pom.model.IGuiElementWithEvent
import org.openxma.dsl.pom.model.Label
import org.openxma.dsl.pom.model.LabelText
import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.Seperator
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.TabFolder
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.TableColumn
import org.openxma.dsl.pom.model.TableCombo
import org.openxma.dsl.pom.model.Text
import org.openxma.dsl.pom.model.Tree
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.XmadslPage
import org.openxma.dsl.pom.model.TabulatorPosition
import org.openxma.dsl.pom.model.Command
import org.openxma.dsl.pom.model.ControlEventMapping
import org.openxma.dsl.pom.model.ObjectProperty
import org.openxma.dsl.pom.model.InvokeStatement
import org.openxma.dsl.pom.model.Statement
import java.util.ArrayList
import org.openxma.dsl.pom.model.Invokeable
import org.openxma.dsl.pom.model.FieldPart
import org.openxma.dsl.pom.model.FieldPartSpecification
import org.openxma.dsl.pom.model.EnabledFlag
import org.openxma.dsl.dom.model.ReadOnlyFlag
import org.openxma.dsl.dom.model.Expression
import org.openxma.dsl.core.model.FalseLiteral
import java.util.List
import org.openxma.dsl.pom.model.LayoutDataProperty
import org.eclipse.emf.ecore.util.EcoreUtil
import org.openxma.dsl.pom.model.HeightProperty

@Singleton
class FaceletImpl implements Facelet {
	
	@Inject
	extension JsfData data
	@Inject
	extension Files files
	@Inject
	extension JsfNames jsfNames
	
	def dispatch CharSequence createContents(XmadslPage page) {	'''
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html
        PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
	xmlns:f="http://java.sun.com/jsf/core"
	xmlns:h="http://java.sun.com/jsf/html"
	xmlns:ui="http://java.sun.com/jsf/facelets"
	xmlns:p="http://primefaces.org/ui"
	xmlns:i18n="http://fwv.spardat.at/i18n"
	xmlns:fn="http://java.sun.com/jsp/jstl/functions">
<head>
</head>
<body>
<ui:composition template="${template}">
		<ui:param name="title" value="page.labelText?.label" />
		<ui:param name="dialogId" value="page.name.toFirstLower" />
		<ui:param name="dialogWidgetVar" value="page.dialogWidgetVar" />
		<ui:define name="dialogs">
			FOR invokeablePage : page.invokablePages
			<ui:remove>TODO: Copy the section 'dialogs' to custom facelet. (otherwise not recognized)</ui:remove>
			<ui:include src="/invokeablePage.faceletPageFileName(false)" />
			ENDFOR
		</ui:define>
		<ui:define name="genscripts">
			<script type="text/javascript">
				if (!App.page.component.JSClass) {
					App.page.component.JSClass = Xma.Component.extend( {
						page.component.commands?.toUiCommandPrototypes
						getMainPage : function() { return App.page.component.mainPage.widgetVar; }
					});
					App.page.component.widgetVar = new App.page.component.JSClass();
				}
				App.page.component.JSClass.page.JSClass = Xma.Page.extend({
					page.commands?.toUiCommandPrototypes
					getDialogWidget : 	function() { return page.dialogWidgetVar;}
				});
				App.page.widgetVar = new App.page.component.JSClass.page.JSClass();
				
			/*
				page.JSClass = function() {};
				page.JSClass.prototype = new xmadslpage('page.name');
				page.commands?.toUiCommandPrototypes
				var page.widgetVar = new page.JSClass('page.name');
				page.widgetVar.getDialogWidget = function() {return page.dialogWidgetVar;};
				page.component.JSClass = function() {};
				page.component.JSClass.prototype = new xmadslcomponent('page.component.xmaComponent.namClass');
				var page.component.widgetVar = new page.component.JSClass(page.component.xmaComponent.namClass);
				page.component.widgetVar.getMainPage = function() { return page.component.mainPage.widgetVar;};
				*/
			</script>
		</ui:define>
		<ui:define name="center">
			<h:form id="page.nameForm">
				page.content?.createContents
				page.commands?.toRemoteCommands
				page.pageProperties?.toRemoteProperties
				<ui:insert name="additionalWidgets" />
			</h:form>
		</ui:define>
		
		<ui:define name="dialogs">
		</ui:define>

	</ui:composition>
</body>
</html>
'''	
	}
	
	def Iterable<XmadslPage> getInvokablePages(XmadslPage page) {
		val implementations = page.commands.map([implementation]).filterNull
        val statements = implementations.map([statements.filter(typeof(InvokeStatement))]).flatten
        val invokeables = statements.map([invokeAble])
		
		invokeables.filter(typeof(XmadslPage))
	}
	
	
	def CharSequence toRemoteProperties(EList<ObjectProperty> properties) { '''
	    FOR property : properties
	<p:inputText value="#{property.parentPage.bean.property.name}" style="display:none;" widgetVar="property.widgetVar" />
    	ENDFOR
	'''
	}
	def CharSequence toRemoteCommands(EList<Command> commands) { '''
	    FOR command : commands
			IF !command.clientOnly
	<p:remoteCommand action="#{command.parentPage.bean.command.name}" update="@form" name="command.widgetVar" />
    		ENDIF
    	ENDFOR
	'''
	}
	
	
	def dispatch CharSequence createContents(Table table) { '''
		    <p:dataTable id="table.name"
		    IF table.multiselectionFlag?.multiSelection instanceof TrueLiteral || table.checkSelection instanceof TrueLiteral 
		     	selectionMode="multiple" selection="#{table.modelProperty.selectedItems}"
		     	ELSE
		     	selectionMode="single" selection="#{table.modelProperty.selectedItem}"
		     	ENDIF
		     	widgetVar="table.widgetVar"
		    value="#{table.modelProperty}" var="item"
		    rowKey="#{item.oid}"
			    IF table.pageable instanceof TrueLiteral
			    paginator="true" rows="10" paginatorPosition="top" paginatorTemplate="{CurrentPageReport}  {FirstPageLink} {PreviousPageLink} {PageLinks} {NextPageLink} {LastPageLink}"
			    ENDIF
			    resizableColumns="true" >
			    table.createEventMappingsForControl
		    FOR column : table.columns
			    column.createContents
		    ENDFOR
		    </p:dataTable>
'''	
	}
	
	def dispatch CharSequence createContents(Tree tree) { '''
		    <p:tree id="tree.name" 
		     	selectionMode="single" widgetVar="tree.nameWidgetVar"
			    value="#{tree.rootNodeProperty}" var="node"
			    selection="#{tree.selectedNodeProperty}">
			    tree.createEventMappingsForControl
			    <p:treeNode>
			    	<h:outputText value="#{node}" />
			    </p:treeNode>
		    </p:tree>
'''	
	}
	
	def CharSequence getModelProperty(Table table) {
		'''table.parentPage.bean.table.modelPropertyName'''
	}
	def CharSequence getRootNodeProperty(Tree tree) {
		'''tree.parentPage.bean.tree.rootNodePropertyName'''
	}
	def CharSequence getSelectedNodeProperty(Tree tree) {
		'''tree.parentPage.bean.tree.selectedNodePropertyName'''
	}
	def CharSequence getSelectItemsProperty(CustomizeableField field) {
		'''field.parentPage.bean.field.selectItemsPropertyName'''
	}
	def dispatch CharSequence createEventMapping(IGuiElementWithEvent control, GuiElementEventMapping mapping) { '''
		<!-- error not control event mapping for control type control.getClass().name -->
	'''
	}
	def dispatch CharSequence createEventMapping(Table table, GuiElementEventMapping mapping) {
		switch mapping.event {
			case EventType::ON_SELECTION : '''<p:ajax event="rowSelect" listener="#{table.parentPage.bean.table.name_onSelection}" update="@form" />'''
			default : ''''''
		}
	}
	def CharSequence createEventMappingsForControl(IGuiElementWithEvent control) { '''
		FOR mapping : control.eventMappingsForControl
			control.createEventMapping(mapping)
		ENDFOR	
	'''
	}
	def Iterable<GuiElementEventMapping> getEventMappingsForControl(IGuiElementWithEvent control) {
		control.parentPage.events.mapping.filter(typeof(GuiElementEventMapping)).filter[m | m.control == control]
	}
	def dispatch CharSequence createContents(TableCombo table) {
		''' <p:outputLabel>not supported: TableCombo</p:outputLabel>''' 
	/*'''
		    <p:selectOneMenu id="table.name"> 
					<f:selectItem itemLabel="Select value" itemValue="" />
				    <f:selectItems value="#{table.parentPage.bean.table.nameTableModel}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
		    FOR column : table.columns
			    column.createContents
		    ENDFOR
		    </p:selectOneMenu>
'''	*/
	}
	def ParameterValue getParameter(EList<ParameterValue> list, String name) {
		list.findFirst[p | p.referencedParameter.name == name]
	}
	def int getParameterValue(EList<ParameterValue> list, String name) {
		(list.getParameter(name) as IntegerParameterValue).intValue
	}
	def CharSequence createConverter(ValidatorReference validatorRef) {
		if (validatorRef == null)
			''''''
		else if (validatorRef.dateValidator)
			'''<f:convertDateTime pattern="dd.MM.yyyy" />'''
		else if (validatorRef.numberValidator) {
						
			'''<f:convertNumber minFractionDigits="2" maxFractionDigits="2" 
						maxIntegerDigits="10" />
						/*validatorRef.rootValidatorReference.parameterValues.getParameterValue("precision")*/
						/*validatorRef.rootValidatorReference.parameterValues.getParameterValue("scale")*/
						'''
		}
		else
			''''''
	}
	
	def dispatch CharSequence createContents(TableColumn column) { '''
			    <p:column headerText="column.labelText.toEncodedHtml"
			    	IF "styling".equals("styling") && column.attribute != nullstyleClass="column.attribute.name"ENDIF
				    IF (column.feature != null && column.feature.attributeHolder != null)sortBy="#{column.columnValueExpression}">
				    <h:outputText value="#{column.columnValueExpression}">
				    	column.feature.attributeHolder.attribute.type.validatorReferenceDefinition.createConverter
				    </h:outputText>
				    ELSE
				    >
				    <h:outputText value="hardcodedtext" />
				    ENDIF
			    </p:column>
''' 
	}

	
	def dispatch CharSequence createContents(Text text) { '''
		    <p:inputText id="text.name" />
''' 
	}
	
	
	def dispatch CharSequence createContents(TabFolder tabFolder) { '''
	    <p:tabView id="tabFolder.name.asIdentifier">
	    FOR item : tabFolder.eContents
		    item.createContents
	    ENDFOR
	    </p:tabView>
''' 
	}
	
	def dispatch CharSequence createContents(TabPage tabPage) { '''
			<p:tab id="tabPage.name" title="tabPage.labelText.string">
			FOR item : tabPage.eContents
				item.createContents
			ENDFOR
			</p:tab>
''' 
	}
	
	def dispatch CharSequence createContents(Button button) { 
		'''<p:commandButton id="button.name" value="button.labelText.label" onclick="button.expandCommandMappings" type="button" />'''
	}
	
	def dispatch CharSequence createContents(Seperator seperator) { '''
		    <p:separator />
''' 
	}
	
	def dispatch CharSequence createContents(FieldReference field) { '''
		    <h:field attribute="field.attribute.name"/>
''' 
	}
	
	def dispatch CharSequence createContents(Combo combo) { '''
		    <p:selectOneMenu id="combo.name"/>
''' 
	}
	
	def dispatch CharSequence createContents(Label label) {	'''
		    <h:outputLabel value="label.labelText.label.toEncodedHtml" IF label.name != nullid="label.name"ENDIF />
'''	
	}
	
	def dispatch CharSequence createContents(LabelText label) {	'''
		IF label.eContainer instanceof TabPage && (label.eContainer as TabPage).labelText == label
		ELSE
		    <h:outputLabel value="label.label.toEncodedHtml" />
		ENDIF
'''	
	}
	def CharSequence debugFieldPart(FieldPartSpecification part) {
		"lit" + part.part.literal + "pname" + part.part.name
	}
	def dispatch CharSequence createContents(CustomizeableField field) {	'''
		IF field.parts == null || field.parts.size == 0 || field.parts.filter(fps|fps.part == FieldPart::LABEL).size > 0
			<p:column>
				<p:outputLabel value="field.labelText.toEncodedHtml: " for="field.attribute.name1" />
			</p:column>
		ENDIF
		IF field.parts == null || field.parts.size == 0 ||  field.parts.filter(fps|fps.part == FieldPart::CONTROL).size > 0
			<p:column>
			    IF field.attribute.type.dataType instanceof EnumType
			    <p:selectOneMenu id="field.attribute.name1" value="#{field.fieldValueExpression}" field.disabledFlag>
			    	<f:selectItem itemLabel="Select value" itemValue="" />
			        <f:selectItems value="#{field.selectItemsProperty}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
			    </p:selectOneMenu>
			    ELSEIF field.attribute.type.validatorReferenceDefinition.booleanValidator
			    <p:selectBooleanCheckbox id="field.attribute.name1" value="#{field.fieldValueExpression}" field.disabledFlag />
			    ELSEIF field.attribute.type.validatorReferenceDefinition.dateValidator
			    <p:calendar id="field.attribute.name1" value="#{field.fieldValueExpression}" field.disabledFlag />
			    ELSE
			    <p:inputText id="field.attribute.name1" value="#{field.fieldValueExpression}" field.disabledFlag>field.format.createConverter
			    field.feature.attributeHolder.attribute.type.validatorReferenceDefinition.createConverter 
			    </p:inputText>
			    ENDIF
			</p:column>
		ENDIF
		<p:column>
			<p:message for="field.attribute.name1" display="icon" />
		</p:column>	
'''	
	}
	
	def CharSequence disabledFlag(CustomizeableField field) {
		val flag = field.fieldFlags.findFirst(f|f instanceof EnabledFlag)
		if (flag != null && !(flag instanceof TrueLiteral))
			'''disabled="!flag.expression.getExpressionLanguage("false")"'''
	}
	def CharSequence readonlyFlag(CustomizeableField field) {
		val flag = field.fieldFlags.findFirst(f|f instanceof ReadOnlyFlag)
		if (flag != null && !(flag instanceof FalseLiteral))
			'''readonly="flag.expression.getExpressionLanguage("false")"'''
	}
	def dispatch CharSequence getExpressionLanguage(EqualityExpr it, CharSequence defaultVal) {
		defaultVal
	}
	def dispatch CharSequence getExpressionLanguage(FalseLiteral it, CharSequence defaultVal) {
		"false"
	}
	def dispatch CharSequence getExpressionLanguage(TrueLiteral it, CharSequence defaultVal) {
		"true"
	}
	
	/*   	field.fieldFlags.createDisabledFlag(field.pageBean) */
	
	def CharSequence createDisabledFlag(EList<FieldFlag> fieldFlags, CharSequence pageBean) {''' 
		FOR flag : fieldFlags.filter(typeof(EditableFlag))
		disabled="#{not pageBean.flag.expression.toExpressionLanguage}" 
		ENDFOR
'''
	}
	
	def CharSequence toExpressionLanguage(EqualityExpr expr) { '''
	expr.left
	expr.op
	expr.right
'''		
	}
	
	def dispatch CharSequence createContents(Content content) {	'''
		FOR item : content.eContents
			item.createContents
		ENDFOR
'''	
	}
	
	def dispatch CharSequence createContents(SetOfGuiElements s) { ''' 
		FOR item : s.elements
			item.createContents
		ENDFOR
''' 
	} 
	
	def dispatch CharSequence createContents(ReferencedXMAComposite composite) { 
		composite.createChildContents
	}
	
	def dispatch CharSequence createContents(XmadslComposite composite) { '''
		IF !composite.horizontalPartitioning && !composite.verticalPartitioning
			IF composite.compositeLabel != null
			<p:panel id="pgcomposite.name.asIdentifier">
				<f:facet name="header">composite.compositeLabel</f:facet>
				composite.createChildContents
			</p:panel>
			ELSE
			<h:panelGroup id="pgcomposite.name.asIdentifier">
				composite.createChildContents
			</h:panelGroup>			
			ENDIF
		ELSEIF composite.horizontalPartitioning
			<p:panelGrid columns="getColumns(composite)" id="pgcomposite.name.asIdentifier">
				IF composite.compositeLabel != null<f:facet name="header">composite.compositeLabel</f:facet>ENDIF
				composite.createChildContents 
			</p:panelGrid>
		ELSE
			<p:panelGrid id="pgcomposite.name.asIdentifier">
				IF composite.compositeLabel != null<f:facet name="header"><p:row><p:column colspan="getColumns(composite)">composite.compositeLabel</p:column></p:row></f:facet>ENDIF
				composite.createChildContents 
			</p:panelGrid>			
		ENDIF
	'''	
	}	
			
	
	/**
	 * This methods returns the number of columns a PanelGrid should have.
	 * 
	 * In case of vertical panel, PanelGrid should have 3 columns if there is a customizeable field and 1 if there is no customizeable field
	 * In case of horizontal panel, PanelGrid should have 3 columns for each customizeable field plus one column of every other field
	 */
	def Integer getColumns(XmadslComposite composite) {		
		var columns = 1;
		var numberOfCustomizableFields = numberOfCustomizableFields(composite);
		
		if (composite.horizontalPartitioning) {
			if (composite.content != null && composite.content.eContents != null) {
				val totalComponents = composite.content.eContents.size
				columns = totalComponents + numberOfCustomizableFields * 2 //one each for label and message
			} else {
				columns = 0
			}
		} else if (composite.verticalPartitioning) {
			if (numberOfCustomizableFields > 0) {
				columns = 3
			}
		}
		 
		return columns
	}
	
	/**
	 * This method returns the number of customizeable fields in the composite.
	 */
	def Integer numberOfCustomizableFields(XmadslComposite composite) {
		if (composite != null && composite.content != null && composite.content.eContents != null) {
			composite.content.eContents.filter[item | 
				if (item.eContents != null) {
					item.eContents.filter(typeof(CustomizeableField)).size > 0
				} else {
					false
				}
			].size
		} else {
			0
		}
	}
	
	def dispatch CharSequence createContents(ComposeData data) { 
		/* Tabulator (ComposieData) should be handled by parent component --> */
	}
	def dispatch CharSequence createContents(EObject eObject) { '''
		    <!-- !!! No implementation for type 'eObject.eClass.name' !!! -->
	''' 
	}
	
	def CharSequence beforeChildContents(boolean vertical, int tabulators, CharSequence label) {

	}
	
	def CharSequence afterChildContents(boolean vertical, int tabulators) {

	}
	
	def CharSequence beforeChildItem(boolean vertical, boolean isCustomizeableField, int numberOfColumns) {
		if (vertical) {
			'''<p:row>
					IF !isCustomizeableField
				<p:column colspan="numberOfColumns">
					ENDIF
			'''			
		} else {
			''''''
		}
	}
	
	def CharSequence afterChildItem(boolean vertical, boolean isCustomizeableField, int numberOfColumns) {
		if (vertical) {
			'''		
				IF !isCustomizeableField
					</p:column>
				ENDIF
				</p:row>
				
			'''			
		} else {
			''''''
		}
	}
	
	def dispatch CharSequence getCompositeLabel(ReferencedXMAComposite composite) {
		null
	}
	def dispatch CharSequence getCompositeLabel(XmadslComposite composite) {
		composite.labelText?.label
	}
	
	def dispatch CharSequence createChildContents(ReferencedXMAComposite composite) { 
		if (composite.content == null)
			return ''''''
		var tabulators = 1;
		if (composite.content.composeLayout != null && composite.content.composeLayout.tabulators != null )
			tabulators = composite.content.composeLayout.tabulators.size
		'''
		beforeChildContents(composite.verticalPartitioning, tabulators, null)
		FOR item : composite.content.eContents
			beforeChildItem(composite.verticalPartitioning, false, tabulators)
			item.createContents
			afterChildItem(composite.verticalPartitioning, false, tabulators)
		ENDFOR
		afterChildContents(composite.verticalPartitioning, tabulators)
		'''
	}
	def dispatch CharSequence createChildContents(XmadslComposite composite) {
		if (composite.content == null)
			return ''''''
			
		val numberOfColumns = getColumns(composite)
		var tabulators = 1;
		if (composite.content.composeLayout != null && composite.content.composeLayout.tabulators != null )
			tabulators = composite.content.composeLayout.tabulators.size
		'''
		beforeChildContents(composite.verticalPartitioning, tabulators, composite.compositeLabel)
		FOR item : composite.content.eContents
			val isCustomizeableField = isCustomizeableField(item)
			beforeChildItem(composite.verticalPartitioning, isCustomizeableField, numberOfColumns)
			item.createContents
			afterChildItem(composite.verticalPartitioning, isCustomizeableField, numberOfColumns)
		ENDFOR
		afterChildContents(composite.verticalPartitioning, tabulators)
		'''
	}
	
	/**
	 * Checks whether the item is a customizeable field
	 */
	def boolean isCustomizeableField(EObject item) {
		item.eContents.filter(typeof(CustomizeableField)).size > 0
	}
	
	def boolean hasEventMappings(IGuiElementWithEvent control, boolean client) {
		for (mapping : control.parentPage.events.mapping.filter(typeof(GuiElementEventMapping))) {
			for (function : mapping.eventFunctions)
				if (control == mapping.control) {
					if (function.command != null && function.command.clientOnly == client)
						return true
					if (function.predefinedCommand != null && function.command == null && client)
						return true
				}
		}
		false
	} 
	
	def CharSequence expandCommandMappings(IGuiElementWithEvent control) {
		val calls = new ArrayList<String>()
		
		for (mapping : control.parentPage.events.mapping.filter(typeof(GuiElementEventMapping))) {
			for (function : mapping.eventFunctions)
				if (control == mapping.control) {
					if (function.command != null) {
						calls.add('''App.function.command.eContainer.widgetVar.function.command.name()''')
					}
					else if (function.predefinedCommand != null)
						calls.add('''App.control.parentPage.widgetVar.function.predefinedCommand.literal()''')
				}
		}
		/* '''queue = new xmaqueue(calls.join(','));return false''' */
		'''calls.join(';');return false'''
	} 
	
	
	def CharSequence columnValueExpression(TableColumn column) { 
		if (column.feature != null) {
			if (column.feature.feature != null && column.feature.feature.name != null)
				'''item.column.feature.feature.name'''
			else if (column.feature.feature != null && column.feature.feature.attribute != null && column.feature.feature.attribute.name != null) 
				'''item.column.feature.feature.attribute.name'''
		}
		 
	}
	def CharSequence fieldValueExpression(CustomizeableField field) { 
		'''IF (field.feature != null && field.feature.attributeHolder != null)field.object.eContainer.referencedBean.field.object.name.faceletName.field.feature.attributeHolder.name.faceletNameENDIF''' 
	} 

	
	
	def CharSequence toUiCommandPrototypes(EList<Command> commands) { '''
	    FOR command : commands
    command.name : function() {
			IF command.clientOnly
				IF command.implementation != null && !command.implementation.statements?.empty
					command.implementation.statements.forEach(statement | statement.createContents)
	    		ELSE
				alert('uicommand "command.name" is not implemented. Please define it in custom .xhtml page ');
	    		ENDIF
    		ELSE
	    					command.widgetVar();
    		ENDIF
    },
    	ENDFOR
	'''
	}
	
	
	def dispatch CharSequence createContents(Statement stmt) {
		'''Other statement:stmt'''
	}
	def dispatch CharSequence createContents(InvokeStatement stmt) {
		'''// InvokeStatement
		FOR propertyJoin : stmt.propertyJoin
			//TODO: /*propertyJoin.property.parentPage.widgetVar*/
		ENDFOR
		App.stmt.invokeAble.widgetVar.invoke();'''
	}
	
	/** Remove method after introducing i18n internationlization (texts are outsourced to language files!) */
	def CharSequence toEncodedHtml(String seq) {
		return seq.replaceAll("&", "&amp;");
	}
}