package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.common.collect.Iterators;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.ParameterDefinition;
import org.openxma.dsl.core.model.ParameterValue;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.SqlType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.AggregateFunction;
import org.openxma.dsl.dom.model.AliasedExpression;
import org.openxma.dsl.dom.model.AltWhenClause;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeGroup;
import org.openxma.dsl.dom.model.AttributeSortOrder;
import org.openxma.dsl.dom.model.BetweenExpression;
import org.openxma.dsl.dom.model.BinaryExpression;
import org.openxma.dsl.dom.model.BooleanLiteralValue;
import org.openxma.dsl.dom.model.CallableStatement;
import org.openxma.dsl.dom.model.CaseExpression;
import org.openxma.dsl.dom.model.CastFunction;
import org.openxma.dsl.dom.model.CollectionFunction;
import org.openxma.dsl.dom.model.Column;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DataBaseConstraint;
import org.openxma.dsl.dom.model.DeleteStatement;
import org.openxma.dsl.dom.model.EmptyLiteralValue;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.Expression;
import org.openxma.dsl.dom.model.FromClass;
import org.openxma.dsl.dom.model.FromRange;
import org.openxma.dsl.dom.model.FunctionCall;
import org.openxma.dsl.dom.model.InClass;
import org.openxma.dsl.dom.model.InCollection;
import org.openxma.dsl.dom.model.InCollectionElements;
import org.openxma.dsl.dom.model.InExpression;
import org.openxma.dsl.dom.model.InsertStatement;
import org.openxma.dsl.dom.model.IntegerLiteralValue;
import org.openxma.dsl.dom.model.Join;
import org.openxma.dsl.dom.model.JoinEntity;
import org.openxma.dsl.dom.model.LikeExpression;
import org.openxma.dsl.dom.model.LiteralValue;
import org.openxma.dsl.dom.model.ManyToMany;
import org.openxma.dsl.dom.model.ManyToOne;
import org.openxma.dsl.dom.model.MemberOfExpression;
import org.openxma.dsl.dom.model.NotExpression;
import org.openxma.dsl.dom.model.NullLiteralValue;
import org.openxma.dsl.dom.model.OneToMany;
import org.openxma.dsl.dom.model.OneToOne;
import org.openxma.dsl.dom.model.Operation;
import org.openxma.dsl.dom.model.Parameter;
import org.openxma.dsl.dom.model.ParenthesizedExpression;
import org.openxma.dsl.dom.model.PropertyAssignment;
import org.openxma.dsl.dom.model.PropertyValue;
import org.openxma.dsl.dom.model.QlStatement;
import org.openxma.dsl.dom.model.QuantifiedExpression;
import org.openxma.dsl.dom.model.QueryOperation;
import org.openxma.dsl.dom.model.QueryParameterReference;
import org.openxma.dsl.dom.model.QueryParameterValue;
import org.openxma.dsl.dom.model.RealLiteralValue;
import org.openxma.dsl.dom.model.SelectClass;
import org.openxma.dsl.dom.model.SelectObject;
import org.openxma.dsl.dom.model.SelectProperties;
import org.openxma.dsl.dom.model.SelectStatement;
import org.openxma.dsl.dom.model.SortOrderElement;
import org.openxma.dsl.dom.model.StringLiteralValue;
import org.openxma.dsl.dom.model.SubQuery;
import org.openxma.dsl.dom.model.TrimFunction;
import org.openxma.dsl.dom.model.UnaryExpression;
import org.openxma.dsl.dom.model.UpdateStatement;
import org.openxma.dsl.dom.model.WhenClause;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.EntityExtension;
import org.openxma.dsl.generator.helper.ValidatorExtension;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.impl.Types;
import org.openxma.dsl.generator.lib.Index;
import org.openxma.dsl.generator.lib.IterableExtensions2;
import org.openxma.dsl.generator.lib.ValidatorExtensions;

@SuppressWarnings("all")
public class HbmXmlFileGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Types _types;
  
  @Inject
  private Features _features;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    final Iterable<Dao> daos = Iterables.<Dao>filter(_elements, Dao.class);
    final Function1<Dao,Boolean> _function = new Function1<Dao,Boolean>() {
        public Boolean apply(final Dao d1) {
          final Function1<Dao,Boolean> _function = new Function1<Dao,Boolean>() {
              public Boolean apply(final Dao d2) {
                Entity _entity = d1.getEntity();
                Entity _superType = _entity.getSuperType();
                Entity _entity_1 = d2.getEntity();
                boolean _equals = Objects.equal(_superType, _entity_1);
                return Boolean.valueOf(_equals);
              }
            };
          boolean _exists = IterableExtensions.<Dao>exists(daos, _function);
          boolean _not = (!_exists);
          return Boolean.valueOf(_not);
        }
      };
    Iterable<Dao> _filter = IterableExtensions.<Dao>filter(daos, _function);
    final Procedure1<Dao> _function_1 = new Procedure1<Dao>() {
        public void apply(final Dao it) {
          final CharSequence hibernateMapping = HbmXmlFileGenerator.this.toHibernateMapping(it);
          String _hbmXmlFileName = EntityExtension.getHbmXmlFileName(it);
          fsa.generateFile(_hbmXmlFileName, GeneratorConstants.OUTLET_GENERATED_RESOURCES, hibernateMapping);
          String _generatedHbmXmlFileName = EntityExtension.getGeneratedHbmXmlFileName(it);
          fsa.generateFile(_generatedHbmXmlFileName, GeneratorConstants.OUTLET_GENERATED_RESOURCES, hibernateMapping);
          String _fragmentHbmXmlFileName = EntityExtension.getFragmentHbmXmlFileName(it);
          fsa.generateFile(_fragmentHbmXmlFileName, GeneratorConstants.OUTLET_RESOURCES, hibernateMapping);
        }
      };
    IterableExtensions.<Dao>forEach(_filter, _function_1);
  }
  
  public CharSequence toHibernateMapping(final Dao it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    _builder.newLine();
    _builder.append("<!DOCTYPE hibernate-mapping PUBLIC \"-//Hibernate/Hibernate Mapping DTD//EN\"");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("\"http://hibernate.sourceforge.net/hibernate-mapping-3.0.dtd\">");
    _builder.newLine();
    _builder.append("<hibernate-mapping default-access=\"field\" default-lazy=\"true\">");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("<class name=\"");
    Entity _entity = it.getEntity();
    String _qualifiedClassName = this._names.getQualifiedClassName(_entity);
    _builder.append(_qualifiedClassName, "    ");
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    _builder.append("           ");
    _builder.append("table=\"");
    String _tableName = it.getTableName();
    String _upperCase = _tableName.toUpperCase();
    _builder.append(_upperCase, "           ");
    _builder.append("\" discriminator-value=\"");
    String _qualifier = it.getQualifier();
    _builder.append(_qualifier, "           ");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    {
      Entity _entity_1 = it.getEntity();
      String _txtDescription = _entity_1.getTxtDescription();
      boolean _notEquals = (!Objects.equal(_txtDescription, null));
      if (_notEquals) {
        _builder.append("        ");
        _builder.append("<comment><![CDATA[");
        Entity _entity_2 = it.getEntity();
        String _txtDescription_1 = _entity_2.getTxtDescription();
        _builder.append(_txtDescription_1, "        ");
        _builder.append("]]></comment>");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      DataBaseConstraint _primaryKey = it.getPrimaryKey();
      boolean _notEquals_1 = (!Objects.equal(_primaryKey, null));
      if (_notEquals_1) {
        _builder.append("        ");
        _builder.append("<id name=\"");
        Column _primaryKeyColumn = it.getPrimaryKeyColumn();
        Attribute _attribute = _primaryKeyColumn.getAttribute();
        String _name = _attribute.getName();
        _builder.append(_name, "        ");
        _builder.append("\" access=\"property\">");
        _builder.newLineIfNotEmpty();
        _builder.append("        ");
        _builder.append("    ");
        Column _primaryKeyColumn_1 = it.getPrimaryKeyColumn();
        CharSequence _columnMapping = this.toColumnMapping(_primaryKeyColumn_1);
        _builder.append(_columnMapping, "            ");
        _builder.newLineIfNotEmpty();
        _builder.append("        ");
        _builder.append("    ");
        Column _primaryKeyColumn_2 = it.getPrimaryKeyColumn();
        CharSequence _typeMapping = this.toTypeMapping(_primaryKeyColumn_2);
        _builder.append(_typeMapping, "            ");
        _builder.newLineIfNotEmpty();
        _builder.append("        ");
        _builder.append("    ");
        _builder.append("<generator class=\"assigned\" />");
        _builder.newLine();
        _builder.append("        ");
        _builder.append("</id>");
        _builder.newLine();
      } else {
        _builder.append("        ");
        _builder.append("<composite-id class=\"");
        Entity _entity_3 = it.getEntity();
        AttributeGroup _key = _entity_3.getKey();
        String _qualifiedKeyClassName = this._features.getQualifiedKeyClassName(_key);
        _builder.append(_qualifiedKeyClassName, "        ");
        _builder.append("\" mapped=\"true\">");
        _builder.newLineIfNotEmpty();
        {
          EList<Column> _naturalKeyColumns = it.getNaturalKeyColumns();
          for(final Column naturalKeyColumn : _naturalKeyColumns) {
            _builder.append("        ");
            _builder.append("<key-property name=\"");
            Attribute _attribute_1 = naturalKeyColumn.getAttribute();
            String _name_1 = _attribute_1.getName();
            _builder.append(_name_1, "        ");
            _builder.append("\">");
            _builder.newLineIfNotEmpty();
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _columnMapping_1 = this.toColumnMapping(naturalKeyColumn);
            _builder.append(_columnMapping_1, "            ");
            _builder.newLineIfNotEmpty();
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _typeMapping_1 = this.toTypeMapping(naturalKeyColumn);
            _builder.append(_typeMapping_1, "            ");
            _builder.newLineIfNotEmpty();
            _builder.append("        ");
            _builder.append("</key-property>");
            _builder.newLine();
          }
        }
        _builder.append("        ");
        _builder.append("</composite-id>");
        _builder.newLine();
      }
    }
    {
      List<Dao> _subClassDaos = this.getSubClassDaos(it);
      boolean _isEmpty = _subClassDaos.isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        _builder.append("        ");
        _builder.append("<discriminator column=\"");
        String _discriminator = it.getDiscriminator();
        _builder.append(_discriminator, "        ");
        _builder.append("\" type=\"string\"/>");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      Column _versionColumn = it.getVersionColumn();
      boolean _notEquals_2 = (!Objects.equal(_versionColumn, null));
      if (_notEquals_2) {
        _builder.append("        ");
        Column _versionColumn_1 = it.getVersionColumn();
        Attribute _attribute_2 = _versionColumn_1.getAttribute();
        DataTypeAndTypeParameter _type = _attribute_2.getType();
        final String versionType = this._names.getQualifiedName(_type);
        _builder.newLineIfNotEmpty();
        _builder.append("        ");
        _builder.append("<version name=\"");
        Column _versionColumn_2 = it.getVersionColumn();
        Attribute _attribute_3 = _versionColumn_2.getAttribute();
        String _name_2 = _attribute_3.getName();
        _builder.append(_name_2, "        ");
        _builder.append("\" access=\"field\" unsaved-value=\"null\" ");
        {
          boolean _or = false;
          boolean _endsWith = versionType.endsWith("Date");
          if (_endsWith) {
            _or = true;
          } else {
            boolean _endsWith_1 = versionType.endsWith("Timestamp");
            _or = (_endsWith || _endsWith_1);
          }
          if (_or) {
            _builder.append("type=\"timestamp\"");
          } else {
            boolean _endsWith_2 = versionType.endsWith("Long");
            if (_endsWith_2) {
              _builder.append("type=\"long\"");
            }
          }
        }
        _builder.append(">");
        _builder.newLineIfNotEmpty();
        _builder.append("        ");
        _builder.append("    ");
        Column _versionColumn_3 = it.getVersionColumn();
        CharSequence _columnMapping_2 = this.toColumnMapping(_versionColumn_3);
        _builder.append(_columnMapping_2, "            ");
        _builder.newLineIfNotEmpty();
        _builder.append("        ");
        _builder.append("</version>");
        _builder.newLine();
      }
    }
    _builder.append("        ");
    EList<Column> _columns = it.getColumns();
    final Function1<Column,Boolean> _function = new Function1<Column,Boolean>() {
        public Boolean apply(final Column e) {
          boolean _or = false;
          boolean _or_1 = false;
          Attribute _attribute = e.getAttribute();
          boolean _isIdentifier = _attribute.isIdentifier();
          if (_isIdentifier) {
            _or_1 = true;
          } else {
            Attribute _attribute_1 = e.getAttribute();
            boolean _isVersion = _attribute_1.isVersion();
            _or_1 = (_isIdentifier || _isVersion);
          }
          if (_or_1) {
            _or = true;
          } else {
            EList<ManyToOne> _manyToOneAssociations = it.getManyToOneAssociations();
            final Function1<ManyToOne,String> _function = new Function1<ManyToOne,String>() {
                public String apply(final ManyToOne m) {
                  String _columnName = m.getColumnName();
                  return _columnName;
                }
              };
            List<String> _map = ListExtensions.<ManyToOne, String>map(_manyToOneAssociations, _function);
            String _columnName = e.getColumnName();
            boolean _contains = _map.contains(_columnName);
            _or = (_or_1 || _contains);
          }
          return Boolean.valueOf(_or);
        }
      };
    final Iterable<Column> propertyColumns = IterableExtensions2.<Column>reject(_columns, _function);
    _builder.newLineIfNotEmpty();
    {
      DataBaseConstraint _primaryKey_1 = it.getPrimaryKey();
      boolean _notEquals_3 = (!Objects.equal(_primaryKey_1, null));
      if (_notEquals_3) {
        {
          for(final Column property : propertyColumns) {
            _builder.append("        ");
            CharSequence _propertyMapping = this.toPropertyMapping(property);
            _builder.append(_propertyMapping, "        ");
            _builder.newLineIfNotEmpty();
          }
        }
      } else {
        {
          final Function1<Column,Boolean> _function_1 = new Function1<Column,Boolean>() {
              public Boolean apply(final Column e) {
                EList<Column> _naturalKeyColumns = it.getNaturalKeyColumns();
                boolean _contains = _naturalKeyColumns.contains(e);
                return Boolean.valueOf(_contains);
              }
            };
          Iterable<Column> _reject = IterableExtensions2.<Column>reject(propertyColumns, _function_1);
          for(final Column property_1 : _reject) {
            _builder.append("        ");
            CharSequence _propertyMapping_1 = this.toPropertyMapping(property_1);
            _builder.append(_propertyMapping_1, "        ");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    {
      EList<OneToMany> _oneToManyAssociations = it.getOneToManyAssociations();
      for(final OneToMany relation : _oneToManyAssociations) {
        _builder.append("        ");
        CharSequence _relationMapping = this.toRelationMapping(relation);
        _builder.append(_relationMapping, "        ");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<ManyToOne> _manyToOneAssociations = it.getManyToOneAssociations();
      for(final ManyToOne relation_1 : _manyToOneAssociations) {
        _builder.append("        ");
        CharSequence _relationMapping_1 = this.toRelationMapping(relation_1);
        _builder.append(_relationMapping_1, "        ");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<OneToOne> _oneToOneAssociations = it.getOneToOneAssociations();
      for(final OneToOne relation_2 : _oneToOneAssociations) {
        _builder.append("        ");
        CharSequence _relationMapping_2 = this.toRelationMapping(relation_2);
        _builder.append(_relationMapping_2, "        ");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<ManyToMany> _manyToManyAssociations = it.getManyToManyAssociations();
      for(final ManyToMany relation_3 : _manyToManyAssociations) {
        _builder.append("        ");
        CharSequence _relationMapping_3 = this.toRelationMapping(relation_3);
        _builder.append(_relationMapping_3, "        ");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      List<Dao> _subClassDaos_1 = this.getSubClassDaos(it);
      for(final Dao subClassDao : _subClassDaos_1) {
        _builder.append("        ");
        _builder.append("<subclass name=\"");
        Entity _entity_4 = subClassDao.getEntity();
        String _qualifiedClassName_1 = this._names.getQualifiedClassName(_entity_4);
        _builder.append(_qualifiedClassName_1, "        ");
        _builder.append("\" discriminator-value=\"");
        String _qualifier_1 = subClassDao.getQualifier();
        _builder.append(_qualifier_1, "        ");
        _builder.append("\">");
        _builder.newLineIfNotEmpty();
        {
          EList<Column> _columns_1 = subClassDao.getColumns();
          final Function1<Column,Boolean> _function_2 = new Function1<Column,Boolean>() {
              public Boolean apply(final Column e) {
                Entity _entity = subClassDao.getEntity();
                EList<Attribute> _attributes = _entity.getAttributes();
                final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
                    public Boolean apply(final Attribute sca) {
                      String _name = sca.getName();
                      Attribute _attribute = e.getAttribute();
                      String _name_1 = _attribute.getName();
                      boolean _equals = Objects.equal(_name, _name_1);
                      return Boolean.valueOf(_equals);
                    }
                  };
                boolean _exists = IterableExtensions.<Attribute>exists(_attributes, _function);
                return Boolean.valueOf(_exists);
              }
            };
          Iterable<Column> _filter = IterableExtensions.<Column>filter(_columns_1, _function_2);
          for(final Column property_2 : _filter) {
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _propertyMapping_2 = this.toPropertyMapping(property_2);
            _builder.append(_propertyMapping_2, "            ");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          EList<OneToMany> _oneToManyAssociations_1 = subClassDao.getOneToManyAssociations();
          for(final OneToMany relation_4 : _oneToManyAssociations_1) {
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _relationMapping_4 = this.toRelationMapping(relation_4);
            _builder.append(_relationMapping_4, "            ");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          EList<ManyToOne> _manyToOneAssociations_1 = subClassDao.getManyToOneAssociations();
          for(final ManyToOne relation_5 : _manyToOneAssociations_1) {
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _relationMapping_5 = this.toRelationMapping(relation_5);
            _builder.append(_relationMapping_5, "            ");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          EList<OneToOne> _oneToOneAssociations_1 = subClassDao.getOneToOneAssociations();
          for(final OneToOne relation_6 : _oneToOneAssociations_1) {
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _relationMapping_6 = this.toRelationMapping(relation_6);
            _builder.append(_relationMapping_6, "            ");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          EList<ManyToMany> _manyToManyAssociations_1 = subClassDao.getManyToManyAssociations();
          for(final ManyToMany relation_7 : _manyToManyAssociations_1) {
            _builder.append("        ");
            _builder.append("    ");
            CharSequence _relationMapping_7 = this.toRelationMapping(relation_7);
            _builder.append(_relationMapping_7, "            ");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("        ");
        _builder.append("</subclass>");
        _builder.newLine();
      }
    }
    _builder.append("    ");
    _builder.append("</class>");
    _builder.newLine();
    {
      EList<Operation> _operations = it.getOperations();
      final Function1<Operation,Boolean> _function_3 = new Function1<Operation,Boolean>() {
          public Boolean apply(final Operation e) {
            String _expression = e.getExpression();
            boolean _notEquals = (!Objects.equal(_expression, null));
            return Boolean.valueOf(_notEquals);
          }
        };
      Iterable<Operation> _filter_1 = IterableExtensions.<Operation>filter(_operations, _function_3);
      for(final Operation operation : _filter_1) {
        _builder.append("    ");
        _builder.append("<query name=\"");
        Entity _entity_5 = it.getEntity();
        String _name_3 = _entity_5.getName();
        _builder.append(_name_3, "    ");
        _builder.append(".");
        String _name_4 = operation.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name_4);
        _builder.append(_firstUpper, "    ");
        _builder.append("\"><![CDATA[ ");
        String _expression = operation.getExpression();
        _builder.append(_expression, "    ");
        _builder.append(" ]]></query>");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<QueryOperation> _queryOperation = it.getQueryOperation();
      final Function1<QueryOperation,Boolean> _function_4 = new Function1<QueryOperation,Boolean>() {
          public Boolean apply(final QueryOperation e) {
            QlStatement _statement = e.getStatement();
            return Boolean.valueOf((_statement instanceof CallableStatement));
          }
        };
      Iterable<QueryOperation> _reject_1 = IterableExtensions2.<QueryOperation>reject(_queryOperation, _function_4);
      for(final QueryOperation queryOperation : _reject_1) {
        _builder.append("    ");
        _builder.append("<query name=\"");
        String _qualifiedQueryName = this._features.getQualifiedQueryName(queryOperation);
        _builder.append(_qualifiedQueryName, "    ");
        _builder.append("\">");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("<![CDATA[");
        _builder.newLine();
        QlStatement _statement = queryOperation.getStatement();
        CharSequence _qlString = this.toQlString(_statement);
        _builder.append(_qlString, "");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("]]>");
        _builder.newLine();
        _builder.append("    ");
        _builder.append("</query>");
        _builder.newLine();
      }
    }
    {
      List<Dao> _subClassDaos_2 = this.getSubClassDaos(it);
      for(final Dao subClassDao_1 : _subClassDaos_2) {
        _builder.append("    ");
        CharSequence _namedQueryMapping = this.toNamedQueryMapping(subClassDao_1);
        _builder.append(_namedQueryMapping, "    ");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("</hibernate-mapping>");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toNamedQueryMapping(final Dao it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Operation> _operations = it.getOperations();
      final Function1<Operation,Boolean> _function = new Function1<Operation,Boolean>() {
          public Boolean apply(final Operation e) {
            String _expression = e.getExpression();
            boolean _notEquals = (!Objects.equal(_expression, null));
            return Boolean.valueOf(_notEquals);
          }
        };
      Iterable<Operation> _filter = IterableExtensions.<Operation>filter(_operations, _function);
      for(final Operation operation : _filter) {
        _builder.append("<query name=\"");
        Entity _entity = it.getEntity();
        String _name = _entity.getName();
        _builder.append(_name, "");
        _builder.append(".");
        String _name_1 = operation.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name_1);
        _builder.append(_firstUpper, "");
        _builder.append("\"><![CDATA[ ");
        String _expression = operation.getExpression();
        _builder.append(_expression, "");
        _builder.append(" ]]></query>");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<QueryOperation> _queryOperation = it.getQueryOperation();
      final Function1<QueryOperation,Boolean> _function_1 = new Function1<QueryOperation,Boolean>() {
          public Boolean apply(final QueryOperation e) {
            QlStatement _statement = e.getStatement();
            return Boolean.valueOf((_statement instanceof CallableStatement));
          }
        };
      Iterable<QueryOperation> _reject = IterableExtensions2.<QueryOperation>reject(_queryOperation, _function_1);
      for(final QueryOperation queryOperation : _reject) {
        _builder.append("<query name=\"");
        String _qualifiedQueryName = this._features.getQualifiedQueryName(queryOperation);
        _builder.append(_qualifiedQueryName, "");
        _builder.append("\">");
        _builder.newLineIfNotEmpty();
        _builder.append("<![CDATA[");
        _builder.newLine();
        QlStatement _statement = queryOperation.getStatement();
        CharSequence _qlString = this.toQlString(_statement);
        _builder.append(_qlString, "");
        _builder.newLineIfNotEmpty();
        _builder.append("]]>");
        _builder.newLine();
        _builder.append("</query>");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence toColumnMapping(final Column it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<column name=\"");
    String _columnName = it.getColumnName();
    _builder.append(_columnName, "");
    _builder.append("\"");
    {
      Attribute _attribute = it.getAttribute();
      boolean _isRequired = _attribute.isRequired();
      if (_isRequired) {
        _builder.append(" not-null=\"true\"");
      }
    }
    {
      Type _columnType = it.getColumnType();
      boolean _notEquals = (!Objects.equal(_columnType, null));
      if (_notEquals) {
        _builder.append(" sql-type=\"");
        Type _columnType_1 = it.getColumnType();
        String _qualifiedName = this._names.getQualifiedName(_columnType_1);
        _builder.append(_qualifiedName, "");
        _builder.append("\"");
      }
    }
    {
      Attribute _attribute_1 = it.getAttribute();
      DataTypeAndTypeParameter _type = _attribute_1.getType();
      Integer _maxLength = ValidatorExtension.getMaxLength(_type);
      boolean _notEquals_1 = (!Objects.equal(_maxLength, null));
      if (_notEquals_1) {
        _builder.append(" length=\"");
        Attribute _attribute_2 = it.getAttribute();
        DataTypeAndTypeParameter _type_1 = _attribute_2.getType();
        Integer _maxLength_1 = ValidatorExtension.getMaxLength(_type_1);
        _builder.append(_maxLength_1, "");
        _builder.append("\"");
      }
    }
    {
      Attribute _attribute_3 = it.getAttribute();
      DataTypeAndTypeParameter _type_2 = _attribute_3.getType();
      Integer _precision = ValidatorExtensions.getPrecision(_type_2);
      boolean _notEquals_2 = (!Objects.equal(_precision, null));
      if (_notEquals_2) {
        _builder.append(" precision=\"");
        Attribute _attribute_4 = it.getAttribute();
        DataTypeAndTypeParameter _type_3 = _attribute_4.getType();
        Integer _precision_1 = ValidatorExtensions.getPrecision(_type_3);
        _builder.append(_precision_1, "");
        _builder.append("\"");
      }
    }
    {
      Attribute _attribute_5 = it.getAttribute();
      DataTypeAndTypeParameter _type_4 = _attribute_5.getType();
      Integer _scale = ValidatorExtensions.getScale(_type_4);
      boolean _notEquals_3 = (!Objects.equal(_scale, null));
      if (_notEquals_3) {
        _builder.append(" scale=\"");
        Attribute _attribute_6 = it.getAttribute();
        DataTypeAndTypeParameter _type_5 = _attribute_6.getType();
        Integer _scale_1 = ValidatorExtensions.getScale(_type_5);
        _builder.append(_scale_1, "");
        _builder.append("\"");
      }
    }
    {
      String _comment = EntityExtension.getComment(it);
      boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(_comment);
      if (_isNullOrEmpty) {
        _builder.append("/");
      }
    }
    _builder.append(">");
    _builder.newLineIfNotEmpty();
    {
      String _comment_1 = EntityExtension.getComment(it);
      boolean _isNullOrEmpty_1 = StringExtensions.isNullOrEmpty(_comment_1);
      boolean _not = (!_isNullOrEmpty_1);
      if (_not) {
        _builder.append("    ");
        _builder.append("<comment><![CDATA[");
        String _comment_2 = EntityExtension.getComment(it);
        _builder.append(_comment_2, "    ");
        _builder.append("]]></comment>");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      String _comment_3 = EntityExtension.getComment(it);
      boolean _isNullOrEmpty_2 = StringExtensions.isNullOrEmpty(_comment_3);
      boolean _not_1 = (!_isNullOrEmpty_2);
      if (_not_1) {
        _builder.append("</column>");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence toTypeMapping(final Column it) {
    CharSequence _xblockexpression = null;
    {
      DataTypeAndTypeParameter _userType = it.getUserType();
      boolean _equals = Objects.equal(_userType, null);
      if (_equals) {
        return "";
      }
      StringConcatenation _builder = new StringConcatenation();
      {
        DataTypeAndTypeParameter _userType_1 = it.getUserType();
        EList<ParameterValue> _parameterValues = _userType_1.getParameterValues();
        boolean _isEmpty = _parameterValues.isEmpty();
        if (_isEmpty) {
          _builder.append("<type name=\"");
          DataTypeAndTypeParameter _userType_2 = it.getUserType();
          String _qualifiedClassName = this.getQualifiedClassName(_userType_2);
          _builder.append(_qualifiedClassName, "");
          _builder.append("\"/>");
          _builder.newLineIfNotEmpty();
        } else {
          _builder.append("<type name=\"");
          DataTypeAndTypeParameter _userType_3 = it.getUserType();
          String _qualifiedClassName_1 = this.getQualifiedClassName(_userType_3);
          _builder.append(_qualifiedClassName_1, "");
          _builder.append("\">");
          _builder.newLineIfNotEmpty();
          Index _index = new Index();
          final Index index = _index;
          _builder.newLineIfNotEmpty();
          {
            DataTypeAndTypeParameter _userType_4 = it.getUserType();
            EList<ParameterValue> _parameterValues_1 = _userType_4.getParameterValues();
            for(final ParameterValue parameterValue : _parameterValues_1) {
              _builder.append("<param name=\"");
              DataTypeAndTypeParameter _userType_5 = it.getUserType();
              Type _dataType = _userType_5.getDataType();
              EList<ParameterDefinition> _typeParameter = ((SimpleType) _dataType).getTypeParameter();
              int _counter0 = index.getCounter0();
              ParameterDefinition _get = _typeParameter.get(_counter0);
              String _name = _get.getName();
              _builder.append(_name, "");
              _builder.append("\">");
              String _valueAsString = ValidatorExtensions.getValueAsString(parameterValue);
              _builder.append(_valueAsString, "");
              _builder.append("</param>");
              _builder.newLineIfNotEmpty();
              index.increment();
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("</type>");
          _builder.newLine();
        }
      }
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence toPropertyMapping(final Column it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Column> _columns = it.getColumns();
      boolean _isEmpty = _columns.isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        _builder.append("<component name=\"");
        Attribute _attribute = it.getAttribute();
        String _name = _attribute.getName();
        _builder.append(_name, "");
        _builder.append("\" class=\"");
        Attribute _attribute_1 = it.getAttribute();
        DataTypeAndTypeParameter _type = _attribute_1.getType();
        String _qualifiedClassName = this.getQualifiedClassName(_type);
        _builder.append(_qualifiedClassName, "");
        _builder.append("\" access=\"field\">");
        _builder.newLineIfNotEmpty();
        {
          EList<Column> _columns_1 = it.getColumns();
          for(final Column column : _columns_1) {
            _builder.append("<property name=\"");
            Attribute _attribute_2 = column.getAttribute();
            String _name_1 = _attribute_2.getName();
            _builder.append(_name_1, "");
            _builder.append("\">");
            _builder.newLineIfNotEmpty();
            _builder.append("    ");
            CharSequence _columnMapping = this.toColumnMapping(column);
            _builder.append(_columnMapping, "    ");
            _builder.newLineIfNotEmpty();
            _builder.append("    ");
            CharSequence _typeMapping = this.toTypeMapping(column);
            _builder.append(_typeMapping, "    ");
            _builder.newLineIfNotEmpty();
            _builder.append("</property>");
            _builder.newLine();
          }
        }
        _builder.append("</component>");
        _builder.newLine();
      } else {
        _builder.append("<property name=\"");
        Attribute _attribute_3 = it.getAttribute();
        String _name_2 = _attribute_3.getName();
        _builder.append(_name_2, "");
        _builder.append("\">");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        CharSequence _columnMapping_1 = this.toColumnMapping(it);
        _builder.append(_columnMapping_1, "    ");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        CharSequence _typeMapping_1 = this.toTypeMapping(it);
        _builder.append(_typeMapping_1, "    ");
        _builder.newLineIfNotEmpty();
        _builder.append("</property>");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence toRelationMapping(final OneToMany it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<set name=\"");
    Attribute _attribute = it.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("\" inverse=\"");
    Attribute _attribute_1 = it.getAttribute();
    Attribute _opposite = _attribute_1.getOpposite();
    boolean _notEquals = (!Objects.equal(_opposite, null));
    _builder.append(_notEquals, "");
    _builder.append("\" cascade=\"all");
    {
      Attribute _attribute_2 = it.getAttribute();
      boolean _isComposition = _attribute_2.isComposition();
      if (_isComposition) {
        _builder.append(",delete-orphan");
      }
    }
    _builder.append("\"");
    {
      Attribute _attribute_3 = it.getAttribute();
      AttributeGroup _sortOrder = _attribute_3.getSortOrder();
      boolean _notEquals_1 = (!Objects.equal(_sortOrder, null));
      if (_notEquals_1) {
        _builder.append(" order-by=\"");
        {
          Attribute _attribute_4 = it.getAttribute();
          AttributeGroup _sortOrder_1 = _attribute_4.getSortOrder();
          EList<AttributeSortOrder> _attributes = _sortOrder_1.getAttributes();
          boolean _hasElements = false;
          for(final AttributeSortOrder order : _attributes) {
            if (!_hasElements) {
              _hasElements = true;
            } else {
              _builder.appendImmediate(",", "");
            }
            EObject _eContainer = it.eContainer();
            Entity _entity = ((Dao) _eContainer).getEntity();
            Attribute _attribute_5 = order.getAttribute();
            String _columnName = EntityExtension.getColumnName(_entity, _attribute_5);
            _builder.append(_columnName, "");
            _builder.append(" ");
            String _xifexpression = null;
            boolean _isAsc = order.isAsc();
            if (_isAsc) {
              _xifexpression = "ASC";
            } else {
              _xifexpression = "DESC";
            }
            _builder.append(_xifexpression, "");
          }
        }
        _builder.append("\"");
      }
    }
    {
      Attribute _attribute_6 = it.getAttribute();
      boolean _isComposition_1 = _attribute_6.isComposition();
      boolean _not = (!_isComposition_1);
      if (_not) {
        _builder.append(" optimistic-lock=\"false\"");
      }
    }
    _builder.append(">");
    _builder.newLineIfNotEmpty();
    {
      EList<Column> _columns = it.getColumns();
      boolean _isEmpty = _columns.isEmpty();
      boolean _not_1 = (!_isEmpty);
      if (_not_1) {
        _builder.append("    ");
        _builder.append("<key>");
        _builder.newLine();
        {
          EList<Column> _columns_1 = it.getColumns();
          for(final Column foreignKeyColumn : _columns_1) {
            _builder.append("    ");
            _builder.append("<column name=\"");
            String _columnName_1 = foreignKeyColumn.getColumnName();
            _builder.append(_columnName_1, "    ");
            _builder.append("\"/>");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("    ");
        _builder.append("</key>");
        _builder.newLine();
      } else {
        _builder.append("    ");
        _builder.append("<key column=\"");
        String _columnName_2 = it.getColumnName();
        _builder.append(_columnName_2, "    ");
        _builder.append("\"/>");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("    ");
    _builder.append("<one-to-many class=\"");
    Attribute _attribute_7 = it.getAttribute();
    DataTypeAndTypeParameter _type = _attribute_7.getType();
    String _qualifiedClassName = this.getQualifiedClassName(_type);
    _builder.append(_qualifiedClassName, "    ");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("</set>");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toRelationMapping(final ManyToMany it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<set name=\"");
    Attribute _attribute = it.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("\" inverse=\"");
    boolean _isInverse = it.isInverse();
    _builder.append(_isInverse, "");
    _builder.append("\" table=\"");
    String _tableName = it.getTableName();
    _builder.append(_tableName, "");
    _builder.append("\" cascade=\"save-update\">");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("<key column=\"");
    String _columnName = it.getColumnName();
    _builder.append(_columnName, "    ");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("<many-to-many class=\"");
    Attribute _attribute_1 = it.getAttribute();
    DataTypeAndTypeParameter _type = _attribute_1.getType();
    String _qualifiedClassName = this.getQualifiedClassName(_type);
    _builder.append(_qualifiedClassName, "    ");
    _builder.append("\"  column=\"");
    String _joinColumnName = EntityExtension.getJoinColumnName(it);
    _builder.append(_joinColumnName, "    ");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("</set>");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toRelationMapping(final ManyToOne it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<many-to-one name=\"");
    Attribute _attribute = it.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("\" class=\"");
    Attribute _attribute_1 = it.getAttribute();
    DataTypeAndTypeParameter _type = _attribute_1.getType();
    String _qualifiedClassName = this.getQualifiedClassName(_type);
    _builder.append(_qualifiedClassName, "");
    _builder.append("\"");
    {
      boolean _isDerived = it.isDerived();
      if (_isDerived) {
        _builder.append(" insert=\"false\" update=\"false\"");
      }
    }
    _builder.append(">");
    _builder.newLineIfNotEmpty();
    {
      EList<Column> _columns = it.getColumns();
      boolean _isEmpty = _columns.isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        {
          EList<Column> _columns_1 = it.getColumns();
          for(final Column foreignKeyColumn : _columns_1) {
            _builder.append("    ");
            _builder.append("<column name=\"");
            String _columnName = foreignKeyColumn.getColumnName();
            _builder.append(_columnName, "    ");
            _builder.append("\"/>");
            _builder.newLineIfNotEmpty();
          }
        }
      } else {
        _builder.append("    ");
        _builder.append("<column name=\"");
        String _columnName_1 = it.getColumnName();
        _builder.append(_columnName_1, "    ");
        _builder.append("\"");
        {
          SqlType _sqlType = it.getSqlType();
          boolean _notEquals = (!Objects.equal(_sqlType, null));
          if (_notEquals) {
            _builder.append(" sql-type=\"");
            SqlType _sqlType_1 = it.getSqlType();
            String _ddl = _sqlType_1.getDdl();
            _builder.append(_ddl, "    ");
            _builder.append("\"");
          }
        }
        {
          Attribute _attribute_2 = it.getAttribute();
          boolean _isRequired = _attribute_2.isRequired();
          if (_isRequired) {
            _builder.append(" not-null=\"true\"");
          }
        }
        _builder.append("/>");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("</many-to-one>");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toRelationMapping(final OneToOne it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<one-to-one name=\"");
    Attribute _attribute = it.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("\" class=\"");
    Attribute _attribute_1 = it.getAttribute();
    DataTypeAndTypeParameter _type = _attribute_1.getType();
    String _qualifiedClassName = this.getQualifiedClassName(_type);
    _builder.append(_qualifiedClassName, "");
    _builder.append("\" ");
    {
      boolean _or = false;
      Attribute _attribute_2 = it.getAttribute();
      boolean _isComposition = _attribute_2.isComposition();
      if (_isComposition) {
        _or = true;
      } else {
        Attribute _attribute_3 = it.getAttribute();
        Attribute _opposite = _attribute_3.getOpposite();
        boolean _isRequired = _opposite.isRequired();
        _or = (_isComposition || _isRequired);
      }
      if (_or) {
        _builder.append("cascade=\"all\"");
      }
    }
    _builder.append(" property-ref=\"");
    Attribute _attribute_4 = it.getAttribute();
    Attribute _opposite_1 = _attribute_4.getOpposite();
    String _name_1 = _opposite_1.getName();
    _builder.append(_name_1, "");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public List<Dao> getSubClassDaos(final Dao it) {
    Entity _entity = it.getEntity();
    Model _model = this._types.model(_entity);
    TreeIterator<EObject> _eAllContents = _model.eAllContents();
    Iterator<Dao> _filter = Iterators.<Dao>filter(_eAllContents, Dao.class);
    final Function1<Dao,Boolean> _function = new Function1<Dao,Boolean>() {
        public Boolean apply(final Dao e) {
          Entity _entity = e.getEntity();
          Entity _superType = _entity.getSuperType();
          Entity _entity_1 = it.getEntity();
          boolean _equals = Objects.equal(_superType, _entity_1);
          return Boolean.valueOf(_equals);
        }
      };
    Iterator<Dao> _filter_1 = IteratorExtensions.<Dao>filter(_filter, _function);
    List<Dao> _list = IteratorExtensions.<Dao>toList(_filter_1);
    final Function1<Dao,String> _function_1 = new Function1<Dao,String>() {
        public String apply(final Dao e) {
          String _name = e.getName();
          return _name;
        }
      };
    List<Dao> _sortBy = IterableExtensions.<Dao, String>sortBy(_list, _function_1);
    return _sortBy;
  }
  
  protected CharSequence _toQlString(final DeleteStatement it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("delete ");
    Entity _entity = it.getEntity();
    String _className = this._names.getClassName(_entity);
    _builder.append(_className, "");
    _builder.append(" as ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" ");
    {
      Expression _where = it.getWhere();
      boolean _notEquals = (!Objects.equal(_where, null));
      if (_notEquals) {
        _builder.append("where ");
        Expression _where_1 = it.getWhere();
        CharSequence _expressionString = this.toExpressionString(_where_1);
        _builder.append(_expressionString, "");
        _builder.append(" ");
      }
    }
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toQlString(final InsertStatement it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("insert into ");
    Entity _entity = it.getEntity();
    String _className = this._names.getClassName(_entity);
    _builder.append(_className, "");
    _builder.append(" (");
    {
      EList<Expression> _expression = it.getExpression();
      boolean _hasElements = false;
      for(final Expression expression : _expression) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _expressionString = this.toExpressionString(expression);
        _builder.append(_expressionString, "");
      }
    }
    _builder.append(")  ");
    SelectStatement _selectStatement = it.getSelectStatement();
    Object _qlString = this.toQlString(_selectStatement);
    _builder.append(_qlString, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toQlString(final UpdateStatement it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("update");
    {
      boolean _isVersioned = it.isVersioned();
      if (_isVersioned) {
        _builder.append(" versioned");
      }
    }
    _builder.append(" ");
    Entity _entity = it.getEntity();
    String _className = this._names.getClassName(_entity);
    _builder.append(_className, "");
    _builder.append(" as ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.newLineIfNotEmpty();
    _builder.append("set ");
    {
      EList<PropertyAssignment> _assignment = it.getAssignment();
      boolean _hasElements = false;
      for(final PropertyAssignment propertyAssignment : _assignment) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        PropertyValue _property = propertyAssignment.getProperty();
        CharSequence _expressionString = this.toExpressionString(_property);
        _builder.append(_expressionString, "");
        _builder.append(" = ");
        Expression _expression = propertyAssignment.getExpression();
        CharSequence _expressionString_1 = this.toExpressionString(_expression);
        _builder.append(_expressionString_1, "");
      }
    }
    _builder.newLineIfNotEmpty();
    {
      Expression _where = it.getWhere();
      boolean _notEquals = (!Objects.equal(_where, null));
      if (_notEquals) {
        _builder.append("where ");
        Expression _where_1 = it.getWhere();
        CharSequence _expressionString_2 = this.toExpressionString(_where_1);
        _builder.append(_expressionString_2, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toQlString(final SelectStatement it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _selectString = this.toSelectString(it);
    _builder.append(_selectString, "");
    return _builder;
  }
  
  protected CharSequence _toQlString(final SelectProperties it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("select ");
    {
      boolean _isDistinct = it.isDistinct();
      if (_isDistinct) {
        _builder.append("distinct ");
      }
    }
    {
      EList<Expression> _properties = it.getProperties();
      boolean _hasElements = false;
      for(final Expression property : _properties) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _expressionString = this.toExpressionString(property);
        _builder.append(_expressionString, "");
      }
    }
    _builder.append(" ");
    CharSequence _selectString = this.toSelectString(it);
    _builder.append(_selectString, "");
    return _builder;
  }
  
  protected CharSequence _toQlString(final SelectClass it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("select new ");
    String _class_ = it.getClass_();
    _builder.append(_class_, "");
    _builder.append("(");
    {
      EList<Expression> _arguments = it.getArguments();
      boolean _hasElements = false;
      for(final Expression argument : _arguments) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _expressionString = this.toExpressionString(argument);
        _builder.append(_expressionString, "");
      }
    }
    _builder.append(")");
    _builder.newLineIfNotEmpty();
    _builder.append("        ");
    CharSequence _selectString = this.toSelectString(it);
    _builder.append(_selectString, "        ");
    return _builder;
  }
  
  protected CharSequence _toQlString(final SelectObject it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("select object(");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(") ");
    CharSequence _selectString = this.toSelectString(it);
    _builder.append(_selectString, "");
    return _builder;
  }
  
  public CharSequence toSelectString(final SelectStatement it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<FromRange> _from = it.getFrom();
      boolean _isEmpty = _from.isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        _builder.append("from ");
        {
          EList<FromRange> _from_1 = it.getFrom();
          boolean _hasElements = false;
          for(final FromRange from : _from_1) {
            if (!_hasElements) {
              _hasElements = true;
            } else {
              _builder.appendImmediate(",", "");
            }
            CharSequence _fromRangeString = this.toFromRangeString(from);
            _builder.append(_fromRangeString, "");
          }
        }
      }
    }
    {
      EList<Join> _join = it.getJoin();
      for(final Join join : _join) {
        _builder.newLineIfNotEmpty();
        {
          String _type = join.getType();
          boolean _notEquals = (!Objects.equal(_type, null));
          if (_notEquals) {
            _builder.append(" ");
            String _type_1 = join.getType();
            _builder.append(_type_1, "");
          }
        }
        _builder.append(" join ");
        {
          boolean _isFetch = join.isFetch();
          if (_isFetch) {
            _builder.append("fetch ");
          }
        }
        JoinEntity _entity = join.getEntity();
        String _name = _entity.getName();
        _builder.append(_name, "");
        _builder.append(".");
        Attribute _reference = join.getReference();
        String _name_1 = _reference.getName();
        _builder.append(_name_1, "");
        _builder.append(" as ");
        String _name_2 = join.getName();
        _builder.append(_name_2, "");
        _builder.append(" ");
        {
          boolean _isPropertyFetch = join.isPropertyFetch();
          if (_isPropertyFetch) {
            _builder.append("fetch all properties");
          }
        }
        {
          Expression _expression = join.getExpression();
          boolean _notEquals_1 = (!Objects.equal(_expression, null));
          if (_notEquals_1) {
            _builder.append(" with ");
            Expression _expression_1 = join.getExpression();
            CharSequence _expressionString = this.toExpressionString(_expression_1);
            _builder.append(_expressionString, "");
          }
        }
      }
    }
    {
      Expression _where = it.getWhere();
      boolean _notEquals_2 = (!Objects.equal(_where, null));
      if (_notEquals_2) {
        _builder.newLineIfNotEmpty();
        _builder.append("where ");
        Expression _where_1 = it.getWhere();
        CharSequence _expressionString_1 = this.toExpressionString(_where_1);
        _builder.append(_expressionString_1, "");
      }
    }
    {
      EList<Expression> _groupBy = it.getGroupBy();
      boolean _isEmpty_1 = _groupBy.isEmpty();
      boolean _not_1 = (!_isEmpty_1);
      if (_not_1) {
        _builder.newLineIfNotEmpty();
        _builder.append("group by ");
        {
          EList<Expression> _groupBy_1 = it.getGroupBy();
          boolean _hasElements_1 = false;
          for(final Expression groupBy : _groupBy_1) {
            if (!_hasElements_1) {
              _hasElements_1 = true;
            } else {
              _builder.appendImmediate(",", "");
            }
            CharSequence _expressionString_2 = this.toExpressionString(groupBy);
            _builder.append(_expressionString_2, "");
          }
        }
        {
          Expression _having = it.getHaving();
          boolean _notEquals_3 = (!Objects.equal(_having, null));
          if (_notEquals_3) {
            _builder.append(" having ");
            Expression _having_1 = it.getHaving();
            CharSequence _expressionString_3 = this.toExpressionString(_having_1);
            _builder.append(_expressionString_3, "");
            _builder.append(" ");
          }
        }
      }
    }
    {
      EList<SortOrderElement> _orderBy = it.getOrderBy();
      boolean _isEmpty_2 = _orderBy.isEmpty();
      boolean _not_2 = (!_isEmpty_2);
      if (_not_2) {
        _builder.newLineIfNotEmpty();
        _builder.append("order by ");
        {
          EList<SortOrderElement> _orderBy_1 = it.getOrderBy();
          boolean _hasElements_2 = false;
          for(final SortOrderElement orderBy : _orderBy_1) {
            if (!_hasElements_2) {
              _hasElements_2 = true;
            } else {
              _builder.appendImmediate(",", "");
            }
            Expression _expression_2 = orderBy.getExpression();
            CharSequence _expressionString_4 = this.toExpressionString(_expression_2);
            _builder.append(_expressionString_4, "");
            _builder.append(" ");
            String _sortOrder = orderBy.getSortOrder();
            _builder.append(_sortOrder, "");
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _toFromRangeString(final InClass it) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" in class ");
    String _class_ = it.getClass_();
    _builder.append(_class_, "");
    return _builder;
  }
  
  protected CharSequence _toFromRangeString(final InCollection it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("in (");
    String _path = it.getPath();
    _builder.append(_path, "");
    _builder.append(") ");
    String _alias = it.getAlias();
    _builder.append(_alias, "");
    return _builder;
  }
  
  protected CharSequence _toFromRangeString(final InCollectionElements it) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" in elements (");
    String _reference = it.getReference();
    _builder.append(_reference, "");
    _builder.append(")");
    return _builder;
  }
  
  protected CharSequence _toFromRangeString(final FromClass it) {
    StringConcatenation _builder = new StringConcatenation();
    Entity _entity = it.getEntity();
    String _className = this._names.getClassName(_entity);
    _builder.append(_className, "");
    _builder.append(" as ");
    String _name = it.getName();
    _builder.append(_name, "");
    {
      boolean _isPopertyFetch = it.isPopertyFetch();
      if (_isPopertyFetch) {
        _builder.append(" fetch all properties");
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final Expression it) {
    return null;
  }
  
  protected CharSequence _toExpressionString(final AliasedExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    _builder.append(" ");
    {
      String _name = it.getName();
      boolean _notEquals = (!Objects.equal(_name, null));
      if (_notEquals) {
        _builder.append(" as ");
        String _name_1 = it.getName();
        _builder.append(_name_1, "");
        _builder.append(" ");
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final BinaryExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    Expression _left = it.getLeft();
    Object _expressionString = this.toExpressionString(_left);
    _builder.append(_expressionString, "");
    _builder.append(" ");
    String _operator = it.getOperator();
    _builder.append(_operator, "");
    _builder.append(" ");
    Expression _right = it.getRight();
    Object _expressionString_1 = this.toExpressionString(_right);
    _builder.append(_expressionString_1, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final UnaryExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    String _operator = it.getOperator();
    _builder.append(_operator, "");
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final NotExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(" ");
    _builder.append("not ");
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, " ");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final InExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    {
      boolean _isNot = it.isNot();
      if (_isNot) {
        _builder.append(" not");
      }
    }
    _builder.append(" ");
    String _operator = it.getOperator();
    _builder.append(_operator, "");
    _builder.append(" ");
    Expression _in = it.getIn();
    Object _expressionString_1 = this.toExpressionString(_in);
    _builder.append(_expressionString_1, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final BetweenExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    {
      boolean _isNot = it.isNot();
      if (_isNot) {
        _builder.append(" not");
      }
    }
    _builder.append(" ");
    String _operator = it.getOperator();
    _builder.append(_operator, "");
    _builder.append(" ");
    Expression _left = it.getLeft();
    Object _expressionString_1 = this.toExpressionString(_left);
    _builder.append(_expressionString_1, "");
    _builder.append(" and ");
    Expression _right = it.getRight();
    Object _expressionString_2 = this.toExpressionString(_right);
    _builder.append(_expressionString_2, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final LikeExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    {
      boolean _isNot = it.isNot();
      if (_isNot) {
        _builder.append(" not");
      }
    }
    _builder.append(" ");
    String _operator = it.getOperator();
    _builder.append(_operator, "");
    _builder.append(" ");
    Expression _like = it.getLike();
    Object _expressionString_1 = this.toExpressionString(_like);
    _builder.append(_expressionString_1, "");
    {
      Expression _escape = it.getEscape();
      boolean _notEquals = (!Objects.equal(_escape, null));
      if (_notEquals) {
        _builder.append(" escape ");
        Expression _escape_1 = it.getEscape();
        Object _expressionString_2 = this.toExpressionString(_escape_1);
        _builder.append(_expressionString_2, "");
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final MemberOfExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    {
      boolean _isNot = it.isNot();
      if (_isNot) {
        _builder.append(" not");
      }
    }
    _builder.append(" ");
    String _operator = it.getOperator();
    _builder.append(_operator, "");
    _builder.append(" of ");
    String _memberOf = it.getMemberOf();
    _builder.append(_memberOf, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final PropertyValue it) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = it.getName();
    _builder.append(_name, "");
    {
      EList<String> _segments = it.getSegments();
      for(final String segment : _segments) {
        _builder.append(".");
        _builder.append(segment, "");
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final FunctionCall it) {
    StringConcatenation _builder = new StringConcatenation();
    String _function = it.getFunction();
    _builder.append(_function, "");
    _builder.append("(");
    {
      EList<Expression> _arguments = it.getArguments();
      boolean _hasElements = false;
      for(final Expression argument : _arguments) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        Object _expressionString = this.toExpressionString(argument);
        _builder.append(_expressionString, "");
      }
    }
    _builder.append(")");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final TrimFunction it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("trim(");
    {
      String _mode = it.getMode();
      boolean _notEquals = (!Objects.equal(_mode, null));
      if (_notEquals) {
        String _mode_1 = it.getMode();
        _builder.append(_mode_1, "");
        _builder.append(" ");
      }
    }
    {
      StringLiteralValue _character = it.getCharacter();
      boolean _notEquals_1 = (!Objects.equal(_character, null));
      if (_notEquals_1) {
        StringLiteralValue _character_1 = it.getCharacter();
        _builder.append(_character_1, "");
        _builder.append(" from ");
      }
    }
    Expression _from = it.getFrom();
    Object _expressionString = this.toExpressionString(_from);
    _builder.append(_expressionString, "");
    _builder.append(")");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final CastFunction it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    {
      String _function = it.getFunction();
      boolean _equalsIgnoreCase = "cast".equalsIgnoreCase(_function);
      if (_equalsIgnoreCase) {
        String _function_1 = it.getFunction();
        _builder.append(_function_1, "");
        _builder.append("(");
        Expression _from = it.getFrom();
        Object _expressionString = this.toExpressionString(_from);
        _builder.append(_expressionString, "");
        _builder.append(" as ");
        String _name = it.getName();
        _builder.append(_name, "");
        _builder.append(")");
        _builder.newLineIfNotEmpty();
      } else {
        String _function_2 = it.getFunction();
        _builder.append(_function_2, "");
        _builder.append("(");
        String _name_1 = it.getName();
        _builder.append(_name_1, "");
        _builder.append(" from ");
        Expression _from_1 = it.getFrom();
        Object _expressionString_1 = this.toExpressionString(_from_1);
        _builder.append(_expressionString_1, "");
        _builder.append(")");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final AggregateFunction it) {
    CharSequence _xifexpression = null;
    Expression _aggregateExpression = it.getAggregateExpression();
    boolean _notEquals = (!Objects.equal(_aggregateExpression, null));
    if (_notEquals) {
      StringConcatenation _builder = new StringConcatenation();
      String _function = it.getFunction();
      _builder.append(_function, "");
      _builder.append("(");
      Expression _aggregateExpression_1 = it.getAggregateExpression();
      Object _expressionString = this.toExpressionString(_aggregateExpression_1);
      _builder.append(_expressionString, "");
      _builder.append(")");
      _xifexpression = _builder;
    } else {
      StringConcatenation _builder_1 = new StringConcatenation();
      String _function_1 = it.getFunction();
      _builder_1.append(_function_1, "");
      _builder_1.append("(");
      {
        boolean _isAll = it.isAll();
        if (_isAll) {
          _builder_1.append("*");
        } else {
          boolean _isDistinct = it.isDistinct();
          if (_isDistinct) {
            _builder_1.append("distinct ");
          }
        }
      }
      {
        String _from = it.getFrom();
        boolean _notEquals_1 = (!Objects.equal(_from, null));
        if (_notEquals_1) {
          String _from_1 = it.getFrom();
          _builder_1.append(_from_1, "");
        } else {
          CollectionFunction _collection = it.getCollection();
          boolean _notEquals_2 = (!Objects.equal(_collection, null));
          if (_notEquals_2) {
            CollectionFunction _collection_1 = it.getCollection();
            Object _expressionString_1 = this.toExpressionString(_collection_1);
            _builder_1.append(_expressionString_1, "");
          }
        }
      }
      _builder_1.append(")");
      _xifexpression = _builder_1;
    }
    return _xifexpression;
  }
  
  protected CharSequence _toExpressionString(final QueryParameterValue it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(":");
    String _name = this.getName(it);
    _builder.append(_name, "");
    {
      Attribute _attribute = it.getAttribute();
      boolean _notEquals = (!Objects.equal(_attribute, null));
      if (_notEquals) {
        _builder.append("_");
        Attribute _attribute_1 = it.getAttribute();
        String _name_1 = _attribute_1.getName();
        _builder.append(_name_1, "");
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final QuantifiedExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    String _quantifier = it.getQuantifier();
    _builder.append(_quantifier, "");
    _builder.append(" ");
    Expression _expression = it.getExpression();
    Object _expressionString = this.toExpressionString(_expression);
    _builder.append(_expressionString, "");
    _builder.append(" ");
    String _name = it.getName();
    _builder.append(_name, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final CaseExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    {
      Expression _expression = it.getExpression();
      boolean _equals = Objects.equal(_expression, null);
      if (_equals) {
        _builder.append("case ");
        {
          EList<WhenClause> _whenClause = it.getWhenClause();
          for(final WhenClause whenClause : _whenClause) {
            Object _expressionString = this.toExpressionString(whenClause);
            _builder.append(_expressionString, "");
          }
        }
        _builder.append(" ");
        {
          Expression _elseExpression = it.getElseExpression();
          boolean _notEquals = (!Objects.equal(null, _elseExpression));
          if (_notEquals) {
            _builder.append(" else ");
            Expression _elseExpression_1 = it.getElseExpression();
            Object _expressionString_1 = this.toExpressionString(_elseExpression_1);
            _builder.append(_expressionString_1, "");
            _builder.append(" ");
          }
        }
        _builder.append(" end");
        _builder.newLineIfNotEmpty();
      } else {
        _builder.append("case ");
        Expression _expression_1 = it.getExpression();
        Object _expressionString_2 = this.toExpressionString(_expression_1);
        _builder.append(_expressionString_2, "");
        _builder.append(" ");
        {
          EList<AltWhenClause> _altWhenClause = it.getAltWhenClause();
          for(final AltWhenClause expression : _altWhenClause) {
            Object _expressionString_3 = this.toExpressionString(expression);
            _builder.append(_expressionString_3, "");
          }
        }
        {
          Expression _elseExpression_2 = it.getElseExpression();
          boolean _notEquals_1 = (!Objects.equal(_elseExpression_2, null));
          if (_notEquals_1) {
            _builder.append(" else ");
            Expression _elseExpression_3 = it.getElseExpression();
            Object _expressionString_4 = this.toExpressionString(_elseExpression_3);
            _builder.append(_expressionString_4, "");
            _builder.append(" ");
          }
        }
        _builder.append(" end");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final WhenClause it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(" ");
    _builder.append("when ");
    Expression _whenExpression = it.getWhenExpression();
    Object _expressionString = this.toExpressionString(_whenExpression);
    _builder.append(_expressionString, " ");
    _builder.append(" then ");
    Expression _thenExpression = it.getThenExpression();
    Object _expressionString_1 = this.toExpressionString(_thenExpression);
    _builder.append(_expressionString_1, " ");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final AltWhenClause it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(" ");
    _builder.append("when ");
    Expression _whenExpression = it.getWhenExpression();
    Object _expressionString = this.toExpressionString(_whenExpression);
    _builder.append(_expressionString, " ");
    _builder.append(" then ");
    Expression _thenExpression = it.getThenExpression();
    Object _expressionString_1 = this.toExpressionString(_thenExpression);
    _builder.append(_expressionString_1, " ");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final CollectionFunction it) {
    StringConcatenation _builder = new StringConcatenation();
    String _function = it.getFunction();
    _builder.append(_function, "");
    _builder.append(" ");
    PropertyValue _property = it.getProperty();
    Object _expressionString = this.toExpressionString(_property);
    _builder.append(_expressionString, "");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final SubQuery it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("(");
    _builder.newLine();
    {
      EList<SelectStatement> _queries = it.getQueries();
      boolean _hasElements = false;
      for(final SelectStatement statement : _queries) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate("union", "");
        }
        Object _qlString = this.toQlString(statement);
        _builder.append(_qlString, "");
      }
    }
    _builder.append(")");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final ParenthesizedExpression it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("(");
    {
      EList<Expression> _expressions = it.getExpressions();
      boolean _hasElements = false;
      for(final Expression expression : _expressions) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        Object _expressionString = this.toExpressionString(expression);
        _builder.append(_expressionString, "");
      }
    }
    _builder.append(")");
    return _builder;
  }
  
  protected CharSequence _toExpressionString(final StringLiteralValue it) {
    String _value = it.getValue();
    String _plus = ("\'" + _value);
    String _plus_1 = (_plus + "\'");
    return _plus_1;
  }
  
  protected CharSequence _toExpressionString(final IntegerLiteralValue it) {
    String _value = it.getValue();
    return _value;
  }
  
  protected CharSequence _toExpressionString(final LiteralValue it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof StringLiteralValue) {
        final StringLiteralValue _stringLiteralValue = (StringLiteralValue)it;
        _matched=true;
        String _value = _stringLiteralValue.getValue();
        String _plus = ("\'" + _value);
        String _plus_1 = (_plus + "\'");
        _switchResult = _plus_1;
      }
    }
    if (!_matched) {
      if (it instanceof BooleanLiteralValue) {
        final BooleanLiteralValue _booleanLiteralValue = (BooleanLiteralValue)it;
        boolean _isIsTrue = _booleanLiteralValue.isIsTrue();
        if (_isIsTrue) {
          _matched=true;
          _switchResult = "true";
        }
      }
    }
    if (!_matched) {
      if (it instanceof BooleanLiteralValue) {
        final BooleanLiteralValue _booleanLiteralValue = (BooleanLiteralValue)it;
        boolean _isIsTrue = _booleanLiteralValue.isIsTrue();
        boolean _not = (!_isIsTrue);
        if (_not) {
          _matched=true;
          _switchResult = "false";
        }
      }
    }
    if (!_matched) {
      if (it instanceof EmptyLiteralValue) {
        final EmptyLiteralValue _emptyLiteralValue = (EmptyLiteralValue)it;
        _matched=true;
        _switchResult = "empty";
      }
    }
    if (!_matched) {
      if (it instanceof NullLiteralValue) {
        final NullLiteralValue _nullLiteralValue = (NullLiteralValue)it;
        _matched=true;
        _switchResult = "null";
      }
    }
    if (!_matched) {
      if (it instanceof IntegerLiteralValue) {
        final IntegerLiteralValue _integerLiteralValue = (IntegerLiteralValue)it;
        _matched=true;
        String _value = _integerLiteralValue.getValue();
        _switchResult = _value;
      }
    }
    if (!_matched) {
      if (it instanceof RealLiteralValue) {
        final RealLiteralValue _realLiteralValue = (RealLiteralValue)it;
        _matched=true;
        String _value = _realLiteralValue.getValue();
        _switchResult = _value;
      }
    }
    return _switchResult;
  }
  
  public String getName(final QueryParameterValue it) {
    String _xblockexpression = null;
    {
      final QueryParameterReference type = it.getParameter();
      String _switchResult = null;
      boolean _matched = false;
      if (!_matched) {
        if (type instanceof Parameter) {
          final Parameter _parameter = (Parameter)type;
          _matched=true;
          String _name = _parameter.getName();
          _switchResult = _name;
        }
      }
      if (!_matched) {
        if (type instanceof Attribute) {
          final Attribute _attribute = (Attribute)type;
          _matched=true;
          String _name = _attribute.getName();
          _switchResult = _name;
        }
      }
      _xblockexpression = (_switchResult);
    }
    return _xblockexpression;
  }
  
  private String getQualifiedClassName(final DataTypeAndTypeParameter it) {
    Type _dataType = it.getDataType();
    String _qualifiedClassName = this._names.getQualifiedClassName(_dataType);
    return _qualifiedClassName;
  }
  
  public CharSequence toQlString(final QlStatement it) {
    if (it instanceof SelectClass) {
      return _toQlString((SelectClass)it);
    } else if (it instanceof SelectObject) {
      return _toQlString((SelectObject)it);
    } else if (it instanceof SelectProperties) {
      return _toQlString((SelectProperties)it);
    } else if (it instanceof DeleteStatement) {
      return _toQlString((DeleteStatement)it);
    } else if (it instanceof InsertStatement) {
      return _toQlString((InsertStatement)it);
    } else if (it instanceof SelectStatement) {
      return _toQlString((SelectStatement)it);
    } else if (it instanceof UpdateStatement) {
      return _toQlString((UpdateStatement)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public CharSequence toFromRangeString(final FromRange it) {
    if (it instanceof FromClass) {
      return _toFromRangeString((FromClass)it);
    } else if (it instanceof InClass) {
      return _toFromRangeString((InClass)it);
    } else if (it instanceof InCollection) {
      return _toFromRangeString((InCollection)it);
    } else if (it instanceof InCollectionElements) {
      return _toFromRangeString((InCollectionElements)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public CharSequence toExpressionString(final EObject it) {
    if (it instanceof IntegerLiteralValue) {
      return _toExpressionString((IntegerLiteralValue)it);
    } else if (it instanceof StringLiteralValue) {
      return _toExpressionString((StringLiteralValue)it);
    } else if (it instanceof AggregateFunction) {
      return _toExpressionString((AggregateFunction)it);
    } else if (it instanceof AliasedExpression) {
      return _toExpressionString((AliasedExpression)it);
    } else if (it instanceof BetweenExpression) {
      return _toExpressionString((BetweenExpression)it);
    } else if (it instanceof BinaryExpression) {
      return _toExpressionString((BinaryExpression)it);
    } else if (it instanceof CaseExpression) {
      return _toExpressionString((CaseExpression)it);
    } else if (it instanceof CastFunction) {
      return _toExpressionString((CastFunction)it);
    } else if (it instanceof CollectionFunction) {
      return _toExpressionString((CollectionFunction)it);
    } else if (it instanceof FunctionCall) {
      return _toExpressionString((FunctionCall)it);
    } else if (it instanceof InExpression) {
      return _toExpressionString((InExpression)it);
    } else if (it instanceof LikeExpression) {
      return _toExpressionString((LikeExpression)it);
    } else if (it instanceof LiteralValue) {
      return _toExpressionString((LiteralValue)it);
    } else if (it instanceof MemberOfExpression) {
      return _toExpressionString((MemberOfExpression)it);
    } else if (it instanceof NotExpression) {
      return _toExpressionString((NotExpression)it);
    } else if (it instanceof ParenthesizedExpression) {
      return _toExpressionString((ParenthesizedExpression)it);
    } else if (it instanceof PropertyValue) {
      return _toExpressionString((PropertyValue)it);
    } else if (it instanceof QuantifiedExpression) {
      return _toExpressionString((QuantifiedExpression)it);
    } else if (it instanceof QueryParameterValue) {
      return _toExpressionString((QueryParameterValue)it);
    } else if (it instanceof SubQuery) {
      return _toExpressionString((SubQuery)it);
    } else if (it instanceof TrimFunction) {
      return _toExpressionString((TrimFunction)it);
    } else if (it instanceof UnaryExpression) {
      return _toExpressionString((UnaryExpression)it);
    } else if (it instanceof AltWhenClause) {
      return _toExpressionString((AltWhenClause)it);
    } else if (it instanceof Expression) {
      return _toExpressionString((Expression)it);
    } else if (it instanceof WhenClause) {
      return _toExpressionString((WhenClause)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
