package org.openxma.dsl.generator.impl;

import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import java.util.Arrays;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.dom.model.Service;
import org.openxma.dsl.generator.impl.JsfData;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.impl.Types;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;

@SuppressWarnings("all")
public class JsfDataImpl implements JsfData {
  @Inject
  private Names names;
  
  @Inject
  private Types type;
  
  public CharSequence toDataObjectVars(final EList<DataObjectVariable> variables) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final DataObjectVariable dataObject : variables) {
        {
          boolean _isIsCollection = dataObject.isIsCollection();
          if (_isIsCollection) {
            _builder.append("protected ");
            String _addImport = this.names.addImport(dataObject, "java.util.Collection");
            _builder.append(_addImport, "");
            _builder.append("<");
            ComplexType _type = dataObject.getType();
            Type _pomType = this.type.pomType(_type);
            String _qualifiedName = this.names.getQualifiedName(_pomType);
            _builder.append(_qualifiedName, "");
            _builder.append("> ");
            String _name = dataObject.getName();
            _builder.append(_name, "");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("protected ");
            ComplexType _type_1 = dataObject.getType();
            Type _pomType_1 = this.type.pomType(_type_1);
            String _qualifiedName_1 = this.names.getQualifiedName(_pomType_1);
            _builder.append(_qualifiedName_1, "");
            _builder.append(" ");
            String _name_1 = dataObject.getName();
            _builder.append(_name_1, "");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  public CharSequence toObjectPropertyVars(final EList<ObjectProperty> properties) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final ObjectProperty objProperty : properties) {
        _builder.append("protected ");
        SimpleType _type = objProperty.getType();
        String _qualifiedName = this.names.getQualifiedName(_type);
        _builder.append(_qualifiedName, "");
        _builder.append(" ");
        String _name = objProperty.getName();
        _builder.append(_name, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toInjectedDependencies(final EList dependencies) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Iterable<Service> _filter = Iterables.<Service>filter(dependencies, Service.class);
      for(final Service service : _filter) {
        _builder.append("\t");
        _builder.append("@");
        String _addImport = this.names.addImport(service, "org.springframework.beans.factory.annotation.Autowired");
        _builder.append(_addImport, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("protected ");
        String _qualifiedName = this.names.getQualifiedName(service);
        _builder.append(_qualifiedName, "");
        _builder.append(" ");
        String _name = service.getName();
        String _firstLower = StringExtensions.toFirstLower(_name);
        _builder.append(_firstLower, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toGetterAndSetter(final ObjectProperty objProperty) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    SimpleType _type = objProperty.getType();
    String _qualifiedName = this.names.getQualifiedName(_type);
    _builder.append(_qualifiedName, "");
    _builder.append(" get");
    String _name = objProperty.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("return ");
    String _name_1 = objProperty.getName();
    _builder.append(_name_1, "    ");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    _builder.append("public void set");
    String _name_2 = objProperty.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_2);
    _builder.append(_firstUpper_1, "");
    _builder.append("(");
    SimpleType _type_1 = objProperty.getType();
    String _qualifiedName_1 = this.names.getQualifiedName(_type_1);
    _builder.append(_qualifiedName_1, "");
    _builder.append(" ");
    String _name_3 = objProperty.getName();
    _builder.append(_name_3, "");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("this.");
    String _name_4 = objProperty.getName();
    _builder.append(_name_4, "    ");
    _builder.append(" = ");
    String _name_5 = objProperty.getName();
    _builder.append(_name_5, "    ");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _toGetterAndSetter(final DataObjectVariable dataObject) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isIsCollection = dataObject.isIsCollection();
      if (_isIsCollection) {
        _builder.append("public ");
        String _addImport = this.names.addImport(dataObject, "java.util.Collection");
        _builder.append(_addImport, "");
        _builder.append("<");
        ComplexType _type = dataObject.getType();
        Type _pomType = this.type.pomType(_type);
        String _qualifiedName = this.names.getQualifiedName(_pomType);
        _builder.append(_qualifiedName, "");
        _builder.append("> get");
        String _name = dataObject.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("return ");
        String _name_1 = dataObject.getName();
        _builder.append(_name_1, "    ");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
        _builder.append("public void set");
        String _name_2 = dataObject.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_2);
        _builder.append(_firstUpper_1, "");
        _builder.append("(");
        String _addImport_1 = this.names.addImport(dataObject, "java.util.Collection");
        _builder.append(_addImport_1, "");
        _builder.append("<");
        ComplexType _type_1 = dataObject.getType();
        Type _pomType_1 = this.type.pomType(_type_1);
        String _qualifiedName_1 = this.names.getQualifiedName(_pomType_1);
        _builder.append(_qualifiedName_1, "");
        _builder.append("> ");
        String _name_3 = dataObject.getName();
        _builder.append(_name_3, "");
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("this.");
        String _name_4 = dataObject.getName();
        _builder.append(_name_4, "	");
        _builder.append(" = ");
        String _name_5 = dataObject.getName();
        _builder.append(_name_5, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
      } else {
        _builder.append("public ");
        ComplexType _type_2 = dataObject.getType();
        Type _pomType_2 = this.type.pomType(_type_2);
        String _qualifiedName_2 = this.names.getQualifiedName(_pomType_2);
        _builder.append(_qualifiedName_2, "");
        _builder.append(" get");
        String _name_6 = dataObject.getName();
        String _firstUpper_2 = StringExtensions.toFirstUpper(_name_6);
        _builder.append(_firstUpper_2, "");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("return ");
        String _name_7 = dataObject.getName();
        _builder.append(_name_7, "    ");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
        _builder.append("public void set");
        String _name_8 = dataObject.getName();
        String _firstUpper_3 = StringExtensions.toFirstUpper(_name_8);
        _builder.append(_firstUpper_3, "");
        _builder.append("(");
        ComplexType _type_3 = dataObject.getType();
        Type _pomType_3 = this.type.pomType(_type_3);
        String _qualifiedName_3 = this.names.getQualifiedName(_pomType_3);
        _builder.append(_qualifiedName_3, "");
        _builder.append(" ");
        String _name_9 = dataObject.getName();
        _builder.append(_name_9, "");
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("this.");
        String _name_10 = dataObject.getName();
        _builder.append(_name_10, "	");
        _builder.append(" = ");
        String _name_11 = dataObject.getName();
        _builder.append(_name_11, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  protected CharSequence _getModelPropertyName(final Table table) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = table.getName();
    _builder.append(_name, "");
    _builder.append("TableModel");
    return _builder;
  }
  
  protected CharSequence _getRootNodePropertyName(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    _builder.append(_name, "");
    _builder.append("TreeRootNode");
    return _builder;
  }
  
  protected CharSequence _getSelectedNodePropertyName(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    _builder.append(_name, "");
    _builder.append("TreeSelectedNode");
    return _builder;
  }
  
  protected CharSequence _getModelPropertyNameFirstUpper(final Table table) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = table.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("TableModel");
    return _builder;
  }
  
  protected CharSequence _getRootNodePropertyNameFirstUpper(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("TreeRootNode");
    return _builder;
  }
  
  protected CharSequence _getSelectedNodePropertyNameFirstUpper(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("TreeSelectedNode");
    return _builder;
  }
  
  protected CharSequence _getSelectItemsPropertyName(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    Attribute _attribute = field.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("1SelectItems");
    return _builder;
  }
  
  protected CharSequence _getSelectItemsPropertyNameFirstUpper(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    Attribute _attribute = field.getAttribute();
    String _name = _attribute.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("1SelectItems");
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Tree it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _addImport = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport, "");
    _builder.append(" ");
    CharSequence _rootNodePropertyName = this.getRootNodePropertyName(it);
    _builder.append(_rootNodePropertyName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("protected ");
    String _addImport_1 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport_1, "");
    _builder.append(" ");
    CharSequence _selectedNodePropertyName = this.getSelectedNodePropertyName(it);
    _builder.append(_selectedNodePropertyName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public void set");
    CharSequence _rootNodePropertyNameFirstUpper = this.getRootNodePropertyNameFirstUpper(it);
    _builder.append(_rootNodePropertyNameFirstUpper, "");
    _builder.append("(");
    String _addImport_2 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport_2, "");
    _builder.append(" node) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("this.");
    CharSequence _rootNodePropertyName_1 = this.getRootNodePropertyName(it);
    _builder.append(_rootNodePropertyName_1, "	");
    _builder.append(" = node;  ");
    _builder.newLineIfNotEmpty();
    _builder.append("}\t");
    _builder.newLine();
    _builder.append("public ");
    String _addImport_3 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport_3, "");
    _builder.append(" get");
    CharSequence _rootNodePropertyNameFirstUpper_1 = this.getRootNodePropertyNameFirstUpper(it);
    _builder.append(_rootNodePropertyNameFirstUpper_1, "");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    CharSequence _rootNodePropertyName_2 = this.getRootNodePropertyName(it);
    _builder.append(_rootNodePropertyName_2, "	");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("} ");
    _builder.newLine();
    _builder.append("public void set");
    CharSequence _selectedNodePropertyNameFirstUpper = this.getSelectedNodePropertyNameFirstUpper(it);
    _builder.append(_selectedNodePropertyNameFirstUpper, "");
    _builder.append("(");
    String _addImport_4 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport_4, "");
    _builder.append(" node) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("this.");
    CharSequence _selectedNodePropertyName_1 = this.getSelectedNodePropertyName(it);
    _builder.append(_selectedNodePropertyName_1, "	");
    _builder.append(" = node;  ");
    _builder.newLineIfNotEmpty();
    _builder.append("}\t");
    _builder.newLine();
    _builder.append("public ");
    String _addImport_5 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport_5, "");
    _builder.append(" get");
    CharSequence _selectedNodePropertyNameFirstUpper_1 = this.getSelectedNodePropertyNameFirstUpper(it);
    _builder.append(_selectedNodePropertyNameFirstUpper_1, "");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    CharSequence _selectedNodePropertyName_2 = this.getSelectedNodePropertyName(it);
    _builder.append(_selectedNodePropertyName_2, "	");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("} ");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Table it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _addImport = this.names.addImport(it, "org.openxma.dsl.platform.jsf.model.MapDataModel");
    _builder.append(_addImport, "");
    _builder.append("<");
    PresentableFeature _key = it.getKey();
    Attribute _attribute = _key.getAttribute();
    Type _dataType = _attribute.getDataType();
    String _qualifiedName = this.names.getQualifiedName(_dataType);
    _builder.append(_qualifiedName, "");
    _builder.append(",");
    DataObjectVariable _object = it.getObject();
    ComplexType _type = _object.getType();
    Type _pomType = this.type.pomType(_type);
    String _qualifiedName_1 = this.names.getQualifiedName(_pomType);
    _builder.append(_qualifiedName_1, "");
    _builder.append("> ");
    CharSequence _modelPropertyName = this.getModelPropertyName(it);
    _builder.append(_modelPropertyName, "");
    _builder.append(" = new MapDataModel<");
    PresentableFeature _key_1 = it.getKey();
    Attribute _attribute_1 = _key_1.getAttribute();
    Type _dataType_1 = _attribute_1.getDataType();
    String _qualifiedName_2 = this.names.getQualifiedName(_dataType_1);
    _builder.append(_qualifiedName_2, "");
    _builder.append(",");
    DataObjectVariable _object_1 = it.getObject();
    ComplexType _type_1 = _object_1.getType();
    Type _pomType_1 = this.type.pomType(_type_1);
    String _qualifiedName_3 = this.names.getQualifiedName(_pomType_1);
    _builder.append(_qualifiedName_3, "");
    _builder.append(">() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public ");
    PresentableFeature _key_2 = it.getKey();
    Attribute _attribute_2 = _key_2.getAttribute();
    Type _dataType_2 = _attribute_2.getDataType();
    String _qualifiedName_4 = this.names.getQualifiedName(_dataType_2);
    _builder.append(_qualifiedName_4, "	");
    _builder.append(" getRowKey(");
    DataObjectVariable _object_2 = it.getObject();
    ComplexType _type_2 = _object_2.getType();
    Type _pomType_2 = this.type.pomType(_type_2);
    String _qualifiedName_5 = this.names.getQualifiedName(_pomType_2);
    _builder.append(_qualifiedName_5, "	");
    _builder.append(" row) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return row.get");
    PresentableFeature _key_3 = it.getKey();
    Attribute _attribute_3 = _key_3.getAttribute();
    String _name = _attribute_3.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "		");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("};");
    _builder.newLine();
    _builder.newLine();
    _builder.append("public void set");
    CharSequence _modelPropertyNameFirstUpper = this.getModelPropertyNameFirstUpper(it);
    _builder.append(_modelPropertyNameFirstUpper, "");
    _builder.append("(");
    String _addImport_1 = this.names.addImport(it, "org.openxma.dsl.platform.jsf.model.MapDataModel");
    _builder.append(_addImport_1, "");
    _builder.append(" model) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("this.");
    CharSequence _modelPropertyName_1 = this.getModelPropertyName(it);
    _builder.append(_modelPropertyName_1, "	");
    _builder.append(" = model;  ");
    _builder.newLineIfNotEmpty();
    _builder.append("}\t");
    _builder.newLine();
    _builder.newLine();
    _builder.append("public ");
    String _addImport_2 = this.names.addImport(it, "org.openxma.dsl.platform.jsf.model.MapDataModel");
    _builder.append(_addImport_2, "");
    _builder.append(" get");
    CharSequence _modelPropertyNameFirstUpper_1 = this.getModelPropertyNameFirstUpper(it);
    _builder.append(_modelPropertyNameFirstUpper_1, "");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    CharSequence _modelPropertyName_2 = this.getModelPropertyName(it);
    _builder.append(_modelPropertyName_2, "	");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("} ");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final CustomizeableField it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Attribute _attribute = it.getAttribute();
      DataTypeAndTypeParameter _type = _attribute.getType();
      Type _dataType = _type.getDataType();
      if ((_dataType instanceof EnumType)) {
        _builder.append("protected ");
        String _addImport = this.names.addImport(it, "java.util.List");
        _builder.append(_addImport, "");
        _builder.append("<");
        String _addImport_1 = this.names.addImport(it, "javax.faces.model.SelectItem");
        _builder.append(_addImport_1, "");
        _builder.append("> ");
        CharSequence _selectItemsPropertyName = this.getSelectItemsPropertyName(it);
        _builder.append(_selectItemsPropertyName, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
        _builder.append("public void set");
        CharSequence _selectItemsPropertyNameFirstUpper = this.getSelectItemsPropertyNameFirstUpper(it);
        _builder.append(_selectItemsPropertyNameFirstUpper, "");
        _builder.append("(");
        String _addImport_2 = this.names.addImport(it, "java.util.List");
        _builder.append(_addImport_2, "");
        _builder.append(" items) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("this.");
        CharSequence _selectItemsPropertyName_1 = this.getSelectItemsPropertyName(it);
        _builder.append(_selectItemsPropertyName_1, "	");
        _builder.append(" = items;  ");
        _builder.newLineIfNotEmpty();
        _builder.append("}\t");
        _builder.newLine();
        _builder.newLine();
        _builder.append("public ");
        String _addImport_3 = this.names.addImport(it, "java.util.List");
        _builder.append(_addImport_3, "");
        _builder.append(" get");
        CharSequence _selectItemsPropertyNameFirstUpper_1 = this.getSelectItemsPropertyNameFirstUpper(it);
        _builder.append(_selectItemsPropertyNameFirstUpper_1, "");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("return ");
        CharSequence _selectItemsPropertyName_2 = this.getSelectItemsPropertyName(it);
        _builder.append(_selectItemsPropertyName_2, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("} ");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final EObject o) {
    return null;
  }
  
  protected CharSequence _toJsfDataModel(final XmadslPage page) {
    Content _content = page.getContent();
    CharSequence _jsfDataModel = _content==null?(CharSequence)null:this.toJsfDataModel(_content);
    return _jsfDataModel;
  }
  
  protected CharSequence _toJsfDataModel(final XmadslComposite composite) {
    Content _content = composite.getContent();
    CharSequence _jsfDataModel = _content==null?(CharSequence)null:this.toJsfDataModel(_content);
    return _jsfDataModel;
  }
  
  protected CharSequence _toJsfDataModel(final ReferencedXMAComposite composite) {
    Content _content = composite.getContent();
    CharSequence _jsfDataModel = this.toJsfDataModel(_content);
    return _jsfDataModel;
  }
  
  protected CharSequence _toJsfDataModel(final SetOfGuiElements composite) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = composite.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final TabFolder tabFolder) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = tabFolder.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final TabPage page) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = page.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Content content) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = content.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toGetterAndSetter(final EObject dataObject) {
    if (dataObject instanceof DataObjectVariable) {
      return _toGetterAndSetter((DataObjectVariable)dataObject);
    } else if (dataObject instanceof ObjectProperty) {
      return _toGetterAndSetter((ObjectProperty)dataObject);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(dataObject).toString());
    }
  }
  
  public CharSequence getModelPropertyName(final Table table) {
    {
      return _getModelPropertyName(table);
    }
  }
  
  public CharSequence getRootNodePropertyName(final Tree tree) {
    {
      return _getRootNodePropertyName(tree);
    }
  }
  
  public CharSequence getSelectedNodePropertyName(final Tree tree) {
    {
      return _getSelectedNodePropertyName(tree);
    }
  }
  
  public CharSequence getModelPropertyNameFirstUpper(final Table table) {
    {
      return _getModelPropertyNameFirstUpper(table);
    }
  }
  
  public CharSequence getRootNodePropertyNameFirstUpper(final Tree tree) {
    {
      return _getRootNodePropertyNameFirstUpper(tree);
    }
  }
  
  public CharSequence getSelectedNodePropertyNameFirstUpper(final Tree tree) {
    {
      return _getSelectedNodePropertyNameFirstUpper(tree);
    }
  }
  
  public CharSequence getSelectItemsPropertyName(final CustomizeableField field) {
    {
      return _getSelectItemsPropertyName(field);
    }
  }
  
  public CharSequence getSelectItemsPropertyNameFirstUpper(final CustomizeableField field) {
    {
      return _getSelectItemsPropertyNameFirstUpper(field);
    }
  }
  
  public CharSequence toJsfDataModel(final EObject composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _toJsfDataModel((ReferencedXMAComposite)composite);
    } else if (composite instanceof TabFolder) {
      return _toJsfDataModel((TabFolder)composite);
    } else if (composite instanceof TabPage) {
      return _toJsfDataModel((TabPage)composite);
    } else if (composite instanceof XmadslComposite) {
      return _toJsfDataModel((XmadslComposite)composite);
    } else if (composite instanceof CustomizeableField) {
      return _toJsfDataModel((CustomizeableField)composite);
    } else if (composite instanceof Table) {
      return _toJsfDataModel((Table)composite);
    } else if (composite instanceof Tree) {
      return _toJsfDataModel((Tree)composite);
    } else if (composite instanceof XmadslPage) {
      return _toJsfDataModel((XmadslPage)composite);
    } else if (composite instanceof Content) {
      return _toJsfDataModel((Content)composite);
    } else if (composite instanceof SetOfGuiElements) {
      return _toJsfDataModel((SetOfGuiElements)composite);
    } else if (composite != null) {
      return _toJsfDataModel(composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
}
