package org.openxma.dsl.generator.impl

import com.google.inject.Inject
import org.eclipse.emf.common.util.EList
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.core.model.EnumType
import org.openxma.dsl.dom.model.Service
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.DataObjectVariable
import org.openxma.dsl.pom.model.ObjectProperty
import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.TabFolder
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.TableCombo
import org.openxma.dsl.pom.model.Tree
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.XmadslPage

class JsfDataImpl implements JsfData {
	@Inject
	extension Names names
	@Inject
	extension Types type
	
	override CharSequence toDataObjectVars(EList<DataObjectVariable> variables) { '''
		FOR dataObject : variables
			IF dataObject.isCollection
	protected dataObject.addImport("java.util.Collection")<dataObject.type.pomType.qualifiedName> dataObject.name;
			ELSE
	protected dataObject.type.pomType.qualifiedName dataObject.name;
			ENDIF
		ENDFOR
	'''
	}
	
	override CharSequence toObjectPropertyVars(EList<ObjectProperty> properties) { '''
	FOR objProperty : properties
	protected objProperty.type.qualifiedName objProperty.name;
	ENDFOR
	'''
	}
	
	override CharSequence toInjectedDependencies(EList dependencies) { '''
	FOR service : dependencies.filter(typeof(Service))
		@service.addImport("org.springframework.beans.factory.annotation.Autowired")
	protected service.qualifiedName service.name.toFirstLower();
	ENDFOR
	'''
	}

	def dispatch CharSequence toGetterAndSetter(ObjectProperty objProperty){ '''
    public objProperty.type.qualifiedName getobjProperty.name.toFirstUpper() {
        return objProperty.name;
    }
    public void setobjProperty.name.toFirstUpper(objProperty.type.qualifiedName objProperty.name) {
        this.objProperty.name = objProperty.name;
    }
	'''
	}
	def dispatch CharSequence toGetterAndSetter(DataObjectVariable dataObject){ '''
		IF dataObject.isCollection
	public dataObject.addImport("java.util.Collection")<dataObject.type.pomType.qualifiedName> getdataObject.name.toFirstUpper() {
	    return dataObject.name;
	}
	public void setdataObject.name.toFirstUpper(dataObject.addImport("java.util.Collection")<dataObject.type.pomType.qualifiedName> dataObject.name) {
		this.dataObject.name = dataObject.name;
	}
	ELSE
	public dataObject.type.pomType.qualifiedName getdataObject.name.toFirstUpper() {
	    return dataObject.name;
	}
	public void setdataObject.name.toFirstUpper(dataObject.type.pomType.qualifiedName dataObject.name) {
		this.dataObject.name = dataObject.name;
	}
		ENDIF
		'''
	}
	
	
	def dispatch CharSequence getModelPropertyName(Table table) {
		'''table.nameTableModel'''
	}
	def dispatch CharSequence getRootNodePropertyName(Tree tree) {
		'''tree.nameTreeRootNode'''
	}
	def dispatch CharSequence getSelectedNodePropertyName(Tree tree) {
		'''tree.nameTreeSelectedNode'''
	}
	def dispatch CharSequence getModelPropertyNameFirstUpper(Table table) {
		'''table.name.toFirstUpperTableModel'''
	}
	def dispatch CharSequence getRootNodePropertyNameFirstUpper(Tree tree) {
		'''tree.name.toFirstUpperTreeRootNode'''
	}
	def dispatch CharSequence getSelectedNodePropertyNameFirstUpper(Tree tree) {
		'''tree.name.toFirstUpperTreeSelectedNode'''
	}
	def dispatch CharSequence getSelectItemsPropertyName(CustomizeableField field) {
		'''field.attribute.name1SelectItems'''
	}
	def dispatch CharSequence getSelectItemsPropertyNameFirstUpper(CustomizeableField field) {
		'''field.attribute.name.toFirstUpper1SelectItems'''
	}
	
	def dispatch CharSequence toJsfDataModel(Tree it) { '''
		protected addImport("org.primefaces.model.TreeNode") rootNodePropertyName;
		protected addImport("org.primefaces.model.TreeNode") selectedNodePropertyName;
		
		public void setrootNodePropertyNameFirstUpper(addImport("org.primefaces.model.TreeNode") node) {
			this.rootNodePropertyName = node;  
		}	
		public addImport("org.primefaces.model.TreeNode") getrootNodePropertyNameFirstUpper() {
			return rootNodePropertyName;
		} 
		public void setselectedNodePropertyNameFirstUpper(addImport("org.primefaces.model.TreeNode") node) {
			this.selectedNodePropertyName = node;  
		}	
		public addImport("org.primefaces.model.TreeNode") getselectedNodePropertyNameFirstUpper() {
			return selectedNodePropertyName;
		} 
	'''
	}
	def dispatch CharSequence toJsfDataModel(Table it) { '''
		protected addImport("org.openxma.dsl.platform.jsf.model.MapDataModel")<key.attribute.dataType.qualifiedName,object.type.pomType.qualifiedName> modelPropertyName = new MapDataModel<key.attribute.dataType.qualifiedName,object.type.pomType.qualifiedName>() {
			public key.attribute.dataType.qualifiedName getRowKey(object.type.pomType.qualifiedName row) {
				return row.getkey.attribute.name.toFirstUpper();
			}
		};
		
		public void setmodelPropertyNameFirstUpper(addImport("org.openxma.dsl.platform.jsf.model.MapDataModel") model) {
			this.modelPropertyName = model;  
		}	
		
		public addImport("org.openxma.dsl.platform.jsf.model.MapDataModel") getmodelPropertyNameFirstUpper() {
			return modelPropertyName;
		} 
	'''
	}
	def dispatch CharSequence toJsfDataModel(CustomizeableField it) {
		 '''
		IF (attribute.type.dataType instanceof EnumType)
		protected addImport("java.util.List")<addImport("javax.faces.model.SelectItem")> selectItemsPropertyName;
		
		public void setselectItemsPropertyNameFirstUpper(addImport("java.util.List") items) {
			this.selectItemsPropertyName = items;  
		}	
		
		public addImport("java.util.List") getselectItemsPropertyNameFirstUpper() {
			return selectItemsPropertyName;
		} 
		ENDIF
	'''
	}
	def dispatch CharSequence toJsfDataModel(EObject o) {
		/* nothing  */  
	}
	def dispatch CharSequence toJsfDataModel(XmadslPage page) { 
		page.content?.toJsfDataModel		
	} 
	def dispatch CharSequence toJsfDataModel(XmadslComposite composite) { 
		composite.content?.toJsfDataModel		
	} 
	def dispatch CharSequence toJsfDataModel(ReferencedXMAComposite composite) { 
		composite.content.toJsfDataModel		
	} 
	def dispatch CharSequence toJsfDataModel(SetOfGuiElements composite) {'''
		FOR item : composite.eContents
			item.toJsfDataModel
		ENDFOR
		'''
	} 
	def dispatch CharSequence toJsfDataModel(TabFolder tabFolder) { '''
		FOR item : tabFolder.eContents
			item.toJsfDataModel
		ENDFOR
		'''
	} 
	def dispatch CharSequence toJsfDataModel(TabPage page) { ''' 
		FOR item : page.eContents
			item.toJsfDataModel
		ENDFOR
		'''
	} 
	def dispatch CharSequence toJsfDataModel(Content content) { '''
		FOR item : content.eContents
			item.toJsfDataModel
		ENDFOR
	'''
	} 
}