package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.util.Arrays;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.openxma.dsl.generator.impl.JsfEvents;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.EnterEventMapping;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.InitEventMapping;
import org.openxma.dsl.pom.model.LeaveEventMapping;
import org.openxma.dsl.pom.model.MenuItem;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.XmadslComposite;

@Singleton
@SuppressWarnings("all")
public class JsfEventsImpl implements JsfEvents {
  @Inject
  private Names names;
  
  protected CharSequence _toEventMappings(final GuiElementEventMapping eventMapping) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public void ");
    IGuiElementWithEvent _control = eventMapping.getControl();
    CharSequence _controlName = this.controlName(_control);
    _builder.append(_controlName, "");
    _builder.append("_");
    CharSequence _eventMethodSuffix = this.toEventMethodSuffix(eventMapping);
    _builder.append(_eventMethodSuffix, "");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    {
      EList<EventFunction> _eventFunctions = eventMapping.getEventFunctions();
      for(final EventFunction function : _eventFunctions) {
        {
          boolean _and = false;
          Command _command = function.getCommand();
          boolean _notEquals = (!Objects.equal(_command, null));
          if (!_notEquals) {
            _and = false;
          } else {
            Command _command_1 = function.getCommand();
            boolean _isClientOnly = _command_1.isClientOnly();
            boolean _not = (!_isClientOnly);
            _and = (_notEquals && _not);
          }
          if (_and) {
            _builder.append("\t");
            CharSequence _componentReference = this.componentReference(function);
            _builder.append(_componentReference, "	");
            Command _command_2 = function.getCommand();
            String _name = _command_2.getName();
            _builder.append(_name, "	");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence componentReference(final EventFunction function) {
    CharSequence _xifexpression = null;
    Command _command = function.getCommand();
    EObject _eContainer = _command.eContainer();
    if ((_eContainer instanceof Component)) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("component.");
      _xifexpression = _builder;
    } else {
      StringConcatenation _builder_1 = new StringConcatenation();
      _xifexpression = _builder_1;
    }
    return _xifexpression;
  }
  
  protected CharSequence _toEventMappings(final EventMapping eventMapping) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// uninplemented event mapping for class ");
    Class<? extends EventMapping> _class = eventMapping.getClass();
    String _name = _class.getName();
    _builder.append(_name, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toEventMappings(final EventMappingList eventMappings) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EventMapping> _mapping = eventMappings.getMapping();
      for(final EventMapping event : _mapping) {
        CharSequence _eventMappings = this.toEventMappings(event);
        _builder.append(_eventMappings, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toEventMappings(final EnterEventMapping eventMapping) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  protected CharSequence _toEventMappings(final InitEventMapping eventMapping) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  public CharSequence toEventMethodSuffix(final GuiElementEventMapping eventMapping) {
    String _xifexpression = null;
    EventType _event = eventMapping.getEvent();
    boolean _notEquals = (!Objects.equal(_event, EventType.NULL));
    if (_notEquals) {
      EventType _event_1 = eventMapping.getEvent();
      String _literal = _event_1.getLiteral();
      _xifexpression = _literal;
    } else {
      String _literal_1 = EventType.ON_CLICK.getLiteral();
      _xifexpression = _literal_1;
    }
    return _xifexpression;
  }
  
  protected CharSequence _createChildEventMappings(final XmadslComposite composite, final EventMappingList events) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = composite.eContents();
      for(final EObject c : _eContents) {
        CharSequence _createChildEventMappings = this.createChildEventMappings(c, events);
        _builder.append(_createChildEventMappings, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createChildEventMappings(final Content composite, final EventMappingList events) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = composite.eContents();
      for(final EObject c : _eContents) {
        CharSequence _createChildEventMappings = this.createChildEventMappings(c, events);
        _builder.append(_createChildEventMappings, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createChildEventMappings(final SetOfGuiElements composite, final EventMappingList events) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = composite.eContents();
      for(final EObject c : _eContents) {
        CharSequence _createChildEventMappings = this.createChildEventMappings(c, events);
        _builder.append(_createChildEventMappings, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createChildEventMappings(final Button button, final EventMappingList events) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EventMapping> _mapping = events.getMapping();
      Iterable<GuiElementEventMapping> _filter = Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
      for(final GuiElementEventMapping event : _filter) {
        {
          IGuiElementWithEvent _control = event.getControl();
          boolean _equals = Objects.equal(_control, button);
          if (_equals) {
            _builder.append("public void ");
            String _name = button.getName();
            _builder.append(_name, "");
            _builder.append("_onClick() {");
            _builder.newLineIfNotEmpty();
            _builder.append("// TODO: find target method for this certain event");
            _builder.newLine();
            _builder.append("\t");
            _builder.append("// event");
            _builder.newLine();
            _builder.newLine();
            _builder.append("// TODO: generate eventmappings here");
            _builder.newLine();
            _builder.append("}");
            _builder.newLine();
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _createChildEventMappings(final EObject object, final EventMappingList events) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  public CharSequence toLifecycleMethods(final EventMappingList it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@");
    String _addImport = this.names.addImport(it, "javax.annotation.PostConstruct");
    _builder.append(_addImport, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public void beanInit() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("onInit();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("onEnter();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* PML onLoad event");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("protected void onEnter() {");
    _builder.newLine();
    {
      EList<EventMapping> _mapping = it.getMapping();
      Iterable<EnterEventMapping> _filter = Iterables.<EnterEventMapping>filter(_mapping, EnterEventMapping.class);
      for(final EnterEventMapping mapping : _filter) {
        {
          EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
          for(final EventFunction function : _eventFunctions) {
            {
              Command _command = function.getCommand();
              boolean _isClientOnly = _command.isClientOnly();
              boolean _not = (!_isClientOnly);
              if (_not) {
                _builder.append("        ");
                Command _command_1 = function.getCommand();
                String _name = _command_1.getName();
                _builder.append(_name, "        ");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* PML onLeave event");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("protected void onLeave() {");
    _builder.newLine();
    {
      EList<EventMapping> _mapping_1 = it.getMapping();
      Iterable<LeaveEventMapping> _filter_1 = Iterables.<LeaveEventMapping>filter(_mapping_1, LeaveEventMapping.class);
      for(final LeaveEventMapping mapping_1 : _filter_1) {
        {
          EList<EventFunction> _eventFunctions_1 = mapping_1.getEventFunctions();
          for(final EventFunction function_1 : _eventFunctions_1) {
            {
              Command _command_2 = function_1.getCommand();
              boolean _isClientOnly_1 = _command_2.isClientOnly();
              boolean _not_1 = (!_isClientOnly_1);
              if (_not_1) {
                _builder.append("        ");
                Command _command_3 = function_1.getCommand();
                String _name_1 = _command_3.getName();
                _builder.append(_name_1, "        ");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* PML onInit event");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("protected void onInit() {");
    _builder.newLine();
    {
      EList<EventMapping> _mapping_2 = it.getMapping();
      Iterable<InitEventMapping> _filter_2 = Iterables.<InitEventMapping>filter(_mapping_2, InitEventMapping.class);
      for(final InitEventMapping mapping_2 : _filter_2) {
        {
          EList<EventFunction> _eventFunctions_2 = mapping_2.getEventFunctions();
          for(final EventFunction function_2 : _eventFunctions_2) {
            {
              Command _command_4 = function_2.getCommand();
              boolean _isClientOnly_2 = _command_4.isClientOnly();
              boolean _not_2 = (!_isClientOnly_2);
              if (_not_2) {
                _builder.append("        ");
                Command _command_5 = function_2.getCommand();
                String _name_2 = _command_5.getName();
                _builder.append(_name_2, "        ");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _controlName(final IGuiElementWithEvent control) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<!-- ERROR: no control name for control with mapping -->");
    return _builder;
  }
  
  protected CharSequence _controlName(final MenuItem control) {
    String _name = control.getName();
    return _name;
  }
  
  protected CharSequence _controlName(final Button control) {
    String _name = control.getName();
    return _name;
  }
  
  protected CharSequence _controlName(final Table control) {
    String _name = control.getName();
    return _name;
  }
  
  public CharSequence toEventMappings(final EObject eventMapping) {
    if (eventMapping instanceof GuiElementEventMapping) {
      return _toEventMappings((GuiElementEventMapping)eventMapping);
    } else if (eventMapping instanceof EnterEventMapping) {
      return _toEventMappings((EnterEventMapping)eventMapping);
    } else if (eventMapping instanceof InitEventMapping) {
      return _toEventMappings((InitEventMapping)eventMapping);
    } else if (eventMapping instanceof EventMapping) {
      return _toEventMappings((EventMapping)eventMapping);
    } else if (eventMapping instanceof EventMappingList) {
      return _toEventMappings((EventMappingList)eventMapping);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(eventMapping).toString());
    }
  }
  
  public CharSequence createChildEventMappings(final EObject composite, final EventMappingList events) {
    if (composite instanceof XmadslComposite) {
      return _createChildEventMappings((XmadslComposite)composite, events);
    } else if (composite instanceof Button) {
      return _createChildEventMappings((Button)composite, events);
    } else if (composite instanceof Content) {
      return _createChildEventMappings((Content)composite, events);
    } else if (composite instanceof SetOfGuiElements) {
      return _createChildEventMappings((SetOfGuiElements)composite, events);
    } else if (composite != null) {
      return _createChildEventMappings(composite, events);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite, events).toString());
    }
  }
  
  public CharSequence controlName(final IGuiElementWithEvent control) {
    if (control instanceof Button) {
      return _controlName((Button)control);
    } else if (control instanceof Table) {
      return _controlName((Table)control);
    } else if (control instanceof MenuItem) {
      return _controlName((MenuItem)control);
    } else if (control != null) {
      return _controlName(control);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(control).toString());
    }
  }
}
