package org.openxma.dsl.generator.impl

import com.google.inject.Inject
import com.google.inject.Singleton
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.pom.model.Button
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.EnterEventMapping
import org.openxma.dsl.pom.model.EventFunction
import org.openxma.dsl.pom.model.EventMapping
import org.openxma.dsl.pom.model.EventMappingList
import org.openxma.dsl.pom.model.EventType
import org.openxma.dsl.pom.model.GuiElementEventMapping
import org.openxma.dsl.pom.model.IGuiElementWithEvent
import org.openxma.dsl.pom.model.InitEventMapping
import org.openxma.dsl.pom.model.MenuItem
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.LeaveEventMapping

@Singleton
class JsfEventsImpl implements JsfEvents {
	@Inject
	extension Names names
	
	
	def dispatch CharSequence toEventMappings(GuiElementEventMapping eventMapping) {'''
		public void eventMapping.control.controlName_eventMapping.toEventMethodSuffix() {
			FOR function : eventMapping.eventFunctions
				IF function.command != null && !function.command.clientOnly
				function.componentReferencefunction.command.name();
				ENDIF
			ENDFOR
		}
'''
	}
	override CharSequence componentReference(EventFunction function) {
		if (function.command.eContainer instanceof Component)
			'''component.'''
		else
			''''''
	}
	def dispatch CharSequence toEventMappings(EventMapping eventMapping) {'''
		// uninplemented event mapping for class eventMapping.getClass().name
	'''
	}
	def dispatch CharSequence toEventMappings(EventMappingList eventMappings) {'''
	FOR event : eventMappings.mapping
		event.toEventMappings
	ENDFOR
'''
	}
	
	def dispatch CharSequence toEventMappings(EnterEventMapping eventMapping) {'''
'''
	}
	def dispatch CharSequence toEventMappings(InitEventMapping eventMapping) {'''
'''
	}
	
	override CharSequence toEventMethodSuffix(GuiElementEventMapping eventMapping) {
		if (eventMapping.event != EventType::NULL)
			eventMapping.event.literal
		else
			EventType::ON_CLICK.literal
	}
	
	def dispatch CharSequence createChildEventMappings(XmadslComposite composite, EventMappingList events) {'''
		FOR c : composite.eContents
			c.createChildEventMappings(events)
		ENDFOR
''' }
	def dispatch CharSequence createChildEventMappings(Content composite, EventMappingList events) {'''
		FOR c : composite.eContents
			c.createChildEventMappings(events)
		ENDFOR
''' }
	def dispatch CharSequence createChildEventMappings(SetOfGuiElements composite, EventMappingList events) {'''
		FOR c : composite.eContents
			c.createChildEventMappings(events)
		ENDFOR
''' }
	def dispatch CharSequence createChildEventMappings(Button button, EventMappingList events) {'''
		FOR event : events.mapping.filter(typeof(GuiElementEventMapping))
			IF event.control == button
			public void button.name_onClick() {
			// TODO: find target method for this certain event
				// event
			
			// TODO: generate eventmappings here
			}
			ENDIF
		ENDFOR
''' }
	def dispatch CharSequence createChildEventMappings(EObject object, EventMappingList events) {'''
''' }

	override CharSequence toLifecycleMethods(EventMappingList it) {
	'''
	@addImport("javax.annotation.PostConstruct")
	public void beanInit() {
		onInit();
		onEnter();
	}
	
	/**
	 * PML onLoad event
	 */
	protected void onEnter() {
        FOR mapping : mapping.filter(typeof(EnterEventMapping))
        FOR function : mapping.eventFunctions
        IF !function.command.clientOnly
        function.command.name();
        ENDIF    
        ENDFOR    
        ENDFOR    
	}
	
	/**
	 * PML onLeave event
	 */
	protected void onLeave() {
        FOR mapping : mapping.filter(typeof(LeaveEventMapping))
        FOR function : mapping.eventFunctions
        IF !function.command.clientOnly
        function.command.name();
        ENDIF    
        ENDFOR    
        ENDFOR    
	}
	
	/**
	 * PML onInit event
	 */
	protected void onInit() {
        FOR mapping : mapping.filter(typeof(InitEventMapping))
        FOR function : mapping.eventFunctions
        IF !function.command.clientOnly
        function.command.name();
        ENDIF    
        ENDFOR    
        ENDFOR 
	}
	'''
	/*
	@addImport("org.apache.myfaces.orchestra.viewController.annotations.PreProcess")
	public void preProcess() {
		
	}
	@addImport("org.apache.myfaces.orchestra.viewController.annotations.PreRenderView")
	public void preRenderView() {
	
	}*/
	}
	
	
	def dispatch CharSequence controlName(IGuiElementWithEvent control) {
		'''<!-- ERROR: no control name for control with mapping -->'''
	}
	
	def dispatch CharSequence controlName(MenuItem control) {
		control.name
	}
	def dispatch CharSequence controlName(Button control) {
		control.name
	}
	def dispatch CharSequence controlName(Table control) {
		control.name
	}
	
}