package org.openxma.dsl.generator.impl


import com.google.inject.Singleton
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.XmadslPage
import org.openxma.dsl.pom.model.ObjectProperty
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.TableColumn
import org.openxma.dsl.pom.model.Command
import java.beans.Introspector

@Singleton
class JsfNamesImpl implements JsfNames {
	
	def dispatch CharSequence getJSClass(Component component) {
		component.xmaComponent.namClass
	} 
	def dispatch CharSequence getJSClass(XmadslPage page) { 
		page.name
	}
	
	override CharSequence asIdentifier(String s) {
		s +	(Math::random).toString.substring(2)
	}
	
	override CharSequence getBean(XmadslPage page) { 
		page.name.toFirstLower
	}
	override CharSequence getDialogWidgetVar(XmadslPage page) { 
		page.name.toFirstLower + "Var"

	}
	def dispatch CharSequence getWidgetVar(EObject page) { 
		throw new RuntimeException("No widget var for this element type!");
	}
	def dispatch CharSequence getWidgetVar(Table table) {
		table.parentPage.JSClass + "_" + table.name + "WidgetVar"
	}
	def dispatch CharSequence getWidgetVar(Component component) { 
		component.xmaComponent.namClass.toFirstLower
	}
	def dispatch CharSequence getWidgetVar(XmadslPage page) { 
		page.name.toFirstLower
	}
	def dispatch CharSequence getWidgetVar(ObjectProperty property) {
		property.parentPage.JSClass + "_" + property.name + "Prop"
	}
	def dispatch CharSequence getWidgetVar(Command command) {
		if (command.eContainer instanceof Component)
			(command.eContainer as Component).JSClass + "_" + command.name + "Cmd"
		else
			command.parentPage.JSClass + "_" + command.name + "Cmd"
	}
	
	
	def dispatch CharSequence referencedBean(Component component) { 
		component.xmaComponent.namClass.toFirstLower
	}
	def dispatch CharSequence referencedBean(XmadslPage page) {
		page.name.toFirstLower
	}
	
	override Component component(XmadslPage page) {
		page.eContainer as Component
	}
	override XmadslPage mainPage(Component component) {
		component.pages?.head as XmadslPage
	}
	
	
	override XmadslPage getParentPage(TableColumn tableColumn) {
		var EObject parent = tableColumn.eContainer
		do {
			parent = parent.eContainer
		} while (!(parent instanceof XmadslPage))
		parent as XmadslPage
	}
	override XmadslPage getParentPage(EObject container) {
		var EObject parent = container
		while (!(parent instanceof XmadslPage))
			parent = parent.eContainer
		parent as XmadslPage
	}
	
	/**
	 * Takes care of JSF specific capitalization
	 * @see http://www.rqna.net/qna/uwzkp-jsf-2-managed-beans-property-not-found.html
	 * @see http://www.oracle.com/technetwork/java/javase/documentation/spec-136004.html
	 */
	override getFaceletName(String n) {
		Introspector::decapitalize(n.toFirstUpper)
	}
}