package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Sets;
import com.google.inject.Singleton;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import javax.inject.Inject;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.util.SimpleAttributeResolver;
import org.eclipse.xtext.xbase.lib.Functions.Function0;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.model.TypeDefinition;
import org.openxma.dsl.dom.model.ApplicationSession;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.Service;
import org.openxma.dsl.generator.helper.JavaUtilExtension;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.impl.Types;
import org.openxma.dsl.generator.lib.ImportAdapter;

@Singleton
@SuppressWarnings("all")
public class NamesImpl implements Names {
  private final Set<String> _qualifiedNames = new Function0<Set<String>>() {
    public Set<String> apply() {
      HashSet<String> _newHashSet = Sets.<String>newHashSet();
      return _newHashSet;
    }
  }.apply();
  
  public Set<String> getQualifiedNames() {
    return this._qualifiedNames;
  }
  
  @Inject
  private Types _types;
  
  public String toVariableName(final EObject eObject) {
    String _xblockexpression = null;
    {
      String name = SimpleAttributeResolver.NAME_RESOLVER.apply(eObject);
      String _variableName = name==null?(String)null:this.toVariableName(name);
      _xblockexpression = (_variableName);
    }
    return _xblockexpression;
  }
  
  public String toVariableName(final String name) {
    String _variableName = JavaUtilExtension.toVariableName(name);
    return _variableName;
  }
  
  protected String _getQualifiedName(final DataTypeAndTypeParameter it) {
    Type _dataType = it.getDataType();
    String _qualifiedName = this.getQualifiedName(_dataType);
    return _qualifiedName;
  }
  
  protected String _getQualifiedName(final EnumType it) {
    String _packageName = this.getPackageName(it);
    String _plus = (_packageName + ".");
    String _name = it.getName();
    String _plus_1 = (_plus + _name);
    String _addImport = this.addImport(it, _plus_1);
    return _addImport;
  }
  
  protected String _getQualifiedName(final SimpleType it) {
    String _xifexpression = null;
    boolean _and = false;
    boolean _isCustom = it.isCustom();
    if (!_isCustom) {
      _and = false;
    } else {
      TypeDefinition _typeDefinition = it.getTypeDefinition();
      DataTypeAndTypeParameter _definition = _typeDefinition.getDefinition();
      boolean _notEquals = (!Objects.equal(_definition, null));
      _and = (_isCustom && _notEquals);
    }
    if (_and) {
      String _packageName = this.getPackageName(it);
      String _plus = (_packageName + ".");
      String _name = it.getName();
      String _plus_1 = (_plus + _name);
      String _addImport = this.addImport(it, _plus_1);
      _xifexpression = _addImport;
    } else {
      String _instanceTypeFullyQualifiedName = it.getInstanceTypeFullyQualifiedName();
      String _addImport_1 = this.addImport(it, _instanceTypeFullyQualifiedName);
      _xifexpression = _addImport_1;
    }
    return _xifexpression;
  }
  
  protected String _getQualifiedName(final ModelElement it) {
    String _packageName = this.getPackageName(it);
    String _plus = (_packageName + ".");
    String _name = it.getName();
    String _plus_1 = (_plus + _name);
    String _addImport = this.addImport(it, _plus_1);
    return _addImport;
  }
  
  public String getPackageName(final ModelElement modelElement) {
    String _xblockexpression = null;
    {
      final Model model = this._types.model(modelElement);
      String _switchResult = null;
      boolean _matched = false;
      if (!_matched) {
        if (modelElement instanceof Dao) {
          final Dao _dao = (Dao)modelElement;
          _matched=true;
          _switchResult = ".dao";
        }
      }
      if (!_matched) {
        if (modelElement instanceof DataView) {
          final DataView _dataView = (DataView)modelElement;
          _matched=true;
          _switchResult = ".dto";
        }
      }
      if (!_matched) {
        if (modelElement instanceof Entity) {
          final Entity _entity = (Entity)modelElement;
          _matched=true;
          _switchResult = ".model";
        }
      }
      if (!_matched) {
        if (modelElement instanceof Service) {
          final Service _service = (Service)modelElement;
          _matched=true;
          _switchResult = ".service";
        }
      }
      if (!_matched) {
        if (modelElement instanceof ApplicationSession) {
          final ApplicationSession _applicationSession = (ApplicationSession)modelElement;
          _matched=true;
          _switchResult = ".context";
        }
      }
      if (!_matched) {
        if (modelElement instanceof SimpleType) {
          final SimpleType _simpleType = (SimpleType)modelElement;
          boolean _isCustom = _simpleType.isCustom();
          if (_isCustom) {
            _matched=true;
            _switchResult = ".valueobject";
          }
        }
      }
      if (!_matched) {
        _switchResult = "";
      }
      String packageSuffix = _switchResult;
      String _xifexpression = null;
      String _name = model.getName();
      boolean _endsWith = _name.endsWith(packageSuffix);
      if (_endsWith) {
        return model.getName();
      } else {
        String _name_1 = model.getName();
        String _plus = (_name_1 + packageSuffix);
        _xifexpression = _plus;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public String getClassPackageName(final ModelElement it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Entity) {
        final Entity _entity = (Entity)it;
        _matched=true;
        String _packageName = this.getPackageName(_entity);
        String _plus = (_packageName + ".impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Dao) {
        final Dao _dao = (Dao)it;
        _matched=true;
        String _packageName = this.getPackageName(_dao);
        String _plus = (_packageName + ".impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Service) {
        final Service _service = (Service)it;
        _matched=true;
        String _packageName = this.getPackageName(_service);
        String _plus = (_packageName + ".impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof ApplicationSession) {
        final ApplicationSession _applicationSession = (ApplicationSession)it;
        _matched=true;
        String _packageName = this.getPackageName(_applicationSession);
        String _plus = (_packageName + ".impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      String _packageName = this.getPackageName(it);
      _switchResult = _packageName;
    }
    return _switchResult;
  }
  
  public String getGenInterfaceName(final ModelElement it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Entity) {
        final Entity _entity = (Entity)it;
        _matched=true;
        String _name = _entity.getName();
        String _plus = (_name + "Gen");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Dao) {
        final Dao _dao = (Dao)it;
        _matched=true;
        String _name = _dao.getName();
        String _plus = (_name + "Gen");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Service) {
        final Service _service = (Service)it;
        _matched=true;
        String _name = _service.getName();
        String _plus = (_name + "Gen");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof ApplicationSession) {
        final ApplicationSession _applicationSession = (ApplicationSession)it;
        _matched=true;
        String _name = _applicationSession.getName();
        String _plus = (_name + "Gen");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      String _name = it.getName();
      _switchResult = _name;
    }
    return _switchResult;
  }
  
  public String getInterfaceName(final ModelElement it) {
    String _name = it.getName();
    return _name;
  }
  
  public String getGenClassName(final ModelElement it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Entity) {
        final Entity _entity = (Entity)it;
        _matched=true;
        String _name = _entity.getName();
        String _plus = (_name + "GenImpl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Dao) {
        final Dao _dao = (Dao)it;
        _matched=true;
        String _name = _dao.getName();
        String _plus = (_name + "GenImpl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Service) {
        final Service _service = (Service)it;
        _matched=true;
        String _name = _service.getName();
        String _plus = (_name + "GenImpl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof ApplicationSession) {
        final ApplicationSession _applicationSession = (ApplicationSession)it;
        _matched=true;
        String _name = _applicationSession.getName();
        String _plus = (_name + "GenImpl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof DataView) {
        final DataView _dataView = (DataView)it;
        _matched=true;
        String _name = _dataView.getName();
        String _plus = (_name + "Gen");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      String _name = it.getName();
      _switchResult = _name;
    }
    return _switchResult;
  }
  
  public String getClassName(final ModelElement it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Entity) {
        final Entity _entity = (Entity)it;
        _matched=true;
        String _name = _entity.getName();
        String _plus = (_name + "Impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Dao) {
        final Dao _dao = (Dao)it;
        _matched=true;
        String _name = _dao.getName();
        String _plus = (_name + "Impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof Service) {
        final Service _service = (Service)it;
        _matched=true;
        String _name = _service.getName();
        String _plus = (_name + "Impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof ApplicationSession) {
        final ApplicationSession _applicationSession = (ApplicationSession)it;
        _matched=true;
        String _name = _applicationSession.getName();
        String _plus = (_name + "Impl");
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      String _name = it.getName();
      _switchResult = _name;
    }
    return _switchResult;
  }
  
  public String getQualifiedGenInterfaceName(final ModelElement it) {
    String _packageName = this.getPackageName(it);
    String _plus = (_packageName + ".");
    String _genInterfaceName = this.getGenInterfaceName(it);
    String _plus_1 = (_plus + _genInterfaceName);
    return _plus_1;
  }
  
  public String getQualifiedInterfaceName(final ModelElement it) {
    String _packageName = this.getPackageName(it);
    String _plus = (_packageName + ".");
    String _interfaceName = this.getInterfaceName(it);
    String _plus_1 = (_plus + _interfaceName);
    return _plus_1;
  }
  
  public String getQualifiedGenClassName(final ModelElement it) {
    String _classPackageName = this.getClassPackageName(it);
    String _plus = (_classPackageName + ".");
    String _genClassName = this.getGenClassName(it);
    String _plus_1 = (_plus + _genClassName);
    return _plus_1;
  }
  
  public String getQualifiedClassName(final ModelElement it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof SimpleType) {
        final SimpleType _simpleType = (SimpleType)it;
        _matched=true;
        String _xifexpression = null;
        boolean _and = false;
        boolean _isCustom = _simpleType.isCustom();
        if (!_isCustom) {
          _and = false;
        } else {
          TypeDefinition _typeDefinition = _simpleType.getTypeDefinition();
          DataTypeAndTypeParameter _definition = _typeDefinition.getDefinition();
          boolean _notEquals = (!Objects.equal(_definition, null));
          _and = (_isCustom && _notEquals);
        }
        if (_and) {
          String _packageName = this.getPackageName(_simpleType);
          String _plus = (_packageName + ".");
          String _name = _simpleType.getName();
          String _plus_1 = (_plus + _name);
          _xifexpression = _plus_1;
        } else {
          String _instanceTypeFullyQualifiedName = _simpleType.getInstanceTypeFullyQualifiedName();
          _xifexpression = _instanceTypeFullyQualifiedName;
        }
        _switchResult = _xifexpression;
      }
    }
    if (!_matched) {
      String _classPackageName = this.getClassPackageName(it);
      String _plus = (_classPackageName + ".");
      String _className = this.getClassName(it);
      String _plus_1 = (_plus + _className);
      _switchResult = _plus_1;
    }
    return _switchResult;
  }
  
  public String addImport(final EObject eObject, final String qualifiedName) {
    String _addImport = ImportAdapter.addImport(eObject, qualifiedName);
    return _addImport;
  }
  
  public String getQualifiedName(final EObject it) {
    if (it instanceof EnumType) {
      return _getQualifiedName((EnumType)it);
    } else if (it instanceof SimpleType) {
      return _getQualifiedName((SimpleType)it);
    } else if (it instanceof DataTypeAndTypeParameter) {
      return _getQualifiedName((DataTypeAndTypeParameter)it);
    } else if (it instanceof ModelElement) {
      return _getQualifiedName((ModelElement)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
