package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.common.collect.Iterators;
import com.google.common.collect.Lists;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.common.Strings;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.AliasedExpression;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeGroup;
import org.openxma.dsl.dom.model.AttributeSortOrder;
import org.openxma.dsl.dom.model.CallInputParameter;
import org.openxma.dsl.dom.model.CallOutputParameter;
import org.openxma.dsl.dom.model.CallableStatement;
import org.openxma.dsl.dom.model.Column;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DataBaseConstraint;
import org.openxma.dsl.dom.model.DataBaseConstraintType;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.DeleteStatement;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.Expression;
import org.openxma.dsl.dom.model.Join;
import org.openxma.dsl.dom.model.Operation;
import org.openxma.dsl.dom.model.Parameter;
import org.openxma.dsl.dom.model.PropertyValue;
import org.openxma.dsl.dom.model.QlStatement;
import org.openxma.dsl.dom.model.QueryOperation;
import org.openxma.dsl.dom.model.QueryParameter;
import org.openxma.dsl.dom.model.QueryParameterReference;
import org.openxma.dsl.dom.model.QueryParameterValue;
import org.openxma.dsl.dom.model.SelectProperties;
import org.openxma.dsl.dom.model.UpdateStatement;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.GeneratorConfigurationExtension;
import org.openxma.dsl.generator.helper.NameExtension;
import org.openxma.dsl.generator.helper.SimpleTypeExtension;
import org.openxma.dsl.generator.helper.ValidatorExtension;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;
import org.openxma.dsl.generator.lib.Index;
import org.openxma.dsl.generator.lib.IterableExtensions2;

@SuppressWarnings("all")
public class RepositoryGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<Dao> _filter = Iterables.<Dao>filter(_elements, Dao.class);
    final Procedure1<Dao> _function = new Procedure1<Dao>() {
        public void apply(final Dao it) {
          String _qualifiedGenInterfaceName = RepositoryGenerator.this._names.getQualifiedGenInterfaceName(it);
          String _javaFilePath = RepositoryGenerator.this._files.toJavaFilePath(_qualifiedGenInterfaceName);
          CharSequence _genInterface = RepositoryGenerator.this.toGenInterface(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genInterface);
          String _qualifiedInterfaceName = RepositoryGenerator.this._names.getQualifiedInterfaceName(it);
          String _javaFilePath_1 = RepositoryGenerator.this._files.toJavaFilePath(_qualifiedInterfaceName);
          CharSequence _interface = RepositoryGenerator.this.toInterface(it);
          fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _interface);
          String _qualifiedGenClassName = RepositoryGenerator.this._names.getQualifiedGenClassName(it);
          String _javaFilePath_2 = RepositoryGenerator.this._files.toJavaFilePath(_qualifiedGenClassName);
          CharSequence _genClass = RepositoryGenerator.this.toGenClass(it);
          fsa.generateFile(_javaFilePath_2, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _qualifiedClassName = RepositoryGenerator.this._names.getQualifiedClassName(it);
          String _javaFilePath_3 = RepositoryGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _class = RepositoryGenerator.this.toClass(it);
          fsa.generateFile(_javaFilePath_3, GeneratorConstants.OUTLET_JAVA, _class);
        }
      };
    IterableExtensions.<Dao>forEach(_filter, _function);
  }
  
  public CharSequence toClass(final Dao it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.springframework.stereotype.Repository;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("@Repository(\"");
    String _variableName = this._names.toVariableName(it);
    _builder.append(_variableName, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _className = this._names.getClassName(it);
    _builder.append(_className, "");
    _builder.append(" extends ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      EList<Operation> _operations = it.getOperations();
      final Function1<Operation,Boolean> _function = new Function1<Operation,Boolean>() {
          public Boolean apply(final Operation e) {
            String _expression = e.getExpression();
            boolean _equals = Objects.equal(_expression, null);
            return Boolean.valueOf(_equals);
          }
        };
      Iterable<Operation> _filter = IterableExtensions.<Operation>filter(_operations, _function);
      for(final Operation operation : _filter) {
        CharSequence _daoOperationSignature = this.daoOperationSignature(operation, "public");
        _builder.append(_daoOperationSignature, "");
        _builder.append(" {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("throw new IllegalStateException(\"not implemented\");");
        _builder.newLine();
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenInterface(final Dao it) {
    CharSequence _xblockexpression = null;
    {
      Entity _entity = it.getEntity();
      EList<Attribute> _requiredReferences = _entity.getRequiredReferences();
      final List<Attribute> notNullFeatures = IterableExtensions.<Attribute>toList(_requiredReferences);
      Entity _entity_1 = it.getEntity();
      EList<Attribute> _requiredAttributes = _entity_1.getRequiredAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isIdOrVersionAttribute = RepositoryGenerator.this._features.isIdOrVersionAttribute(it);
            return Boolean.valueOf(_isIdOrVersionAttribute);
          }
        };
      Iterable<Attribute> _reject = IterableExtensions2.<Attribute>reject(_requiredAttributes, _function);
      Iterables.<Attribute>addAll(notNullFeatures, _reject);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("package ");
      String _packageName = this._names.getPackageName(it);
      _builder.append(_packageName, "");
      _builder.append(";");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("import org.openxma.dsl.platform.dao.*;");
      _builder.newLine();
      String _addImportsHere = ImportAdapter.addImportsHere(it);
      _builder.append(_addImportsHere, "");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("/**");
      _builder.newLine();
      CharSequence _documentationLines = this._features.getDocumentationLines(it);
      _builder.append(_documentationLines, "");
      _builder.newLineIfNotEmpty();
      _builder.append(" ");
      _builder.append("* Represents a Data Access Object, as defined in the");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* to abstract and encapsulate all access to entity ");
      String _name = it.getName();
      _builder.append(_name, " ");
      _builder.append(".");
      _builder.newLineIfNotEmpty();
      _builder.append(" ");
      _builder.append("* @see ");
      String _qualifiedName = this._names.getQualifiedName(it);
      _builder.append(_qualifiedName, " ");
      _builder.newLineIfNotEmpty();
      _builder.append(" ");
      _builder.append("*/");
      _builder.newLine();
      _builder.append("public interface ");
      String _genInterfaceName = this._names.getGenInterfaceName(it);
      _builder.append(_genInterfaceName, "");
      _builder.append(" extends GenericDao<");
      Entity _entity_2 = it.getEntity();
      String _qualifiedName_1 = this._names.getQualifiedName(_entity_2);
      _builder.append(_qualifiedName_1, "");
      _builder.append(",");
      String _idClassName = this.getIdClassName(it);
      _builder.append(_idClassName, "");
      _builder.append(">, EntityFactory<");
      Entity _entity_3 = it.getEntity();
      String _qualifiedName_2 = this._names.getQualifiedName(_entity_3);
      _builder.append(_qualifiedName_2, "");
      _builder.append("> {");
      _builder.newLineIfNotEmpty();
      {
        Entity _entity_4 = it.getEntity();
        EList<AttributeGroup> _sortOrders = _entity_4.getSortOrders();
        boolean _isEmpty = _sortOrders.isEmpty();
        boolean _not = (!_isEmpty);
        if (_not) {
          _builder.append("\t");
          _builder.append("/**");
          _builder.newLine();
          _builder.append("\t");
          _builder.append(" ");
          _builder.append("* Sort order constants");
          _builder.newLine();
          _builder.append("\t");
          _builder.append(" ");
          _builder.append("*/");
          _builder.newLine();
        }
      }
      {
        Entity _entity_5 = it.getEntity();
        EList<AttributeGroup> _sortOrders_1 = _entity_5.getSortOrders();
        for(final AttributeGroup sortOrder : _sortOrders_1) {
          _builder.append("\t");
          _builder.append("static final SortOrderSpecification ");
          String _constantName = this.getConstantName(sortOrder);
          _builder.append(_constantName, "	");
          _builder.append(" = SortOrderSpecification.create(\"");
          Entity _entity_6 = it.getEntity();
          String _name_1 = _entity_6.getName();
          _builder.append(_name_1, "	");
          _builder.append(".");
          String _name_2 = sortOrder.getName();
          _builder.append(_name_2, "	");
          _builder.append("\")");
          {
            EList<AttributeSortOrder> _attributes = sortOrder.getAttributes();
            for(final AttributeSortOrder orderDefinition : _attributes) {
              {
                boolean _isAsc = orderDefinition.isAsc();
                if (_isAsc) {
                  _builder.append(".asc(\"");
                  Attribute _attribute = orderDefinition.getAttribute();
                  String _name_3 = _attribute.getName();
                  _builder.append(_name_3, "	");
                  _builder.append("\")");
                } else {
                  _builder.append(" .desc(\"");
                  Attribute _attribute_1 = orderDefinition.getAttribute();
                  String _name_4 = _attribute_1.getName();
                  _builder.append(_name_4, "	");
                  _builder.append("\")");
                }
              }
            }
          }
          _builder.append(";");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        EList<QueryOperation> _queryOperation = it.getQueryOperation();
        boolean _isEmpty_1 = _queryOperation.isEmpty();
        boolean _not_1 = (!_isEmpty_1);
        if (_not_1) {
          _builder.append("\t");
          _builder.append("/**");
          _builder.newLine();
          _builder.append("\t");
          _builder.append(" ");
          _builder.append("* Query name constants");
          _builder.newLine();
          _builder.append("\t");
          _builder.append(" ");
          _builder.append("*/");
          _builder.newLine();
          {
            EList<QueryOperation> _queryOperation_1 = it.getQueryOperation();
            final Function1<QueryOperation,Boolean> _function_1 = new Function1<QueryOperation,Boolean>() {
                public Boolean apply(final QueryOperation it) {
                  QlStatement _statement = it.getStatement();
                  return Boolean.valueOf((_statement instanceof CallableStatement));
                }
              };
            Iterable<QueryOperation> _reject_1 = IterableExtensions2.<QueryOperation>reject(_queryOperation_1, _function_1);
            for(final QueryOperation queryOperation : _reject_1) {
              _builder.append("\t");
              _builder.append("static final String ");
              String _queryConstant = this._features.getQueryConstant(queryOperation);
              _builder.append(_queryConstant, "	");
              _builder.append(" = \"");
              String _qualifiedQueryName = this._features.getQualifiedQueryName(queryOperation);
              _builder.append(_qualifiedQueryName, "	");
              _builder.append("\";");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("/**");
          _builder.newLine();
          _builder.append("\t");
          _builder.append(" ");
          _builder.append("* Stored procedure name constants");
          _builder.newLine();
          _builder.append("\t");
          _builder.append(" ");
          _builder.append("*/");
          _builder.newLine();
          {
            EList<QueryOperation> _queryOperation_2 = it.getQueryOperation();
            final Function1<QueryOperation,Boolean> _function_2 = new Function1<QueryOperation,Boolean>() {
                public Boolean apply(final QueryOperation it) {
                  QlStatement _statement = it.getStatement();
                  return Boolean.valueOf((_statement instanceof CallableStatement));
                }
              };
            Iterable<QueryOperation> _filter = IterableExtensions.<QueryOperation>filter(_queryOperation_2, _function_2);
            for(final QueryOperation queryOperation_1 : _filter) {
              _builder.append("\t");
              _builder.append("static final String ");
              String _storedProcedureConstantName = this.getStoredProcedureConstantName(queryOperation_1);
              _builder.append(_storedProcedureConstantName, "	");
              _builder.append(" = \"");
              QlStatement _statement = queryOperation_1.getStatement();
              String _name_5 = ((CallableStatement) _statement).getName();
              String _simpleName = this.toSimpleName(_name_5);
              _builder.append(_simpleName, "	");
              _builder.append("\";");
              _builder.newLineIfNotEmpty();
            }
          }
        }
      }
      _builder.append("\t");
      CharSequence _createOperationInterface = this.createOperationInterface(it);
      _builder.append(_createOperationInterface, "	");
      _builder.newLineIfNotEmpty();
      {
        EList<DataBaseConstraint> _dataBaseConstraints = it.getDataBaseConstraints();
        final Function1<DataBaseConstraint,Boolean> _function_3 = new Function1<DataBaseConstraint,Boolean>() {
            public Boolean apply(final DataBaseConstraint it) {
              DataBaseConstraintType _type = it.getType();
              boolean _equals = Objects.equal(_type, DataBaseConstraintType.PRIMARY);
              return Boolean.valueOf(_equals);
            }
          };
        Iterable<DataBaseConstraint> _reject_2 = IterableExtensions2.<DataBaseConstraint>reject(_dataBaseConstraints, _function_3);
        for(final DataBaseConstraint constraint : _reject_2) {
          _builder.append("\t");
          CharSequence _dbConstraintFinderSignature = this.dbConstraintFinderSignature(constraint, null);
          _builder.append(_dbConstraintFinderSignature, "	");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        EList<Operation> _operations = it.getOperations();
        for(final Operation operation : _operations) {
          _builder.append("\t");
          CharSequence _daoOperationSignature = this.daoOperationSignature(operation, null);
          _builder.append(_daoOperationSignature, "	");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        EList<QueryOperation> _queryOperation_3 = it.getQueryOperation();
        for(final QueryOperation queryOperation_2 : _queryOperation_3) {
          _builder.append("\t");
          CharSequence _queryOperationSignature = this.queryOperationSignature(queryOperation_2, null);
          _builder.append(_queryOperationSignature, "	");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence createOperationInterface(final Dao it) {
    CharSequence _xblockexpression = null;
    {
      Entity _entity = it.getEntity();
      EList<Attribute> _requiredReferences = _entity.getRequiredReferences();
      final List<Attribute> notNullFeatures = IterableExtensions.<Attribute>toList(_requiredReferences);
      Entity _entity_1 = it.getEntity();
      EList<Attribute> _requiredAttributes = _entity_1.getRequiredAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isIdOrVersionAttribute = RepositoryGenerator.this._features.isIdOrVersionAttribute(it);
            return Boolean.valueOf(_isIdOrVersionAttribute);
          }
        };
      Iterable<Attribute> _reject = IterableExtensions2.<Attribute>reject(_requiredAttributes, _function);
      Iterables.<Attribute>addAll(notNullFeatures, _reject);
      CharSequence _xifexpression = null;
      DataBaseConstraint _primaryKey = it.getPrimaryKey();
      boolean _notEquals = (!Objects.equal(_primaryKey, null));
      if (_notEquals) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("/**");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("* Factory method to create an empty <code>");
        Entity _entity_2 = it.getEntity();
        String _name = _entity_2.getName();
        _builder.append(_name, " ");
        _builder.append("</code> entity with an already assigned identifier.");
        _builder.newLineIfNotEmpty();
        _builder.append(" ");
        _builder.append("* The advantage of this approach, coupled with the fact that we\'re using the id for equals,");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("* is that equality test\'s are immediately possible and therefore independent from the persistence state");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("* of the entity. This in contrast to other solutions where auto generated Keys are used");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("* which must wait until the object is persisted or differentiate between transient and managed states.");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("* @return a new (transient), empty <code>");
        Entity _entity_3 = it.getEntity();
        String _name_1 = _entity_3.getName();
        _builder.append(_name_1, " ");
        _builder.append("</code> entity with an already assigned identifier.");
        _builder.newLineIfNotEmpty();
        _builder.append(" ");
        _builder.append("*/");
        _builder.newLine();
        Entity _entity_4 = it.getEntity();
        String _name_2 = _entity_4.getName();
        _builder.append(_name_2, "");
        _builder.append(" create();");
        _builder.newLineIfNotEmpty();
        {
          boolean _isEmpty = notNullFeatures.isEmpty();
          boolean _not = (!_isEmpty);
          if (_not) {
            _builder.append("/**");
            _builder.newLine();
            _builder.append(" ");
            _builder.append("* Factory method to create an empty <code>");
            Entity _entity_5 = it.getEntity();
            String _name_3 = _entity_5.getName();
            _builder.append(_name_3, " ");
            _builder.append("</code> taking all not null attributes.");
            _builder.newLineIfNotEmpty();
            _builder.append(" ");
            _builder.append("*");
            _builder.newLine();
            {
              for(final Attribute feature : notNullFeatures) {
                _builder.append(" ");
                String _methodParameterJavaDoc = this._features.methodParameterJavaDoc(feature);
                _builder.append(_methodParameterJavaDoc, " ");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append(" ");
            _builder.append("* @return a new (transient ) <code>");
            Entity _entity_6 = it.getEntity();
            String _name_4 = _entity_6.getName();
            _builder.append(_name_4, " ");
            _builder.append("</code> entity with the given natural identifier and not null attributes set.");
            _builder.newLineIfNotEmpty();
            _builder.append(" ");
            _builder.append("*/");
            _builder.newLine();
            Entity _entity_7 = it.getEntity();
            String _name_5 = _entity_7.getName();
            _builder.append(_name_5, "");
            _builder.append(" create(");
            {
              boolean _hasElements = false;
              for(final Attribute field : notNullFeatures) {
                if (!_hasElements) {
                  _hasElements = true;
                } else {
                  _builder.appendImmediate(",", "");
                }
                CharSequence _methodParameter = this._features.methodParameter(field);
                _builder.append(_methodParameter, "");
              }
            }
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            _builder.append("/**");
            _builder.newLine();
            _builder.append(" ");
            _builder.append("* Factory method to create and save an empty <code>");
            Entity _entity_8 = it.getEntity();
            String _name_6 = _entity_8.getName();
            _builder.append(_name_6, " ");
            _builder.append("</code> taking all not null attributes.");
            _builder.newLineIfNotEmpty();
            _builder.append(" ");
            _builder.append("*");
            _builder.newLine();
            {
              for(final Attribute feature_1 : notNullFeatures) {
                _builder.append(" ");
                String _methodParameterJavaDoc_1 = this._features.methodParameterJavaDoc(feature_1);
                _builder.append(_methodParameterJavaDoc_1, " ");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append(" ");
            _builder.append("* @return a new (transient) <code>");
            Entity _entity_9 = it.getEntity();
            String _name_7 = _entity_9.getName();
            _builder.append(_name_7, " ");
            _builder.append("</code> entity with the given natural identifier and not null attributes set.");
            _builder.newLineIfNotEmpty();
            _builder.append(" ");
            _builder.append("*/");
            _builder.newLine();
            Entity _entity_10 = it.getEntity();
            String _name_8 = _entity_10.getName();
            _builder.append(_name_8, "");
            _builder.append(" createAndSave(");
            {
              boolean _hasElements_1 = false;
              for(final Attribute field_1 : notNullFeatures) {
                if (!_hasElements_1) {
                  _hasElements_1 = true;
                } else {
                  _builder.appendImmediate(",", "");
                }
                CharSequence _methodParameter_1 = this._features.methodParameter(field_1);
                _builder.append(_methodParameter_1, "");
              }
            }
            _builder.append(");");
            _builder.newLineIfNotEmpty();
          }
        }
        _xifexpression = _builder;
      } else {
        CharSequence _xblockexpression_1 = null;
        {
          DataBaseConstraint _naturalKey = it.getNaturalKey();
          final EList<Attribute> naturalKeyFields = _naturalKey.getAttributes();
          final Set<Attribute> notNullFields = IterableExtensions.<Attribute>toSet(naturalKeyFields);
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("/**");
          _builder_1.newLine();
          _builder_1.append(" ");
          _builder_1.append("* Factory method to create an empty <code>");
          Entity _entity_11 = it.getEntity();
          String _name_9 = _entity_11.getName();
          _builder_1.append(_name_9, " ");
          _builder_1.append("</code> entity with an already assigned identifier.");
          _builder_1.newLineIfNotEmpty();
          _builder_1.append(" ");
          _builder_1.append("* The advantage of this approach, coupled with the fact that we\'re using the id for equals,");
          _builder_1.newLine();
          _builder_1.append(" ");
          _builder_1.append("* is that equality test\'s are immediately possible and therefore independent from the persistence state");
          _builder_1.newLine();
          _builder_1.append(" ");
          _builder_1.append("* of the entity. This in contrast to other solutions where auto generated Keys are used");
          _builder_1.newLine();
          _builder_1.append(" ");
          _builder_1.append("* which must wait until the object is persisted or differentiate between transient and managed states.");
          _builder_1.newLine();
          {
            for(final Attribute feature_2 : naturalKeyFields) {
              _builder_1.append(" ");
              String _methodParameterJavaDoc_2 = this._features.methodParameterJavaDoc(feature_2);
              _builder_1.append(_methodParameterJavaDoc_2, " ");
              _builder_1.newLineIfNotEmpty();
            }
          }
          _builder_1.append(" ");
          _builder_1.append("* @return a new (transient), empty <code>");
          Entity _entity_12 = it.getEntity();
          String _name_10 = _entity_12.getName();
          _builder_1.append(_name_10, " ");
          _builder_1.append("</code> entity with an already assigned identifier.");
          _builder_1.newLineIfNotEmpty();
          _builder_1.append(" ");
          _builder_1.append("*/");
          _builder_1.newLine();
          Entity _entity_13 = it.getEntity();
          String _name_11 = _entity_13.getName();
          _builder_1.append(_name_11, "");
          _builder_1.append(" create(");
          {
            boolean _hasElements_2 = false;
            for(final Attribute field_2 : naturalKeyFields) {
              if (!_hasElements_2) {
                _hasElements_2 = true;
              } else {
                _builder_1.appendImmediate(",", "");
              }
              CharSequence _methodParameter_2 = this._features.methodParameter(field_2);
              _builder_1.append(_methodParameter_2, "");
            }
          }
          _builder_1.append(");");
          _builder_1.newLineIfNotEmpty();
          {
            boolean _addAll = notNullFields.addAll(notNullFeatures);
            if (_addAll) {
              _builder_1.append("/**");
              _builder_1.newLine();
              _builder_1.append(" ");
              _builder_1.append("* Factory method to create an empty <code>");
              Entity _entity_14 = it.getEntity();
              String _name_12 = _entity_14.getName();
              _builder_1.append(_name_12, " ");
              _builder_1.append("</code> taking all required (not-null) attributes.");
              _builder_1.newLineIfNotEmpty();
              _builder_1.append(" ");
              _builder_1.append("*");
              _builder_1.newLine();
              {
                for(final Attribute feature_3 : notNullFields) {
                  _builder_1.append(" ");
                  String _methodParameterJavaDoc_3 = this._features.methodParameterJavaDoc(feature_3);
                  _builder_1.append(_methodParameterJavaDoc_3, " ");
                  _builder_1.newLineIfNotEmpty();
                }
              }
              _builder_1.append(" ");
              _builder_1.append("* @return a new, transient (i.e. is not associated with a persistence context and has no persistent representation in the database) <code>");
              Entity _entity_15 = it.getEntity();
              String _name_13 = _entity_15.getName();
              _builder_1.append(_name_13, " ");
              _builder_1.append("</code> entity with the given natural identifier and not null attributes set.");
              _builder_1.newLineIfNotEmpty();
              _builder_1.append("*/");
              _builder_1.newLine();
              Entity _entity_16 = it.getEntity();
              String _name_14 = _entity_16.getName();
              _builder_1.append(_name_14, "");
              _builder_1.append(" create(");
              {
                boolean _hasElements_3 = false;
                for(final Attribute field_3 : notNullFields) {
                  if (!_hasElements_3) {
                    _hasElements_3 = true;
                  } else {
                    _builder_1.appendImmediate(",", "");
                  }
                  CharSequence _methodParameter_3 = this._features.methodParameter(field_3);
                  _builder_1.append(_methodParameter_3, "");
                }
              }
              _builder_1.append(");");
              _builder_1.newLineIfNotEmpty();
            }
          }
          _builder_1.append("/**");
          _builder_1.newLine();
          _builder_1.append(" ");
          _builder_1.append("* Factory method to create and save an empty <code>");
          Entity _entity_17 = it.getEntity();
          String _name_15 = _entity_17.getName();
          _builder_1.append(_name_15, " ");
          _builder_1.append("</code> taking all required (not-null) attributes.");
          _builder_1.newLineIfNotEmpty();
          _builder_1.append(" ");
          _builder_1.append("*");
          _builder_1.newLine();
          {
            for(final Attribute feature_4 : notNullFields) {
              String _methodParameterJavaDoc_4 = this._features.methodParameterJavaDoc(feature_4);
              _builder_1.append(_methodParameterJavaDoc_4, "");
              _builder_1.newLineIfNotEmpty();
            }
          }
          _builder_1.append(" ");
          _builder_1.append("* @return a new, transient (i.e. is not associated with a persistence context and has no persistent representation in the database) <code>");
          Entity _entity_18 = it.getEntity();
          String _name_16 = _entity_18.getName();
          _builder_1.append(_name_16, " ");
          _builder_1.append("</code> entity with the given natural identifier and not null attributes set.");
          _builder_1.newLineIfNotEmpty();
          _builder_1.append(" ");
          _builder_1.append("*/");
          _builder_1.newLine();
          Entity _entity_19 = it.getEntity();
          String _name_17 = _entity_19.getName();
          _builder_1.append(_name_17, "");
          _builder_1.append(" createAndSave(");
          {
            boolean _hasElements_4 = false;
            for(final Attribute field_4 : notNullFields) {
              if (!_hasElements_4) {
                _hasElements_4 = true;
              } else {
                _builder_1.appendImmediate(",", "");
              }
              CharSequence _methodParameter_4 = this._features.methodParameter(field_4);
              _builder_1.append(_methodParameter_4, "");
            }
          }
          _builder_1.append(");");
          _builder_1.newLineIfNotEmpty();
          _xblockexpression_1 = (_builder_1);
        }
        _xifexpression = _xblockexpression_1;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public CharSequence dbConstraintFinderSignature(final DataBaseConstraint it, final String visibilityModifier) {
    CharSequence _xblockexpression = null;
    {
      final EList<Attribute> resolvedAttributeList = it.getResolvedAttributeList();
      StringConcatenation _builder = new StringConcatenation();
      {
        DataBaseConstraintType _type = it.getType();
        boolean _equals = Objects.equal(_type, DataBaseConstraintType.NATURAL);
        if (_equals) {
          {
            boolean _equals_1 = Objects.equal(visibilityModifier, null);
            if (_equals_1) {
              _builder.append("/**");
              _builder.newLine();
              _builder.append(" ");
              _builder.append("* Return the persistent instance of the ");
              Entity _entity = this.getEntity(it);
              String _name = _entity.getName();
              _builder.append(_name, " ");
              _builder.append(" entity class");
              _builder.newLineIfNotEmpty();
              _builder.append(" ");
              _builder.append("* with the given natural identifier or <code>null</code> if not found.");
              _builder.newLine();
              _builder.append(" ");
              _builder.append("*");
              _builder.newLine();
              {
                for(final Attribute feature : resolvedAttributeList) {
                  _builder.append(" ");
                  String _methodParameterJavaDoc = this._features.methodParameterJavaDoc(feature);
                  _builder.append(_methodParameterJavaDoc, " ");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append(" ");
              _builder.append("* @return the persistent ");
              Entity _entity_1 = this.getEntity(it);
              String _name_1 = _entity_1.getName();
              _builder.append(_name_1, " ");
              _builder.append(" instance, or <code>null</code> if not found");
              _builder.newLineIfNotEmpty();
              _builder.append(" ");
              _builder.append("*/");
              _builder.newLine();
            }
          }
          {
            boolean _notEquals = (!Objects.equal(visibilityModifier, null));
            if (_notEquals) {
              _builder.append(visibilityModifier, "");
              _builder.append(" ");
            }
          }
          Entity _entity_2 = this.getEntity(it);
          String _name_2 = _entity_2.getName();
          _builder.append(_name_2, "");
          _builder.append(" ");
          String _methodName = this._features.getMethodName(it);
          _builder.append(_methodName, "");
          _builder.append("(");
          {
            boolean _hasElements = false;
            for(final Attribute parameter : resolvedAttributeList) {
              if (!_hasElements) {
                _hasElements = true;
              } else {
                _builder.appendImmediate(",", "");
              }
              CharSequence _methodParameter = this._features.methodParameter(parameter);
              _builder.append(_methodParameter, "");
            }
          }
          _builder.append(")");
          {
            boolean _equals_2 = Objects.equal(visibilityModifier, null);
            if (_equals_2) {
              _builder.append(";");
            }
          }
        } else {
          DataBaseConstraintType _type_1 = it.getType();
          boolean _equals_3 = Objects.equal(_type_1, DataBaseConstraintType.UNIQUE);
          if (_equals_3) {
            _builder.newLineIfNotEmpty();
            {
              boolean _equals_4 = Objects.equal(visibilityModifier, null);
              if (_equals_4) {
                _builder.append("/**");
                _builder.newLine();
                _builder.append(" ");
                _builder.append("* Return the persistent instance of the ");
                Entity _entity_3 = this.getEntity(it);
                String _name_3 = _entity_3.getName();
                _builder.append(_name_3, " ");
                _builder.append(" entity class");
                _builder.newLineIfNotEmpty();
                _builder.append(" ");
                _builder.append("* with the given unique-key or <code>null</code> if not found.");
                _builder.newLine();
                _builder.append(" ");
                _builder.append("*");
                _builder.newLine();
                {
                  EList<Attribute> _attributes = it.getAttributes();
                  for(final Attribute feature_1 : _attributes) {
                    _builder.append(" ");
                    String _methodParameterJavaDoc_1 = this._features.methodParameterJavaDoc(feature_1);
                    _builder.append(_methodParameterJavaDoc_1, " ");
                    _builder.newLineIfNotEmpty();
                  }
                }
                _builder.append(" ");
                _builder.append("* @return the persistent ");
                Entity _entity_4 = this.getEntity(it);
                String _name_4 = _entity_4.getName();
                _builder.append(_name_4, " ");
                _builder.append(" instance, or <code>null</code> if not found");
                _builder.newLineIfNotEmpty();
                _builder.append(" ");
                _builder.append("*/");
                _builder.newLine();
              }
            }
            {
              boolean _notEquals_1 = (!Objects.equal(visibilityModifier, null));
              if (_notEquals_1) {
                _builder.append(visibilityModifier, "");
                _builder.append(" ");
              }
            }
            Entity _entity_5 = this.getEntity(it);
            String _name_5 = _entity_5.getName();
            _builder.append(_name_5, "");
            _builder.append(" ");
            String _methodName_1 = this._features.getMethodName(it);
            _builder.append(_methodName_1, "");
            _builder.append("(");
            {
              EList<Attribute> _attributes_1 = it.getAttributes();
              boolean _hasElements_1 = false;
              for(final Attribute parameter_1 : _attributes_1) {
                if (!_hasElements_1) {
                  _hasElements_1 = true;
                } else {
                  _builder.appendImmediate(",", "");
                }
                CharSequence _methodParameter_1 = this._features.methodParameter(parameter_1);
                _builder.append(_methodParameter_1, "");
              }
            }
            _builder.append(")");
            {
              boolean _equals_5 = Objects.equal(visibilityModifier, null);
              if (_equals_5) {
                _builder.append(";");
              }
            }
          } else {
            DataBaseConstraintType _type_2 = it.getType();
            boolean _equals_6 = Objects.equal(_type_2, DataBaseConstraintType.INDEX);
            if (_equals_6) {
              _builder.newLineIfNotEmpty();
              {
                boolean _equals_7 = Objects.equal(visibilityModifier, null);
                if (_equals_7) {
                  _builder.append("/**");
                  _builder.newLine();
                  _builder.append(" ");
                  _builder.append("* Execute a query based on the given index attributes.");
                  _builder.newLine();
                  _builder.append(" ");
                  _builder.append("*");
                  _builder.newLine();
                  {
                    for(final Attribute feature_2 : resolvedAttributeList) {
                      _builder.append(" ");
                      String _methodParameterJavaDoc_2 = this._features.methodParameterJavaDoc(feature_2);
                      _builder.append(_methodParameterJavaDoc_2, " ");
                      _builder.newLineIfNotEmpty();
                    }
                  }
                  _builder.append(" ");
                  _builder.append("* @param firstResult the index of the first result object to be retrieved (numbered from 0 or null if ignored)");
                  _builder.newLine();
                  _builder.append(" ");
                  _builder.append("* @param maxResults the maximum number of result objects to retrieved (null or <=0 for no limit)");
                  _builder.newLine();
                  _builder.append(" ");
                  _builder.append("* @return a {@link Set} containing 0 or more persistent ");
                  Entity _entity_6 = this.getEntity(it);
                  String _name_6 = _entity_6.getName();
                  _builder.append(_name_6, " ");
                  _builder.append(" instances");
                  _builder.newLineIfNotEmpty();
                  _builder.append(" ");
                  _builder.append("*/");
                  _builder.newLine();
                }
              }
              {
                boolean _notEquals_2 = (!Objects.equal(visibilityModifier, null));
                if (_notEquals_2) {
                  _builder.append(visibilityModifier, "");
                  _builder.append(" ");
                }
              }
              String _addImport = this._names.addImport(it, "java.util.Set");
              _builder.append(_addImport, "");
              _builder.append("<");
              Entity _entity_7 = this.getEntity(it);
              String _name_7 = _entity_7.getName();
              _builder.append(_name_7, "");
              _builder.append("> ");
              String _methodName_2 = this._features.getMethodName(it);
              _builder.append(_methodName_2, "");
              _builder.append("(");
              {
                boolean _hasElements_2 = false;
                for(final Attribute parameter_2 : resolvedAttributeList) {
                  if (!_hasElements_2) {
                    _hasElements_2 = true;
                  } else {
                    _builder.appendImmediate(",", "");
                  }
                  CharSequence _methodParameter_2 = this._features.methodParameter(parameter_2);
                  _builder.append(_methodParameter_2, "");
                }
              }
              _builder.append(",Integer firstResult,Integer maxResults");
              _builder.append(")", "");
              {
                boolean _equals_8 = Objects.equal(visibilityModifier, null);
                if (_equals_8) {
                  _builder.append(";");
                }
              }
            }
          }
        }
      }
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence daoOperationSignature(final Operation it, final String visibilityModifier) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _notEquals = (!Objects.equal(visibilityModifier, null));
      if (_notEquals) {
      } else {
        _builder.append("/**");
        _builder.newLine();
        _builder.append(" ");
        _builder.append("*");
        _builder.newLine();
        {
          EList<Parameter> _parameters = it.getParameters();
          for(final Parameter parameter : _parameters) {
            _builder.append(" ");
            String _methodParameterJavaDoc = this._features.methodParameterJavaDoc(parameter);
            _builder.append(_methodParameterJavaDoc, " ");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          boolean _isMany = it.isMany();
          if (_isMany) {
            _builder.append(" ");
            _builder.append("* @return a collection containing 0 or more persistent ");
            Type _type = it.getType();
            String _name = _type.getName();
            _builder.append(_name, " ");
            _builder.append(" instances");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append(" ");
            _builder.append("* @return the ");
            String _elvis = null;
            Type _type_1 = it.getType();
            String _name_1 = _type_1==null?(String)null:_type_1.getName();
            String _firstLower = StringExtensions.toFirstLower(_name_1);
            if (_firstLower != null) {
              _elvis = _firstLower;
            } else {
              _elvis = ObjectExtensions.<String>operator_elvis(_firstLower, "");
            }
            _builder.append(_elvis, " ");
            _builder.append(" instance, or <code>null</code> if not found");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append(" ");
        _builder.append("*/");
        _builder.newLine();
      }
    }
    {
      boolean _notEquals_1 = (!Objects.equal(visibilityModifier, null));
      if (_notEquals_1) {
        _builder.append(visibilityModifier, "");
        _builder.append(" ");
      }
    }
    String _type_2 = this._features.toType(it);
    _builder.append(_type_2, "");
    _builder.append(" ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "");
    _builder.append("(");
    {
      EList<Parameter> _parameters_1 = it.getParameters();
      boolean _hasElements = false;
      for(final Parameter parameter_1 : _parameters_1) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _methodParameter = this._features.methodParameter(parameter_1);
        _builder.append(_methodParameter, "");
      }
    }
    _builder.append(")");
    {
      boolean _equals = Objects.equal(visibilityModifier, null);
      if (_equals) {
        _builder.append(";");
      }
    }
    return _builder;
  }
  
  public CharSequence queryOperationSignature(final QueryOperation it, final String visibilityModifier) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _notEquals = (!Objects.equal(visibilityModifier, null));
      if (_notEquals) {
      } else {
        _builder.append("/**");
        _builder.newLine();
        CharSequence _documentationLinesIndented = this._features.getDocumentationLinesIndented(it);
        _builder.append(_documentationLinesIndented, "");
        _builder.newLineIfNotEmpty();
        _builder.append(" ");
        _builder.append("*");
        _builder.newLine();
        {
          EList<QueryParameter> _queryParameters = it.getQueryParameters();
          for(final QueryParameter parameter : _queryParameters) {
            _builder.append(" ");
            String _methodParameterJavaDoc = this._features.methodParameterJavaDoc(parameter);
            _builder.append(_methodParameterJavaDoc, " ");
            _builder.newLineIfNotEmpty();
          }
        }
        {
          boolean _isMany = it.isMany();
          if (_isMany) {
            _builder.append(" ");
            _builder.append("* @return a collection containing 0 or more persistent ");
            Type _type = it.getType();
            String _name = _type.getName();
            _builder.append(_name, " ");
            _builder.append(" instances");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append(" ");
            _builder.append("* @return the ");
            String _elvis = null;
            Type _type_1 = it.getType();
            String _name_1 = _type_1==null?(String)null:_type_1.getName();
            String _firstLower = StringExtensions.toFirstLower(_name_1);
            if (_firstLower != null) {
              _elvis = _firstLower;
            } else {
              _elvis = ObjectExtensions.<String>operator_elvis(_firstLower, "");
            }
            _builder.append(_elvis, " ");
            _builder.append(" instance, or <code>null</code> if not found");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append(" ");
        _builder.append("*/");
        _builder.newLine();
      }
    }
    {
      boolean _notEquals_1 = (!Objects.equal(visibilityModifier, null));
      if (_notEquals_1) {
        _builder.append(visibilityModifier, "");
        _builder.append(" ");
      }
    }
    String _type_2 = this._features.toType(it);
    _builder.append(_type_2, "");
    _builder.append(" ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "");
    _builder.append("(");
    {
      EList<QueryParameter> _queryParameters_1 = it.getQueryParameters();
      boolean _hasElements = false;
      for(final QueryParameter parameter_1 : _queryParameters_1) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _methodParameter = this._features.methodParameter(parameter_1);
        _builder.append(_methodParameter, "");
      }
    }
    _builder.append(")");
    {
      boolean _equals = Objects.equal(visibilityModifier, null);
      if (_equals) {
        _builder.append(visibilityModifier, "");
        _builder.append(";");
      }
    }
    return _builder;
  }
  
  public String getStoredProcedureConstantName(final QueryOperation it) {
    String _queryConstant = this._features.getQueryConstant(it);
    String _plus = ("SP_" + _queryConstant);
    return _plus;
  }
  
  public String getIdClassName(final Dao it) {
    String _xifexpression = null;
    DataBaseConstraint _primaryKey = it.getPrimaryKey();
    boolean _notEquals = (!Objects.equal(_primaryKey, null));
    if (_notEquals) {
      Column _primaryKeyColumn = it.getPrimaryKeyColumn();
      Attribute _attribute = _primaryKeyColumn.getAttribute();
      Type _dataType = _attribute.getDataType();
      String _qualifiedName = this._names.getQualifiedName(_dataType);
      _xifexpression = _qualifiedName;
    } else {
      Entity _entity = it.getEntity();
      AttributeGroup _key = _entity.getKey();
      String _qualifiedKeyClassName = this._features.getQualifiedKeyClassName(_key);
      String _addImport = this._names.addImport(it, _qualifiedKeyClassName);
      _xifexpression = _addImport;
    }
    return _xifexpression;
  }
  
  public CharSequence toInterface(final Dao it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" extends ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final Dao it) {
    CharSequence _xblockexpression = null;
    {
      Entity _entity = it.getEntity();
      EList<Attribute> _requiredReferences = _entity.getRequiredReferences();
      final List<Attribute> notNullFeatures = IterableExtensions.<Attribute>toList(_requiredReferences);
      Entity _entity_1 = it.getEntity();
      EList<Attribute> _requiredAttributes = _entity_1.getRequiredAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isIdOrVersionAttribute = RepositoryGenerator.this._features.isIdOrVersionAttribute(it);
            return Boolean.valueOf(_isIdOrVersionAttribute);
          }
        };
      Iterable<Attribute> _reject = IterableExtensions2.<Attribute>reject(_requiredAttributes, _function);
      Iterables.<Attribute>addAll(notNullFeatures, _reject);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("package ");
      String _classPackageName = this._names.getClassPackageName(it);
      _builder.append(_classPackageName, "");
      _builder.append(";");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("import org.openxma.dsl.platform.dao.*;");
      _builder.newLine();
      _builder.append("import org.openxma.dsl.platform.dao.impl.*;");
      _builder.newLine();
      _builder.append("import org.openxma.dsl.platform.service.Mapper;");
      _builder.newLine();
      _builder.newLine();
      _builder.append("import org.hibernate.Criteria;");
      _builder.newLine();
      _builder.append("import org.hibernate.Query;");
      _builder.newLine();
      _builder.append("import org.hibernate.SessionFactory;");
      _builder.newLine();
      _builder.append("import org.hibernate.criterion.Criterion;");
      _builder.newLine();
      _builder.append("import org.hibernate.criterion.Expression;");
      _builder.newLine();
      _builder.append("import org.hibernate.criterion.Restrictions;");
      _builder.newLine();
      _builder.append("import org.springframework.beans.factory.annotation.Autowired;");
      _builder.newLine();
      _builder.append("import org.springframework.beans.factory.annotation.Qualifier;");
      _builder.newLine();
      _builder.append("import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;");
      _builder.newLine();
      _builder.append("import org.springframework.stereotype.Repository;");
      _builder.newLine();
      _builder.append("import org.springframework.util.Assert;");
      _builder.newLine();
      String _addImportsHere = ImportAdapter.addImportsHere(it);
      _builder.append(_addImportsHere, "");
      _builder.newLineIfNotEmpty();
      _builder.append("/**");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* @see ");
      String _interfaceName = this._names.getInterfaceName(it);
      _builder.append(_interfaceName, " ");
      _builder.newLineIfNotEmpty();
      _builder.append(" ");
      _builder.append("*/");
      _builder.newLine();
      _builder.append("@SuppressWarnings(\"unused\")");
      _builder.newLine();
      _builder.append("public abstract class ");
      String _genClassName = this._names.getGenClassName(it);
      _builder.append(_genClassName, "");
      _builder.append(" extends ");
      {
        String _daoBaseClassname = GeneratorConfigurationExtension.getDaoBaseClassname();
        String _lastToken = Strings.lastToken(_daoBaseClassname, ".");
        boolean _equals = "GenericDaoHibernateImpl".equals(_lastToken);
        if (_equals) {
          _builder.append("GenericDaoHibernateImpl");
        } else {
          String _daoBaseClassname_1 = GeneratorConfigurationExtension.getDaoBaseClassname();
          String _addImport = this._names.addImport(it, _daoBaseClassname_1);
          _builder.append(_addImport, "");
        }
      }
      _builder.append("<");
      Entity _entity_2 = it.getEntity();
      String _qualifiedName = this._names.getQualifiedName(_entity_2);
      _builder.append(_qualifiedName, "");
      _builder.append(",");
      String _idClassName = this.getIdClassName(it);
      _builder.append(_idClassName, "");
      _builder.append("> implements ");
      String _qualifiedName_1 = this._names.getQualifiedName(it);
      _builder.append(_qualifiedName_1, "");
      _builder.append(" {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("public ");
      String _genClassName_1 = this._names.getGenClassName(it);
      _builder.append(_genClassName_1, "	");
      _builder.append("() {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("super(");
      Entity _entity_3 = it.getEntity();
      String _qualifiedClassName = this._names.getQualifiedClassName(_entity_3);
      String _addImport_1 = this._names.addImport(it, _qualifiedClassName);
      _builder.append(_addImport_1, "		");
      _builder.append(".class);");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("/**");
      _builder.newLine();
      _builder.append("\t ");
      _builder.append("* @param context the context for the object creation");
      _builder.newLine();
      _builder.append("\t ");
      _builder.append("* @return a transient instance of entity <code>");
      Entity _entity_4 = it.getEntity();
      String _name = _entity_4.getName();
      _builder.append(_name, "	 ");
      _builder.append("</code> with an already assigned identifier.");
      _builder.newLineIfNotEmpty();
      _builder.append("\t ");
      _builder.append("*/");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("public ");
      Entity _entity_5 = it.getEntity();
      String _name_1 = _entity_5.getName();
      _builder.append(_name_1, "	");
      _builder.append(" createEntity(Object context) {");
      _builder.newLineIfNotEmpty();
      {
        DataBaseConstraint _primaryKey = it.getPrimaryKey();
        boolean _notEquals = (!Objects.equal(_primaryKey, null));
        if (_notEquals) {
          _builder.append("\t\t");
          _builder.append("return create();");
          _builder.newLine();
        } else {
          _builder.append("\t\t");
          _builder.append("return createEntityInstance();");
          _builder.newLine();
        }
      }
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t");
      _builder.newLine();
      {
        DataBaseConstraint _primaryKey_1 = it.getPrimaryKey();
        boolean _notEquals_1 = (!Objects.equal(_primaryKey_1, null));
        if (_notEquals_1) {
          _builder.append("\t");
          _builder.append("public ");
          Entity _entity_6 = it.getEntity();
          String _name_2 = _entity_6.getName();
          _builder.append(_name_2, "	");
          _builder.append(" create() ");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("{");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("\t");
          Entity _entity_7 = it.getEntity();
          String _name_3 = _entity_7.getName();
          _builder.append(_name_3, "		");
          _builder.append(" ");
          Entity _entity_8 = it.getEntity();
          String _name_4 = _entity_8.getName();
          String _variableName = this._names.toVariableName(_name_4);
          _builder.append(_variableName, "		");
          _builder.append(" = createEntityInstance();");
          _builder.newLineIfNotEmpty();
          {
            Column _primaryKeyColumn = it.getPrimaryKeyColumn();
            Attribute _attribute = _primaryKeyColumn.getAttribute();
            DataTypeAndTypeParameter _type = _attribute.getType();
            String _fullyQualifiedName = NameExtension.getFullyQualifiedName(_type, Boolean.valueOf(true));
            boolean _equals_1 = _fullyQualifiedName.equals("Long");
            if (_equals_1) {
              _builder.append("\t");
              _builder.append("\t");
              Entity _entity_9 = it.getEntity();
              String _name_5 = _entity_9.getName();
              String _variableName_1 = this._names.toVariableName(_name_5);
              _builder.append(_variableName_1, "		");
              _builder.append(".");
              Column _primaryKeyColumn_1 = it.getPrimaryKeyColumn();
              Attribute _attribute_1 = _primaryKeyColumn_1.getAttribute();
              String _mutatorMethodName = this._features.getMutatorMethodName(_attribute_1);
              _builder.append(_mutatorMethodName, "		");
              _builder.append("(Long.valueOf(dataFieldMaxValueIncrementer.nextLongValue()));");
              _builder.newLineIfNotEmpty();
            } else {
              Column _primaryKeyColumn_2 = it.getPrimaryKeyColumn();
              Attribute _attribute_2 = _primaryKeyColumn_2.getAttribute();
              DataTypeAndTypeParameter _type_1 = _attribute_2.getType();
              String _fullyQualifiedName_1 = NameExtension.getFullyQualifiedName(_type_1, Boolean.valueOf(true));
              boolean _equals_2 = _fullyQualifiedName_1.equals("String");
              if (_equals_2) {
                _builder.append("\t");
                _builder.append("\t");
                Entity _entity_10 = it.getEntity();
                String _name_6 = _entity_10.getName();
                String _variableName_2 = this._names.toVariableName(_name_6);
                _builder.append(_variableName_2, "		");
                _builder.append(".");
                Column _primaryKeyColumn_3 = it.getPrimaryKeyColumn();
                Attribute _attribute_3 = _primaryKeyColumn_3.getAttribute();
                String _mutatorMethodName_1 = this._features.getMutatorMethodName(_attribute_3);
                _builder.append(_mutatorMethodName_1, "		");
                _builder.append("(dataFieldMaxValueIncrementer.nextStringValue());");
                _builder.newLineIfNotEmpty();
              } else {
                Column _primaryKeyColumn_4 = it.getPrimaryKeyColumn();
                Attribute _attribute_4 = _primaryKeyColumn_4.getAttribute();
                DataTypeAndTypeParameter _type_2 = _attribute_4.getType();
                String _fullyQualifiedName_2 = NameExtension.getFullyQualifiedName(_type_2, Boolean.valueOf(true));
                boolean _equals_3 = _fullyQualifiedName_2.equals("Integer");
                if (_equals_3) {
                  _builder.append("\t");
                  _builder.append("\t");
                  Entity _entity_11 = it.getEntity();
                  String _name_7 = _entity_11.getName();
                  String _variableName_3 = this._names.toVariableName(_name_7);
                  _builder.append(_variableName_3, "		");
                  _builder.append(".");
                  Column _primaryKeyColumn_5 = it.getPrimaryKeyColumn();
                  Attribute _attribute_5 = _primaryKeyColumn_5.getAttribute();
                  String _mutatorMethodName_2 = this._features.getMutatorMethodName(_attribute_5);
                  _builder.append(_mutatorMethodName_2, "		");
                  _builder.append("(dataFieldMaxValueIncrementer.nextIntValue());");
                  _builder.newLineIfNotEmpty();
                } else {
                  Column _primaryKeyColumn_6 = it.getPrimaryKeyColumn();
                  Attribute _attribute_6 = _primaryKeyColumn_6.getAttribute();
                  Type _dataType = _attribute_6.getDataType();
                  String _javaType = SimpleTypeExtension.getJavaType(((SimpleType) _dataType));
                  boolean _equals_4 = "org.openxma.dsl.platform.valueobject.ValueObjectString".equals(_javaType);
                  if (_equals_4) {
                    _builder.append("\t");
                    _builder.append("\t");
                    Entity _entity_12 = it.getEntity();
                    String _name_8 = _entity_12.getName();
                    String _variableName_4 = this._names.toVariableName(_name_8);
                    _builder.append(_variableName_4, "		");
                    _builder.append(".");
                    Column _primaryKeyColumn_7 = it.getPrimaryKeyColumn();
                    Attribute _attribute_7 = _primaryKeyColumn_7.getAttribute();
                    String _mutatorMethodName_3 = this._features.getMutatorMethodName(_attribute_7);
                    _builder.append(_mutatorMethodName_3, "		");
                    _builder.append("(new ");
                    Column _primaryKeyColumn_8 = it.getPrimaryKeyColumn();
                    Attribute _attribute_8 = _primaryKeyColumn_8.getAttribute();
                    Type _dataType_1 = _attribute_8.getDataType();
                    String _qualifiedName_2 = this._names.getQualifiedName(_dataType_1);
                    _builder.append(_qualifiedName_2, "		");
                    _builder.append("(dataFieldMaxValueIncrementer.nextStringValue()));");
                    _builder.newLineIfNotEmpty();
                  } else {
                    Column _primaryKeyColumn_9 = it.getPrimaryKeyColumn();
                    Attribute _attribute_9 = _primaryKeyColumn_9.getAttribute();
                    Type _dataType_2 = _attribute_9.getDataType();
                    String _javaType_1 = SimpleTypeExtension.getJavaType(((SimpleType) _dataType_2));
                    boolean _equals_5 = "org.openxma.dsl.platform.valueobject.ValueObjectLong".equals(_javaType_1);
                    if (_equals_5) {
                      _builder.append("\t");
                      _builder.append("\t");
                      Entity _entity_13 = it.getEntity();
                      String _name_9 = _entity_13.getName();
                      String _variableName_5 = this._names.toVariableName(_name_9);
                      _builder.append(_variableName_5, "		");
                      _builder.append(".");
                      Column _primaryKeyColumn_10 = it.getPrimaryKeyColumn();
                      Attribute _attribute_10 = _primaryKeyColumn_10.getAttribute();
                      String _mutatorMethodName_4 = this._features.getMutatorMethodName(_attribute_10);
                      _builder.append(_mutatorMethodName_4, "		");
                      _builder.append("(new ");
                      Column _primaryKeyColumn_11 = it.getPrimaryKeyColumn();
                      Attribute _attribute_11 = _primaryKeyColumn_11.getAttribute();
                      Type _dataType_3 = _attribute_11.getDataType();
                      String _qualifiedName_3 = this._names.getQualifiedName(_dataType_3);
                      _builder.append(_qualifiedName_3, "		");
                      _builder.append("(dataFieldMaxValueIncrementer.nextLongValue()));");
                      _builder.newLineIfNotEmpty();
                    }
                  }
                }
              }
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return ");
          Entity _entity_14 = it.getEntity();
          String _name_10 = _entity_14.getName();
          String _variableName_6 = this._names.toVariableName(_name_10);
          _builder.append(_variableName_6, "		");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          {
            boolean _isEmpty = notNullFeatures.isEmpty();
            boolean _not = (!_isEmpty);
            if (_not) {
              _builder.append("\t");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("public ");
              Entity _entity_15 = it.getEntity();
              String _name_11 = _entity_15.getName();
              _builder.append(_name_11, "	");
              _builder.append(" create(");
              {
                boolean _hasElements = false;
                for(final Attribute field : notNullFeatures) {
                  if (!_hasElements) {
                    _hasElements = true;
                  } else {
                    _builder.appendImmediate(",", "	");
                  }
                  CharSequence _methodParameter = this._features.methodParameter(field);
                  _builder.append(_methodParameter, "	");
                }
              }
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
              {
                for(final Attribute field_1 : notNullFeatures) {
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("Assert.notNull(");
                  String _resolvedName = this._features.getResolvedName(field_1);
                  String _firstLower = StringExtensions.toFirstLower(_resolvedName);
                  _builder.append(_firstLower, "		");
                  _builder.append(", \"Parameter \'");
                  String _resolvedName_1 = this._features.getResolvedName(field_1);
                  String _firstLower_1 = StringExtensions.toFirstLower(_resolvedName_1);
                  _builder.append(_firstLower_1, "		");
                  _builder.append("\' must not be null\");");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              _builder.append("\t");
              Entity _entity_16 = it.getEntity();
              String _name_12 = _entity_16.getName();
              _builder.append(_name_12, "		");
              _builder.append(" ");
              Entity _entity_17 = it.getEntity();
              String _name_13 = _entity_17.getName();
              String _variableName_7 = this._names.toVariableName(_name_13);
              _builder.append(_variableName_7, "		");
              _builder.append(" = (");
              Entity _entity_18 = it.getEntity();
              String _name_14 = _entity_18.getName();
              _builder.append(_name_14, "		");
              _builder.append(") create();");
              _builder.newLineIfNotEmpty();
              {
                for(final Attribute field_2 : notNullFeatures) {
                  _builder.append("\t");
                  _builder.append("\t");
                  Entity _entity_19 = it.getEntity();
                  String _name_15 = _entity_19.getName();
                  String _variableName_8 = this._names.toVariableName(_name_15);
                  _builder.append(_variableName_8, "		");
                  _builder.append(".");
                  String _mutatorMethodName_5 = this._features.getMutatorMethodName(field_2);
                  _builder.append(_mutatorMethodName_5, "		");
                  _builder.append("(");
                  String _resolvedName_2 = this._features.getResolvedName(field_2);
                  String _firstLower_2 = StringExtensions.toFirstLower(_resolvedName_2);
                  _builder.append(_firstLower_2, "		");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("return ");
              Entity _entity_20 = it.getEntity();
              String _name_16 = _entity_20.getName();
              String _variableName_9 = this._names.toVariableName(_name_16);
              _builder.append(_variableName_9, "		");
              _builder.append(";");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
              _builder.append("\t");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("public ");
              Entity _entity_21 = it.getEntity();
              String _name_17 = _entity_21.getName();
              _builder.append(_name_17, "	");
              _builder.append(" createAndSave(");
              {
                boolean _hasElements_1 = false;
                for(final Attribute field_3 : notNullFeatures) {
                  if (!_hasElements_1) {
                    _hasElements_1 = true;
                  } else {
                    _builder.appendImmediate(",", "	");
                  }
                  CharSequence _methodParameter_1 = this._features.methodParameter(field_3);
                  _builder.append(_methodParameter_1, "	");
                }
              }
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              Entity _entity_22 = it.getEntity();
              String _name_18 = _entity_22.getName();
              _builder.append(_name_18, "		");
              _builder.append(" ");
              Entity _entity_23 = it.getEntity();
              String _name_19 = _entity_23.getName();
              String _variableName_10 = this._names.toVariableName(_name_19);
              _builder.append(_variableName_10, "		");
              _builder.append(" = create(");
              {
                boolean _hasElements_2 = false;
                for(final Attribute field_4 : notNullFeatures) {
                  if (!_hasElements_2) {
                    _hasElements_2 = true;
                  } else {
                    _builder.appendImmediate(",", "		");
                  }
                  String _resolvedName_3 = this._features.getResolvedName(field_4);
                  String _firstLower_3 = StringExtensions.toFirstLower(_resolvedName_3);
                  _builder.append(_firstLower_3, "		");
                }
              }
              _builder.append(");");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("saveOrUpdate(");
              Entity _entity_24 = it.getEntity();
              String _name_20 = _entity_24.getName();
              String _variableName_11 = this._names.toVariableName(_name_20);
              _builder.append(_variableName_11, "		");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("return ");
              Entity _entity_25 = it.getEntity();
              String _name_21 = _entity_25.getName();
              String _variableName_12 = this._names.toVariableName(_name_21);
              _builder.append(_variableName_12, "		");
              _builder.append(";");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            }
          }
        } else {
          _builder.append("\t");
          DataBaseConstraint _naturalKey = it.getNaturalKey();
          final EList<Attribute> naturalKeyFields = _naturalKey.getAttributes();
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          final Set<Attribute> notNullFields = IterableExtensions.<Attribute>toSet(naturalKeyFields);
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("public ");
          Entity _entity_26 = it.getEntity();
          String _name_22 = _entity_26.getName();
          _builder.append(_name_22, "	");
          _builder.append(" create(");
          {
            boolean _hasElements_3 = false;
            for(final Attribute field_5 : naturalKeyFields) {
              if (!_hasElements_3) {
                _hasElements_3 = true;
              } else {
                _builder.appendImmediate(",", "	");
              }
              CharSequence _methodParameter_2 = this._features.methodParameter(field_5);
              _builder.append(_methodParameter_2, "	");
            }
          }
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          {
            for(final Attribute field_6 : naturalKeyFields) {
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("Assert.notNull(");
              String _variableName_13 = this._names.toVariableName(field_6);
              _builder.append(_variableName_13, "		");
              _builder.append(", \"Parameter \'");
              String _resolvedName_4 = this._features.getResolvedName(field_6);
              String _firstLower_4 = StringExtensions.toFirstLower(_resolvedName_4);
              _builder.append(_firstLower_4, "		");
              _builder.append("\' must not be null\");");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          Entity _entity_27 = it.getEntity();
          String _className = this._names.getClassName(_entity_27);
          _builder.append(_className, "		");
          _builder.append(" ");
          Entity _entity_28 = it.getEntity();
          String _name_23 = _entity_28.getName();
          String _variableName_14 = this._names.toVariableName(_name_23);
          _builder.append(_variableName_14, "		");
          _builder.append(" = new ");
          Entity _entity_29 = it.getEntity();
          String _className_1 = this._names.getClassName(_entity_29);
          _builder.append(_className_1, "		");
          _builder.append("();");
          _builder.newLineIfNotEmpty();
          {
            for(final Attribute field_7 : naturalKeyFields) {
              _builder.append("\t");
              _builder.append("\t");
              Entity _entity_30 = it.getEntity();
              String _name_24 = _entity_30.getName();
              String _variableName_15 = this._names.toVariableName(_name_24);
              _builder.append(_variableName_15, "		");
              _builder.append(".");
              String _mutatorMethodName_6 = this._features.getMutatorMethodName(field_7);
              _builder.append(_mutatorMethodName_6, "		");
              _builder.append("(");
              String _resolvedName_5 = this._features.getResolvedName(field_7);
              String _firstLower_5 = StringExtensions.toFirstLower(_resolvedName_5);
              _builder.append(_firstLower_5, "		");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return ");
          Entity _entity_31 = it.getEntity();
          String _name_25 = _entity_31.getName();
          String _variableName_16 = this._names.toVariableName(_name_25);
          _builder.append(_variableName_16, "		");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          {
            boolean _addAll = notNullFields.addAll(notNullFeatures);
            if (_addAll) {
              _builder.append("\t");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("public ");
              Entity _entity_32 = it.getEntity();
              String _name_26 = _entity_32.getName();
              _builder.append(_name_26, "	");
              _builder.append(" create(");
              {
                boolean _hasElements_4 = false;
                for(final Attribute field_8 : notNullFields) {
                  if (!_hasElements_4) {
                    _hasElements_4 = true;
                  } else {
                    _builder.appendImmediate(",", "	");
                  }
                  CharSequence _methodParameter_3 = this._features.methodParameter(field_8);
                  _builder.append(_methodParameter_3, "	");
                }
              }
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              Entity _entity_33 = it.getEntity();
              String _name_27 = _entity_33.getName();
              _builder.append(_name_27, "		");
              _builder.append(" ");
              Entity _entity_34 = it.getEntity();
              String _name_28 = _entity_34.getName();
              String _variableName_17 = this._names.toVariableName(_name_28);
              _builder.append(_variableName_17, "		");
              _builder.append(" = (");
              Entity _entity_35 = it.getEntity();
              String _name_29 = _entity_35.getName();
              _builder.append(_name_29, "		");
              _builder.append(") create(");
              {
                boolean _hasElements_5 = false;
                for(final Attribute field_9 : naturalKeyFields) {
                  if (!_hasElements_5) {
                    _hasElements_5 = true;
                  } else {
                    _builder.appendImmediate(",", "		");
                  }
                  String _name_30 = field_9.getName();
                  _builder.append(_name_30, "		");
                }
              }
              _builder.append(");");
              _builder.newLineIfNotEmpty();
              {
                for(final Attribute field_10 : notNullFeatures) {
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("Assert.notNull(");
                  String _resolvedName_6 = this._features.getResolvedName(field_10);
                  String _firstLower_6 = StringExtensions.toFirstLower(_resolvedName_6);
                  _builder.append(_firstLower_6, "		");
                  _builder.append(", \"Parameter \'");
                  String _resolvedName_7 = this._features.getResolvedName(field_10);
                  String _firstLower_7 = StringExtensions.toFirstLower(_resolvedName_7);
                  _builder.append(_firstLower_7, "		");
                  _builder.append("\' must not be null\");");
                  _builder.newLineIfNotEmpty();
                }
              }
              {
                for(final Attribute field_11 : notNullFeatures) {
                  _builder.append("\t");
                  _builder.append("\t");
                  Entity _entity_36 = it.getEntity();
                  String _name_31 = _entity_36.getName();
                  String _variableName_18 = this._names.toVariableName(_name_31);
                  _builder.append(_variableName_18, "		");
                  _builder.append(".");
                  String _mutatorMethodName_7 = this._features.getMutatorMethodName(field_11);
                  _builder.append(_mutatorMethodName_7, "		");
                  _builder.append("(");
                  String _resolvedName_8 = this._features.getResolvedName(field_11);
                  String _firstLower_8 = StringExtensions.toFirstLower(_resolvedName_8);
                  _builder.append(_firstLower_8, "		");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("return ");
              Entity _entity_37 = it.getEntity();
              String _name_32 = _entity_37.getName();
              String _variableName_19 = this._names.toVariableName(_name_32);
              _builder.append(_variableName_19, "		");
              _builder.append(";\t");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            }
          }
          _builder.append("\t");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("public ");
          Entity _entity_38 = it.getEntity();
          String _name_33 = _entity_38.getName();
          _builder.append(_name_33, "	");
          _builder.append(" createAndSave(");
          {
            boolean _hasElements_6 = false;
            for(final Attribute field_12 : notNullFields) {
              if (!_hasElements_6) {
                _hasElements_6 = true;
              } else {
                _builder.appendImmediate(",", "	");
              }
              CharSequence _methodParameter_4 = this._features.methodParameter(field_12);
              _builder.append(_methodParameter_4, "	");
            }
          }
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          Entity _entity_39 = it.getEntity();
          String _name_34 = _entity_39.getName();
          _builder.append(_name_34, "		");
          _builder.append(" ");
          Entity _entity_40 = it.getEntity();
          String _name_35 = _entity_40.getName();
          String _variableName_20 = this._names.toVariableName(_name_35);
          _builder.append(_variableName_20, "		");
          _builder.append(" = create(");
          {
            boolean _hasElements_7 = false;
            for(final Attribute field_13 : notNullFields) {
              if (!_hasElements_7) {
                _hasElements_7 = true;
              } else {
                _builder.appendImmediate(",", "		");
              }
              String _resolvedName_9 = this._features.getResolvedName(field_13);
              String _firstLower_9 = StringExtensions.toFirstLower(_resolvedName_9);
              _builder.append(_firstLower_9, "		");
            }
          }
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("saveOrUpdate(");
          Entity _entity_41 = it.getEntity();
          String _name_36 = _entity_41.getName();
          String _variableName_21 = this._names.toVariableName(_name_36);
          _builder.append(_variableName_21, "		");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return ");
          Entity _entity_42 = it.getEntity();
          String _name_37 = _entity_42.getName();
          String _variableName_22 = this._names.toVariableName(_name_37);
          _builder.append(_variableName_22, "		");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      {
        EList<DataBaseConstraint> _dataBaseConstraints = it.getDataBaseConstraints();
        final Function1<DataBaseConstraint,Boolean> _function_1 = new Function1<DataBaseConstraint,Boolean>() {
            public Boolean apply(final DataBaseConstraint it) {
              DataBaseConstraintType _type = it.getType();
              boolean _equals = Objects.equal(_type, DataBaseConstraintType.PRIMARY);
              return Boolean.valueOf(_equals);
            }
          };
        Iterable<DataBaseConstraint> _reject_1 = IterableExtensions2.<DataBaseConstraint>reject(_dataBaseConstraints, _function_1);
        for(final DataBaseConstraint constraint : _reject_1) {
          _builder.append("\t");
          _builder.newLine();
          _builder.append("\t");
          CharSequence _finderQueryCall = this.toFinderQueryCall(constraint);
          _builder.append(_finderQueryCall, "	");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        EList<Operation> _operations = it.getOperations();
        final Function1<Operation,Boolean> _function_2 = new Function1<Operation,Boolean>() {
            public Boolean apply(final Operation e) {
              String _expression = e.getExpression();
              boolean _notEquals = (!Objects.equal(_expression, null));
              return Boolean.valueOf(_notEquals);
            }
          };
        Iterable<Operation> _filter = IterableExtensions.<Operation>filter(_operations, _function_2);
        for(final Operation operation : _filter) {
          _builder.append("\t");
          _builder.newLine();
          _builder.append("\t");
          CharSequence _namedQueryCall = this.toNamedQueryCall(operation);
          _builder.append(_namedQueryCall, "	");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        EList<QueryOperation> _queryOperation = it.getQueryOperation();
        for(final QueryOperation queryOperation : _queryOperation) {
          {
            QlStatement _statement = queryOperation.getStatement();
            if ((_statement instanceof CallableStatement)) {
              _builder.append("\t");
              _builder.newLine();
              _builder.append("\t");
              CharSequence _simpleJdbcCall = this.toSimpleJdbcCall(queryOperation);
              _builder.append(_simpleJdbcCall, "	");
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("\t");
              _builder.newLine();
              _builder.append("\t");
              CharSequence _namedQueryCall_1 = this.toNamedQueryCall(queryOperation);
              _builder.append(_namedQueryCall_1, "	");
              _builder.newLineIfNotEmpty();
            }
          }
        }
      }
      _builder.append("\t");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("protected Mapper getNamedQueryMapper(String queryName) {");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("return new ");
      String _name_38 = it.getName();
      _builder.append(_name_38, "		");
      _builder.append("QueryMapper();");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("protected class ");
      String _name_39 = it.getName();
      _builder.append(_name_39, "	");
      _builder.append("QueryMapper extends NamedQueryMapper {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("@SuppressWarnings(\"unchecked\")");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("public <E> E mapOne(Object sourceObject, E targetObject, Object context) {");
      _builder.newLine();
      {
        EList<QueryOperation> _queryOperation_1 = it.getQueryOperation();
        final Function1<QueryOperation,Boolean> _function_3 = new Function1<QueryOperation,Boolean>() {
            public Boolean apply(final QueryOperation e) {
              boolean _and = false;
              boolean _and_1 = false;
              boolean _isMany = e.isMany();
              if (!_isMany) {
                _and_1 = false;
              } else {
                Type _type = e.getType();
                _and_1 = (_isMany && (_type instanceof DataView));
              }
              if (!_and_1) {
                _and = false;
              } else {
                QlStatement _statement = e.getStatement();
                _and = (_and_1 && (_statement instanceof SelectProperties));
              }
              return Boolean.valueOf(_and);
            }
          };
        Iterable<QueryOperation> _filter_1 = IterableExtensions.<QueryOperation>filter(_queryOperation_1, _function_3);
        for(final QueryOperation qlOperation : _filter_1) {
          _builder.append("\t\t");
          _builder.append("if (\"");
          String _qualifiedQueryName = this._features.getQualifiedQueryName(qlOperation);
          _builder.append(_qualifiedQueryName, "		");
          _builder.append("\".equals(getQueryName(context))) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t");
          _builder.append("\t");
          _builder.append("return (E) map");
          String _name_40 = qlOperation.getName();
          String _firstUpper = StringExtensions.toFirstUpper(_name_40);
          _builder.append(_firstUpper, "			");
          _builder.append("Tuple((Object[]) sourceObject, 1);");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      _builder.append("\t\t");
      _builder.append("return super.mapOne(sourceObject, targetObject);");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence toFinderQueryCall(final DataBaseConstraint it) {
    CharSequence _xblockexpression = null;
    {
      final EList<Attribute> resolvedAttributes = it.getResolvedAttributeList();
      StringConcatenation _builder = new StringConcatenation();
      CharSequence _dbConstraintFinderSignature = this.dbConstraintFinderSignature(it, "public");
      _builder.append(_dbConstraintFinderSignature, "");
      _builder.append(" {");
      _builder.newLineIfNotEmpty();
      {
        boolean _or = false;
        DataBaseConstraintType _type = it.getType();
        boolean _equals = Objects.equal(DataBaseConstraintType.NATURAL, _type);
        if (_equals) {
          _or = true;
        } else {
          DataBaseConstraintType _type_1 = it.getType();
          boolean _equals_1 = Objects.equal(DataBaseConstraintType.INDEX, _type_1);
          _or = (_equals || _equals_1);
        }
        if (_or) {
          _builder.append("\t");
          Index _index = new Index();
          final Index index = _index;
          _builder.newLineIfNotEmpty();
          {
            for(final Attribute attribute : resolvedAttributes) {
              {
                boolean _firstIteration = index.firstIteration();
                if (_firstIteration) {
                  _builder.append("\t");
                  _builder.append("Criterion criterion = equal(\"");
                  String _finderQueryParameterName = this.getFinderQueryParameterName(it, attribute);
                  _builder.append(_finderQueryParameterName, "	");
                  _builder.append("\", ");
                  String _resolvedName = this._features.getResolvedName(attribute);
                  _builder.append(_resolvedName, "	");
                  _builder.append(", ");
                  boolean _isRequired = attribute.isRequired();
                  boolean _not = (!_isRequired);
                  _builder.append(_not, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                } else {
                  _builder.append("\t");
                  _builder.append("criterion = and(criterion,\"");
                  String _finderQueryParameterName_1 = this.getFinderQueryParameterName(it, attribute);
                  _builder.append(_finderQueryParameterName_1, "	");
                  _builder.append("\", ");
                  String _resolvedName_1 = this._features.getResolvedName(attribute);
                  _builder.append(_resolvedName_1, "	");
                  _builder.append(", ");
                  boolean _isRequired_1 = attribute.isRequired();
                  boolean _not_1 = (!_isRequired_1);
                  _builder.append(_not_1, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              index.increment();
              _builder.newLineIfNotEmpty();
            }
          }
        } else {
          _builder.append("\t");
          Index _index_1 = new Index();
          final Index index_1 = _index_1;
          _builder.newLineIfNotEmpty();
          {
            EList<Attribute> _attributes = it.getAttributes();
            for(final Attribute field : _attributes) {
              {
                boolean _firstIteration_1 = index_1.firstIteration();
                if (_firstIteration_1) {
                  _builder.append("\t");
                  _builder.append("Criterion criterion = equal(\"");
                  String _finderQueryPropertyName = this.getFinderQueryPropertyName(it, field);
                  _builder.append(_finderQueryPropertyName, "	");
                  _builder.append("\", ");
                  String _resolvedName_2 = this._features.getResolvedName(field);
                  _builder.append(_resolvedName_2, "	");
                  _builder.append(", ");
                  boolean _isRequired_2 = field.isRequired();
                  boolean _not_2 = (!_isRequired_2);
                  _builder.append(_not_2, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                } else {
                  _builder.append("\t");
                  _builder.append("criterion = and(criterion,\"");
                  String _finderQueryPropertyName_1 = this.getFinderQueryPropertyName(it, field);
                  _builder.append(_finderQueryPropertyName_1, "	");
                  _builder.append("\", ");
                  String _resolvedName_3 = this._features.getResolvedName(field);
                  _builder.append(_resolvedName_3, "	");
                  _builder.append(", ");
                  boolean _isRequired_3 = field.isRequired();
                  boolean _not_3 = (!_isRequired_3);
                  _builder.append(_not_3, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              index_1.increment();
              _builder.newLineIfNotEmpty();
            }
          }
        }
      }
      _builder.append("\t");
      _builder.append("Criteria criteria = createCriteria(criterion);");
      _builder.newLine();
      {
        boolean _or_1 = false;
        DataBaseConstraintType _type_2 = it.getType();
        boolean _equals_2 = Objects.equal(DataBaseConstraintType.NATURAL, _type_2);
        if (_equals_2) {
          _or_1 = true;
        } else {
          DataBaseConstraintType _type_3 = it.getType();
          boolean _equals_3 = Objects.equal(DataBaseConstraintType.INDEX, _type_3);
          _or_1 = (_equals_2 || _equals_3);
        }
        if (_or_1) {
          {
            final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
                public Boolean apply(final Attribute e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  Entity _entity = RepositoryGenerator.this.getEntity(e);
                  Entity _entity_1 = RepositoryGenerator.this.getEntity(it);
                  boolean _notEquals = (!Objects.equal(_entity, _entity_1));
                  if (!_notEquals) {
                    _and_1 = false;
                  } else {
                    Entity _entity_2 = RepositoryGenerator.this.getEntity(e);
                    Dao _repository = _entity_2==null?(Dao)null:_entity_2.getRepository();
                    boolean _notEquals_1 = (!Objects.equal(_repository, null));
                    _and_1 = (_notEquals && _notEquals_1);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    Entity _entity_3 = RepositoryGenerator.this.getEntity(e);
                    Entity _entity_4 = RepositoryGenerator.this.getEntity(it);
                    Entity _superType = _entity_4.getSuperType();
                    boolean _notEquals_2 = (!Objects.equal(_entity_3, _superType));
                    _and = (_and_1 && _notEquals_2);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(resolvedAttributes, _function);
            final Function1<Attribute,String> _function_1 = new Function1<Attribute,String>() {
                public String apply(final Attribute e) {
                  String _fQNProperty = RepositoryGenerator.this._features.getFQNProperty(e);
                  return _fQNProperty;
                }
              };
            Iterable<String> _map = IterableExtensions.<Attribute, String>map(_filter, _function_1);
            Set<String> _set = IterableExtensions.<String>toSet(_map);
            for(final String alias : _set) {
              _builder.append("\t");
              _builder.append("criteria.createAlias(\"");
              String _firstLower = StringExtensions.toFirstLower(alias);
              _builder.append(_firstLower, "	");
              _builder.append("\",\"");
              String _firstLower_1 = StringExtensions.toFirstLower(alias);
              _builder.append(_firstLower_1, "	");
              _builder.append("\");");
              _builder.newLineIfNotEmpty();
            }
          }
        } else {
          {
            Iterable<String> _aliasNames = this.getAliasNames(it);
            for(final String alias_1 : _aliasNames) {
              _builder.append("\t");
              _builder.append("criteria.createAlias(\"");
              _builder.append(alias_1, "	");
              _builder.append("\",\"");
              _builder.append(alias_1, "	");
              _builder.append("\");");
              _builder.newLineIfNotEmpty();
            }
          }
        }
      }
      {
        boolean _or_2 = false;
        DataBaseConstraintType _type_4 = it.getType();
        boolean _equals_4 = Objects.equal(DataBaseConstraintType.NATURAL, _type_4);
        if (_equals_4) {
          _or_2 = true;
        } else {
          DataBaseConstraintType _type_5 = it.getType();
          boolean _equals_5 = Objects.equal(DataBaseConstraintType.UNIQUE, _type_5);
          _or_2 = (_equals_4 || _equals_5);
        }
        if (_or_2) {
          _builder.append("\t");
          _builder.append("return (");
          Entity _entity = this.getEntity(it);
          String _qualifiedName = this._names.getQualifiedName(_entity);
          _builder.append(_qualifiedName, "	");
          _builder.append(") unique(criteria);");
          _builder.newLineIfNotEmpty();
        } else {
          _builder.append("\t");
          _builder.append("applyPagingParameters(criteria,firstResult,maxResults);");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("return new ");
          String _addImport = this._names.addImport(it, "java.util.LinkedHashSet");
          _builder.append(_addImport, "	");
          _builder.append("<");
          Entity _entity_1 = this.getEntity(it);
          String _qualifiedName_1 = this._names.getQualifiedName(_entity_1);
          _builder.append(_qualifiedName_1, "	");
          _builder.append(">(list(criteria));");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public String getFinderQueryParameterName(final DataBaseConstraint it, final Attribute attribute) {
    String _xifexpression = null;
    boolean _and = false;
    boolean _and_1 = false;
    Entity _entity = this.getEntity(attribute);
    Entity _entity_1 = this.getEntity(it);
    boolean _notEquals = (!Objects.equal(_entity, _entity_1));
    if (!_notEquals) {
      _and_1 = false;
    } else {
      Entity _entity_2 = this.getEntity(attribute);
      Dao _repository = _entity_2==null?(Dao)null:_entity_2.getRepository();
      boolean _notEquals_1 = (!Objects.equal(_repository, null));
      _and_1 = (_notEquals && _notEquals_1);
    }
    if (!_and_1) {
      _and = false;
    } else {
      Entity _entity_3 = this.getEntity(attribute);
      Entity _entity_4 = this.getEntity(it);
      Entity _superType = _entity_4.getSuperType();
      boolean _notEquals_2 = (!Objects.equal(_entity_3, _superType));
      _and = (_and_1 && _notEquals_2);
    }
    if (_and) {
      String _resolvedParameterName = this.getResolvedParameterName(attribute);
      _xifexpression = _resolvedParameterName;
    } else {
      String _name = attribute.getName();
      _xifexpression = _name;
    }
    return _xifexpression;
  }
  
  public String getFinderQueryPropertyName(final DataBaseConstraint it, final Attribute field) {
    String _xifexpression = null;
    boolean _and = false;
    String _fQNProperty = this._features.getFQNProperty(field);
    boolean _notEquals = (!Objects.equal(_fQNProperty, null));
    if (!_notEquals) {
      _and = false;
    } else {
      String _fQNProperty_1 = this._features.getFQNProperty(field);
      Entity _entity = this.getEntity(it);
      String _name = _entity.getName();
      boolean _notEquals_1 = (!Objects.equal(_fQNProperty_1, _name));
      _and = (_notEquals && _notEquals_1);
    }
    if (_and) {
      String _resolvedParameterName = this.getResolvedParameterName(field);
      _xifexpression = _resolvedParameterName;
    } else {
      String _name_1 = field.getName();
      _xifexpression = _name_1;
    }
    return _xifexpression;
  }
  
  private String getResolvedParameterName(final Attribute it) {
    String _xifexpression = null;
    String _fQNProperty = this._features.getFQNProperty(it);
    boolean _notEquals = (!Objects.equal(_fQNProperty, null));
    if (_notEquals) {
      String _fQNProperty_1 = this._features.getFQNProperty(it);
      String _string = _fQNProperty_1.toString();
      String _firstLower = StringExtensions.toFirstLower(_string);
      String _plus = (_firstLower + ".");
      String _name = it.getName();
      String _firstLower_1 = StringExtensions.toFirstLower(_name);
      String _plus_1 = (_plus + _firstLower_1);
      _xifexpression = _plus_1;
    } else {
      String _name_1 = it.getName();
      _xifexpression = _name_1;
    }
    return _xifexpression;
  }
  
  public CharSequence toSimpleJdbcCall(final QueryOperation it) {
    CharSequence _xblockexpression = null;
    {
      QlStatement _statement = it.getStatement();
      final CallableStatement callableStatement = ((CallableStatement) _statement);
      StringConcatenation _builder = new StringConcatenation();
      CharSequence _queryOperationSignature = this.queryOperationSignature(it, "public");
      _builder.append(_queryOperationSignature, "");
      _builder.append(" {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("SimpleJdbcCall jdbcCall = prepare");
      String _name = it.getName();
      String _firstUpper = StringExtensions.toFirstUpper(_name);
      _builder.append(_firstUpper, "	");
      _builder.append("JdbcCall(");
      {
        EList<QueryParameter> _queryParameters = it.getQueryParameters();
        boolean _hasElements = false;
        for(final QueryParameter parameter : _queryParameters) {
          if (!_hasElements) {
            _hasElements = true;
          } else {
            _builder.appendImmediate(",", "	");
          }
          String _name_1 = this._features.getName(parameter);
          _builder.append(_name_1, "	");
        }
      }
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("Map<String, Object> _resultMap = execute");
      String _name_2 = it.getName();
      String _firstUpper_1 = StringExtensions.toFirstUpper(_name_2);
      _builder.append(_firstUpper_1, "	");
      _builder.append("(jdbcCall");
      {
        EList<QueryParameter> _queryParameters_1 = it.getQueryParameters();
        boolean _isEmpty = _queryParameters_1.isEmpty();
        boolean _not = (!_isEmpty);
        if (_not) {
          _builder.append(",");
        }
      }
      {
        EList<QueryParameter> _queryParameters_2 = it.getQueryParameters();
        boolean _hasElements_1 = false;
        for(final QueryParameter parameter_1 : _queryParameters_2) {
          if (!_hasElements_1) {
            _hasElements_1 = true;
          } else {
            _builder.appendImmediate(",", "	");
          }
          String _name_3 = this._features.getName(parameter_1);
          _builder.append(_name_3, "	");
        }
      }
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      {
        Type _type = it.getType();
        boolean _notEquals = (!Objects.equal(_type, null));
        if (_notEquals) {
          _builder.append("\t");
          _builder.append("return map");
          String _name_4 = it.getName();
          String _firstUpper_2 = StringExtensions.toFirstUpper(_name_4);
          _builder.append(_firstUpper_2, "	");
          _builder.append("Result(_resultMap");
          {
            EList<QueryParameter> _queryParameters_3 = it.getQueryParameters();
            boolean _isEmpty_1 = _queryParameters_3.isEmpty();
            boolean _not_1 = (!_isEmpty_1);
            if (_not_1) {
              _builder.append(",");
            }
          }
          {
            EList<QueryParameter> _queryParameters_4 = it.getQueryParameters();
            boolean _hasElements_2 = false;
            for(final QueryParameter parameter_2 : _queryParameters_4) {
              if (!_hasElements_2) {
                _hasElements_2 = true;
              } else {
                _builder.appendImmediate(",", "	");
              }
              String _name_5 = this._features.getName(parameter_2);
              _builder.append(_name_5, "	");
            }
          }
          _builder.append(");");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _builder.append("protected SimpleJdbcCall prepare");
      String _name_6 = it.getName();
      String _firstUpper_3 = StringExtensions.toFirstUpper(_name_6);
      _builder.append(_firstUpper_3, "");
      _builder.append("JdbcCall(");
      {
        EList<QueryParameter> _queryParameters_5 = it.getQueryParameters();
        boolean _hasElements_3 = false;
        for(final QueryParameter parameter_3 : _queryParameters_5) {
          if (!_hasElements_3) {
            _hasElements_3 = true;
          } else {
            _builder.appendImmediate(",", "");
          }
          CharSequence _methodParameter = this._features.methodParameter(parameter_3);
          _builder.append(_methodParameter, "");
        }
      }
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      String _addImport = this._names.addImport(it, "org.springframework.jdbc.core.simple.SimpleJdbcCall");
      _builder.append(_addImport, "	");
      _builder.append(" jdbcCall = createJdbcCall(");
      String _storedProcedureConstantName = this.getStoredProcedureConstantName(it);
      _builder.append(_storedProcedureConstantName, "	");
      {
        String _name_7 = callableStatement.getName();
        boolean _contains = _name_7.contains(".");
        if (_contains) {
          _builder.append(",\"");
          String _name_8 = callableStatement.getName();
          String _packageName = this.toPackageName(_name_8);
          _builder.append(_packageName, "	");
          _builder.append("\"");
        }
      }
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      {
        boolean _isFunctionCall = callableStatement.isFunctionCall();
        if (_isFunctionCall) {
          _builder.append("\t");
          _builder.append("jdbcCall.setFunction(true);");
          _builder.newLine();
        }
      }
      {
        boolean _isMany = it.isMany();
        if (_isMany) {
          {
            Type _type_1 = it.getType();
            if ((_type_1 instanceof ComplexType)) {
              _builder.append("\t");
              _builder.append("jdbcCall.addDeclaredRowMapper(\"_resultSet\", new ");
              String _addImport_1 = this._names.addImport(it, "org.springframework.jdbc.core.simple.ParameterizedRowMapper");
              _builder.append(_addImport_1, "	");
              _builder.append("<");
              Type _type_2 = it.getType();
              String _name_9 = _type_2.getName();
              _builder.append(_name_9, "	");
              _builder.append(">() {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("public ");
              Type _type_3 = it.getType();
              String _name_10 = _type_3.getName();
              _builder.append(_name_10, "		");
              _builder.append(" mapRow(");
              String _addImport_2 = this._names.addImport(it, "java.sql.ResultSet");
              _builder.append(_addImport_2, "		");
              _builder.append(" rs, int rowNum) throws ");
              String _addImport_3 = this._names.addImport(it, "java.sql.SQLException");
              _builder.append(_addImport_3, "		");
              _builder.append(" {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t\t");
              Type _type_4 = it.getType();
              String _name_11 = _type_4.getName();
              _builder.append(_name_11, "			");
              _builder.append(" _");
              Type _type_5 = it.getType();
              String _name_12 = _type_5.getName();
              String _firstLower = StringExtensions.toFirstLower(_name_12);
              _builder.append(_firstLower, "			");
              _builder.append(" = new ");
              Type _type_6 = it.getType();
              String _name_13 = _type_6.getName();
              _builder.append(_name_13, "			");
              _builder.append("();");
              _builder.newLineIfNotEmpty();
              {
                EList<CallOutputParameter> _outParameter = callableStatement.getOutParameter();
                final Function1<CallOutputParameter,Boolean> _function = new Function1<CallOutputParameter,Boolean>() {
                    public Boolean apply(final CallOutputParameter e) {
                      Attribute _attribute = e.getAttribute();
                      boolean _notEquals = (!Objects.equal(_attribute, null));
                      return Boolean.valueOf(_notEquals);
                    }
                  };
                Iterable<CallOutputParameter> _filter = IterableExtensions.<CallOutputParameter>filter(_outParameter, _function);
                for(final CallOutputParameter outParameter : _filter) {
                  _builder.append("\t");
                  _builder.append("\t\t");
                  _builder.append("_");
                  Type _type_7 = it.getType();
                  String _name_14 = _type_7.getName();
                  String _firstLower_1 = StringExtensions.toFirstLower(_name_14);
                  _builder.append(_firstLower_1, "			");
                  _builder.append(".");
                  Attribute _attribute = outParameter.getAttribute();
                  String _mutatorMethodName = this._features.getMutatorMethodName(_attribute);
                  _builder.append(_mutatorMethodName, "			");
                  _builder.append("(");
                  String _addImport_4 = this._names.addImport(it, "org.openxma.dsl.platform.jdbc.JdbcUtils2");
                  _builder.append(_addImport_4, "			");
                  _builder.append(".getResultSetValue(rs,\"");
                  String _name_15 = outParameter.getName();
                  _builder.append(_name_15, "			");
                  _builder.append("\",");
                  Attribute _attribute_1 = outParameter.getAttribute();
                  DataTypeAndTypeParameter _type_8 = _attribute_1.getType();
                  String _qualifiedName = this._names.getQualifiedName(_type_8);
                  _builder.append(_qualifiedName, "			");
                  _builder.append(".class));");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              _builder.append("\t\t");
              _builder.append("return _");
              Type _type_9 = it.getType();
              String _name_16 = _type_9.getName();
              String _firstLower_2 = StringExtensions.toFirstLower(_name_16);
              _builder.append(_firstLower_2, "			");
              _builder.append(";");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("});");
              _builder.newLine();
            } else {
              _builder.append("\t");
              _builder.append("jdbcCall.addDeclaredRowMapper(\"_resultSet\", new ");
              String _addImport_5 = this._names.addImport(it, "org.springframework.jdbc.core.simple.ParameterizedRowMapper");
              _builder.append(_addImport_5, "	");
              _builder.append("<");
              Type _type_10 = it.getType();
              String _name_17 = _type_10.getName();
              _builder.append(_name_17, "	");
              _builder.append(">() {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("public ");
              Type _type_11 = it.getType();
              String _name_18 = _type_11.getName();
              _builder.append(_name_18, "		");
              _builder.append(" mapRow(");
              String _addImport_6 = this._names.addImport(it, "java.sql.ResultSet");
              _builder.append(_addImport_6, "		");
              _builder.append(" rs, int rowNum) throws ");
              String _addImport_7 = this._names.addImport(it, "java.sql.SQLException");
              _builder.append(_addImport_7, "		");
              _builder.append(" {");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t\t");
              _builder.append("return ");
              String _addImport_8 = this._names.addImport(it, "org.openxma.dsl.platform.jdbc.JdbcUtils2");
              _builder.append(_addImport_8, "			");
              _builder.append(".getResultSetValue(rs,\"");
              EList<CallOutputParameter> _outParameter_1 = callableStatement.getOutParameter();
              CallOutputParameter _head = IterableExtensions.<CallOutputParameter>head(_outParameter_1);
              String _name_19 = _head.getName();
              _builder.append(_name_19, "			");
              _builder.append("\",");
              Type _type_12 = it.getType();
              String _qualifiedName_1 = this._names.getQualifiedName(_type_12);
              _builder.append(_qualifiedName_1, "			");
              _builder.append(".class);");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("});");
              _builder.newLine();
            }
          }
        } else {
          Type _type_13 = it.getType();
          if ((_type_13 instanceof ComplexType)) {
            {
              EList<CallOutputParameter> _outParameter_2 = callableStatement.getOutParameter();
              final Function1<CallOutputParameter,Boolean> _function_1 = new Function1<CallOutputParameter,Boolean>() {
                  public Boolean apply(final CallOutputParameter e) {
                    boolean _and = false;
                    Attribute _attribute = e.getAttribute();
                    boolean _isMany = _attribute.isMany();
                    if (!_isMany) {
                      _and = false;
                    } else {
                      Attribute _attribute_1 = e.getAttribute();
                      Type _dataType = _attribute_1.getDataType();
                      boolean _not = (!(_dataType instanceof ComplexType));
                      _and = (_isMany && _not);
                    }
                    return Boolean.valueOf(_and);
                  }
                };
              Iterable<CallOutputParameter> _filter_1 = IterableExtensions.<CallOutputParameter>filter(_outParameter_2, _function_1);
              for(final CallOutputParameter outParameter_1 : _filter_1) {
                _builder.append("\t");
                _builder.append("jdbcCall.addDeclaredRowMapper(\"_");
                Attribute _attribute_2 = outParameter_1.getAttribute();
                String _name_20 = _attribute_2.getName();
                _builder.append(_name_20, "	");
                _builder.append("_resultSet\", new ");
                String _addImport_9 = this._names.addImport(it, "org.springframework.jdbc.core.simple.ParameterizedRowMapper");
                _builder.append(_addImport_9, "	");
                _builder.append("<");
                Attribute _attribute_3 = outParameter_1.getAttribute();
                DataTypeAndTypeParameter _type_14 = _attribute_3.getType();
                String _qualifiedName_2 = this._names.getQualifiedName(_type_14);
                _builder.append(_qualifiedName_2, "	");
                _builder.append(">() {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("\t");
                _builder.append("public ");
                Attribute _attribute_4 = outParameter_1.getAttribute();
                DataTypeAndTypeParameter _type_15 = _attribute_4.getType();
                String _qualifiedName_3 = this._names.getQualifiedName(_type_15);
                _builder.append(_qualifiedName_3, "		");
                _builder.append(" mapRow(");
                String _addImport_10 = this._names.addImport(it, "java.sql.ResultSet");
                _builder.append(_addImport_10, "		");
                _builder.append(" rs, int rowNum) throws ");
                String _addImport_11 = this._names.addImport(it, "java.sql.SQLException");
                _builder.append(_addImport_11, "		");
                _builder.append(" {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("\t\t");
                _builder.append("return ");
                String _addImport_12 = this._names.addImport(it, "org.openxma.dsl.platform.jdbc.JdbcUtils2");
                _builder.append(_addImport_12, "			");
                _builder.append(".getResultSetValue(rs,\"");
                String _name_21 = outParameter_1.getName();
                _builder.append(_name_21, "			");
                _builder.append("\",");
                Attribute _attribute_5 = outParameter_1.getAttribute();
                DataTypeAndTypeParameter _type_16 = _attribute_5.getType();
                String _qualifiedName_4 = this._names.getQualifiedName(_type_16);
                _builder.append(_qualifiedName_4, "			");
                _builder.append(".class);");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("\t");
                _builder.append("}");
                _builder.newLine();
                _builder.append("\t");
                _builder.append("});");
                _builder.newLine();
              }
            }
            _builder.append("\t");
            EList<CallOutputParameter> _outParameter_3 = callableStatement.getOutParameter();
            final Function1<CallOutputParameter,Attribute> _function_2 = new Function1<CallOutputParameter,Attribute>() {
                public Attribute apply(final CallOutputParameter p) {
                  Attribute _attribute = p.getAttribute();
                  return _attribute;
                }
              };
            final List<Attribute> outParameterAttributes = ListExtensions.<CallOutputParameter, Attribute>map(_outParameter_3, _function_2);
            _builder.append(" ");
            _builder.newLineIfNotEmpty();
            {
              final Function1<Attribute,Boolean> _function_3 = new Function1<Attribute,Boolean>() {
                  public Boolean apply(final Attribute e) {
                    boolean _and = false;
                    boolean _isMany = e.isMany();
                    if (!_isMany) {
                      _and = false;
                    } else {
                      Type _dataType = e.getDataType();
                      _and = (_isMany && (_dataType instanceof ComplexType));
                    }
                    return Boolean.valueOf(_and);
                  }
                };
              Iterable<Attribute> _filter_2 = IterableExtensions.<Attribute>filter(outParameterAttributes, _function_3);
              Set<Attribute> _set = IterableExtensions.<Attribute>toSet(_filter_2);
              for(final Attribute complexTypeAttribute : _set) {
                _builder.append("\t");
                Index _index = new Index();
                final Index index = _index;
                _builder.newLineIfNotEmpty();
                {
                  EList<CallOutputParameter> _outParameter_4 = callableStatement.getOutParameter();
                  final Function1<CallOutputParameter,Boolean> _function_4 = new Function1<CallOutputParameter,Boolean>() {
                      public Boolean apply(final CallOutputParameter e) {
                        Attribute _attribute = e.getAttribute();
                        boolean _equals = Objects.equal(_attribute, complexTypeAttribute);
                        return Boolean.valueOf(_equals);
                      }
                    };
                  Iterable<CallOutputParameter> _filter_3 = IterableExtensions.<CallOutputParameter>filter(_outParameter_4, _function_4);
                  for(final CallOutputParameter outParameter_2 : _filter_3) {
                    {
                      boolean _firstIteration = index.firstIteration();
                      if (_firstIteration) {
                        _builder.append("\t");
                        _builder.append("jdbcCall.addDeclaredRowMapper(\"_");
                        Attribute _attribute_6 = outParameter_2.getAttribute();
                        String _name_22 = _attribute_6.getName();
                        _builder.append(_name_22, "	");
                        _builder.append("_resultSet\", new ");
                        String _addImport_13 = this._names.addImport(it, "org.springframework.jdbc.core.simple.ParameterizedRowMapper");
                        _builder.append(_addImport_13, "	");
                        _builder.append("<");
                        DataTypeAndTypeParameter _type_17 = complexTypeAttribute.getType();
                        String _qualifiedName_5 = this._names.getQualifiedName(_type_17);
                        _builder.append(_qualifiedName_5, "	");
                        _builder.append(">() {");
                        _builder.newLineIfNotEmpty();
                        _builder.append("\t");
                        _builder.append("\t");
                        _builder.append("public ");
                        Attribute _attribute_7 = outParameter_2.getAttribute();
                        DataTypeAndTypeParameter _type_18 = _attribute_7.getType();
                        String _qualifiedName_6 = this._names.getQualifiedName(_type_18);
                        _builder.append(_qualifiedName_6, "		");
                        _builder.append(" mapRow(");
                        String _addImport_14 = this._names.addImport(it, "java.sql.ResultSet");
                        _builder.append(_addImport_14, "		");
                        _builder.append(" rs, int rowNum) throws ");
                        String _addImport_15 = this._names.addImport(it, "java.sql.SQLException");
                        _builder.append(_addImport_15, "		");
                        _builder.append(" {");
                        _builder.newLineIfNotEmpty();
                        _builder.append("\t");
                        _builder.append("\t\t");
                        DataTypeAndTypeParameter _type_19 = complexTypeAttribute.getType();
                        String _qualifiedName_7 = this._names.getQualifiedName(_type_19);
                        _builder.append(_qualifiedName_7, "			");
                        _builder.append(" _");
                        String _name_23 = complexTypeAttribute.getName();
                        String _firstLower_3 = StringExtensions.toFirstLower(_name_23);
                        _builder.append(_firstLower_3, "			");
                        _builder.append(" = new ");
                        DataTypeAndTypeParameter _type_20 = complexTypeAttribute.getType();
                        String _qualifiedName_8 = this._names.getQualifiedName(_type_20);
                        _builder.append(_qualifiedName_8, "			");
                        _builder.append("();\t   ");
                        _builder.newLineIfNotEmpty();
                      }
                    }
                    _builder.append("\t");
                    _builder.append("\t\t");
                    _builder.append("_");
                    String _name_24 = complexTypeAttribute.getName();
                    String _firstLower_4 = StringExtensions.toFirstLower(_name_24);
                    _builder.append(_firstLower_4, "			");
                    _builder.append(".");
                    Attribute _nestedAttribute = outParameter_2.getNestedAttribute();
                    String _mutatorMethodName_1 = this._features.getMutatorMethodName(_nestedAttribute);
                    _builder.append(_mutatorMethodName_1, "			");
                    _builder.append("(");
                    String _addImport_16 = this._names.addImport(it, "org.openxma.dsl.platform.jdbc.JdbcUtils2");
                    _builder.append(_addImport_16, "			");
                    _builder.append(".getResultSetValue(rs,\"");
                    String _name_25 = outParameter_2.getName();
                    _builder.append(_name_25, "			");
                    _builder.append("\",");
                    Attribute _nestedAttribute_1 = outParameter_2.getNestedAttribute();
                    DataTypeAndTypeParameter _type_21 = _nestedAttribute_1.getType();
                    String _qualifiedName_9 = this._names.getQualifiedName(_type_21);
                    _builder.append(_qualifiedName_9, "			");
                    _builder.append(".class));\t ");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    index.increment();
                    _builder.newLineIfNotEmpty();
                  }
                }
                _builder.append("\t");
                _builder.append("\t\t");
                _builder.append("return _");
                String _name_26 = complexTypeAttribute.getName();
                String _firstLower_5 = StringExtensions.toFirstLower(_name_26);
                _builder.append(_firstLower_5, "			");
                _builder.append(";");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("\t");
                _builder.append("}");
                _builder.newLine();
                _builder.append("\t");
                _builder.append("});");
                _builder.newLine();
              }
            }
          }
        }
      }
      _builder.append("\t");
      _builder.append("return jdbcCall;");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _builder.append("protected Map<String, Object> execute");
      String _name_27 = it.getName();
      String _firstUpper_4 = StringExtensions.toFirstUpper(_name_27);
      _builder.append(_firstUpper_4, "");
      _builder.append("(SimpleJdbcCall jdbcCall");
      {
        EList<QueryParameter> _queryParameters_6 = it.getQueryParameters();
        boolean _isEmpty_2 = _queryParameters_6.isEmpty();
        boolean _not_2 = (!_isEmpty_2);
        if (_not_2) {
          _builder.append(",");
        }
      }
      {
        EList<QueryParameter> _queryParameters_7 = it.getQueryParameters();
        boolean _hasElements_4 = false;
        for(final QueryParameter parameter_4 : _queryParameters_7) {
          if (!_hasElements_4) {
            _hasElements_4 = true;
          } else {
            _builder.appendImmediate(",", "");
          }
          CharSequence _methodParameter_1 = this._features.methodParameter(parameter_4);
          _builder.append(_methodParameter_1, "");
        }
      }
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      String _addImport_17 = this._names.addImport(it, "java.util.Map");
      _builder.append(_addImport_17, "	");
      _builder.append("<String,Object> parametersMap = new ");
      String _addImport_18 = this._names.addImport(it, "java.util.HashMap");
      _builder.append(_addImport_18, "	");
      _builder.append("<String,Object>();");
      _builder.newLineIfNotEmpty();
      {
        EList<CallInputParameter> _inParameter = callableStatement.getInParameter();
        for(final CallInputParameter inParameter : _inParameter) {
          _builder.append("\t");
          _builder.append("parametersMap.put(\"");
          String _parameterName = this.getParameterName(inParameter);
          _builder.append(_parameterName, "	");
          _builder.append("\",");
          {
            Attribute _attribute_8 = inParameter.getAttribute();
            boolean _notEquals_1 = (!Objects.equal(_attribute_8, null));
            if (_notEquals_1) {
              QueryParameter _parameter = inParameter.getParameter();
              String _name_28 = this._features.getName(_parameter);
              _builder.append(_name_28, "	");
              _builder.append("==null ? null:");
              QueryParameter _parameter_1 = inParameter.getParameter();
              String _name_29 = this._features.getName(_parameter_1);
              _builder.append(_name_29, "	");
              _builder.append(".");
              Attribute _attribute_9 = inParameter.getAttribute();
              String _accessorMethodName = this._features.getAccessorMethodName(_attribute_9);
              _builder.append(_accessorMethodName, "	");
              _builder.append("()");
            } else {
              QueryParameter _parameter_2 = inParameter.getParameter();
              String _name_30 = this._features.getName(_parameter_2);
              _builder.append(_name_30, "	");
            }
          }
          _builder.append(");");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("\t");
      _builder.append("return executeJdbcCall(jdbcCall,parametersMap);");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      {
        Type _type_22 = it.getType();
        boolean _notEquals_2 = (!Objects.equal(_type_22, null));
        if (_notEquals_2) {
          _builder.newLine();
          _builder.append("protected ");
          String _type_23 = this._features.toType(it);
          _builder.append(_type_23, "");
          _builder.append(" map");
          String _name_31 = it.getName();
          String _firstUpper_5 = StringExtensions.toFirstUpper(_name_31);
          _builder.append(_firstUpper_5, "");
          _builder.append("Result(Map<String, Object> _resultMap");
          {
            EList<QueryParameter> _queryParameters_8 = it.getQueryParameters();
            boolean _isEmpty_3 = _queryParameters_8.isEmpty();
            boolean _not_3 = (!_isEmpty_3);
            if (_not_3) {
              _builder.append(",");
            }
          }
          {
            EList<QueryParameter> _queryParameters_9 = it.getQueryParameters();
            boolean _hasElements_5 = false;
            for(final QueryParameter parameter_5 : _queryParameters_9) {
              if (!_hasElements_5) {
                _hasElements_5 = true;
              } else {
                _builder.appendImmediate(",", "");
              }
              CharSequence _methodParameter_2 = this._features.methodParameter(parameter_5);
              _builder.append(_methodParameter_2, "");
            }
          }
          _builder.append(") {");
          _builder.newLineIfNotEmpty();
          {
            boolean _isMany_1 = it.isMany();
            if (_isMany_1) {
              _builder.append("\t");
              _builder.append("@SuppressWarnings(\"unchecked\")");
              _builder.newLine();
              _builder.append("\t");
              String _type_24 = this._features.toType(it);
              _builder.append(_type_24, "	");
              _builder.append(" _resultSet = (");
              String _type_25 = this._features.toType(it);
              _builder.append(_type_25, "	");
              _builder.append(") _resultMap.get(\"_resultSet\");");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("return _resultSet;");
              _builder.newLine();
            } else {
              Type _type_26 = it.getType();
              if ((_type_26 instanceof ComplexType)) {
                _builder.append("\t");
                Type _type_27 = it.getType();
                String _name_32 = _type_27.getName();
                _builder.append(_name_32, "	");
                _builder.append(" _");
                Type _type_28 = it.getType();
                String _name_33 = _type_28.getName();
                String _firstLower_6 = StringExtensions.toFirstLower(_name_33);
                _builder.append(_firstLower_6, "	");
                _builder.append(" = new ");
                Type _type_29 = it.getType();
                String _name_34 = _type_29.getName();
                _builder.append(_name_34, "	");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
                {
                  EList<CallOutputParameter> _outParameter_5 = callableStatement.getOutParameter();
                  final Function1<CallOutputParameter,Boolean> _function_5 = new Function1<CallOutputParameter,Boolean>() {
                      public Boolean apply(final CallOutputParameter e) {
                        Attribute _attribute = e.getAttribute();
                        Type _dataType = _attribute.getDataType();
                        boolean _not = (!(_dataType instanceof ComplexType));
                        return Boolean.valueOf(_not);
                      }
                    };
                  Iterable<CallOutputParameter> _filter_4 = IterableExtensions.<CallOutputParameter>filter(_outParameter_5, _function_5);
                  for(final CallOutputParameter outParameter_3 : _filter_4) {
                    {
                      Attribute _attribute_10 = outParameter_3.getAttribute();
                      boolean _isMany_2 = _attribute_10.isMany();
                      if (_isMany_2) {
                        _builder.append("\t");
                        _builder.append("@SuppressWarnings(\"unchecked\")");
                        _builder.newLine();
                        _builder.append("\t");
                        String _addImport_19 = this._names.addImport(it, "java.util.List");
                        _builder.append(_addImport_19, "	");
                        _builder.append("<");
                        Attribute _attribute_11 = outParameter_3.getAttribute();
                        DataTypeAndTypeParameter _type_30 = _attribute_11.getType();
                        String _qualifiedName_10 = this._names.getQualifiedName(_type_30);
                        _builder.append(_qualifiedName_10, "	");
                        _builder.append("> _");
                        Attribute _attribute_12 = outParameter_3.getAttribute();
                        String _name_35 = _attribute_12.getName();
                        _builder.append(_name_35, "	");
                        _builder.append("ResultSet = (List<");
                        Attribute _attribute_13 = outParameter_3.getAttribute();
                        DataTypeAndTypeParameter _type_31 = _attribute_13.getType();
                        String _qualifiedName_11 = this._names.getQualifiedName(_type_31);
                        _builder.append(_qualifiedName_11, "	");
                        _builder.append(">) _resultMap.get(\"_");
                        Attribute _attribute_14 = outParameter_3.getAttribute();
                        String _name_36 = _attribute_14.getName();
                        _builder.append(_name_36, "	");
                        _builder.append("_resultSet\");");
                        _builder.newLineIfNotEmpty();
                        _builder.append("\t");
                        _builder.append("_");
                        Type _type_32 = it.getType();
                        String _name_37 = _type_32.getName();
                        String _firstLower_7 = StringExtensions.toFirstLower(_name_37);
                        _builder.append(_firstLower_7, "	");
                        _builder.append(".");
                        Attribute _attribute_15 = outParameter_3.getAttribute();
                        String _mutatorMethodName_2 = this._features.getMutatorMethodName(_attribute_15);
                        _builder.append(_mutatorMethodName_2, "	");
                        _builder.append("(_");
                        Attribute _attribute_16 = outParameter_3.getAttribute();
                        String _name_38 = _attribute_16.getName();
                        _builder.append(_name_38, "	");
                        _builder.append("ResultSet);");
                        _builder.newLineIfNotEmpty();
                      } else {
                        Attribute _attribute_17 = outParameter_3.getAttribute();
                        boolean _isMany_3 = _attribute_17.isMany();
                        boolean _not_4 = (!_isMany_3);
                        if (_not_4) {
                          _builder.append("\t");
                          _builder.append("_");
                          Type _type_33 = it.getType();
                          String _name_39 = _type_33.getName();
                          String _firstLower_8 = StringExtensions.toFirstLower(_name_39);
                          _builder.append(_firstLower_8, "	");
                          _builder.append(".");
                          Attribute _attribute_18 = outParameter_3.getAttribute();
                          String _mutatorMethodName_3 = this._features.getMutatorMethodName(_attribute_18);
                          _builder.append(_mutatorMethodName_3, "	");
                          _builder.append("((");
                          Attribute _attribute_19 = outParameter_3.getAttribute();
                          DataTypeAndTypeParameter _type_34 = _attribute_19.getType();
                          String _qualifiedName_12 = this._names.getQualifiedName(_type_34);
                          _builder.append(_qualifiedName_12, "	");
                          _builder.append(")_resultMap.get(\"");
                          String _name_40 = outParameter_3.getName();
                          _builder.append(_name_40, "	");
                          _builder.append("\"));");
                          _builder.newLineIfNotEmpty();
                        }
                      }
                    }
                  }
                }
                {
                  EList<CallOutputParameter> _outParameter_6 = callableStatement.getOutParameter();
                  final Function1<CallOutputParameter,Attribute> _function_6 = new Function1<CallOutputParameter,Attribute>() {
                      public Attribute apply(final CallOutputParameter a) {
                        Attribute _attribute = a.getAttribute();
                        return _attribute;
                      }
                    };
                  List<Attribute> _map = ListExtensions.<CallOutputParameter, Attribute>map(_outParameter_6, _function_6);
                  final Function1<Attribute,Boolean> _function_7 = new Function1<Attribute,Boolean>() {
                      public Boolean apply(final Attribute e) {
                        boolean _and = false;
                        boolean _isMany = e.isMany();
                        if (!_isMany) {
                          _and = false;
                        } else {
                          Type _dataType = e.getDataType();
                          _and = (_isMany && (_dataType instanceof ComplexType));
                        }
                        return Boolean.valueOf(_and);
                      }
                    };
                  Iterable<Attribute> _filter_5 = IterableExtensions.<Attribute>filter(_map, _function_7);
                  Set<Attribute> _set_1 = IterableExtensions.<Attribute>toSet(_filter_5);
                  for(final Attribute complexTypeAttribute_1 : _set_1) {
                    _builder.append("\t");
                    _builder.append("@SuppressWarnings(\"unchecked\")");
                    _builder.newLine();
                    _builder.append("\t");
                    String _addImport_20 = this._names.addImport(it, "java.util.List");
                    _builder.append(_addImport_20, "	");
                    _builder.append("<");
                    DataTypeAndTypeParameter _type_35 = complexTypeAttribute_1.getType();
                    String _qualifiedName_13 = this._names.getQualifiedName(_type_35);
                    _builder.append(_qualifiedName_13, "	");
                    _builder.append("> _");
                    String _name_41 = complexTypeAttribute_1.getName();
                    _builder.append(_name_41, "	");
                    _builder.append("ResultSet = (List<");
                    DataTypeAndTypeParameter _type_36 = complexTypeAttribute_1.getType();
                    String _qualifiedName_14 = this._names.getQualifiedName(_type_36);
                    _builder.append(_qualifiedName_14, "	");
                    _builder.append(">) _resultMap.get(\"_");
                    String _name_42 = complexTypeAttribute_1.getName();
                    _builder.append(_name_42, "	");
                    _builder.append("_resultSet\");");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("_");
                    Type _type_37 = it.getType();
                    String _name_43 = _type_37.getName();
                    String _firstLower_9 = StringExtensions.toFirstLower(_name_43);
                    _builder.append(_firstLower_9, "	");
                    _builder.append(".");
                    String _mutatorMethodName_4 = this._features.getMutatorMethodName(complexTypeAttribute_1);
                    _builder.append(_mutatorMethodName_4, "	");
                    _builder.append("(_");
                    String _name_44 = complexTypeAttribute_1.getName();
                    _builder.append(_name_44, "	");
                    _builder.append("ResultSet);");
                    _builder.newLineIfNotEmpty();
                  }
                }
                {
                  EList<CallOutputParameter> _outParameter_7 = callableStatement.getOutParameter();
                  final Function1<CallOutputParameter,Attribute> _function_8 = new Function1<CallOutputParameter,Attribute>() {
                      public Attribute apply(final CallOutputParameter a) {
                        Attribute _attribute = a.getAttribute();
                        return _attribute;
                      }
                    };
                  List<Attribute> _map_1 = ListExtensions.<CallOutputParameter, Attribute>map(_outParameter_7, _function_8);
                  final Function1<Attribute,Boolean> _function_9 = new Function1<Attribute,Boolean>() {
                      public Boolean apply(final Attribute e) {
                        boolean _and = false;
                        boolean _isMany = e.isMany();
                        boolean _not = (!_isMany);
                        if (!_not) {
                          _and = false;
                        } else {
                          Type _dataType = e.getDataType();
                          _and = (_not && (_dataType instanceof ComplexType));
                        }
                        return Boolean.valueOf(_and);
                      }
                    };
                  Iterable<Attribute> _filter_6 = IterableExtensions.<Attribute>filter(_map_1, _function_9);
                  Set<Attribute> _set_2 = IterableExtensions.<Attribute>toSet(_filter_6);
                  for(final Attribute complexTypeAttribute_2 : _set_2) {
                    _builder.append("\t");
                    Index _index_1 = new Index();
                    final Index index_1 = _index_1;
                    _builder.newLineIfNotEmpty();
                    {
                      EList<CallOutputParameter> _outParameter_8 = callableStatement.getOutParameter();
                      final Function1<CallOutputParameter,Boolean> _function_10 = new Function1<CallOutputParameter,Boolean>() {
                          public Boolean apply(final CallOutputParameter e) {
                            boolean _and = false;
                            Attribute _attribute = e.getAttribute();
                            boolean _equals = Objects.equal(_attribute, complexTypeAttribute_2);
                            if (!_equals) {
                              _and = false;
                            } else {
                              Attribute _nestedAttribute = e.getNestedAttribute();
                              boolean _notEquals = (!Objects.equal(_nestedAttribute, null));
                              _and = (_equals && _notEquals);
                            }
                            return Boolean.valueOf(_and);
                          }
                        };
                      Iterable<CallOutputParameter> _filter_7 = IterableExtensions.<CallOutputParameter>filter(_outParameter_8, _function_10);
                      for(final CallOutputParameter outParameter_4 : _filter_7) {
                        {
                          boolean _firstIteration_1 = index_1.firstIteration();
                          if (_firstIteration_1) {
                            _builder.append("\t");
                            Type _dataType = complexTypeAttribute_2.getDataType();
                            String _qualifiedName_15 = this._names.getQualifiedName(_dataType);
                            _builder.append(_qualifiedName_15, "	");
                            _builder.append(" _");
                            String _name_45 = complexTypeAttribute_2.getName();
                            _builder.append(_name_45, "	");
                            _builder.append(" = new ");
                            Type _dataType_1 = complexTypeAttribute_2.getDataType();
                            String _qualifiedName_16 = this._names.getQualifiedName(_dataType_1);
                            _builder.append(_qualifiedName_16, "	");
                            _builder.append("();");
                            _builder.newLineIfNotEmpty();
                            _builder.append("\t");
                            _builder.append("_");
                            Type _type_38 = it.getType();
                            String _name_46 = _type_38.getName();
                            String _firstLower_10 = StringExtensions.toFirstLower(_name_46);
                            _builder.append(_firstLower_10, "	");
                            _builder.append(".");
                            String _mutatorMethodName_5 = this._features.getMutatorMethodName(complexTypeAttribute_2);
                            _builder.append(_mutatorMethodName_5, "	");
                            _builder.append("(_");
                            String _name_47 = complexTypeAttribute_2.getName();
                            _builder.append(_name_47, "	");
                            _builder.append(");");
                            _builder.newLineIfNotEmpty();
                          }
                        }
                        _builder.append("\t");
                        _builder.append("_");
                        String _name_48 = complexTypeAttribute_2.getName();
                        _builder.append(_name_48, "	");
                        _builder.append(".");
                        Attribute _nestedAttribute_2 = outParameter_4.getNestedAttribute();
                        String _mutatorMethodName_6 = this._features.getMutatorMethodName(_nestedAttribute_2);
                        _builder.append(_mutatorMethodName_6, "	");
                        _builder.append("((");
                        Attribute _nestedAttribute_3 = outParameter_4.getNestedAttribute();
                        DataTypeAndTypeParameter _type_39 = _nestedAttribute_3.getType();
                        String _qualifiedName_17 = this._names.getQualifiedName(_type_39);
                        _builder.append(_qualifiedName_17, "	");
                        _builder.append(")_resultMap.get(\"");
                        String _name_49 = outParameter_4.getName();
                        _builder.append(_name_49, "	");
                        _builder.append("\"));");
                        _builder.newLineIfNotEmpty();
                        _builder.append("\t");
                        index_1.increment();
                        _builder.newLineIfNotEmpty();
                      }
                    }
                  }
                }
                _builder.append("\t");
                _builder.append("return _");
                Type _type_40 = it.getType();
                String _name_50 = _type_40.getName();
                String _firstLower_11 = StringExtensions.toFirstLower(_name_50);
                _builder.append(_firstLower_11, "	");
                _builder.append(";");
                _builder.newLineIfNotEmpty();
              } else {
                _builder.append("\t");
                _builder.append("return (");
                String _type_41 = this._features.toType(it);
                _builder.append(_type_41, "	");
                _builder.append(") ");
                {
                  boolean _isFunctionCall_1 = callableStatement.isFunctionCall();
                  if (_isFunctionCall_1) {
                    _builder.append("_resultMap.get(\"return\")");
                  } else {
                    _builder.append("_resultMap.get(\"");
                    EList<CallOutputParameter> _outParameter_9 = callableStatement.getOutParameter();
                    CallOutputParameter _head_1 = IterableExtensions.<CallOutputParameter>head(_outParameter_9);
                    String _name_51 = _head_1.getName();
                    _builder.append(_name_51, "	");
                    _builder.append("\")");
                  }
                }
                _builder.append(";");
                _builder.newLineIfNotEmpty();
              }
            }
          }
          _builder.append("}");
          _builder.newLine();
        }
      }
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  private String getParameterName(final CallInputParameter it) {
    String _xifexpression = null;
    String _name = it.getName();
    boolean _notEquals = (!Objects.equal(_name, null));
    if (_notEquals) {
      String _name_1 = it.getName();
      _xifexpression = _name_1;
    } else {
      String _xifexpression_1 = null;
      Attribute _attribute = it.getAttribute();
      boolean _notEquals_1 = (!Objects.equal(_attribute, null));
      if (_notEquals_1) {
        Attribute _attribute_1 = it.getAttribute();
        String _name_2 = _attribute_1.getName();
        _xifexpression_1 = _name_2;
      } else {
        QueryParameter _parameter = it.getParameter();
        String _name_3 = _parameter==null?(String)null:this._features.getName(_parameter);
        _xifexpression_1 = _name_3;
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  public CharSequence toNamedQueryCall(final Operation it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _daoOperationSignature = this.daoOperationSignature(it, "public");
    _builder.append(_daoOperationSignature, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      EList<Parameter> _parameters = it.getParameters();
      final Function1<Parameter,Boolean> _function = new Function1<Parameter,Boolean>() {
          public Boolean apply(final Parameter e) {
            boolean _and = false;
            String _name = e.getName();
            boolean _notEquals = (!Objects.equal(_name, "firstResult"));
            if (!_notEquals) {
              _and = false;
            } else {
              String _name_1 = e.getName();
              boolean _notEquals_1 = (!Objects.equal(_name_1, "maxResults"));
              _and = (_notEquals && _notEquals_1);
            }
            return Boolean.valueOf(_and);
          }
        };
      Iterable<Parameter> _filter = IterableExtensions.<Parameter>filter(_parameters, _function);
      for(final Parameter parameter : _filter) {
        _builder.append("\t");
        _builder.append("Assert.notNull(");
        String _variableName = this._names.toVariableName(parameter);
        _builder.append(_variableName, "	");
        _builder.append(", \"Parameter \'");
        String _variableName_1 = this._names.toVariableName(parameter);
        _builder.append(_variableName_1, "	");
        _builder.append("\' must not be null\");");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(\"");
    EObject _eContainer = it.eContainer();
    Entity _entity = ((Dao) _eContainer).getEntity();
    String _name = _entity.getName();
    _builder.append(_name, "	");
    _builder.append(".");
    String _name_1 = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper, "	");
    _builder.append("\");");
    _builder.newLineIfNotEmpty();
    {
      String _expression = it.getExpression();
      boolean _contains = _expression.contains("?");
      if (_contains) {
        _builder.append("\t");
        Index _index = new Index();
        final Index iter = _index;
        _builder.newLineIfNotEmpty();
        {
          EList<Parameter> _parameters_1 = it.getParameters();
          for(final Parameter parameter_1 : _parameters_1) {
            _builder.append("\t");
            _builder.append("namedQuery.setParameter(");
            int _counter0 = iter.getCounter0();
            _builder.append(_counter0, "	");
            _builder.append(", ");
            String _name_2 = parameter_1.getName();
            _builder.append(_name_2, "	");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            iter.increment();
            _builder.newLineIfNotEmpty();
          }
        }
      } else {
        {
          EList<Parameter> _parameters_2 = it.getParameters();
          for(final Parameter parameter_2 : _parameters_2) {
            {
              String _name_3 = parameter_2.getName();
              boolean _equals = "firstResult".equals(_name_3);
              if (_equals) {
                _builder.append("\t");
                _builder.append("if (firstResult!=null) {");
                _builder.newLine();
                _builder.append("\t");
                _builder.append("    ");
                _builder.append("namedQuery.setFirstResult(");
                String _name_4 = parameter_2.getName();
                _builder.append(_name_4, "	    ");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("}");
                _builder.newLine();
              } else {
                String _name_5 = parameter_2.getName();
                boolean _equals_1 = "maxResults".equals(_name_5);
                if (_equals_1) {
                  _builder.append("\t");
                  _builder.append("if (maxResults!=null) {");
                  _builder.newLine();
                  _builder.append("\t");
                  _builder.append("    ");
                  _builder.append("namedQuery.setMaxResults(");
                  String _name_6 = parameter_2.getName();
                  _builder.append(_name_6, "	    ");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                } else {
                  boolean _and = false;
                  Type _type = parameter_2.getType();
                  if (!(_type instanceof SimpleType)) {
                    _and = false;
                  } else {
                    Type _type_1 = parameter_2.getType();
                    ValidatorReference _validatorReference = ((SimpleType) _type_1).getValidatorReference();
                    boolean _isDateValidator = _validatorReference.isDateValidator();
                    _and = ((_type instanceof SimpleType) && _isDateValidator);
                  }
                  if (_and) {
                    _builder.append("\t");
                    _builder.append("namedQuery.setDate");
                    {
                      boolean _isMany = parameter_2.isMany();
                      if (_isMany) {
                        _builder.append("List");
                      }
                    }
                    _builder.append("(\"");
                    String _name_7 = parameter_2.getName();
                    _builder.append(_name_7, "	");
                    _builder.append("\", ");
                    String _name_8 = parameter_2.getName();
                    _builder.append(_name_8, "	");
                    _builder.append(");");
                    _builder.newLineIfNotEmpty();
                  } else {
                    boolean _and_1 = false;
                    Type _type_2 = parameter_2.getType();
                    if (!(_type_2 instanceof SimpleType)) {
                      _and_1 = false;
                    } else {
                      Type _type_3 = parameter_2.getType();
                      ValidatorReference _validatorReference_1 = ((SimpleType) _type_3).getValidatorReference();
                      boolean _isTimestampValidator = ValidatorExtension.isTimestampValidator(_validatorReference_1);
                      _and_1 = ((_type_2 instanceof SimpleType) && _isTimestampValidator);
                    }
                    if (_and_1) {
                      _builder.append("\t");
                      _builder.append("namedQuery.setTimestamp");
                      {
                        boolean _isMany_1 = parameter_2.isMany();
                        if (_isMany_1) {
                          _builder.append("List");
                        }
                      }
                      _builder.append("(\"");
                      String _name_9 = parameter_2.getName();
                      _builder.append(_name_9, "	");
                      _builder.append("\", ");
                      String _name_10 = parameter_2.getName();
                      _builder.append(_name_10, "	");
                      _builder.append(");");
                      _builder.newLineIfNotEmpty();
                    } else {
                      boolean _and_2 = false;
                      Type _type_4 = parameter_2.getType();
                      if (!(_type_4 instanceof Entity)) {
                        _and_2 = false;
                      } else {
                        boolean _isMany_2 = parameter_2.isMany();
                        boolean _not = (!_isMany_2);
                        _and_2 = ((_type_4 instanceof Entity) && _not);
                      }
                      if (_and_2) {
                        _builder.append("\t");
                        _builder.append("namedQuery.setEntity(\"");
                        String _name_11 = parameter_2.getName();
                        _builder.append(_name_11, "	");
                        _builder.append("\", ");
                        String _name_12 = parameter_2.getName();
                        _builder.append(_name_12, "	");
                        _builder.append(");");
                        _builder.newLineIfNotEmpty();
                      } else {
                        _builder.append("\t");
                        _builder.append("namedQuery.setParameter");
                        {
                          boolean _isMany_3 = parameter_2.isMany();
                          if (_isMany_3) {
                            _builder.append("List");
                          }
                        }
                        _builder.append("(\"");
                        String _name_13 = parameter_2.getName();
                        _builder.append(_name_13, "	");
                        _builder.append("\", ");
                        String _name_14 = parameter_2.getName();
                        _builder.append(_name_14, "	");
                        _builder.append(");");
                        _builder.newLineIfNotEmpty();
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    _builder.append("\t");
    _builder.append("apply");
    String _name_15 = it.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_15);
    _builder.append(_firstUpper_1, "	");
    _builder.append("QueryHints(namedQuery");
    {
      EList<Parameter> _parameters_3 = it.getParameters();
      boolean _isEmpty = _parameters_3.isEmpty();
      boolean _not_1 = (!_isEmpty);
      if (_not_1) {
        _builder.append(",");
      }
    }
    {
      EList<Parameter> _parameters_4 = it.getParameters();
      boolean _hasElements = false;
      for(final Parameter parameter_3 : _parameters_4) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "	");
        }
        String _name_16 = parameter_3.getName();
        _builder.append(_name_16, "	");
      }
    }
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    {
      boolean _isMany_4 = it.isMany();
      if (_isMany_4) {
        _builder.append("list(namedQuery)");
      } else {
        _builder.append("unique(namedQuery)");
      }
    }
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("protected void apply");
    String _name_17 = it.getName();
    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_17);
    _builder.append(_firstUpper_2, "");
    _builder.append("QueryHints(Query query");
    {
      EList<Parameter> _parameters_5 = it.getParameters();
      boolean _isEmpty_1 = _parameters_5.isEmpty();
      boolean _not_2 = (!_isEmpty_1);
      if (_not_2) {
        _builder.append(",");
      }
    }
    {
      EList<Parameter> _parameters_6 = it.getParameters();
      boolean _hasElements_1 = false;
      for(final Parameter parameter_4 : _parameters_6) {
        if (!_hasElements_1) {
          _hasElements_1 = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _methodParameter = this._features.methodParameter(parameter_4);
        _builder.append(_methodParameter, "");
      }
    }
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("//apply other hints that influence how a query is executed");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toNamedQueryCall(final QueryOperation it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _queryOperationSignature = this.queryOperationSignature(it, "public");
    _builder.append(_queryOperationSignature, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("String queryName = ");
    String _qualifiedQueryConstant = this._features.getQualifiedQueryConstant(it);
    _builder.append(_qualifiedQueryConstant, "	");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);");
    _builder.newLine();
    {
      EList<QueryParameter> _queryParameters = it.getQueryParameters();
      final Function1<QueryParameter,Boolean> _function = new Function1<QueryParameter,Boolean>() {
          public Boolean apply(final QueryParameter it) {
            boolean _or = false;
            String _name = RepositoryGenerator.this._features.getName(it);
            boolean _equals = "firstResult".equals(_name);
            if (_equals) {
              _or = true;
            } else {
              String _name_1 = RepositoryGenerator.this._features.getName(it);
              boolean _equals_1 = "maxResults".equals(_name_1);
              _or = (_equals || _equals_1);
            }
            return Boolean.valueOf(_or);
          }
        };
      Iterable<QueryParameter> _reject = IterableExtensions2.<QueryParameter>reject(_queryParameters, _function);
      for(final QueryParameter parameter : _reject) {
        _builder.append("\t");
        QlStatement _statement = it.getStatement();
        TreeIterator<EObject> _eAllContents = _statement.eAllContents();
        Iterator<QueryParameterValue> _filter = Iterators.<QueryParameterValue>filter(_eAllContents, QueryParameterValue.class);
        final Function1<QueryParameterValue,Boolean> _function_1 = new Function1<QueryParameterValue,Boolean>() {
            public Boolean apply(final QueryParameterValue value) {
              boolean _or = false;
              QueryParameterReference _parameter = value.getParameter();
              boolean _equals = Objects.equal(_parameter, parameter);
              if (_equals) {
                _or = true;
              } else {
                QueryParameterReference _parameter_1 = value.getParameter();
                Attribute _attribute = parameter.getAttribute();
                boolean _equals_1 = Objects.equal(_parameter_1, _attribute);
                _or = (_equals || _equals_1);
              }
              return Boolean.valueOf(_or);
            }
          };
        final QueryParameterValue value = IteratorExtensions.<QueryParameterValue>findFirst(_filter, _function_1);
        _builder.newLineIfNotEmpty();
        {
          boolean _and = false;
          boolean _isMany = value==null?false:this.isMany(value);
          boolean _not = (!_isMany);
          if (!_not) {
            _and = false;
          } else {
            Type _type = value==null?(Type)null:this.getType(value);
            _and = (_not && (_type instanceof Entity));
          }
          if (_and) {
            _builder.append("\t");
            _builder.append("namedQuery.setEntity(\"");
            String _name = this._features.getName(parameter);
            _builder.append(_name, "	");
            _builder.append("\", ");
            String _name_1 = this._features.getName(parameter);
            _builder.append(_name_1, "	");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
          } else {
            boolean _and_1 = false;
            Type _type_1 = this._features.getType(parameter);
            if (!(_type_1 instanceof ComplexType)) {
              _and_1 = false;
            } else {
              Attribute _attribute = value==null?(Attribute)null:value.getAttribute();
              boolean _notEquals = (!Objects.equal(_attribute, null));
              _and_1 = ((_type_1 instanceof ComplexType) && _notEquals);
            }
            if (_and_1) {
              {
                QlStatement _statement_1 = it.getStatement();
                TreeIterator<EObject> _eAllContents_1 = _statement_1.eAllContents();
                Iterator<QueryParameterValue> _filter_1 = Iterators.<QueryParameterValue>filter(_eAllContents_1, QueryParameterValue.class);
                final Function1<QueryParameterValue,Boolean> _function_2 = new Function1<QueryParameterValue,Boolean>() {
                    public Boolean apply(final QueryParameterValue e) {
                      QueryParameterReference _parameter = e.getParameter();
                      boolean _equals = Objects.equal(_parameter, parameter);
                      return Boolean.valueOf(_equals);
                    }
                  };
                Iterator<QueryParameterValue> _filter_2 = IteratorExtensions.<QueryParameterValue>filter(_filter_1, _function_2);
                final Function1<QueryParameterValue,Attribute> _function_3 = new Function1<QueryParameterValue,Attribute>() {
                    public Attribute apply(final QueryParameterValue e) {
                      Attribute _attribute = e.getAttribute();
                      return _attribute;
                    }
                  };
                Iterator<Attribute> _map = IteratorExtensions.<QueryParameterValue, Attribute>map(_filter_2, _function_3);
                Set<Attribute> _set = IteratorExtensions.<Attribute>toSet(_map);
                final Function1<Attribute,String> _function_4 = new Function1<Attribute,String>() {
                    public String apply(final Attribute e) {
                      String _name = e.getName();
                      return _name;
                    }
                  };
                List<Attribute> _sortBy = IterableExtensions.<Attribute, String>sortBy(_set, _function_4);
                for(final Attribute attribute : _sortBy) {
                  _builder.append("\t");
                  _builder.append("namedQuery.");
                  DataTypeAndTypeParameter _type_2 = attribute.getType();
                  String _setParameterMethodName = this.getSetParameterMethodName(_type_2);
                  _builder.append(_setParameterMethodName, "	");
                  {
                    boolean _isMany_1 = attribute.isMany();
                    if (_isMany_1) {
                      _builder.append("List");
                    }
                  }
                  _builder.append("(\"");
                  String _name_2 = this._features.getName(parameter);
                  _builder.append(_name_2, "	");
                  _builder.append("_");
                  String _name_3 = attribute.getName();
                  _builder.append(_name_3, "	");
                  _builder.append("\", ");
                  String _name_4 = this._features.getName(parameter);
                  _builder.append(_name_4, "	");
                  _builder.append(".");
                  String _accessorMethodName = this._features.getAccessorMethodName(attribute);
                  _builder.append(_accessorMethodName, "	");
                  _builder.append("());");
                  _builder.newLineIfNotEmpty();
                }
              }
            } else {
              boolean _notEquals_1 = (!Objects.equal(value, null));
              if (_notEquals_1) {
                _builder.append("\t");
                _builder.append("namedQuery.");
                String _setParameterMethodName_1 = this.getSetParameterMethodName(parameter);
                _builder.append(_setParameterMethodName_1, "	");
                {
                  boolean _isMany_2 = this.isMany(parameter);
                  if (_isMany_2) {
                    _builder.append("List");
                  }
                }
                _builder.append("(\"");
                String _name_5 = this._features.getName(parameter);
                _builder.append(_name_5, "	");
                _builder.append("\", ");
                String _name_6 = this._features.getName(parameter);
                _builder.append(_name_6, "	");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("\t");
    _builder.append("apply");
    String _name_7 = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name_7);
    _builder.append(_firstUpper, "	");
    _builder.append("QueryHints(namedQuery");
    {
      EList<QueryParameter> _queryParameters_1 = it.getQueryParameters();
      boolean _isEmpty = _queryParameters_1.isEmpty();
      boolean _not_1 = (!_isEmpty);
      if (_not_1) {
        _builder.append(",");
      }
    }
    {
      EList<QueryParameter> _queryParameters_2 = it.getQueryParameters();
      boolean _hasElements = false;
      for(final QueryParameter parameter_1 : _queryParameters_2) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "	");
        }
        String _name_8 = this._features.getName(parameter_1);
        _builder.append(_name_8, "	");
      }
    }
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    {
      boolean _or = false;
      QlStatement _statement_2 = it.getStatement();
      if ((_statement_2 instanceof UpdateStatement)) {
        _or = true;
      } else {
        QlStatement _statement_3 = it.getStatement();
        _or = ((_statement_2 instanceof UpdateStatement) || (_statement_3 instanceof DeleteStatement));
      }
      if (_or) {
        _builder.append("\t");
        {
          Type _type_3 = it.getType();
          boolean _notEquals_2 = (!Objects.equal(_type_3, null));
          if (_notEquals_2) {
            _builder.append("return ");
          }
        }
        _builder.append("namedQuery.executeUpdate();");
        _builder.newLineIfNotEmpty();
      } else {
        boolean _and_2 = false;
        Type _type_4 = it.getType();
        if (!(_type_4 instanceof DataView)) {
          _and_2 = false;
        } else {
          QlStatement _statement_4 = it.getStatement();
          _and_2 = ((_type_4 instanceof DataView) && (_statement_4 instanceof SelectProperties));
        }
        if (_and_2) {
          {
            boolean _isMany_3 = it.isMany();
            if (_isMany_3) {
              _builder.append("\t");
              _builder.append("return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), ");
              Type _type_5 = it.getType();
              String _name_9 = _type_5.getName();
              _builder.append(_name_9, "	");
              _builder.append(".class, ");
              String _addImport = this._names.addImport(it, "java.util.Collections");
              _builder.append(_addImport, "	");
              _builder.append(".singletonMap(QUERY_NAME, queryName));");
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("\t");
              _builder.append("Object[] tuple = unique(namedQuery);");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("return map");
              String _name_10 = it.getName();
              String _firstUpper_1 = StringExtensions.toFirstUpper(_name_10);
              _builder.append(_firstUpper_1, "	");
              _builder.append("Tuple(tuple);");
              _builder.newLineIfNotEmpty();
            }
          }
        } else {
          boolean _isMany_4 = it.isMany();
          if (_isMany_4) {
            {
              QlStatement _statement_5 = it.getStatement();
              TreeIterator<EObject> _eAllContents_2 = _statement_5.eAllContents();
              Iterator<Join> _filter_3 = Iterators.<Join>filter(_eAllContents_2, Join.class);
              boolean _isEmpty_1 = IteratorExtensions.isEmpty(_filter_3);
              boolean _not_2 = (!_isEmpty_1);
              if (_not_2) {
                _builder.append("\t");
                _builder.append("Collection<");
                Type _type_6 = it.getType();
                String _name_11 = _type_6.getName();
                _builder.append(_name_11, "	");
                _builder.append("> result = list(namedQuery);");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("return new ");
                String _addImport_1 = this._names.addImport(it, "java.util.LinkedHashSet");
                _builder.append(_addImport_1, "	");
                _builder.append("<");
                Type _type_7 = it.getType();
                String _name_12 = _type_7.getName();
                _builder.append(_name_12, "	");
                _builder.append(">(result);");
                _builder.newLineIfNotEmpty();
              } else {
                _builder.append("\t");
                _builder.append("return list(namedQuery);");
                _builder.newLine();
              }
            }
          } else {
            Type _type_8 = it.getType();
            boolean _notEquals_3 = (!Objects.equal(_type_8, null));
            if (_notEquals_3) {
              _builder.append("\t");
              _builder.append("return unique(namedQuery);");
              _builder.newLine();
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("protected void apply");
    String _name_13 = it.getName();
    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_13);
    _builder.append(_firstUpper_2, "");
    _builder.append("QueryHints(Query query");
    {
      EList<QueryParameter> _queryParameters_3 = it.getQueryParameters();
      boolean _hasElements_1 = false;
      for(final QueryParameter parameter_2 : _queryParameters_3) {
        if (!_hasElements_1) {
          _hasElements_1 = true;
          _builder.append(",", "");
        } else {
          _builder.appendImmediate(",", "");
        }
        String _type_9 = this._features.toType(parameter_2);
        _builder.append(_type_9, "");
        _builder.append(" ");
        String _name_14 = this._features.getName(parameter_2);
        _builder.append(_name_14, "");
      }
    }
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    {
      EList<QueryParameter> _queryParameters_4 = it.getQueryParameters();
      Iterable<Parameter> _filter_4 = Iterables.<Parameter>filter(_queryParameters_4, Parameter.class);
      for(final Parameter parameter_3 : _filter_4) {
        {
          String _name_15 = parameter_3.getName();
          boolean _equals = Objects.equal(_name_15, "firstResult");
          if (_equals) {
            _builder.append("\t");
            _builder.append("if (firstResult!=null) {");
            _builder.newLine();
            _builder.append("\t");
            _builder.append("\t");
            _builder.append("query.setFirstResult(");
            String _name_16 = parameter_3.getName();
            _builder.append(_name_16, "		");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
          } else {
            String _name_17 = parameter_3.getName();
            boolean _equals_1 = Objects.equal(_name_17, "maxResults");
            if (_equals_1) {
              _builder.append("\t");
              _builder.append("if (maxResults!=null) {");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("\t");
              _builder.append("query.setMaxResults(");
              String _name_18 = parameter_3.getName();
              _builder.append(_name_18, "		");
              _builder.append(");");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
            }
          }
        }
      }
    }
    _builder.append("\t");
    _builder.append("// override this method and apply other hints that influence how a query is executed");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    {
      boolean _and_3 = false;
      Type _type_10 = it.getType();
      if (!(_type_10 instanceof DataView)) {
        _and_3 = false;
      } else {
        QlStatement _statement_6 = it.getStatement();
        _and_3 = ((_type_10 instanceof DataView) && (_statement_6 instanceof SelectProperties));
      }
      if (_and_3) {
        _builder.newLine();
        Type _type_11 = it.getType();
        final DataView dataView = ((DataView) _type_11);
        _builder.newLineIfNotEmpty();
        QlStatement _statement_7 = it.getStatement();
        final SelectProperties selectProperties = ((SelectProperties) _statement_7);
        _builder.newLineIfNotEmpty();
        _builder.append("protected ");
        Type _type_12 = it.getType();
        String _name_19 = _type_12.getName();
        _builder.append(_name_19, "");
        _builder.append(" map");
        String _name_20 = it.getName();
        String _firstUpper_3 = StringExtensions.toFirstUpper(_name_20);
        _builder.append(_firstUpper_3, "");
        _builder.append("Tuple(Object[] tuple");
        {
          boolean _isMany_5 = it.isMany();
          if (_isMany_5) {
            _builder.append(", int rowNumber");
          }
        }
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        Type _type_13 = it.getType();
        String _name_21 = _type_13.getName();
        _builder.append(_name_21, "	");
        _builder.append(" ");
        Type _type_14 = it.getType();
        String _name_22 = _type_14.getName();
        String _firstLower = StringExtensions.toFirstLower(_name_22);
        _builder.append(_firstLower, "	");
        _builder.append(" = null;");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("if (tuple!=null) {");
        _builder.newLine();
        _builder.append("\t\t");
        Type _type_15 = it.getType();
        String _name_23 = _type_15.getName();
        String _firstLower_1 = StringExtensions.toFirstLower(_name_23);
        _builder.append(_firstLower_1, "		");
        _builder.append(" = new ");
        Type _type_16 = it.getType();
        String _name_24 = _type_16.getName();
        _builder.append(_name_24, "		");
        _builder.append("();");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        Index _index = new Index();
        Index index = _index;
        _builder.newLineIfNotEmpty();
        {
          EList<Expression> _properties = selectProperties.getProperties();
          for(final Expression expression : _properties) {
            {
              boolean _and_4 = false;
              if (!(expression instanceof AliasedExpression)) {
                _and_4 = false;
              } else {
                EList<Attribute> _allAttributes = dataView.getAllAttributes();
                final Function1<Attribute,Boolean> _function_5 = new Function1<Attribute,Boolean>() {
                    public Boolean apply(final Attribute e) {
                      String _name = e.getName();
                      String _name_1 = ((AliasedExpression) expression).getName();
                      boolean _equals = _name.equals(_name_1);
                      return Boolean.valueOf(_equals);
                    }
                  };
                Attribute _findFirst = IterableExtensions.<Attribute>findFirst(_allAttributes, _function_5);
                boolean _notEquals_4 = (!Objects.equal(_findFirst, null));
                _and_4 = ((expression instanceof AliasedExpression) && _notEquals_4);
              }
              if (_and_4) {
                _builder.append("\t\t");
                EList<Attribute> _allAttributes_1 = dataView.getAllAttributes();
                final Function1<Attribute,Boolean> _function_6 = new Function1<Attribute,Boolean>() {
                    public Boolean apply(final Attribute e) {
                      String _name = e.getName();
                      String _name_1 = ((AliasedExpression) expression).getName();
                      boolean _equals = _name.equals(_name_1);
                      return Boolean.valueOf(_equals);
                    }
                  };
                final Attribute attribute_1 = IterableExtensions.<Attribute>findFirst(_allAttributes_1, _function_6);
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                Type _type_17 = it.getType();
                String _name_25 = _type_17.getName();
                String _firstLower_2 = StringExtensions.toFirstLower(_name_25);
                _builder.append(_firstLower_2, "		");
                _builder.append(".");
                String _mutatorMethodName = this._features.getMutatorMethodName(attribute_1);
                _builder.append(_mutatorMethodName, "		");
                _builder.append("((");
                DataTypeAndTypeParameter _type_18 = attribute_1.getType();
                String _qualifiedName = this._names.getQualifiedName(_type_18);
                _builder.append(_qualifiedName, "		");
                _builder.append(")tuple[");
                int _counter0 = index.getCounter0();
                _builder.append(_counter0, "		");
                _builder.append("]);");
                _builder.newLineIfNotEmpty();
              } else {
                boolean _and_5 = false;
                if (!(expression instanceof PropertyValue)) {
                  _and_5 = false;
                } else {
                  EList<Attribute> _allAttributes_2 = dataView.getAllAttributes();
                  final Function1<Attribute,Boolean> _function_7 = new Function1<Attribute,Boolean>() {
                      public Boolean apply(final Attribute e) {
                        String _name = e.getName();
                        EList<String> _segments = ((PropertyValue) expression).getSegments();
                        String _last = IterableExtensions.<String>last(_segments);
                        boolean _equals = _name.equals(_last);
                        return Boolean.valueOf(_equals);
                      }
                    };
                  Attribute _findFirst_1 = IterableExtensions.<Attribute>findFirst(_allAttributes_2, _function_7);
                  boolean _notEquals_5 = (!Objects.equal(_findFirst_1, null));
                  _and_5 = ((expression instanceof PropertyValue) && _notEquals_5);
                }
                if (_and_5) {
                  _builder.append("\t\t");
                  EList<Attribute> _allAttributes_3 = dataView.getAllAttributes();
                  final Function1<Attribute,Boolean> _function_8 = new Function1<Attribute,Boolean>() {
                      public Boolean apply(final Attribute e) {
                        String _name = e.getName();
                        EList<String> _segments = ((PropertyValue) expression).getSegments();
                        String _last = IterableExtensions.<String>last(_segments);
                        boolean _equals = _name.equals(_last);
                        return Boolean.valueOf(_equals);
                      }
                    };
                  final Attribute attribute_2 = IterableExtensions.<Attribute>findFirst(_allAttributes_3, _function_8);
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t\t");
                  Type _type_19 = it.getType();
                  String _name_26 = _type_19.getName();
                  String _firstLower_3 = StringExtensions.toFirstLower(_name_26);
                  _builder.append(_firstLower_3, "		");
                  _builder.append(".");
                  String _mutatorMethodName_1 = this._features.getMutatorMethodName(attribute_2);
                  _builder.append(_mutatorMethodName_1, "		");
                  _builder.append("((");
                  DataTypeAndTypeParameter _type_20 = attribute_2.getType();
                  String _qualifiedName_1 = this._names.getQualifiedName(_type_20);
                  _builder.append(_qualifiedName_1, "		");
                  _builder.append(")tuple[");
                  int _counter0_1 = index.getCounter0();
                  _builder.append(_counter0_1, "		");
                  _builder.append("]);");
                  _builder.newLineIfNotEmpty();
                }
              }
            }
            _builder.append("\t\t");
            index.increment();
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("return ");
        Type _type_21 = it.getType();
        String _name_27 = _type_21.getName();
        String _firstLower_4 = StringExtensions.toFirstLower(_name_27);
        _builder.append(_firstLower_4, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public String getQualifiedCompositeIdentifierName(final Entity it) {
    String _packageName = this._names.getPackageName(it);
    String _plus = (_packageName + ".");
    AttributeGroup _key = it.getKey();
    String _name = _key.getName();
    String _plus_1 = (_plus + _name);
    String _addImport = this._names.addImport(it, _plus_1);
    return _addImport;
  }
  
  protected Entity _getEntity(final Attribute it) {
    EObject _eContainer = it.eContainer();
    return ((Entity) _eContainer);
  }
  
  protected Entity _getEntity(final DataBaseConstraint it) {
    EObject _eContainer = it.eContainer();
    Entity _entity = ((Dao) _eContainer).getEntity();
    return _entity;
  }
  
  public boolean isMany(final QueryParameterValue it) {
    boolean _xblockexpression = false;
    {
      Attribute _attribute = it.getAttribute();
      boolean _notEquals = (!Objects.equal(_attribute, null));
      if (_notEquals) {
        Attribute _attribute_1 = it.getAttribute();
        return _attribute_1.isMany();
      }
      final QueryParameterReference type = it.getParameter();
      boolean _switchResult = false;
      boolean _matched = false;
      if (!_matched) {
        if (type instanceof Parameter) {
          final Parameter _parameter = (Parameter)type;
          _matched=true;
          boolean _isMany = _parameter.isMany();
          _switchResult = _isMany;
        }
      }
      if (!_matched) {
        if (type instanceof Attribute) {
          final Attribute _attribute_2 = (Attribute)type;
          _matched=true;
          boolean _isMany = _attribute_2.isMany();
          _switchResult = _isMany;
        }
      }
      if (!_matched) {
        _switchResult = false;
      }
      _xblockexpression = (_switchResult);
    }
    return _xblockexpression;
  }
  
  public Type getType(final QueryParameterValue it) {
    Type _xblockexpression = null;
    {
      Attribute _attribute = it.getAttribute();
      boolean _notEquals = (!Objects.equal(_attribute, null));
      if (_notEquals) {
        Attribute _attribute_1 = it.getAttribute();
        return _attribute_1.getDataType();
      }
      final QueryParameterReference type = it.getParameter();
      Type _switchResult = null;
      boolean _matched = false;
      if (!_matched) {
        if (type instanceof Parameter) {
          final Parameter _parameter = (Parameter)type;
          _matched=true;
          Type _type = _parameter.getType();
          _switchResult = _type;
        }
      }
      if (!_matched) {
        if (type instanceof Attribute) {
          final Attribute _attribute_2 = (Attribute)type;
          _matched=true;
          Type _dataType = _attribute_2.getDataType();
          _switchResult = _dataType;
        }
      }
      _xblockexpression = (_switchResult);
    }
    return _xblockexpression;
  }
  
  public String getSetParameterMethodName(final QueryParameter parameter) {
    String _xblockexpression = null;
    {
      final Type type = this._features.getType(parameter);
      String _switchResult = null;
      boolean _matched = false;
      if (!_matched) {
        if (type instanceof SimpleType) {
          final SimpleType _simpleType = (SimpleType)type;
          boolean _and = false;
          ValidatorReference _validatorReference = _simpleType.getValidatorReference();
          boolean _notEquals = (!Objects.equal(_validatorReference, null));
          if (!_notEquals) {
            _and = false;
          } else {
            ValidatorReference _validatorReference_1 = _simpleType.getValidatorReference();
            boolean _isDateValidator = _validatorReference_1.isDateValidator();
            _and = (_notEquals && _isDateValidator);
          }
          if (_and) {
            _matched=true;
            _switchResult = "setDate";
          }
        }
      }
      if (!_matched) {
        if (type instanceof SimpleType) {
          final SimpleType _simpleType = (SimpleType)type;
          boolean _and = false;
          ValidatorReference _validatorReference = _simpleType.getValidatorReference();
          boolean _notEquals = (!Objects.equal(_validatorReference, null));
          if (!_notEquals) {
            _and = false;
          } else {
            ValidatorReference _validatorReference_1 = _simpleType.getValidatorReference();
            boolean _isTimeStampValidator = _validatorReference_1.isTimeStampValidator();
            _and = (_notEquals && _isTimeStampValidator);
          }
          if (_and) {
            _matched=true;
            _switchResult = "setTimestamp";
          }
        }
      }
      if (!_matched) {
        _switchResult = "setParameter";
      }
      _xblockexpression = (_switchResult);
    }
    return _xblockexpression;
  }
  
  public String getSetParameterMethodName(final DataTypeAndTypeParameter it) {
    String _xifexpression = null;
    boolean _hasDateValidator = ValidatorExtension.hasDateValidator(it);
    if (_hasDateValidator) {
      _xifexpression = "setDate";
    } else {
      String _xifexpression_1 = null;
      boolean _hasTimeStampValidator = ValidatorExtension.hasTimeStampValidator(it);
      if (_hasTimeStampValidator) {
        _xifexpression_1 = "setTimestamp";
      } else {
        _xifexpression_1 = "setParameter";
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  public boolean isMany(final QueryParameter it) {
    boolean _switchResult = false;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Parameter) {
        final Parameter _parameter = (Parameter)it;
        _matched=true;
        boolean _isMany = _parameter.isMany();
        _switchResult = _isMany;
      }
    }
    if (!_matched) {
      Attribute _attribute = it.getAttribute();
      boolean _isMany = _attribute.isMany();
      _switchResult = _isMany;
    }
    return _switchResult;
  }
  
  public String getConstantName(final AttributeGroup it) {
    String _name = it.getName();
    String _underScoreString = Strings.toUnderScoreString(_name);
    String _upperCase = _underScoreString.toUpperCase();
    return _upperCase;
  }
  
  private String toSimpleName(final String fqName) {
    String _lastToken = Strings.lastToken(fqName, ".");
    return _lastToken;
  }
  
  private String toPackageName(final String fqName) {
    String _skipLastToken = Strings.skipLastToken(fqName, ".");
    return _skipLastToken;
  }
  
  private Iterable<String> getAliasNames(final DataBaseConstraint it) {
    ArrayList<String> _xblockexpression = null;
    {
      Object _property = EObjectPropertiesAdapter.getProperty(it, "alias");
      boolean _notEquals = (!Objects.equal(_property, null));
      if (_notEquals) {
        Object _property_1 = EObjectPropertiesAdapter.getProperty(it, "alias");
        return ((Iterable<String>) _property_1);
      }
      ArrayList<String> _newArrayList = Lists.<String>newArrayList();
      _xblockexpression = (_newArrayList);
    }
    return _xblockexpression;
  }
  
  public Entity getEntity(final EObject it) {
    if (it instanceof Attribute) {
      return _getEntity((Attribute)it);
    } else if (it instanceof DataBaseConstraint) {
      return _getEntity((DataBaseConstraint)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
