package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.ConditionsBlock;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.Function;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.dom.model.ApplicationSession;
import org.openxma.dsl.dom.model.Property;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.impl.ExpressionCompiler;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;

@SuppressWarnings("all")
public class SessionContextGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  @Inject
  private ExpressionCompiler _expressionCompiler;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<ApplicationSession> _filter = Iterables.<ApplicationSession>filter(_elements, ApplicationSession.class);
    final Procedure1<ApplicationSession> _function = new Procedure1<ApplicationSession>() {
        public void apply(final ApplicationSession it) {
          String _qualifiedGenInterfaceName = SessionContextGenerator.this._names.getQualifiedGenInterfaceName(it);
          String _javaFilePath = SessionContextGenerator.this._files.toJavaFilePath(_qualifiedGenInterfaceName);
          CharSequence _genInterface = SessionContextGenerator.this.toGenInterface(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genInterface);
          String _qualifiedInterfaceName = SessionContextGenerator.this._names.getQualifiedInterfaceName(it);
          String _javaFilePath_1 = SessionContextGenerator.this._files.toJavaFilePath(_qualifiedInterfaceName);
          CharSequence _interface = SessionContextGenerator.this.toInterface(it);
          fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _interface);
          String _qualifiedGenClassName = SessionContextGenerator.this._names.getQualifiedGenClassName(it);
          String _javaFilePath_2 = SessionContextGenerator.this._files.toJavaFilePath(_qualifiedGenClassName);
          CharSequence _genClass = SessionContextGenerator.this.toGenClass(it);
          fsa.generateFile(_javaFilePath_2, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _qualifiedClassName = SessionContextGenerator.this._names.getQualifiedClassName(it);
          String _javaFilePath_3 = SessionContextGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _class = SessionContextGenerator.this.toClass(it);
          fsa.generateFile(_javaFilePath_3, GeneratorConstants.OUTLET_JAVA, _class);
        }
      };
    IterableExtensions.<ApplicationSession>forEach(_filter, _function);
  }
  
  public CharSequence toGenInterface(final ApplicationSession it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      EList<Property> _properties = it.getProperties();
      for(final Property property : _properties) {
        _builder.append("\t");
        CharSequence _documentationIndented = this._features.getDocumentationIndented(property);
        _builder.append(_documentationIndented, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        SimpleType _type = property.getType();
        String _qualifiedName = this._names.getQualifiedName(_type);
        _builder.append(_qualifiedName, "	");
        _builder.append(" ");
        String _accessorMethodName = this._features.getAccessorMethodName(property);
        _builder.append(_accessorMethodName, "	");
        _builder.append("();");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Function> _functions = it.getFunctions();
      for(final Function function : _functions) {
        _builder.append("\t");
        SimpleType _resultType = function.getResultType();
        String _qualifiedName_1 = this._names.getQualifiedName(_resultType);
        _builder.append(_qualifiedName_1, "	");
        _builder.append(" ");
        String _name = function.getName();
        _builder.append(_name, "	");
        _builder.append("(");
        SimpleType _parameterType = function.getParameterType();
        String _qualifiedName_2 = this._names.getQualifiedName(_parameterType);
        _builder.append(_qualifiedName_2, "	");
        _builder.append(" ");
        String _parameterName = function.getParameterName();
        _builder.append(_parameterName, "	");
        _builder.append(");");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      ConditionsBlock _conditionsBlock = it.getConditionsBlock();
      boolean _notEquals = (!Objects.equal(_conditionsBlock, null));
      if (_notEquals) {
        {
          ConditionsBlock _conditionsBlock_1 = it.getConditionsBlock();
          EList<StatusFlag> _statusFlags = _conditionsBlock_1.getStatusFlags();
          for(final StatusFlag flag : _statusFlags) {
            _builder.append("\t");
            _builder.append("Boolean ");
            String _name_1 = flag.getName();
            String _firstUpper = StringExtensions.toFirstUpper(_name_1);
            _builder.append(_firstUpper, "	");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toInterface(final ApplicationSession it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _interfaceName = this._names.getInterfaceName(it);
    _builder.append(_interfaceName, "");
    _builder.append(" extends ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toClass(final ApplicationSession it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.springframework.stereotype.Service;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _className = this._names.getClassName(it);
    _builder.append(_className, "");
    _builder.append(" extends ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      EList<Property> _properties = it.getProperties();
      for(final Property property : _properties) {
        _builder.append("\t");
        CharSequence _documentationIndented = this._features.getDocumentationIndented(property);
        _builder.append(_documentationIndented, "	");
        _builder.append("\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("public ");
        SimpleType _type = property.getType();
        String _qualifiedName = this._names.getQualifiedName(_type);
        _builder.append(_qualifiedName, "	");
        _builder.append(" get");
        String _name = property.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "	");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("throw new IllegalStateException(\"not yet implemented\");");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.newLine();
    {
      EList<Function> _functions = it.getFunctions();
      for(final Function function : _functions) {
        _builder.append("\t");
        _builder.append("public ");
        SimpleType _resultType = function.getResultType();
        String _qualifiedName_1 = this._names.getQualifiedName(_resultType);
        _builder.append(_qualifiedName_1, "	");
        _builder.append(" ");
        String _name_1 = function.getName();
        _builder.append(_name_1, "	");
        _builder.append("(");
        SimpleType _parameterType = function.getParameterType();
        String _qualifiedName_2 = this._names.getQualifiedName(_parameterType);
        _builder.append(_qualifiedName_2, "	");
        _builder.append(" ");
        String _parameterName = function.getParameterName();
        _builder.append(_parameterName, "	");
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("throw new IllegalStateException(\"not yet implemented\");");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final ApplicationSession it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public abstract class ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" implements ");
    String _qualifiedName = this._names.getQualifiedName(it);
    _builder.append(_qualifiedName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      ConditionsBlock _conditionsBlock = it.getConditionsBlock();
      boolean _notEquals = (!Objects.equal(_conditionsBlock, null));
      if (_notEquals) {
        {
          ConditionsBlock _conditionsBlock_1 = it.getConditionsBlock();
          EList<StatusFlag> _statusFlags = _conditionsBlock_1.getStatusFlags();
          for(final StatusFlag flag : _statusFlags) {
            _builder.append("\t");
            _builder.append("public Boolean ");
            String _name = flag.getName();
            String _firstUpper = StringExtensions.toFirstUpper(_name);
            _builder.append(_firstUpper, "	");
            _builder.append("() {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("\t");
            _builder.append("return ");
            EqualityExpr _expression = flag.getExpression();
            String _compile = this._expressionCompiler.compile(_expression);
            _builder.append(_compile, "		");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
          }
        }
      }
    }
    _builder.append("\t");
    _builder.append("public ");
    String _qualifiedName_1 = this._names.getQualifiedName(it);
    _builder.append(_qualifiedName_1, "	");
    _builder.append(" getContext() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return this;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}\t");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
