package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.generator.IGenerator
import org.openxma.dsl.core.model.Model
import org.openxma.dsl.dom.model.ApplicationSession

import static org.openxma.dsl.generator.GeneratorConstants.*
import static extension org.openxma.dsl.generator.lib.ImportAdapter.*

class SessionContextGenerator implements IGenerator {

	@Inject
	extension Names
	@Inject
	extension Files
	@Inject
	extension Features
	@Inject
	extension ExpressionCompiler
	
	override doGenerate(Resource resource, IFileSystemAccess fsa) {
		val model = resource.getEObject("/") as Model
		model.elements.filter(typeof(ApplicationSession)).forEach(
			[
				fsa.generateFile(qualifiedGenInterfaceName.toJavaFilePath, OUTLET_GENERATED_JAVA, toGenInterface)
				fsa.generateFile(qualifiedInterfaceName.toJavaFilePath, OUTLET_JAVA, toInterface)
				fsa.generateFile(qualifiedGenClassName.toJavaFilePath, OUTLET_GENERATED_JAVA, toGenClass)
				fsa.generateFile(qualifiedClassName.toJavaFilePath, OUTLET_JAVA, toClass)
			]
		)
	}

	def toGenInterface(ApplicationSession it) {
		'''
		package packageName;

		addImportsHere
		
		documentation
		public interface genInterfaceName {
			FOR property : properties
			property.documentationIndented
			property.type.qualifiedName property.getAccessorMethodName()();
			ENDFOR
			FOR function : functions
			function.resultType.qualifiedName function.name(function.parameterType.qualifiedName function.parameterName);
			ENDFOR
			IF conditionsBlock != null
			FOR flag : conditionsBlock.statusFlags
			Boolean flag.name.toFirstUpper()();
			ENDFOR
			ENDIF
		}
		'''
	}

	def toInterface(ApplicationSession it) {
		'''
		package packageName;

		addImportsHere
		
		documentation
		public interface interfaceName extends genInterfaceName {
		}
		'''
	}

	def toClass(ApplicationSession it) {
		'''
		package classPackageName;

		import org.springframework.stereotype.Service;
		addImportsHere
		
		documentation
		public class className extends genClassName {
		
			FOR property : properties
			property.documentationIndented	
			public property.type.qualifiedName getproperty.name.toFirstUpper()() {
				throw new IllegalStateException("not yet implemented");
			}
			ENDFOR
			
			FOR function : functions
			public function.resultType.qualifiedName function.name(function.parameterType.qualifiedName function.parameterName) {
				throw new IllegalStateException("not yet implemented");
			}
			ENDFOR	
		}
		'''
	}
	
	def toGenClass(ApplicationSession it) {
		'''
		package classPackageName;

		addImportsHere
		
		documentation
		public abstract class getGenClassName() implements qualifiedName {
			IF conditionsBlock != null
			FOR flag : conditionsBlock.statusFlags
			public Boolean flag.name.toFirstUpper()() {
				return flag.expression.compile;
			}
			ENDFOR
			ENDIF
			public qualifiedName getContext() {
				return this;
			}	
		}
		'''
	}

}
