package org.openxma.dsl.generator.impl;

import at.spardat.xma.guidesign.BcdValidator;
import at.spardat.xma.guidesign.CustomValidator;
import at.spardat.xma.guidesign.StringValidator;
import at.spardat.xma.guidesign.Validator;
import at.spardat.xma.guidesign.flex.UserDefinedValidator;
import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import java.util.Arrays;
import java.util.List;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.util.SimpleAttributeResolver;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.Constraint;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.ReferenceWithParameter;
import org.openxma.dsl.core.model.ReferenceableByXmadslVariable;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.ApplicationSession;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeProperty;
import org.openxma.dsl.dom.model.AttributeValidationProperty;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.Property;
import org.openxma.dsl.dom.model.RequiredFlag;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.EntityExtension;
import org.openxma.dsl.generator.helper.SimpleTypeExtension;
import org.openxma.dsl.generator.helper.ValidatorExtension;
import org.openxma.dsl.generator.impl.ExpressionCompiler;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.impl.Types;
import org.openxma.dsl.generator.lib.ImportAdapter;
import org.openxma.dsl.generator.lib.Index;

@SuppressWarnings("all")
public class ValidatorGenerator implements IGenerator {
  @Inject
  private Types _types;
  
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  @Inject
  private ExpressionCompiler _expressionCompiler;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<Entity> _filter = Iterables.<Entity>filter(_elements, Entity.class);
    final Procedure1<Entity> _function = new Procedure1<Entity>() {
        public void apply(final Entity it) {
          String _qualifiedGenValidatorClassName = ValidatorGenerator.this.getQualifiedGenValidatorClassName(it);
          String _javaFilePath = ValidatorGenerator.this._files.toJavaFilePath(_qualifiedGenValidatorClassName);
          CharSequence _genClass = ValidatorGenerator.this.toGenClass(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _qualifiedValidatorClassName = ValidatorGenerator.this.getQualifiedValidatorClassName(it);
          String _javaFilePath_1 = ValidatorGenerator.this._files.toJavaFilePath(_qualifiedValidatorClassName);
          CharSequence _class = ValidatorGenerator.this.toClass(it);
          fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _class);
          boolean _isJsr303ValidationEnabled = ValidatorExtension.isJsr303ValidationEnabled();
          if (_isJsr303ValidationEnabled) {
            String _qualifiedGenValidatorAnnotationClassName = ValidatorGenerator.this.getQualifiedGenValidatorAnnotationClassName(it);
            String _javaFilePath_2 = ValidatorGenerator.this._files.toJavaFilePath(_qualifiedGenValidatorAnnotationClassName);
            CharSequence _genAnnotationClass = ValidatorGenerator.this.toGenAnnotationClass(it);
            fsa.generateFile(_javaFilePath_2, GeneratorConstants.OUTLET_GENERATED_JAVA, _genAnnotationClass);
          }
        }
      };
    IterableExtensions.<Entity>forEach(_filter, _function);
    boolean _isJsr303ValidationEnabled = ValidatorExtension.isJsr303ValidationEnabled();
    if (_isJsr303ValidationEnabled) {
      EList<ModelElement> _elements_1 = model.getElements();
      Iterable<DataView> _filter_1 = Iterables.<DataView>filter(_elements_1, DataView.class);
      final Procedure1<DataView> _function_1 = new Procedure1<DataView>() {
          public void apply(final DataView it) {
            String _qualifiedGenValidatorClassName = ValidatorGenerator.this.getQualifiedGenValidatorClassName(it);
            String _javaFilePath = ValidatorGenerator.this._files.toJavaFilePath(_qualifiedGenValidatorClassName);
            CharSequence _viewGenClass = ValidatorGenerator.this.toViewGenClass(it);
            fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _viewGenClass);
            String _qualifiedGenValidatorAnnotationClassName = ValidatorGenerator.this.getQualifiedGenValidatorAnnotationClassName(it);
            String _javaFilePath_1 = ValidatorGenerator.this._files.toJavaFilePath(_qualifiedGenValidatorAnnotationClassName);
            CharSequence _genAnnotationClass = ValidatorGenerator.this.toGenAnnotationClass(it);
            fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_GENERATED_JAVA, _genAnnotationClass);
            String _qualifiedValidatorClassName = ValidatorGenerator.this.getQualifiedValidatorClassName(it);
            String _javaFilePath_2 = ValidatorGenerator.this._files.toJavaFilePath(_qualifiedValidatorClassName);
            CharSequence _class = ValidatorGenerator.this.toClass(it);
            fsa.generateFile(_javaFilePath_2, GeneratorConstants.OUTLET_JAVA, _class);
          }
        };
      IterableExtensions.<DataView>forEach(_filter_1, _function_1);
    }
  }
  
  public CharSequence toGenAnnotationClass(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _validatorPackageName = this.getValidatorPackageName(it);
    _builder.append(_validatorPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import static java.lang.annotation.ElementType.TYPE;");
    _builder.newLine();
    _builder.append("import static java.lang.annotation.RetentionPolicy.RUNTIME;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("import java.lang.annotation.Documented;");
    _builder.newLine();
    _builder.append("import java.lang.annotation.Retention;");
    _builder.newLine();
    _builder.append("import java.lang.annotation.Target;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("import javax.validation.Constraint;");
    _builder.newLine();
    _builder.append("import javax.validation.Payload;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("@Target({ TYPE })");
    _builder.newLine();
    _builder.append("@Retention(RUNTIME)");
    _builder.newLine();
    _builder.append("@Documented");
    _builder.newLine();
    _builder.append("@Constraint(validatedBy = {");
    String _validatorClassName = this.getValidatorClassName(it);
    _builder.append(_validatorClassName, "");
    _builder.append(".class})");
    _builder.newLineIfNotEmpty();
    _builder.append("public @interface ");
    String _genValidatorAnnotationClassName = this.getGenValidatorAnnotationClassName(it);
    _builder.append(_genValidatorAnnotationClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("String message() default \"{");
    String _qualifiedGenValidatorAnnotationClassName = this.getQualifiedGenValidatorAnnotationClassName(it);
    _builder.append(_qualifiedGenValidatorAnnotationClassName, "	");
    _builder.append(".message}\";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("Class<?>[] groups() default { };");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("Class<? extends Payload>[] payload() default {};");
    _builder.newLine();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final Entity it) {
    CharSequence _xblockexpression = null;
    {
      final ApplicationSession applicationContext = EntityExtension.getRequiredApplicationContext(it);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("package ");
      String _validatorPackageName = this.getValidatorPackageName(it);
      _builder.append(_validatorPackageName, "");
      _builder.append(";");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      String _addImportsHere = ImportAdapter.addImportsHere(it);
      _builder.append(_addImportsHere, "");
      _builder.newLineIfNotEmpty();
      {
        boolean _isJsr303ValidationEnabled = ValidatorExtension.isJsr303ValidationEnabled();
        boolean _not = (!_isJsr303ValidationEnabled);
        if (_not) {
          _builder.append("import static org.openxma.dsl.platform.validation.ValidatorsHelper.*;");
        }
      }
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.newLine();
      _builder.append("@SuppressWarnings(\"boxing\")");
      _builder.newLine();
      _builder.append("public abstract class ");
      String _genValidatorClassName = this.getGenValidatorClassName(it);
      _builder.append(_genValidatorClassName, "");
      {
        boolean _isJsr303ValidationEnabled_1 = ValidatorExtension.isJsr303ValidationEnabled();
        if (_isJsr303ValidationEnabled_1) {
          _builder.append(" extends ");
          String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.validation.Jsr303Validators");
          _builder.append(_addImport, "");
          _builder.append("<");
          String _qualifiedGenValidatorAnnotationClassName = this.getQualifiedGenValidatorAnnotationClassName(it);
          String _addImport_1 = this._names.addImport(it, _qualifiedGenValidatorAnnotationClassName);
          _builder.append(_addImport_1, "");
          _builder.append(", ");
          String _qualifiedName = this._names.getQualifiedName(it);
          String _addImport_2 = this._names.addImport(it, _qualifiedName);
          _builder.append(_addImport_2, "");
          _builder.append(">");
        } else {
          Entity _superType = it.getSuperType();
          boolean _notEquals = (!Objects.equal(_superType, null));
          if (_notEquals) {
            _builder.append(" extends ");
            Entity _superType_1 = it.getSuperType();
            String _validatorPackageName_1 = this.getValidatorPackageName(_superType_1);
            String _plus = (_validatorPackageName_1 + ".");
            Entity _superType_2 = it.getSuperType();
            String _validatorClassName = this.getValidatorClassName(_superType_2);
            String _plus_1 = (_plus + _validatorClassName);
            String _addImport_3 = this._names.addImport(it, _plus_1);
            _builder.append(_addImport_3, "");
          } else {
            _builder.append(" extends ");
            String _addImport_4 = this._names.addImport(it, "org.openxma.dsl.platform.validation.Validators");
            _builder.append(_addImport_4, "");
          }
        }
      }
      _builder.append(" {");
      _builder.newLineIfNotEmpty();
      {
        boolean _notEquals_1 = (!Objects.equal(applicationContext, null));
        if (_notEquals_1) {
          _builder.append("\t");
          _builder.append("private ");
          String _qualifiedName_1 = this._names.getQualifiedName(applicationContext);
          _builder.append(_qualifiedName_1, "	");
          _builder.append(" context;");
          _builder.newLineIfNotEmpty();
          _builder.newLine();
          _builder.append("\t");
          _builder.append("public ");
          String _qualifiedName_2 = this._names.getQualifiedName(applicationContext);
          _builder.append(_qualifiedName_2, "	");
          _builder.append(" getContext() {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("return context;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
          _builder.newLine();
          _builder.append("\t");
          _builder.append("public void setContext(");
          String _qualifiedName_3 = this._names.getQualifiedName(applicationContext);
          _builder.append(_qualifiedName_3, "	");
          _builder.append(" context) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("\t");
          _builder.append("this.context = context;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      _builder.append("\t");
      _builder.newLine();
      _builder.append("\t");
      {
        boolean _isJsr303ValidationEnabled_2 = ValidatorExtension.isJsr303ValidationEnabled();
        if (_isJsr303ValidationEnabled_2) {
          CharSequence _createJsr303Validation = this.createJsr303Validation(it);
          _builder.append(_createJsr303Validation, "	");
        } else {
          CharSequence _createSpringValidation = this.createSpringValidation(it);
          _builder.append(_createSpringValidation, "	");
        }
      }
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      CharSequence _createConstraintConditions = this.createConstraintConditions(it);
      _builder.append(_createConstraintConditions, "	");
      _builder.newLineIfNotEmpty();
      _builder.append("}");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence toClass(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _validatorPackageName = this.getValidatorPackageName(it);
    _builder.append(_validatorPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public class ");
    String _validatorClassName = this.getValidatorClassName(it);
    _builder.append(_validatorClassName, "");
    _builder.append(" extends ");
    String _genValidatorClassName = this.getGenValidatorClassName(it);
    _builder.append(_genValidatorClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toViewGenClass(final DataView it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _validatorPackageName = this.getValidatorPackageName(it);
    _builder.append(_validatorPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("@SuppressWarnings(\"boxing\")");
    _builder.newLine();
    _builder.append("public abstract class ");
    String _genValidatorClassName = this.getGenValidatorClassName(it);
    _builder.append(_genValidatorClassName, "");
    {
      boolean _isJsr303ValidationEnabled = ValidatorExtension.isJsr303ValidationEnabled();
      if (_isJsr303ValidationEnabled) {
        _builder.append(" extends ");
        String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.validation.Jsr303Validators");
        _builder.append(_addImport, "");
        _builder.append("<");
        String _qualifiedGenValidatorAnnotationClassName = this.getQualifiedGenValidatorAnnotationClassName(it);
        String _addImport_1 = this._names.addImport(it, _qualifiedGenValidatorAnnotationClassName);
        _builder.append(_addImport_1, "");
        _builder.append(", ");
        String _qualifiedName = this._names.getQualifiedName(it);
        String _addImport_2 = this._names.addImport(it, _qualifiedName);
        _builder.append(_addImport_2, "");
        _builder.append(">");
      }
    }
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _createJsr303Validation = this.createJsr303Validation(it);
    _builder.append(_createJsr303Validation, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _createConstraintConditions = this.createConstraintConditions(it);
    _builder.append(_createConstraintConditions, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence createConstraintConditions(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      TreeIterator<EObject> _eAllContents = it.eAllContents();
      List<EObject> _list = IteratorExtensions.<EObject>toList(_eAllContents);
      Iterable<ReferenceableByXmadslVariable> _filter = Iterables.<ReferenceableByXmadslVariable>filter(_list, ReferenceableByXmadslVariable.class);
      final Function1<ReferenceableByXmadslVariable,String> _function = new Function1<ReferenceableByXmadslVariable,String>() {
          public String apply(final ReferenceableByXmadslVariable e) {
            String _apply = SimpleAttributeResolver.NAME_RESOLVER.apply(e);
            return _apply;
          }
        };
      List<ReferenceableByXmadslVariable> _sortBy = IterableExtensions.<ReferenceableByXmadslVariable, String>sortBy(_filter, _function);
      for(final ReferenceableByXmadslVariable function : _sortBy) {
        CharSequence _function_1 = this.toFunction(function, it);
        _builder.append(_function_1, "");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function_2 = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute e) {
            EList<AttributeProperty> _attributProperties = e.getAttributProperties();
            Iterable<AttributeValidationProperty> _filter = Iterables.<AttributeValidationProperty>filter(_attributProperties, AttributeValidationProperty.class);
            final Function1<AttributeValidationProperty,EList<Constraint>> _function = new Function1<AttributeValidationProperty,EList<Constraint>>() {
                public EList<Constraint> apply(final AttributeValidationProperty a) {
                  EList<Constraint> _constraints = a.getConstraints();
                  return _constraints;
                }
              };
            Iterable<EList<Constraint>> _map = IterableExtensions.<AttributeValidationProperty, EList<Constraint>>map(_filter, _function);
            boolean _isEmpty = IterableExtensions.isEmpty(_map);
            boolean _not = (!_isEmpty);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter_1 = IterableExtensions.<Attribute>filter(_attributes, _function_2);
      for(final Attribute attribute : _filter_1) {
        Index _index = new Index();
        final Index index = _index;
        _builder.newLineIfNotEmpty();
        EList<AttributeProperty> _attributProperties = attribute.getAttributProperties();
        final Iterable<AttributeValidationProperty> attributProperties = Iterables.<AttributeValidationProperty>filter(_attributProperties, AttributeValidationProperty.class);
        _builder.newLineIfNotEmpty();
        final Function1<AttributeValidationProperty,EList<Constraint>> _function_3 = new Function1<AttributeValidationProperty,EList<Constraint>>() {
            public EList<Constraint> apply(final AttributeValidationProperty a) {
              EList<Constraint> _constraints = a.getConstraints();
              return _constraints;
            }
          };
        Iterable<EList<Constraint>> _map = IterableExtensions.<AttributeValidationProperty, EList<Constraint>>map(attributProperties, _function_3);
        final Iterable<Constraint> constraints = Iterables.<Constraint>concat(_map);
        _builder.newLineIfNotEmpty();
        {
          for(final Constraint constraint : constraints) {
            final String compiled = this.compileConstraintCondition(constraint);
            _builder.newLineIfNotEmpty();
            {
              boolean _or = false;
              boolean _isJsr303ValidationEnabled = ValidatorExtension.isJsr303ValidationEnabled();
              boolean _not = (!_isJsr303ValidationEnabled);
              if (_not) {
                _or = true;
              } else {
                boolean _isCompiledConditionTrue = ValidatorGenerator.isCompiledConditionTrue(compiled);
                boolean _not_1 = (!_isCompiledConditionTrue);
                _or = (_not || _not_1);
              }
              if (_or) {
                _builder.newLine();
                _builder.append("public boolean is");
                String _name = attribute.getName();
                String _firstUpper = StringExtensions.toFirstUpper(_name);
                _builder.append(_firstUpper, "");
                _builder.append("Constraint");
                int _counter1 = index.getCounter1();
                _builder.append(_counter1, "");
                _builder.append("Enabled(");
                String _name_1 = it.getName();
                _builder.append(_name_1, "");
                _builder.append(" ");
                String _variableName = this._names.toVariableName(it);
                _builder.append(_variableName, "");
                _builder.append(") {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("return ");
                _builder.append(compiled, "	");
                _builder.append(";");
                _builder.newLineIfNotEmpty();
                _builder.append("}");
                _builder.newLine();
              }
            }
            index.increment();
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  private String compileConstraintCondition(final Constraint constraint) {
    EqualityExpr _condition = constraint.getCondition();
    boolean _equals = Objects.equal(_condition, null);
    if (_equals) {
      return "Boolean.TRUE";
    } else {
      EqualityExpr _condition_1 = constraint.getCondition();
      return this._expressionCompiler.compile(_condition_1);
    }
  }
  
  public boolean isConstraintConditionTrue(final Constraint constraint) {
    String _compileConstraintCondition = this.compileConstraintCondition(constraint);
    return ValidatorGenerator.isCompiledConditionTrue(_compileConstraintCondition);
  }
  
  private static boolean isCompiledConditionTrue(final CharSequence conditionString) {
    return "Boolean.TRUE".equals(conditionString);
  }
  
  private CharSequence createSpringValidation(final Entity it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@SuppressWarnings(\"unchecked\")");
    _builder.newLine();
    _builder.append("public boolean supports(Class clazz) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return ");
    String _qualifiedInterfaceName = this._names.getQualifiedInterfaceName(it);
    String _addImport = this._names.addImport(it, _qualifiedInterfaceName);
    _builder.append(_addImport, "	");
    _builder.append(".class.isAssignableFrom(clazz);");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("public ");
    String _addImport_1 = this._names.addImport(it, "org.springframework.validation.Errors");
    _builder.append(_addImport_1, "");
    _builder.append(" validate(");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append(" ");
    String _variableName = this._names.toVariableName(it);
    _builder.append(_variableName, "");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("Errors errors = createErrors(");
    String _variableName_1 = this._names.toVariableName(it);
    _builder.append(_variableName_1, "	");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    {
      Entity _superType = it.getSuperType();
      boolean _notEquals = (!Objects.equal(_superType, null));
      if (_notEquals) {
        _builder.append("\t");
        _builder.append("super.validate(");
        String _variableName_2 = this._names.toVariableName(it);
        _builder.append(_variableName_2, "	");
        _builder.append(",errors);");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("validate(");
    String _variableName_3 = this._names.toVariableName(it);
    _builder.append(_variableName_3, "	");
    _builder.append(",errors);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return errors;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("public void validate(Object object, Errors errors) {");
    _builder.newLine();
    _builder.append("\t");
    String _name_1 = it.getName();
    _builder.append(_name_1, "	");
    _builder.append(" ");
    String _variableName_4 = this._names.toVariableName(it);
    _builder.append(_variableName_4, "	");
    _builder.append(" = (");
    String _name_2 = it.getName();
    _builder.append(_name_2, "	");
    _builder.append(") object;");
    _builder.newLineIfNotEmpty();
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute e) {
            EList<AttributeProperty> _attributProperties = e.getAttributProperties();
            Iterable<RequiredFlag> _filter = Iterables.<RequiredFlag>filter(_attributProperties, RequiredFlag.class);
            boolean _isEmpty = IterableExtensions.isEmpty(_filter);
            boolean _not = (!_isEmpty);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      for(final Attribute attribute : _filter) {
        {
          boolean _isRequired = attribute.isRequired();
          if (_isRequired) {
            _builder.append("\t");
            String _addImport_2 = this._names.addImport(it, "org.springframework.validation.ValidationUtils");
            _builder.append(_addImport_2, "	");
            _builder.append(".rejectIfEmpty");
            {
              DataTypeAndTypeParameter _type = attribute.getType();
              boolean _hasStringValidator = ValidatorExtension.hasStringValidator(_type);
              if (_hasStringValidator) {
                _builder.append("OrWhitespace");
              }
            }
            _builder.append("(errors, \"");
            String _variableName_5 = this._names.toVariableName(attribute);
            _builder.append(_variableName_5, "	");
            _builder.append("\", \"field.required\");");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    {
      EList<Attribute> _attributes_1 = it.getAttributes();
      for(final Attribute attribute_1 : _attributes_1) {
        {
          DataTypeAndTypeParameter _type_1 = attribute_1.getType();
          boolean _hasStringValidator_1 = ValidatorExtension.hasStringValidator(_type_1);
          if (_hasStringValidator_1) {
            _builder.append("\t");
            DataTypeAndTypeParameter _type_2 = attribute_1.getType();
            ValidatorReference _validatorReferenceDefinition = _type_2.getValidatorReferenceDefinition();
            DataTypeAndTypeParameter _type_3 = attribute_1.getType();
            List<ReferenceWithParameter> _definitionStack = _type_3.getDefinitionStack();
            Validator _createXMAValidator = ValidatorExtension.createXMAValidator(_validatorReferenceDefinition, _definitionStack);
            final StringValidator validator = ((StringValidator) _createXMAValidator);
            _builder.newLineIfNotEmpty();
            {
              String _cntMinLength = validator.getCntMinLength();
              boolean _notEquals_1 = (!Objects.equal(_cntMinLength, null));
              if (_notEquals_1) {
                _builder.append("\t");
                _builder.append("rejectIfMinLength(errors,\"");
                String _name_3 = attribute_1.getName();
                _builder.append(_name_3, "	");
                _builder.append("\",");
                String _variableName_6 = this._names.toVariableName(it);
                _builder.append(_variableName_6, "	");
                _builder.append(".");
                String _accessorMethodName = this._features.getAccessorMethodName(attribute_1);
                _builder.append(_accessorMethodName, "	");
                _builder.append("(),");
                String _cntMinLength_1 = validator.getCntMinLength();
                _builder.append(_cntMinLength_1, "	");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
              }
            }
            {
              String _cntMaxLength = validator.getCntMaxLength();
              boolean _notEquals_2 = (!Objects.equal(_cntMaxLength, null));
              if (_notEquals_2) {
                _builder.append("\t");
                _builder.append("rejectIfMaxLength(errors,\"");
                String _name_4 = attribute_1.getName();
                _builder.append(_name_4, "	");
                _builder.append("\",");
                String _variableName_7 = this._names.toVariableName(it);
                _builder.append(_variableName_7, "	");
                _builder.append(".");
                String _accessorMethodName_1 = this._features.getAccessorMethodName(attribute_1);
                _builder.append(_accessorMethodName_1, "	");
                _builder.append("(),");
                String _cntMaxLength_1 = validator.getCntMaxLength();
                _builder.append(_cntMaxLength_1, "	");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
              }
            }
          } else {
            DataTypeAndTypeParameter _type_4 = attribute_1.getType();
            boolean _hasNumberValidator = ValidatorExtension.hasNumberValidator(_type_4);
            if (_hasNumberValidator) {
              _builder.append("\t");
              DataTypeAndTypeParameter _type_5 = attribute_1.getType();
              ValidatorReference _validatorReferenceDefinition_1 = _type_5.getValidatorReferenceDefinition();
              DataTypeAndTypeParameter _type_6 = attribute_1.getType();
              List<ReferenceWithParameter> _definitionStack_1 = _type_6.getDefinitionStack();
              Validator _createXMAValidator_1 = ValidatorExtension.createXMAValidator(_validatorReferenceDefinition_1, _definitionStack_1);
              final BcdValidator validator_1 = ((BcdValidator) _createXMAValidator_1);
              _builder.newLineIfNotEmpty();
              {
                boolean _and = false;
                boolean _and_1 = false;
                String _cntBeforeComma = validator_1.getCntBeforeComma();
                boolean _notEquals_3 = (!Objects.equal(_cntBeforeComma, null));
                if (!_notEquals_3) {
                  _and_1 = false;
                } else {
                  boolean _or = false;
                  DataTypeAndTypeParameter _type_7 = attribute_1.getType();
                  DataTypeAndTypeParameter _typeForName = SimpleTypeExtension.getTypeForName(_type_7, "Integer");
                  boolean _equals = Objects.equal(_typeForName, null);
                  if (_equals) {
                    _or = true;
                  } else {
                    String _cntBeforeComma_1 = validator_1.getCntBeforeComma();
                    Integer _asInteger = this.asInteger(_cntBeforeComma_1);
                    boolean _notEquals_4 = ((_asInteger).intValue() != 10);
                    _or = (_equals || _notEquals_4);
                  }
                  _and_1 = (_notEquals_3 && _or);
                }
                if (!_and_1) {
                  _and = false;
                } else {
                  boolean _or_1 = false;
                  DataTypeAndTypeParameter _type_8 = attribute_1.getType();
                  DataTypeAndTypeParameter _typeForName_1 = SimpleTypeExtension.getTypeForName(_type_8, "Long");
                  boolean _equals_1 = Objects.equal(_typeForName_1, null);
                  if (_equals_1) {
                    _or_1 = true;
                  } else {
                    String _cntBeforeComma_2 = validator_1.getCntBeforeComma();
                    Integer _asInteger_1 = this.asInteger(_cntBeforeComma_2);
                    boolean _notEquals_5 = ((_asInteger_1).intValue() != 19);
                    _or_1 = (_equals_1 || _notEquals_5);
                  }
                  _and = (_and_1 && _or_1);
                }
                if (_and) {
                  _builder.append("\t");
                  _builder.append("rejectIfMaxIntegerDigits(errors,\"");
                  String _name_5 = attribute_1.getName();
                  _builder.append(_name_5, "	");
                  _builder.append("\",");
                  String _variableName_8 = this._names.toVariableName(it);
                  _builder.append(_variableName_8, "	");
                  _builder.append(".");
                  String _accessorMethodName_2 = this._features.getAccessorMethodName(attribute_1);
                  _builder.append(_accessorMethodName_2, "	");
                  _builder.append("(),");
                  String _cntBeforeComma_3 = validator_1.getCntBeforeComma();
                  _builder.append(_cntBeforeComma_3, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
              {
                boolean _and_2 = false;
                boolean _and_3 = false;
                String _cntAfterComma = validator_1.getCntAfterComma();
                boolean _notEquals_6 = (!Objects.equal(_cntAfterComma, null));
                if (!_notEquals_6) {
                  _and_3 = false;
                } else {
                  DataTypeAndTypeParameter _type_9 = attribute_1.getType();
                  DataTypeAndTypeParameter _typeForName_2 = SimpleTypeExtension.getTypeForName(_type_9, "Integer");
                  boolean _equals_2 = Objects.equal(_typeForName_2, null);
                  _and_3 = (_notEquals_6 && _equals_2);
                }
                if (!_and_3) {
                  _and_2 = false;
                } else {
                  DataTypeAndTypeParameter _type_10 = attribute_1.getType();
                  DataTypeAndTypeParameter _typeForName_3 = SimpleTypeExtension.getTypeForName(_type_10, "Long");
                  boolean _equals_3 = Objects.equal(_typeForName_3, null);
                  _and_2 = (_and_3 && _equals_3);
                }
                if (_and_2) {
                  _builder.append("\t");
                  _builder.append("rejectIfMaxFractionDigits(errors,\"");
                  String _name_6 = attribute_1.getName();
                  _builder.append(_name_6, "	");
                  _builder.append("\",");
                  String _variableName_9 = this._names.toVariableName(it);
                  _builder.append(_variableName_9, "	");
                  _builder.append(".");
                  String _accessorMethodName_3 = this._features.getAccessorMethodName(attribute_1);
                  _builder.append(_accessorMethodName_3, "	");
                  _builder.append("(),");
                  String _cntAfterComma_1 = validator_1.getCntAfterComma();
                  _builder.append(_cntAfterComma_1, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
            }
          }
        }
      }
    }
    {
      EList<Attribute> _attributes_2 = it.getAttributes();
      final Function1<Attribute,Boolean> _function_1 = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute e) {
            EList<AttributeProperty> _attributProperties = e.getAttributProperties();
            Iterable<AttributeValidationProperty> _filter = Iterables.<AttributeValidationProperty>filter(_attributProperties, AttributeValidationProperty.class);
            final Function1<AttributeValidationProperty,EList<Constraint>> _function = new Function1<AttributeValidationProperty,EList<Constraint>>() {
                public EList<Constraint> apply(final AttributeValidationProperty a) {
                  EList<Constraint> _constraints = a.getConstraints();
                  return _constraints;
                }
              };
            Iterable<EList<Constraint>> _map = IterableExtensions.<AttributeValidationProperty, EList<Constraint>>map(_filter, _function);
            boolean _isEmpty = IterableExtensions.isEmpty(_map);
            boolean _not = (!_isEmpty);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter_1 = IterableExtensions.<Attribute>filter(_attributes_2, _function_1);
      for(final Attribute attribute_2 : _filter_1) {
        _builder.append("\t");
        Index _index = new Index();
        final Index iter = _index;
        _builder.newLineIfNotEmpty();
        {
          EList<AttributeProperty> _attributProperties = attribute_2.getAttributProperties();
          Iterable<AttributeValidationProperty> _filter_2 = Iterables.<AttributeValidationProperty>filter(_attributProperties, AttributeValidationProperty.class);
          final Function1<AttributeValidationProperty,EList<Constraint>> _function_2 = new Function1<AttributeValidationProperty,EList<Constraint>>() {
              public EList<Constraint> apply(final AttributeValidationProperty a) {
                EList<Constraint> _constraints = a.getConstraints();
                return _constraints;
              }
            };
          Iterable<EList<Constraint>> _map = IterableExtensions.<AttributeValidationProperty, EList<Constraint>>map(_filter_2, _function_2);
          Iterable<Constraint> _flatten = Iterables.<Constraint>concat(_map);
          for(final Constraint constraint : _flatten) {
            {
              ValidatorReference _validatorReference = constraint.getValidatorReference();
              boolean _isStringValidator = _validatorReference.isStringValidator();
              if (_isStringValidator) {
                _builder.append("\t");
                ValidatorReference _validatorReference_1 = constraint.getValidatorReference();
                Validator _createXMAValidator_2 = ValidatorExtension.createXMAValidator(_validatorReference_1);
                final StringValidator validator_2 = ((StringValidator) _createXMAValidator_2);
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("if (is");
                String _name_7 = attribute_2.getName();
                String _firstUpper = StringExtensions.toFirstUpper(_name_7);
                _builder.append(_firstUpper, "	");
                _builder.append("Constraint");
                int _counter1 = iter.getCounter1();
                _builder.append(_counter1, "	");
                _builder.append("Enabled(");
                String _variableName_10 = this._names.toVariableName(it);
                _builder.append(_variableName_10, "	");
                _builder.append(")) {");
                _builder.newLineIfNotEmpty();
                {
                  String _cntMinLength_2 = validator_2.getCntMinLength();
                  boolean _notEquals_7 = (!Objects.equal(_cntMinLength_2, null));
                  if (_notEquals_7) {
                    _builder.append("\t");
                    _builder.append("\t");
                    _builder.append("rejectIfMinLength(errors,\"");
                    String _name_8 = attribute_2.getName();
                    _builder.append(_name_8, "		");
                    _builder.append("\",");
                    String _variableName_11 = this._names.toVariableName(it);
                    _builder.append(_variableName_11, "		");
                    _builder.append(".");
                    String _accessorMethodName_4 = this._features.getAccessorMethodName(attribute_2);
                    _builder.append(_accessorMethodName_4, "		");
                    _builder.append("(),");
                    String _cntMinLength_3 = validator_2.getCntMinLength();
                    _builder.append(_cntMinLength_3, "		");
                    _builder.append(");");
                    _builder.newLineIfNotEmpty();
                  }
                }
                {
                  String _cntMaxLength_2 = validator_2.getCntMaxLength();
                  boolean _notEquals_8 = (!Objects.equal(_cntMaxLength_2, null));
                  if (_notEquals_8) {
                    _builder.append("\t");
                    _builder.append("\t");
                    _builder.append("rejectIfMaxLength(errors,\"");
                    String _name_9 = attribute_2.getName();
                    _builder.append(_name_9, "		");
                    _builder.append("\",");
                    String _variableName_12 = this._names.toVariableName(it);
                    _builder.append(_variableName_12, "		");
                    _builder.append(".");
                    String _accessorMethodName_5 = this._features.getAccessorMethodName(attribute_2);
                    _builder.append(_accessorMethodName_5, "		");
                    _builder.append("(),");
                    String _cntMaxLength_3 = validator_2.getCntMaxLength();
                    _builder.append(_cntMaxLength_3, "		");
                    _builder.append(");");
                    _builder.newLineIfNotEmpty();
                  }
                }
                {
                  String _txtFormatrange = validator_2.getTxtFormatrange();
                  boolean _notEquals_9 = (!Objects.equal(_txtFormatrange, null));
                  if (_notEquals_9) {
                    _builder.append("\t");
                    _builder.append("\t");
                    _builder.append("rejectIfInvalidFormat(errors,\"");
                    String _name_10 = attribute_2.getName();
                    _builder.append(_name_10, "		");
                    _builder.append("\",");
                    String _variableName_13 = this._names.toVariableName(it);
                    _builder.append(_variableName_13, "		");
                    _builder.append(".");
                    String _accessorMethodName_6 = this._features.getAccessorMethodName(attribute_2);
                    _builder.append(_accessorMethodName_6, "		");
                    _builder.append("(),\"");
                    String _txtFormatrange_1 = validator_2.getTxtFormatrange();
                    _builder.append(_txtFormatrange_1, "		");
                    _builder.append("\");");
                    _builder.newLineIfNotEmpty();
                  }
                }
                _builder.append("\t");
                _builder.append("}");
                _builder.newLine();
              } else {
                ValidatorReference _validatorReference_2 = constraint.getValidatorReference();
                boolean _isNumberValidator = _validatorReference_2.isNumberValidator();
                if (_isNumberValidator) {
                  _builder.append("\t");
                  ValidatorReference _validatorReference_3 = constraint.getValidatorReference();
                  Validator _createXMAValidator_3 = ValidatorExtension.createXMAValidator(_validatorReference_3);
                  final BcdValidator validator_3 = ((BcdValidator) _createXMAValidator_3);
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t");
                  _builder.append("if (is");
                  String _name_11 = attribute_2.getName();
                  String _firstUpper_1 = StringExtensions.toFirstUpper(_name_11);
                  _builder.append(_firstUpper_1, "	");
                  _builder.append("Constraint");
                  int _counter1_1 = iter.getCounter1();
                  _builder.append(_counter1_1, "	");
                  _builder.append("Enabled(");
                  String _variableName_14 = this._names.toVariableName(it);
                  _builder.append(_variableName_14, "	");
                  _builder.append(")) {");
                  _builder.newLineIfNotEmpty();
                  {
                    String _qntMinVal = validator_3.getQntMinVal();
                    boolean _notEquals_10 = (!Objects.equal(_qntMinVal, null));
                    if (_notEquals_10) {
                      _builder.append("\t");
                      _builder.append("\t");
                      _builder.append("rejectIfMinValue(errors,\"");
                      String _name_12 = attribute_2.getName();
                      _builder.append(_name_12, "		");
                      _builder.append("\",");
                      String _variableName_15 = this._names.toVariableName(it);
                      _builder.append(_variableName_15, "		");
                      _builder.append(".");
                      String _accessorMethodName_7 = this._features.getAccessorMethodName(attribute_2);
                      _builder.append(_accessorMethodName_7, "		");
                      _builder.append("(),");
                      String _qntMinVal_1 = validator_3.getQntMinVal();
                      _builder.append(_qntMinVal_1, "		");
                      _builder.append(");");
                      _builder.newLineIfNotEmpty();
                    }
                  }
                  {
                    String _qntMaxVal = validator_3.getQntMaxVal();
                    boolean _notEquals_11 = (!Objects.equal(_qntMaxVal, null));
                    if (_notEquals_11) {
                      _builder.append("\t");
                      _builder.append("\t");
                      _builder.append("rejectIfMaxValue(errors,\"");
                      String _name_13 = attribute_2.getName();
                      _builder.append(_name_13, "		");
                      _builder.append("\",");
                      String _variableName_16 = this._names.toVariableName(it);
                      _builder.append(_variableName_16, "		");
                      _builder.append(".");
                      String _accessorMethodName_8 = this._features.getAccessorMethodName(attribute_2);
                      _builder.append(_accessorMethodName_8, "		");
                      _builder.append("(),");
                      String _qntMaxVal_1 = validator_3.getQntMaxVal();
                      _builder.append(_qntMaxVal_1, "		");
                      _builder.append(");");
                      _builder.newLineIfNotEmpty();
                    }
                  }
                  {
                    boolean _and_4 = false;
                    boolean _and_5 = false;
                    String _cntBeforeComma_4 = validator_3.getCntBeforeComma();
                    boolean _notEquals_12 = (!Objects.equal(_cntBeforeComma_4, null));
                    if (!_notEquals_12) {
                      _and_5 = false;
                    } else {
                      boolean _or_2 = false;
                      DataTypeAndTypeParameter _type_11 = attribute_2.getType();
                      DataTypeAndTypeParameter _typeForName_4 = SimpleTypeExtension.getTypeForName(_type_11, "Integer");
                      boolean _equals_4 = Objects.equal(_typeForName_4, null);
                      if (_equals_4) {
                        _or_2 = true;
                      } else {
                        String _cntBeforeComma_5 = validator_3.getCntBeforeComma();
                        Integer _asInteger_2 = this.asInteger(_cntBeforeComma_5);
                        boolean _notEquals_13 = ((_asInteger_2).intValue() != 10);
                        _or_2 = (_equals_4 || _notEquals_13);
                      }
                      _and_5 = (_notEquals_12 && _or_2);
                    }
                    if (!_and_5) {
                      _and_4 = false;
                    } else {
                      boolean _or_3 = false;
                      DataTypeAndTypeParameter _type_12 = attribute_2.getType();
                      DataTypeAndTypeParameter _typeForName_5 = SimpleTypeExtension.getTypeForName(_type_12, "Long");
                      boolean _equals_5 = Objects.equal(_typeForName_5, null);
                      if (_equals_5) {
                        _or_3 = true;
                      } else {
                        String _cntBeforeComma_6 = validator_3.getCntBeforeComma();
                        Integer _asInteger_3 = this.asInteger(_cntBeforeComma_6);
                        boolean _notEquals_14 = ((_asInteger_3).intValue() != 19);
                        _or_3 = (_equals_5 || _notEquals_14);
                      }
                      _and_4 = (_and_5 && _or_3);
                    }
                    if (_and_4) {
                      _builder.append("\t");
                      _builder.append("\t");
                      _builder.append("rejectIfMaxIntegerDigits(errors,\"");
                      String _name_14 = attribute_2.getName();
                      _builder.append(_name_14, "		");
                      _builder.append("\",");
                      String _variableName_17 = this._names.toVariableName(it);
                      _builder.append(_variableName_17, "		");
                      _builder.append(".");
                      String _accessorMethodName_9 = this._features.getAccessorMethodName(attribute_2);
                      _builder.append(_accessorMethodName_9, "		");
                      _builder.append("(),");
                      String _cntBeforeComma_7 = validator_3.getCntBeforeComma();
                      _builder.append(_cntBeforeComma_7, "		");
                      _builder.append(");");
                      _builder.newLineIfNotEmpty();
                    }
                  }
                  {
                    boolean _and_6 = false;
                    boolean _and_7 = false;
                    String _cntAfterComma_2 = validator_3.getCntAfterComma();
                    boolean _notEquals_15 = (!Objects.equal(_cntAfterComma_2, null));
                    if (!_notEquals_15) {
                      _and_7 = false;
                    } else {
                      DataTypeAndTypeParameter _type_13 = attribute_2.getType();
                      DataTypeAndTypeParameter _typeForName_6 = SimpleTypeExtension.getTypeForName(_type_13, "Integer");
                      boolean _equals_6 = Objects.equal(_typeForName_6, null);
                      _and_7 = (_notEquals_15 && _equals_6);
                    }
                    if (!_and_7) {
                      _and_6 = false;
                    } else {
                      DataTypeAndTypeParameter _type_14 = attribute_2.getType();
                      DataTypeAndTypeParameter _typeForName_7 = SimpleTypeExtension.getTypeForName(_type_14, "Long");
                      boolean _equals_7 = Objects.equal(_typeForName_7, null);
                      _and_6 = (_and_7 && _equals_7);
                    }
                    if (_and_6) {
                      _builder.append("\t");
                      _builder.append("\t");
                      _builder.append("rejectIfMaxFractionDigits(errors,\"");
                      String _name_15 = attribute_2.getName();
                      _builder.append(_name_15, "		");
                      _builder.append("\",");
                      String _variableName_18 = this._names.toVariableName(it);
                      _builder.append(_variableName_18, "		");
                      _builder.append(".");
                      String _accessorMethodName_10 = this._features.getAccessorMethodName(attribute_2);
                      _builder.append(_accessorMethodName_10, "		");
                      _builder.append("(),");
                      String _cntAfterComma_3 = validator_3.getCntAfterComma();
                      _builder.append(_cntAfterComma_3, "		");
                      _builder.append(");");
                      _builder.newLineIfNotEmpty();
                    }
                  }
                  _builder.append("\t");
                  _builder.append("}");
                  _builder.newLine();
                } else {
                  ValidatorReference _validatorReference_4 = constraint.getValidatorReference();
                  boolean _isCustomValidator = _validatorReference_4.isCustomValidator();
                  if (_isCustomValidator) {
                    _builder.append("\t");
                    ValidatorReference _validatorReference_5 = constraint.getValidatorReference();
                    Validator _createXMAValidator_4 = ValidatorExtension.createXMAValidator(_validatorReference_5);
                    final CustomValidator validator_4 = ((CustomValidator) _createXMAValidator_4);
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("if (is");
                    String _name_16 = attribute_2.getName();
                    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_16);
                    _builder.append(_firstUpper_2, "	");
                    _builder.append("Constraint");
                    int _counter1_2 = iter.getCounter1();
                    _builder.append(_counter1_2, "	");
                    _builder.append("Enabled(");
                    String _variableName_19 = this._names.toVariableName(it);
                    _builder.append(_variableName_19, "	");
                    _builder.append(")) {");
                    _builder.newLineIfNotEmpty();
                    {
                      String _txtFormatingString = validator_4.getTxtFormatingString();
                      boolean _notEquals_16 = (!Objects.equal(_txtFormatingString, null));
                      if (_notEquals_16) {
                        _builder.append("\t");
                        _builder.append("\t");
                        _builder.append("rejectIfInvalidFormat(errors,\"");
                        String _name_17 = attribute_2.getName();
                        _builder.append(_name_17, "		");
                        _builder.append("\",");
                        String _variableName_20 = this._names.toVariableName(it);
                        _builder.append(_variableName_20, "		");
                        _builder.append(".");
                        String _accessorMethodName_11 = this._features.getAccessorMethodName(attribute_2);
                        _builder.append(_accessorMethodName_11, "		");
                        _builder.append("(),\"");
                        String _txtFormatingString_1 = validator_4.getTxtFormatingString();
                        _builder.append(_txtFormatingString_1, "		");
                        _builder.append("\");");
                        _builder.newLineIfNotEmpty();
                      }
                    }
                    _builder.append("\t");
                    _builder.append("}");
                    _builder.newLine();
                  } else {
                    ValidatorReference _validatorReference_6 = constraint.getValidatorReference();
                    boolean _isUserDefinedValidator = _validatorReference_6.isUserDefinedValidator();
                    if (_isUserDefinedValidator) {
                      _builder.append("\t");
                      ValidatorReference _validatorReference_7 = constraint.getValidatorReference();
                      Validator _createXMAValidator_5 = ValidatorExtension.createXMAValidator(_validatorReference_7);
                      final UserDefinedValidator validator_5 = ((UserDefinedValidator) _createXMAValidator_5);
                      _builder.newLineIfNotEmpty();
                      _builder.append("\t");
                      _builder.append("if (is");
                      String _name_18 = attribute_2.getName();
                      String _firstUpper_3 = StringExtensions.toFirstUpper(_name_18);
                      _builder.append(_firstUpper_3, "	");
                      _builder.append("Constraint");
                      int _counter1_3 = iter.getCounter1();
                      _builder.append(_counter1_3, "	");
                      _builder.append("Enabled(");
                      String _variableName_21 = this._names.toVariableName(it);
                      _builder.append(_variableName_21, "	");
                      _builder.append(")) {");
                      _builder.newLineIfNotEmpty();
                      {
                        String _namClass = validator_5.getNamClass();
                        boolean _notEquals_17 = (!Objects.equal(_namClass, null));
                        if (_notEquals_17) {
                          _builder.append("\t");
                          _builder.append("\t");
                          _builder.append("rejectIfInvalidFormat(errors,\"");
                          String _name_19 = attribute_2.getName();
                          _builder.append(_name_19, "		");
                          _builder.append("\",");
                          String _variableName_22 = this._names.toVariableName(it);
                          _builder.append(_variableName_22, "		");
                          _builder.append(".");
                          String _accessorMethodName_12 = this._features.getAccessorMethodName(attribute_2);
                          _builder.append(_accessorMethodName_12, "		");
                          _builder.append("(),new ");
                          String _namClass_1 = validator_5.getNamClass();
                          String _replaceAll = _namClass_1.replaceAll("\\^", "");
                          _builder.append(_replaceAll, "		");
                          _builder.append("(\"");
                          String _txtParameterString = validator_5.getTxtParameterString();
                          _builder.append(_txtParameterString, "		");
                          _builder.append("\"));");
                          _builder.newLineIfNotEmpty();
                        }
                      }
                      _builder.append("\t");
                      _builder.append("}");
                      _builder.newLine();
                    }
                  }
                }
              }
            }
            _builder.append("\t");
            iter.increment();
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence createJsr303Validation(final ComplexType it) {
    CharSequence _xblockexpression = null;
    {
      final CharSequence groups = this.createJsr303ValidationGroups(it);
      CharSequence _xifexpression = null;
      int _length = groups.length();
      boolean _greaterThan = (_length > 0);
      if (_greaterThan) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("protected ");
        String _addImport = this._names.addImport(it, "java.util.Set");
        _builder.append(_addImport, "");
        _builder.append("<Class<?>> createGroups(");
        String _qualifiedName = this._names.getQualifiedName(it);
        String _addImport_1 = this._names.addImport(it, _qualifiedName);
        _builder.append(_addImport_1, "");
        _builder.append(" ");
        String _variableName = this._names.toVariableName(it);
        _builder.append(_variableName, "");
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("Set<Class<?>> groups = new ");
        String _addImport_2 = this._names.addImport(it, "java.util.HashSet");
        _builder.append(_addImport_2, "	");
        _builder.append("<Class<?>>();");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append(groups, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("return groups;");
        _builder.newLine();
        _builder.append("}");
        _builder.newLine();
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  private CharSequence createJsr303ValidationGroups(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute e) {
            EList<AttributeProperty> _attributProperties = e.getAttributProperties();
            Iterable<AttributeValidationProperty> _filter = Iterables.<AttributeValidationProperty>filter(_attributProperties, AttributeValidationProperty.class);
            final Function1<AttributeValidationProperty,EList<Constraint>> _function = new Function1<AttributeValidationProperty,EList<Constraint>>() {
                public EList<Constraint> apply(final AttributeValidationProperty a) {
                  EList<Constraint> _constraints = a.getConstraints();
                  return _constraints;
                }
              };
            Iterable<EList<Constraint>> _map = IterableExtensions.<AttributeValidationProperty, EList<Constraint>>map(_filter, _function);
            boolean _isEmpty = IterableExtensions.isEmpty(_map);
            boolean _not = (!_isEmpty);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      for(final Attribute attribute : _filter) {
        Index _index = new Index();
        final Index iter = _index;
        _builder.newLineIfNotEmpty();
        {
          EList<AttributeProperty> _attributProperties = attribute.getAttributProperties();
          Iterable<AttributeValidationProperty> _filter_1 = Iterables.<AttributeValidationProperty>filter(_attributProperties, AttributeValidationProperty.class);
          final Function1<AttributeValidationProperty,EList<Constraint>> _function_1 = new Function1<AttributeValidationProperty,EList<Constraint>>() {
              public EList<Constraint> apply(final AttributeValidationProperty a) {
                EList<Constraint> _constraints = a.getConstraints();
                return _constraints;
              }
            };
          Iterable<EList<Constraint>> _map = IterableExtensions.<AttributeValidationProperty, EList<Constraint>>map(_filter_1, _function_1);
          Iterable<Constraint> _flatten = Iterables.<Constraint>concat(_map);
          for(final Constraint constraint : _flatten) {
            {
              boolean _or = false;
              boolean _or_1 = false;
              boolean _or_2 = false;
              ValidatorReference _validatorReference = constraint.getValidatorReference();
              boolean _isStringValidator = _validatorReference.isStringValidator();
              if (_isStringValidator) {
                _or_2 = true;
              } else {
                ValidatorReference _validatorReference_1 = constraint.getValidatorReference();
                boolean _isNumberValidator = _validatorReference_1.isNumberValidator();
                _or_2 = (_isStringValidator || _isNumberValidator);
              }
              if (_or_2) {
                _or_1 = true;
              } else {
                ValidatorReference _validatorReference_2 = constraint.getValidatorReference();
                boolean _isCustomValidator = _validatorReference_2.isCustomValidator();
                _or_1 = (_or_2 || _isCustomValidator);
              }
              if (_or_1) {
                _or = true;
              } else {
                ValidatorReference _validatorReference_3 = constraint.getValidatorReference();
                boolean _isUserDefinedValidator = _validatorReference_3.isUserDefinedValidator();
                _or = (_or_1 || _isUserDefinedValidator);
              }
              if (_or) {
                {
                  boolean _isConstraintConditionTrue = this.isConstraintConditionTrue(constraint);
                  boolean _not = (!_isConstraintConditionTrue);
                  if (_not) {
                    _builder.append("if (is");
                    String _name = attribute.getName();
                    String _firstUpper = StringExtensions.toFirstUpper(_name);
                    _builder.append(_firstUpper, "");
                    _builder.append("Constraint");
                    int _counter1 = iter.getCounter1();
                    _builder.append(_counter1, "");
                    _builder.append("Enabled(");
                    String _variableName = this._names.toVariableName(it);
                    _builder.append(_variableName, "");
                    _builder.append(")) {");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("groups.add(");
                    String _qualifiedGenClassName = this._names.getQualifiedGenClassName(it);
                    String _plus = (_qualifiedGenClassName + ".Is");
                    String _name_1 = attribute.getName();
                    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
                    String _plus_1 = (_plus + _firstUpper_1);
                    String _plus_2 = (_plus_1 + "Constraint");
                    int _counter1_1 = iter.getCounter1();
                    String _plus_3 = (_plus_2 + Integer.valueOf(_counter1_1));
                    String _plus_4 = (_plus_3 + "Enabled");
                    String _addImport = this._names.addImport(it, _plus_4);
                    _builder.append(_addImport, "	");
                    _builder.append(".class);");
                    _builder.newLineIfNotEmpty();
                    _builder.append("}");
                    _builder.newLine();
                  }
                }
              }
            }
            iter.increment();
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _toFunction(final ReferenceableByXmadslVariable it, final ModelElement element) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  protected CharSequence _toFunction(final StatusFlag it, final ModelElement element) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("public boolean ");
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(");
    String _qualifiedName = this._names.getQualifiedName(element);
    _builder.append(_qualifiedName, "");
    _builder.append(" ");
    String _variableName = this._names.toVariableName(element);
    _builder.append(_variableName, "");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    EqualityExpr _expression = it.getExpression();
    String _compile = this._expressionCompiler.compile(_expression);
    _builder.append(_compile, "	");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _toFunction(final Property it, final ModelElement element) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("public boolean ");
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("(");
    String _qualifiedName = this._names.getQualifiedName(element);
    _builder.append(_qualifiedName, "");
    _builder.append(" ");
    String _variableName = this._names.toVariableName(element);
    _builder.append(_variableName, "");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return false;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private String getQualifiedValidatorClassName(final ModelElement it) {
    String _validatorPackageName = this.getValidatorPackageName(it);
    String _plus = (_validatorPackageName + ".");
    String _validatorClassName = this.getValidatorClassName(it);
    String _plus_1 = (_plus + _validatorClassName);
    return _plus_1;
  }
  
  private String getQualifiedGenValidatorClassName(final ModelElement it) {
    String _validatorPackageName = this.getValidatorPackageName(it);
    String _plus = (_validatorPackageName + ".");
    String _genValidatorClassName = this.getGenValidatorClassName(it);
    String _plus_1 = (_plus + _genValidatorClassName);
    return _plus_1;
  }
  
  public String getQualifiedGenValidatorAnnotationClassName(final ModelElement it) {
    String _validatorPackageName = this.getValidatorPackageName(it);
    String _plus = (_validatorPackageName + ".");
    String _genValidatorAnnotationClassName = this.getGenValidatorAnnotationClassName(it);
    String _plus_1 = (_plus + _genValidatorAnnotationClassName);
    return _plus_1;
  }
  
  private String getValidatorClassName(final ModelElement it) {
    String _name = it.getName();
    String _plus = (_name + "Validator");
    return _plus;
  }
  
  private String getGenValidatorClassName(final ModelElement it) {
    String _name = it.getName();
    String _plus = (_name + "GenValidator");
    return _plus;
  }
  
  private String getGenValidatorAnnotationClassName(final ModelElement it) {
    String _name = it.getName();
    String _plus = ("Valid" + _name);
    return _plus;
  }
  
  private String getValidatorPackageName(final ModelElement it) {
    Model _model = this._types.model(it);
    String _name = _model.getName();
    String _plus = (_name + ".validation");
    return _plus;
  }
  
  private Integer asInteger(final String value) {
    boolean _notEquals = (!Objects.equal(value, null));
    if (_notEquals) {
      return Integer.valueOf(value);
    }
    return 0;
  }
  
  public CharSequence toFunction(final ReferenceableByXmadslVariable it, final ModelElement element) {
    if (it instanceof StatusFlag) {
      return _toFunction((StatusFlag)it, element);
    } else if (it instanceof Property) {
      return _toFunction((Property)it, element);
    } else if (it != null) {
      return _toFunction(it, element);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it, element).toString());
    }
  }
}
