package org.openxma.dsl.generator.impl;

import com.google.common.collect.Iterables;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.openxma.dsl.common.logging.LogUtil;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.dom.model.ValueObject;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;

@SuppressWarnings("all")
public class ValueObjectGenerator implements IGenerator {
  @Inject
  private Names names;
  
  @Inject
  private Features features;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fileSystemAccess) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<ValueObject> _filter = Iterables.<ValueObject>filter(_elements, ValueObject.class);
    final Procedure1<ValueObject> _function = new Procedure1<ValueObject>() {
        public void apply(final ValueObject it) {
          String _genClassFileName = ValueObjectGenerator.this.getGenClassFileName(it);
          String _plus = ("Write " + _genClassFileName);
          LogUtil.info(_plus);
          String _genClassFileName_1 = ValueObjectGenerator.this.getGenClassFileName(it);
          CharSequence _genClass = ValueObjectGenerator.this.toGenClass(it);
          fileSystemAccess.generateFile(_genClassFileName_1, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _classFileName = ValueObjectGenerator.this.getClassFileName(it);
          String _plus_1 = ("Write " + _classFileName);
          LogUtil.info(_plus_1);
          String _classFileName_1 = ValueObjectGenerator.this.getClassFileName(it);
          CharSequence _class = ValueObjectGenerator.this.toClass(it);
          fileSystemAccess.generateFile(_classFileName_1, GeneratorConstants.OUTLET_JAVA, _class);
        }
      };
    IterableExtensions.<ValueObject>forEach(_filter, _function);
  }
  
  public CharSequence toClass(final ValueObject it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this.names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public class ");
    String _className = this.getClassName(it);
    _builder.append(_className, "");
    _builder.append(" extends ");
    String _genClassName = this.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _serialVersionUID = this.features.toSerialVersionUID(it);
    _builder.append(_serialVersionUID, "	");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final ValueObject it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this.names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this.features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _genClassName = this.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" implements ");
    String _addImport = this.names.addImport(it, "java.io.Serializable");
    _builder.append(_addImport, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _serialVersionUID = this.features.toSerialVersionUID(it);
    _builder.append(_serialVersionUID, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _generateAttributes = this.features.generateAttributes(it);
    _builder.append(_generateAttributes, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generateProperties = this.features.generateProperties(it);
    _builder.append(_generateProperties, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generateToString = this.features.generateToString(it);
    _builder.append(_generateToString, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private String getGenClassName(final ValueObject it) {
    String _name = it.getName();
    String _plus = (_name + "Gen");
    return _plus;
  }
  
  private String getClassName(final ValueObject it) {
    String _name = it.getName();
    return _name;
  }
  
  private String getQualifiedGenClassName(final ValueObject it) {
    String _packageName = this.names.getPackageName(it);
    String _plus = (_packageName + ".");
    String _genClassName = this.getGenClassName(it);
    String _plus_1 = (_plus + _genClassName);
    return _plus_1;
  }
  
  private String getQualifiedClassName(final ValueObject it) {
    String _packageName = this.names.getPackageName(it);
    String _plus = (_packageName + ".");
    String _className = this.getClassName(it);
    String _plus_1 = (_plus + _className);
    return _plus_1;
  }
  
  private String getClassFileName(final ValueObject it) {
    String _qualifiedClassName = this.getQualifiedClassName(it);
    String _replaceAll = _qualifiedClassName.replaceAll("\\.", "/");
    String _plus = (_replaceAll + ".java");
    return _plus;
  }
  
  private String getGenClassFileName(final ValueObject it) {
    String _qualifiedGenClassName = this.getQualifiedGenClassName(it);
    String _replaceAll = _qualifiedGenClassName.replaceAll("\\.", "/");
    String _plus = (_replaceAll + ".java");
    return _plus;
  }
}
