package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.generator.IGenerator
import org.openxma.dsl.common.logging.LogUtil
import org.openxma.dsl.core.model.Model
import org.openxma.dsl.dom.model.ValueObject
import org.openxma.dsl.generator.GeneratorConstants
import static extension org.openxma.dsl.generator.lib.ImportAdapter.*


class ValueObjectGenerator implements IGenerator {
    @Inject
    extension Names names
    @Inject
    extension Features features

	override doGenerate(Resource resource, IFileSystemAccess fileSystemAccess) {
		val model = resource.getEObject("/") as Model
		model.elements.filter(typeof(ValueObject)).forEach([ 
			LogUtil::info("Write "+getGenClassFileName())
			fileSystemAccess.generateFile(getGenClassFileName(),GeneratorConstants::OUTLET_GENERATED_JAVA, toGenClass)
			LogUtil::info("Write "+getClassFileName())
			fileSystemAccess.generateFile(getClassFileName(),GeneratorConstants::OUTLET_JAVA, toClass)
		])
	}
	
	def CharSequence toClass(ValueObject it) {
		'''
		package packageName;
		
		public class getClassName() extends getGenClassName() {
			toSerialVersionUID

		}
		'''
	}
	
	def CharSequence toGenClass(ValueObject it) {
		'''
		package packageName;
		
		addImportsHere
		
		documentation
		public class getGenClassName() implements addImport("java.io.Serializable") {
			toSerialVersionUID
			generateAttributes
			
			generateProperties
			
			generateToString
		}
		'''
	}
	
	
	def private getGenClassName(ValueObject it) {
		name+"Gen"
	}
	
	def private getClassName(ValueObject it) {
		name
	}

	def private getQualifiedGenClassName(ValueObject it) {
		packageName+"."+getGenClassName
	}	
	
	def private getQualifiedClassName(ValueObject it) {
		packageName+"."+getClassName
	}	
	
	def private getClassFileName(ValueObject it) {
		getQualifiedClassName.replaceAll("\\.","/")+".java"
	}

	def private getGenClassFileName(ValueObject it) {
		getQualifiedGenClassName.replaceAll("\\.","/")+".java"
	}	
}