package org.openxma.dsl.generator.impl;

import com.google.common.collect.Lists;
import java.util.List;
import javax.inject.Inject;
import javax.inject.Named;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function0;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.generator.Generator;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.IFileSystemAccessProvider;
import org.openxma.dsl.generator.ModelModifier;
import org.openxma.dsl.generator.impl.IFileSystemAccessDelegate;

@SuppressWarnings("all")
public class XtendDomGenerator implements Generator, IGenerator {
  private final Log logger = new Function0<Log>() {
    public Log apply() {
      Class<? extends XtendDomGenerator> _class = XtendDomGenerator.this.getClass();
      String _name = _class.getName();
      Log _log = LogFactory.getLog(_name);
      return _log;
    }
  }.apply();
  
  private GeneratorConfiguration _generatorConfiguration;
  
  public GeneratorConfiguration getGeneratorConfiguration() {
    return this._generatorConfiguration;
  }
  
  public void setGeneratorConfiguration(final GeneratorConfiguration generatorConfiguration) {
    this._generatorConfiguration = generatorConfiguration;
  }
  
  @Inject
  @Named("valueObjectGenerator")
  private IGenerator valueObjectGenerator;
  
  @Inject
  @Named("repositoryGenerator")
  private IGenerator repositoryGenerator;
  
  @Inject
  @Named("hbmXmlFileGenerator")
  private IGenerator hbmXmlFileGenerator;
  
  @Inject
  @Named("entityGenerator")
  private IGenerator entityGenerator;
  
  @Inject
  @Named("dataTypeGenerator")
  private IGenerator dataTypeGenerator;
  
  @Inject
  @Named("serviceGenerator")
  private IGenerator serviceGenerator;
  
  @Inject
  @Named("dataViewGenerator")
  private IGenerator dataViewGenerator;
  
  @Inject
  @Named("validatorGenerator")
  private IGenerator validatorGenerator;
  
  @Inject
  @Named("sessionContextGenerator")
  private IGenerator sessionContextGenerator;
  
  @Inject
  private IFileSystemAccessProvider fileSystemAccessProvider;
  
  private ProgressMonitor monitor;
  
  public void init(final GeneratorConfiguration generatorConfiguration) {
    this.setGeneratorConfiguration(generatorConfiguration);
  }
  
  public void generate(final Resource resource, final ProgressMonitor monitor) {
    this.monitor = monitor;
    try {
      URI _uRI = resource.getURI();
      String _lastSegment = _uRI.lastSegment();
      String _plus = ("on " + _lastSegment);
      monitor.beginTask(_plus, 10);
      GeneratorConfiguration _generatorConfiguration = this.getGeneratorConfiguration();
      final IFileSystemAccess fileSystemAccess = this.fileSystemAccessProvider.get(_generatorConfiguration);
      this.doGenerate(resource, fileSystemAccess);
    } finally {
      monitor.done();
    }
  }
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fileSystemAccess) {
    boolean _or = false;
    URI _uRI = resource.getURI();
    boolean _isArchive = _uRI.isArchive();
    if (_isArchive) {
      _or = true;
    } else {
      boolean _isCanceled = this.monitor.isCanceled();
      _or = (_isArchive || _isCanceled);
    }
    if (_or) {
      return;
    }
    ResourceSet _resourceSet = resource.getResourceSet();
    this.modify(_resourceSet);
    ResourceSet _resourceSet_1 = resource.getResourceSet();
    IFileSystemAccessDelegate _iFileSystemAccessDelegate = new IFileSystemAccessDelegate(fileSystemAccess, _resourceSet_1);
    final IFileSystemAccessDelegate fsa = _iFileSystemAccessDelegate;
    this.subTask("- write ValueObjects", 1);
    this.valueObjectGenerator.doGenerate(resource, fsa);
    this.subTask("- write Entities", 1);
    this.entityGenerator.doGenerate(resource, fsa);
    this.subTask("- write Repositories", 1);
    this.repositoryGenerator.doGenerate(resource, fsa);
    this.subTask("- write ValueObjects", 1);
    this.hbmXmlFileGenerator.doGenerate(resource, fsa);
    this.subTask("- write Validators", 1);
    this.validatorGenerator.doGenerate(resource, fsa);
    this.subTask("- write DataViews", 1);
    this.dataViewGenerator.doGenerate(resource, fsa);
    this.subTask("- write Services", 1);
    this.serviceGenerator.doGenerate(resource, fsa);
    this.subTask("- write Context", 1);
    this.sessionContextGenerator.doGenerate(resource, fsa);
    this.subTask("- write Types", 1);
    this.dataTypeGenerator.doGenerate(resource, fsa);
  }
  
  public void modify(final ResourceSet resourceSet) {
    EList<Resource> _resources = resourceSet.getResources();
    final List<Resource> resources = Lists.<Resource>newArrayList(_resources);
    for (final Resource resource : resources) {
      boolean _and = false;
      EObject _eObject = resource.getEObject("/");
      if (!(_eObject instanceof Model)) {
        _and = false;
      } else {
        boolean _hasProperty = EObjectPropertiesAdapter.hasProperty(resource, GeneratorConstants.MODIFIED_MODEL);
        boolean _not = (!_hasProperty);
        _and = ((_eObject instanceof Model) && _not);
      }
      if (_and) {
        URI _uRI = resource.getURI();
        String _lastSegment = _uRI.lastSegment();
        String _plus = (" - pre-process model " + _lastSegment);
        this.subTask(_plus, 0);
        EObjectPropertiesAdapter.setProperty(resource, GeneratorConstants.MODIFIED_MODEL, Boolean.TRUE);
        URI _uRI_1 = resource.getURI();
        String _lastSegment_1 = _uRI_1.lastSegment();
        String _plus_1 = ("Pre-process model \'" + _lastSegment_1);
        String _plus_2 = (_plus_1 + "\'");
        this.logger.info(_plus_2);
        GeneratorConfiguration _generatorConfiguration = this.getGeneratorConfiguration();
        ModelModifier _modelModifier = _generatorConfiguration.getModelModifier();
        EObject _eObject_1 = resource.getEObject("/");
        _modelModifier.modifyModel(((Model) _eObject_1));
      }
    }
  }
  
  public void subTask(final String name, final int worked) {
    this.monitor.worked(worked);
    this.monitor.subTask(name);
  }
}
