package org.openxma.dsl.generator.impl.substrategy;

import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.openxma.dsl.generator.Config;
import org.openxma.dsl.generator.helper.WidgetExtension;
import org.openxma.dsl.generator.impl.DomPageLogicAttachmentStrategy;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.BrowseControlType;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.StylePropertyBrowseControl;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.scoping.PomElementCollector;
import org.openxma.dsl.pom.util.StyleAccess;

import at.spardat.xma.guidesign.DatePicker;
import at.spardat.xma.guidesign.SimpleCombo;
import at.spardat.xma.guidesign.XMACombo;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAText;
import at.spardat.xma.guidesign.XMAWidget;

public class BrowseModeStrategy {

    private static BrowseModeStrategy instance;

    public static BrowseModeStrategy getInstance() {
        if (instance == null) {
            instance = new BrowseModeStrategy();
        }
        return instance;
    }

    protected boolean canHaveBrowseControl(XMAWidget xmaWidget) {
        if (xmaWidget instanceof XMAText) return true;
        if (xmaWidget instanceof SimpleCombo) return true;
        if (xmaWidget instanceof XMACombo) return true;
        if (xmaWidget instanceof DatePicker) return true;
        return false;
    }

    protected void appendStyleString(StringBuilder buf, String styleString) {
        if (buf.length() > 0) {
            buf.append(" | ");
        }
        buf.append(styleString);
    }

    public void genAdditionalImportsClient(DomPageLogicAttachmentStrategy domPageLogicAttachmentStrategy, List<XMAWidget> xmaElements, LinkedHashSet<String> imports) {
        // Import for browse mode
        for (XMAWidget xmaWidget : xmaElements) {
            if (canHaveBrowseControl(xmaWidget)) {
                GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaWidget);
                if (guiElement != null) {
                    Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(guiElement);
                    if (styleSpecificationProperties != null) {
                        StylePropertyBrowseControl browseControlProp = (StylePropertyBrowseControl)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_BROWSE_CONTROL);
                        if (browseControlProp != null) {
                            imports.add("org.openxma.dsl.platform.xma.client.browse.BrowseMode");
                            break;
                        }
                    }
                }
            }
        }
    }

    public void genAdditionalMethodsClient(StringBuilder buf, XMAPage xmaPage) {
        StringBuilder localBuf = new StringBuilder();
        StringBuilder createWidgetsBuf = new StringBuilder();
        List<XMAWidget> xmaElements = PomElementCollector.getWidgetsOfXMAPage(xmaPage);
        for (XMAWidget xmaWidget : xmaElements) {
            if (canHaveBrowseControl(xmaWidget)) {
                GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaWidget);
                String castString = null;
                if (guiElement != null) {
                    Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(guiElement);
                    if (styleSpecificationProperties != null) {
                        StylePropertyBrowseControl browseControlProp = (StylePropertyBrowseControl)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_BROWSE_CONTROL);
                        if (browseControlProp != null) {
                            StringBuilder styleBuf = new StringBuilder();
                            if (browseControlProp.getControlType() != null && browseControlProp.getControlType() == BrowseControlType.TEXT) {
                                appendStyleString(styleBuf,"BrowseMode.TEXT");
                                castString = "Text";
                            } else {
                                castString = "Label";
                            }
                            if (styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_BACKGROUND_COLOR) != null) {
                                appendStyleString(styleBuf,"BrowseMode.BACKGROUND");

                            }
                            if (styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_FOREGROUND_COLOR) != null) {
                                appendStyleString(styleBuf,"BrowseMode.FOREGROUND");
                            }
                            if (styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_FONT) != null) {
                                appendStyleString(styleBuf,"BrowseMode.FONT");
                            }
                            if (styleBuf.length() ==0) {
                                styleBuf.append("BrowseMode.DEFAULT");
                            }
                            createWidgetsBuf.append("        "+getBrowseWidgetName(xmaWidget)+" = ("+castString+")createBrowseWidget("+xmaWidget.getNamInstance()+"W"+", "+styleBuf+");"+Config.NL);
                            localBuf.append("        attachBrowseWidget("+xmaWidget.getNamInstance()+","+getBrowseWidgetName(xmaWidget)+");"+Config.NL);
                        }
                    }
                }
            }
        }

        if (localBuf.length() > 0) {
            buf.append(Config.NL).append("    public void createBrowseWidgets() {").append(Config.NL);
            buf.append(createWidgetsBuf);
            buf.append("    }").append(Config.NL);
        }

        if (localBuf.length() > 0) {
            buf.append(Config.NL).append("    public void attachBrowseWidgets() {").append(Config.NL);
            buf.append(localBuf);
            buf.append("    }").append(Config.NL);
        }

    }

    protected String getBrowseWidgetName(XMAWidget widget) {
        return widget.getNamInstance()+"_browseW";
    }

    public void genAdditionalMemberVariablesClient(StringBuilder stringBuilder, XMAPage xmaPage) {
        List<XMAWidget> xmaElements = PomElementCollector.getWidgetsOfXMAPage(xmaPage);
        boolean isFirstBrowseControl = true;
        for (XMAWidget xmaWidget : xmaElements) {
            if (canHaveBrowseControl(xmaWidget)) {
                GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaWidget);
                if (guiElement != null) {
                    Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(guiElement);
                    if (styleSpecificationProperties != null) {
                        StylePropertyBrowseControl browseControlProp = (StylePropertyBrowseControl)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_BROWSE_CONTROL);
                        if (browseControlProp != null) {
                            StringBuilder styleBuf = new StringBuilder();
                            if (browseControlProp.getControlType() != null && browseControlProp.getControlType() == BrowseControlType.TEXT) {
                                styleBuf.append("Text "+getBrowseWidgetName(xmaWidget)+";").append(Config.NL);
                            } else if (browseControlProp.getControlType() != null && browseControlProp.getControlType() == BrowseControlType.LABEL) {
                                styleBuf.append("Label "+getBrowseWidgetName(xmaWidget)+";").append(Config.NL);
                            }
                            if (isFirstBrowseControl) {
                                isFirstBrowseControl = false;
                                stringBuilder.append(Config.NL).append("    /* Browse controls */").append(Config.NL);
                            }
                            stringBuilder.append("    "+styleBuf);
                        }
                    }
                }
            }
        }
    }

    public void genCreateWidgetsCodeClient(StringBuilder buf, XMAPage page) {
        List<XMAWidget> xmaElements = PomElementCollector.getWidgetsOfXMAPage(page);
        for (XMAWidget xmaWidget : xmaElements) {
            if (canHaveBrowseControl(xmaWidget)) {
                GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaWidget);
                if (guiElement != null) {
                    Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(guiElement);
                    if (styleSpecificationProperties != null) {
                        StylePropertyBrowseControl browseControlProp = (StylePropertyBrowseControl)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_BROWSE_CONTROL);
                        if (browseControlProp != null) {
                            buf.append("        createBrowseWidgets();").append(Config.NL);
                            break;
                        }
                    }
                }
            }
        }

    }

}
