package org.openxma.dsl.generator.impl.substrategy;

import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.openxma.dsl.generator.Config;
import org.openxma.dsl.generator.helper.WidgetExtension;
import org.openxma.dsl.generator.impl.DomPageLogicAttachmentStrategy;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.scoping.PomElementCollector;
import org.openxma.dsl.pom.util.StyleAccess;

import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.util.WidgetFactoryUtil;

public class EffectStrategy {
    
    private static EffectStrategy instance;
    
    private EffectStrategy() {
        super();
    }

    public static EffectStrategy getInstance() {
        if (instance == null) {
            instance = new EffectStrategy();
        }
        return instance;
    }
    
    abstract class MouseEffectLoop {
        public MouseEffectLoop(XMAPage xmaPage) {
            List<XMAWidget> xmaElements = PomElementCollector.getWidgetsOfXMAPage(xmaPage);                   
            for (XMAWidget xmaWidget : xmaElements) {
                GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaWidget);
                if (guiElement != null) {
                    Map<Integer,StyleSpecificationProperty> styleSpecificationProperties = StyleAccess.getCombinedStyleProperty(guiElement);
                    if (styleSpecificationProperties != null) {
                        StylePropertyOther browseControlProp = (StylePropertyOther)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_OTHER);
                        if (browseControlProp != null && browseControlProp.getWidgetEffects() != null) {
                            boolean canceled = false;
                            for (String effectName : browseControlProp.getWidgetEffects()) {
                                if (!applyLoop(xmaWidget,guiElement,effectName)) {
                                    canceled = true;
                                    break;
                                }
                            }
                            if (canceled) {
                                break;
                            }
                        }
                    }                                                                         
                }
            }                                       
        }
        
        abstract boolean applyLoop(XMAWidget xmaWidget, GuiElement guiElement, String mouseOverEffect);                
    }
    
    public void genAdditionalCreateWidgetsCodeClient(DomPageLogicAttachmentStrategy pageLogicAttachmentStrategy, XMAPage xmaPage, final StringBuilder buf) {            
       new MouseEffectLoop(xmaPage) {
            boolean applyLoop(XMAWidget xmaWidget, GuiElement guiElement, String mouseOverEffect) {
                buf.append("        createEffects();").append(Config.NL);
                return false;
            }            
        };              
    }

    public void genAdditionalMethodsClient(final StringBuilder buf, final XMAPage xmaPage) {
        final StringBuilder methodBuffer = new StringBuilder();
        new MouseEffectLoop(xmaPage) {
            boolean applyLoop(XMAWidget xmaWidget, GuiElement guiElement, String mouseOverEffect) {                
                if (methodBuffer.length() == 0) {
                    String effectFactoryClassName = WidgetFactoryUtil.getName(getEffectFactoryClass(xmaPage));
                    methodBuffer.append(Config.NL).append("    /**");
                    methodBuffer.append(Config.NL).append("     * This method creates effects for widgets.");
                    methodBuffer.append(Config.NL).append("     */");                    
                    methodBuffer.append(Config.NL).append("    protected void createEffects() {");
                    methodBuffer.append(Config.NL).append("        "+effectFactoryClassName+" effectFactory = "+effectFactoryClassName+".getInstance();").append(Config.NL);
                }
                methodBuffer.append("        setEffect("+xmaWidget.getNamInstance()+"W,effectFactory.createEffect(\""+mouseOverEffect+"\"));").append(Config.NL);
                return true;
            }            
        };
        if (methodBuffer.length() > 0) {
            methodBuffer.append("    }").append(Config.NL);
            buf.append(methodBuffer);
            buf.append(Config.NL);
        }
    }

    public void genAdditionalImportsClient(final XMAPage xmaPage, final LinkedHashSet<String> imports) {
        new MouseEffectLoop(xmaPage) {
            boolean applyLoop(XMAWidget xmaWidget, GuiElement guiElement, String mouseOverEffect) {
                imports.add(getEffectFactoryClass(xmaPage));
                return false;
            }            
        };                                    
    }
    
    public String getEffectFactoryClass(XMAPage xmaPage) {
        String effectClass = xmaPage.getComponent().getEffectFactoryClass(); 
        if (effectClass == null) {
            effectClass = "org.openxma.dsl.platform.xma.client.effects.DslEffectFactory";
        }
        return effectClass;        
    }

}
