package org.openxma.dsl.generator.impl.substrategy;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.openxma.dsl.generator.Config;

import at.spardat.xma.guidesign.BDAttribute;
import at.spardat.xma.guidesign.BusinessData;
import at.spardat.xma.guidesign.impl.XMATableImpl;

public class TableLogicStrategy {
	
	private XMATableImpl tableImpl;
	
	public TableLogicStrategy(XMATableImpl tableImpl) {
		this.tableImpl = tableImpl;
	}
	
	public void genAdditionalFillMethod(StringBuilder buffer) {
        List<BusinessData> businessDatas = tableImpl.getBusinessDatas();
        
        /* Key of table */
        BDAttribute keyAttribute = tableImpl.getDataAttribute();
        BusinessData keyDataObject = keyAttribute.getBdObject();
        
        /* Create list of data objects ordered according the access chain */
        List<BusinessData> tableDatas = createOrderedListOfDataObjects(businessDatas, keyDataObject);          
        
        /* Method header */
		buffer.append( 
			"    /**"+Config.NL+
		    "     * This method iterates over the collection and calls <tt>"+tableImpl.getNamInstance()+"AddRow</tt> for"+Config.NL+
		    "     * each element."+Config.NL+
		    "     */"+Config.NL+
		    "     public void "+tableImpl.getNamInstance()+"Fill() {"+Config.NL+
		    "         "+tableImpl.getNamInstance()+".clear();"+Config.NL);
		
		/* Iterators for other "root"-BusinessDatas then the "key"-BusinessData*/
		for (BusinessData bd : tableDatas) {
			if (bd.getIsAttributeOf() == null && bd != keyDataObject) {
				buffer.append("         java.util.Iterator "+getLocalName(bd)+"Iter = "+genDataAccess(bd)+".iterator();").append(Config.NL);
			}
		}
		
		/* Iterator for the "key"-BusinessData */
		buffer.append("         for (java.util.Iterator iter = "+genDataAccess(keyDataObject)+".iterator(); iter.hasNext();) {").append(Config.NL);
		
		/* Create and assign values to temporary variables of the BusinessData */
		Iterator<BusinessData> dataIterator = tableDatas.iterator();
		while (dataIterator.hasNext()) {
			 BusinessData aData = (BusinessData)dataIterator.next();			
			 buffer.append(" 	         ");
			 genCreateInstance(buffer,aData,keyDataObject);
			 buffer.append(";").append(Config.NL);
		}
		
		/* Call addRow and put the temporary variables as parameter */
		buffer.append("             "+tableImpl.getNamInstance()+"AddRow(");
		boolean first = true;
		for (BusinessData bd : businessDatas) {
			if (!first) buffer.append(", ");				
			buffer.append(getLocalName(bd));
			first=false;
		}
		buffer.append(");").append(Config.NL);			
		buffer.append(
		    "         }").append(Config.NL).append(
		    "     }").append(Config.NL);
	}

	private List<BusinessData> createOrderedListOfDataObjects(List<BusinessData> businessDatas, BusinessData keyDataObject) {
		List<BusinessData> tableDatas = new ArrayList<BusinessData>();
        addDataObjectAndTheirChildren(businessDatas, tableDatas, keyDataObject);
        for (BusinessData bd : businessDatas) {
        	if (bd.getIsAttributeOf() == null && tableDatas.indexOf(bd) == -1) {
                addDataObjectAndTheirChildren(businessDatas, tableDatas, bd);        		
        	}
        }
        for (BusinessData bd : businessDatas) {
        	if (tableDatas.indexOf(bd) == -1) {
                throw new RuntimeException("Not supported data mapping for: "+tableImpl.getNamInstance());        		
        	}
        }
		return tableDatas;
	}
	
	private void addDataObjectAndTheirChildren(List<BusinessData> businessDatas, List<BusinessData> tableDatas, BusinessData aBd) {
		tableDatas.add(aBd);		
		int dataIndex = tableDatas.indexOf(aBd);		
		while (dataIndex < tableDatas.size()) {
        	BusinessData parentBd = tableDatas.get(dataIndex);        	            
	        for (BusinessData bd : businessDatas) {        	
	        	if (bd.getIsAttributeOf() == parentBd) {
	        		tableDatas.add(bd);        		        		
	        	}
	        }
	        dataIndex++;
        }
	}	
	
	private void genCreateInstance(StringBuilder buffer, BusinessData bd, BusinessData keyDataObject) {
		buffer.append(bd.getNamBDClass()+" "+getLocalName(bd)+" = ");
		boolean isKeyData = ( bd == keyDataObject);		
		if (isKeyData) {
			buffer.append("("+bd.getNamBDClass()+") iter.next()");
		} else {
			if (bd.getIsAttributeOf() != null) {
				boolean parentIsKey = (bd.getIsAttributeOf() == keyDataObject);
				if (!parentIsKey) {
					buffer.append("("+getLocalName(bd.getIsAttributeOf())+" != null ) ? ");
				}
				buffer.append(getLocalName(bd.getIsAttributeOf()));
	            buffer.append(".get"+bd.getNamInstance().substring(0,1).toUpperCase()+bd.getNamInstance().substring(1)+"()");
	            if (!parentIsKey) {	            
	            	buffer.append(" : null");
	            }
			} else {
				buffer.append("("+getLocalName(bd)+"Iter.hasNext()) ? ("+bd.getNamBDClass()+") "+getLocalName(bd)+"Iter.next() : null");
			}
		}
	}
	
	private String getLocalName(BusinessData bd) {
		String nameOfParent = "";
		if (bd.getIsAttributeOf() != null) {
			nameOfParent = getLocalName(bd.getIsAttributeOf())+"_";
		}
		if (bd.isYnCollection()) {
			return nameOfParent+bd.getNamInstance()+"Element";
		} else {
			return nameOfParent+bd.getNamInstance();
		}
	}
	
	protected String genDataAccess(BusinessData data) {
        StringWriter stringWriter = new StringWriter(); 
        PrintWriter out = new PrintWriter(stringWriter);        
        if(data.isYnGenerated()) {
        	tableImpl.getPageComposite(tableImpl).getPage().genDataAccess(out,data);
        } else {
            out.print(data.getNamInstance());
        }        
        out.flush();
        out.close();        
        return stringWriter.toString();				
	}	

}
