package org.openxma.dsl.generator.lib

import com.google.common.collect.Sets
import java.util.Set
import org.eclipse.emf.common.notify.Notifier
import org.eclipse.emf.common.notify.impl.AdapterImpl
import org.eclipse.emf.ecore.EObject
import org.eclipse.xtext.util.Strings
import org.openxma.dsl.generator.Config;

import static org.apache.commons.logging.LogFactory.*

import static extension org.eclipse.emf.ecore.util.EcoreUtil.*

class ImportAdapter extends AdapterImpl {
	val logger = getLog(getClass().getName());
	var String lastInsertMarker = null;
	val Set<String> imports = Sets::newTreeSet

	def static String addImportsHere(EObject eObject) {
		val adapter = get(eObject)
		if(adapter.lastInsertMarker != null) {
			adapter.logger.warn("Multiple import sections")
		}
		adapter.imports.clear();
		adapter.lastInsertMarker = "_Placeholder_for_a_lazy_import_section_" + System::currentTimeMillis() + "___"
		adapter.lastInsertMarker
	}

	def static addImport(EObject eObject, String qualifiedName) {
		if(!qualifiedName.startsWith("java.lang")) {
			get(eObject).imports.add(qualifiedName);
		}
		Strings::lastToken(qualifiedName, ".")
	}

	def static String resolve(Notifier notifier, String content) {
		var result = content
		val adapter = notifier.getExistingAdapter(typeof(ImportAdapter)) as ImportAdapter
		if(adapter.lastInsertMarker != null) {
			val importBuilder = new StringBuilder()
			val iterator = adapter.imports.iterator();
			while(iterator.hasNext()) {
				val name = iterator.next();
				if(name.indexOf('.') > -1) {
					importBuilder.append("import ").append(name).append(";");
					if(iterator.hasNext()) {
						importBuilder.append(Config::NL);
					}
				}
			}
			if(importBuilder.length() == 0) {
				result = result.replace(adapter.lastInsertMarker + Config::NL, "");
			} else {
				result = result.replace(adapter.lastInsertMarker, importBuilder.toString());
			}
			adapter.lastInsertMarker = null
		}
		result
	}

	def static ImportAdapter get(EObject eObject) {
		val notifier = eObject.eResource.resourceSet
		var ImportAdapter adapter = notifier.getExistingAdapter(typeof(ImportAdapter))  as ImportAdapter
		if(adapter == null) {
			adapter = new ImportAdapter()
			notifier.eAdapters().add(adapter)
		}
		adapter
	}

	override boolean isAdapterForType(Object type) {
		return type == getClass();
	}
}